library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library unisim;
use unisim.vcomponents.all;

entity ddr4_test is
generic(
  simulation         : boolean := false;
  -- Following two generics ignored if not simulation
  simulation_offset  : natural := 99;   -- Value for ctl_offset
  simulation_length  : natural := 213); -- ctl_length is set to (simulation_length - 1)
port(
  refclk300_p        : in    std_logic;
  refclk300_n        : in    std_logic;
  --
  -- synthesis translate_off
  -- Status outputs used in simulation only.
  stat_rst           : out   std_logic_vector(3 downto 0);
  stat_err           : out   std_logic_vector(3 downto 0);
  stat_done          : out   std_logic_vector(3 downto 0);
  stat_init          : out   std_logic_vector(3 downto 0);
  -- synthesis translate_on
  --
  c01_sys_clk_p      : in    std_logic;
  c01_sys_clk_n      : in    std_logic;
  c23_sys_clk_p      : in    std_logic;
  c23_sys_clk_n      : in    std_logic;
  --
  c0_ddr4_par        : out   std_logic;
  c0_ddr4_ten        : out   std_logic;
  c0_ddr4_alert_n    : in    std_logic;
  c0_ddr4_act_n      : out   std_logic;
  c0_ddr4_adr        : out   std_logic_vector (16 downto 0);
  c0_ddr4_ba         : out   std_logic_vector (1 downto 0);
  c0_ddr4_bg         : out   std_logic_vector (0 to 0);
  c0_ddr4_cke        : out   std_logic_vector (0 to 0);
  c0_ddr4_odt        : out   std_logic_vector (0 to 0);
  c0_ddr4_cs_n       : out   std_logic_vector (0 to 0);
  c0_ddr4_ck_t       : out   std_logic_vector (0 to 0);
  c0_ddr4_ck_c       : out   std_logic_vector (0 to 0);
  c0_ddr4_reset_n    : out   std_logic;
  c0_ddr4_dm_dbi_n   : inout std_logic_vector (3 downto 0);
  c0_ddr4_dq         : inout std_logic_vector (31 downto 0);
  c0_ddr4_dqs_t      : inout std_logic_vector (3 downto 0);
  c0_ddr4_dqs_c      : inout std_logic_vector (3 downto 0);
  --
  c1_ddr4_par        : out   std_logic;
  c1_ddr4_ten        : out   std_logic;
  c1_ddr4_alert_n    : in    std_logic;
  c1_ddr4_act_n      : out   std_logic;
  c1_ddr4_adr        : out   std_logic_vector (16 downto 0);
  c1_ddr4_ba         : out   std_logic_vector (1 downto 0);
  c1_ddr4_bg         : out   std_logic_vector (0 to 0);
  c1_ddr4_cke        : out   std_logic_vector (0 to 0);
  c1_ddr4_odt        : out   std_logic_vector (0 to 0);
  c1_ddr4_cs_n       : out   std_logic_vector (0 to 0);
  c1_ddr4_ck_t       : out   std_logic_vector (0 to 0);
  c1_ddr4_ck_c       : out   std_logic_vector (0 to 0);
  c1_ddr4_reset_n    : out   std_logic;
  c1_ddr4_dm_dbi_n   : inout std_logic_vector (3 downto 0);
  c1_ddr4_dq         : inout std_logic_vector (31 downto 0);
  c1_ddr4_dqs_t      : inout std_logic_vector (3 downto 0);
  c1_ddr4_dqs_c      : inout std_logic_vector (3 downto 0);
  --
  c2_ddr4_par        : out   std_logic;
  c2_ddr4_ten        : out   std_logic;
  c2_ddr4_alert_n    : in    std_logic;
  c2_ddr4_act_n      : out   std_logic;
  c2_ddr4_adr        : out   std_logic_vector (16 downto 0);
  c2_ddr4_ba         : out   std_logic_vector (1 downto 0);
  c2_ddr4_bg         : out   std_logic_vector (0 to 0);
  c2_ddr4_cke        : out   std_logic_vector (0 to 0);
  c2_ddr4_odt        : out   std_logic_vector (0 to 0);
  c2_ddr4_cs_n       : out   std_logic_vector (0 to 0);
  c2_ddr4_ck_t       : out   std_logic_vector (0 to 0);
  c2_ddr4_ck_c       : out   std_logic_vector (0 to 0);
  c2_ddr4_reset_n    : out   std_logic;
  c2_ddr4_dm_dbi_n   : inout std_logic_vector (3 downto 0);
  c2_ddr4_dq         : inout std_logic_vector (31 downto 0);
  c2_ddr4_dqs_t      : inout std_logic_vector (3 downto 0);
  c2_ddr4_dqs_c      : inout std_logic_vector (3 downto 0);
  --
  c3_ddr4_par        : out   std_logic;
  c3_ddr4_ten        : out   std_logic;
  c3_ddr4_alert_n    : in    std_logic;
  c3_ddr4_act_n      : out   std_logic;
  c3_ddr4_adr        : out   std_logic_vector (16 downto 0);
  c3_ddr4_ba         : out   std_logic_vector (1 downto 0);
  c3_ddr4_bg         : out   std_logic_vector (0 to 0);
  c3_ddr4_cke        : out   std_logic_vector (0 to 0);
  c3_ddr4_odt        : out   std_logic_vector (0 to 0);
  c3_ddr4_cs_n       : out   std_logic_vector (0 to 0);
  c3_ddr4_ck_t       : out   std_logic_vector (0 to 0);
  c3_ddr4_ck_c       : out   std_logic_vector (0 to 0);
  c3_ddr4_reset_n    : out   std_logic;
  c3_ddr4_dm_dbi_n   : inout std_logic_vector (3 downto 0);
  c3_ddr4_dq         : inout std_logic_vector (31 downto 0);
  c3_ddr4_dqs_t      : inout std_logic_vector (3 downto 0);
  c3_ddr4_dqs_c      : inout std_logic_vector (3 downto 0));
end entity;

architecture admxrcku1 of ddr4_test is

  constant MEMORY_DATA_WIDTH : natural := 256;
  constant MEMORY_ADDR_WIDTH : natural := 26; -- 32-byte data (2 GiB per bank / 8 GiB total)

  -- Calculate pack_factor
  constant PHYS_DATA_WIDTH : natural := 32; -- 4-byte data
  constant PACK_FACTOR : natural := MEMORY_DATA_WIDTH / PHYS_DATA_WIDTH;

  -- Calculate AXI4 address width; should exactly match that of MIG's AXI4 interface
  constant NUM_BYTE_LANES : natural := MEMORY_DATA_WIDTH / 8;
  constant LOG2_NUM_BYTE_LANES : natural := natural(floor(log2(real(NUM_BYTE_LANES) + 0.5)));
  constant M_AXI_AXADDR_WIDTH : natural := MEMORY_ADDR_WIDTH + LOG2_NUM_BYTE_LANES; -- byte address

  -- AXI4 data width to match that of MIG's AXI4 interface
  constant M_AXI_XDATA_WIDTH : natural := MEMORY_DATA_WIDTH;

  -- AXI4 burst length width to match that of MIG's AXI4 interface
  constant M_AXI_AXLEN_WIDTH : natural := 8;

  -- AXI4 TID width to match that of MIG's AXI4 interface
  constant M_AXI_XID_WIDTH : natural := 4;

  -- MIG with AXI4 interface and no ECC appears to correctly handle data masking.
  constant PARTIAL_WRITES : boolean := true;
  -- Ultrascale DDR4
  constant PACK_REARRANGED : boolean := true;

  -- Can count up to 2^32 errors
  constant ERROR_COUNT_WIDTH : natural := 32;
  -- Log up to 256 errors
  constant ERROR_INDEX_WIDTH : natural := 8;


  component ddr4sdram
  port(
    sys_rst                 : in    std_logic;
    c0_sys_clk_i            : in    std_logic;
    --
    c0_ddr4_act_n           : out   std_logic;
    c0_ddr4_adr             : out   std_logic_vector(16 downto 0);
    c0_ddr4_ba              : out   std_logic_vector(1 downto 0);
    c0_ddr4_bg              : out   std_logic_vector(0 to 0);
    c0_ddr4_cke             : out   std_logic_vector(0 to 0);
    c0_ddr4_odt             : out   std_logic_vector(0 to 0);
    c0_ddr4_cs_n            : out   std_logic_vector(0 to 0);
    c0_ddr4_ck_t            : out   std_logic_vector(0 to 0);
    c0_ddr4_ck_c            : out   std_logic_vector(0 to 0);
    c0_ddr4_reset_n         : out   std_logic;
    c0_ddr4_dm_dbi_n        : inout std_logic_vector(3 downto 0);
    c0_ddr4_dq              : inout std_logic_vector(31 downto 0); -- 4-byte data
    c0_ddr4_dqs_t           : inout std_logic_vector(3 downto 0);
    c0_ddr4_dqs_c           : inout std_logic_vector(3 downto 0);
    --
    c0_ddr4_aresetn         : in    std_logic;
    c0_ddr4_ui_clk          : out   std_logic;
    c0_ddr4_ui_clk_sync_rst : out   std_logic;
    c0_init_calib_complete  : out   std_logic;
    --
    c0_ddr4_s_axi_awid      : in    std_logic_vector(3 downto 0);
    c0_ddr4_s_axi_awaddr    : in    std_logic_vector(M_AXI_AXADDR_WIDTH-1 downto 0); -- byte address
    c0_ddr4_s_axi_awlen     : in    std_logic_vector(7 downto 0);
    c0_ddr4_s_axi_awsize    : in    std_logic_vector(2 downto 0);
    c0_ddr4_s_axi_awburst   : in    std_logic_vector(1 downto 0);
    c0_ddr4_s_axi_awlock    : in    std_logic_vector(0 downto 0);
    c0_ddr4_s_axi_awcache   : in    std_logic_vector(3 downto 0);
    c0_ddr4_s_axi_awprot    : in    std_logic_vector(2 downto 0);
    c0_ddr4_s_axi_awqos     : in    std_logic_vector(3 downto 0);
    c0_ddr4_s_axi_awvalid   : in    std_logic;
    c0_ddr4_s_axi_awready   : out   std_logic;
    c0_ddr4_s_axi_wdata     : in    std_logic_vector(255 downto 0);
    c0_ddr4_s_axi_wstrb     : in    std_logic_vector(31 downto 0);
    c0_ddr4_s_axi_wlast     : in    std_logic;
    c0_ddr4_s_axi_wvalid    : in    std_logic;
    c0_ddr4_s_axi_wready    : out   std_logic;
    c0_ddr4_s_axi_bready    : in    std_logic;
    c0_ddr4_s_axi_bid       : out   std_logic_vector(3 downto 0);
    c0_ddr4_s_axi_bresp     : out   std_logic_vector(1 downto 0);
    c0_ddr4_s_axi_bvalid    : out   std_logic;
    c0_ddr4_s_axi_arid      : in    std_logic_vector(3 downto 0);
    c0_ddr4_s_axi_araddr    : in    std_logic_vector(M_AXI_AXADDR_WIDTH-1 downto 0); -- byte address
    c0_ddr4_s_axi_arlen     : in    std_logic_vector(7 downto 0);
    c0_ddr4_s_axi_arsize    : in    std_logic_vector(2 downto 0);
    c0_ddr4_s_axi_arburst   : in    std_logic_vector(1 downto 0);
    c0_ddr4_s_axi_arlock    : in    std_logic_vector(0 downto 0);
    c0_ddr4_s_axi_arcache   : in    std_logic_vector(3 downto 0);
    c0_ddr4_s_axi_arprot    : in    std_logic_vector(2 downto 0);
    c0_ddr4_s_axi_arqos     : in    std_logic_vector(3 downto 0);
    c0_ddr4_s_axi_arvalid   : in    std_logic;
    c0_ddr4_s_axi_arready   : out   std_logic;
    c0_ddr4_s_axi_rready    : in    std_logic;
    c0_ddr4_s_axi_rlast     : out   std_logic;
    c0_ddr4_s_axi_rvalid    : out   std_logic;
    c0_ddr4_s_axi_rresp     : out   std_logic_vector(1 downto 0);
    c0_ddr4_s_axi_rid       : out   std_logic_vector(3 downto 0);
    c0_ddr4_s_axi_rdata     : out   std_logic_vector(255 downto 0));
  end component;

  component memory_exerciser_axi4
    generic(
      memory_data_width  : in    natural;
      memory_addr_width  : in    natural;
      m_axi_axaddr_width : in    natural;
      m_axi_xdata_width  : in    natural;
      m_axi_xid_width    : in    natural;
      m_axi_axlen_width  : in    natural;
      pack_factor        : in    natural;
      pack_rearranged    : in    boolean;
      partial_writes     : in    boolean;
      error_count_width  : in    natural;
      error_index_width  : in    natural);
    port(
      aresetn            : in    std_logic;
      aclk               : in    std_logic;
      -- AXI4 MM interface (master)
      m_axi_araddr       : out   std_logic_vector(m_axi_axaddr_width - 1 downto 0);
      m_axi_arburst      : out   std_logic_vector(1 downto 0);
      m_axi_arcache      : out   std_logic_vector(3 downto 0);
      m_axi_arid         : out   std_logic_vector(m_axi_xid_width - 1 downto 0);
      m_axi_arlen        : out   std_logic_vector(m_axi_axlen_width - 1 downto 0);
      m_axi_arlock       : out   std_logic_vector(0 downto 0);
      m_axi_arprot       : out   std_logic_vector(2 downto 0);
      m_axi_arqos        : out   std_logic_vector(3 downto 0);
      m_axi_arready      : in    std_logic;
      m_axi_arregion     : out   std_logic_vector(3 downto 0);
      m_axi_arsize       : out   std_logic_vector(2 downto 0);
      m_axi_arvalid      : out   std_logic;
      m_axi_awaddr       : out   std_logic_vector(m_axi_axaddr_width - 1 downto 0);
      m_axi_awburst      : out   std_logic_vector(1 downto 0);
      m_axi_awcache      : out   std_logic_vector(3 downto 0);
      m_axi_awid         : out   std_logic_vector(m_axi_xid_width - 1 downto 0);
      m_axi_awlen        : out   std_logic_vector(m_axi_axlen_width - 1 downto 0);
      m_axi_awlock       : out   std_logic_vector(0 downto 0);
      m_axi_awprot       : out   std_logic_vector(2 downto 0);
      m_axi_awqos        : out   std_logic_vector(3 downto 0);
      m_axi_awready      : in    std_logic;
      m_axi_awregion     : out   std_logic_vector(3 downto 0);
      m_axi_awsize       : out   std_logic_vector(2 downto 0);
      m_axi_awvalid      : out   std_logic;
      m_axi_wdata        : out   std_logic_vector(m_axi_xdata_width - 1 downto 0);
      m_axi_wlast        : out   std_logic;
      m_axi_wstrb        : out   std_logic_vector(m_axi_xdata_width / 8 - 1 downto 0);
      m_axi_wready       : in    std_logic;
      m_axi_wvalid       : out   std_logic;
      m_axi_bid          : in    std_logic_vector(m_axi_xid_width - 1 downto 0);
      m_axi_bready       : out   std_logic;
      m_axi_bresp        : in    std_logic_vector(1 downto 0);
      m_axi_bvalid       : in    std_logic;
      m_axi_rdata        : in    std_logic_vector(m_axi_xdata_width - 1 downto 0);
      m_axi_rid          : in    std_logic_vector(m_axi_xid_width - 1 downto 0);
      m_axi_rlast        : in    std_logic;
      m_axi_rready       : out   std_logic;
      m_axi_rresp        : in    std_logic_vector(1 downto 0);
      m_axi_rvalid       : in    std_logic;
      -- Control interface
      ctl_go             : in    std_logic;                                             -- Pulse high for at least one cycle to initiate test
      ctl_offset         : in    std_logic_vector(memory_addr_width - 1 downto 0);      -- Starting logical address of test
      ctl_length         : in    std_logic_vector(memory_addr_width - 1 downto 0);      -- Number of logical words to test, minus 1
      ctl_ph_en          : in    std_logic_vector(7 downto 0) := (others => '1');       -- Enables various phases of test
      ctl_rw_en          : in    std_logic_vector(1 downto 0) := (others => '1');       -- 01 => do writes only, 10 => do reads only, 11 => do reads & writes
      ctl_done           : out   std_logic;                                             -- High indicates finished or idle
      -- Assert to read out an error (latency 2 clock cycles)
      ctl_erd            : in    std_logic;
      -- Index of error to read out
      ctl_eindex         : in    std_logic_vector(error_index_width - 1 downto 0);
      -- Qualified by 'ctl_done', high indicates at least one error occurred
      ctl_error          : out   std_logic;
      -- Qualified by 'ctl_done', actual number of errors
      ctl_ecount         : out   std_logic_vector(error_count_width - 1 downto 0);
      -- Qualified by 'ctl_done', asserted if error log is full
      ctl_efull          : out   std_logic;
      -- Qualified by 'ctl_done', asserted if errors were lost due to error log full
      ctl_elost          : out   std_logic;
      -- Address of selected error
      ctl_eaddr          : out   std_logic_vector(memory_addr_width - 1 downto 0);
      -- Phase in which selected error occurred
      ctl_ephase         : out   std_logic_vector(2 downto 0);
      -- Bits that were incorrect when selected error occurred
      ctl_ebits          : out   std_logic_vector(memory_data_width - 1 downto 0);
      -- Actual data when selected error occurred
      ctl_edata          : out   std_logic_vector(memory_data_width - 1 downto 0));
  end component;


  signal refclk300_ibuf : std_logic;

  signal c0_ddr4_alert_n_nc : std_logic;
  signal c1_ddr4_alert_n_nc : std_logic;
  signal c2_ddr4_alert_n_nc : std_logic;
  signal c3_ddr4_alert_n_nc : std_logic;

  -- Stop unused alert inputs being optimized
  attribute dont_touch : string;
  attribute dont_touch of c0_ddr4_alert_n_nc : signal is "true";
  attribute dont_touch of c1_ddr4_alert_n_nc : signal is "true";
  attribute dont_touch of c2_ddr4_alert_n_nc : signal is "true";
  attribute dont_touch of c3_ddr4_alert_n_nc : signal is "true";

  signal vio_clk : std_logic;
  signal vio0_reset : std_logic;
  signal vio1_reset : std_logic;
  signal vio2_reset : std_logic;
  signal vio3_reset : std_logic;

  -- Stop VIO reset outputs being renamed (for Hardware Manager)
  attribute dont_touch of vio0_reset : signal is "true";
  attribute dont_touch of vio1_reset : signal is "true";
  attribute dont_touch of vio2_reset : signal is "true";
  attribute dont_touch of vio3_reset : signal is "true";

  signal c01_sys_clk_i : std_logic;
  signal c23_sys_clk_i : std_logic;

  signal c0_sys_rst : std_logic;
  signal c0_init_calib_complete : std_logic;
  signal c0_ui_clk : std_logic;
  signal c0_ui_clk_sync_rst : std_logic;
  signal c0_aresetn : std_logic;

  signal axi0_araddr : std_logic_vector(M_AXI_AXADDR_WIDTH - 1 downto 0);
  signal axi0_arburst : std_logic_vector(1 downto 0);
  signal axi0_arcache : std_logic_vector(3 downto 0);
  signal axi0_arid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi0_arlen : std_logic_vector(M_AXI_AXLEN_WIDTH - 1 downto 0);
  signal axi0_arlock : std_logic_vector(0 downto 0);
  signal axi0_arprot : std_logic_vector(2 downto 0);
  signal axi0_arqos : std_logic_vector(3 downto 0);
  signal axi0_arready : std_logic;
  signal axi0_arsize : std_logic_vector(2 downto 0);
  signal axi0_arvalid : std_logic;
  signal axi0_awaddr : std_logic_vector(M_AXI_AXADDR_WIDTH - 1 downto 0);
  signal axi0_awburst : std_logic_vector(1 downto 0);
  signal axi0_awcache : std_logic_vector(3 downto 0);
  signal axi0_awid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi0_awlen : std_logic_vector(M_AXI_AXLEN_WIDTH - 1 downto 0);
  signal axi0_awlock : std_logic_vector(0 downto 0);
  signal axi0_awprot : std_logic_vector(2 downto 0);
  signal axi0_awqos  : std_logic_vector(3 downto 0);
  signal axi0_awready : std_logic;
  signal axi0_awsize : std_logic_vector(2 downto 0);
  signal axi0_awvalid : std_logic;
  signal axi0_wdata : std_logic_vector(M_AXI_XDATA_WIDTH - 1 downto 0);
  signal axi0_wlast : std_logic;
  signal axi0_wstrb : std_logic_vector(M_AXI_XDATA_WIDTH / 8 - 1 downto 0);
  signal axi0_wready : std_logic;
  signal axi0_wvalid : std_logic;
  signal axi0_bid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi0_bready : std_logic;
  signal axi0_bresp : std_logic_vector(1 downto 0);
  signal axi0_bvalid : std_logic;
  signal axi0_rdata : std_logic_vector(M_AXI_XDATA_WIDTH - 1 downto 0);
  signal axi0_rid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi0_rlast : std_logic;
  signal axi0_rready : std_logic;
  signal axi0_rresp : std_logic_vector(1 downto 0);
  signal axi0_rvalid : std_logic;

  signal ctl0_go     : std_logic;
  signal ctl0_offset : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl0_length : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl0_ph_en  : std_logic_vector(7 downto 0);
  signal ctl0_done   : std_logic;
  signal ctl0_erd    : std_logic;
  signal ctl0_eindex : std_logic_vector(ERROR_INDEX_WIDTH - 1 downto 0);
  signal ctl0_error  : std_logic;
  signal ctl0_ecount : std_logic_vector(ERROR_COUNT_WIDTH - 1 downto 0);
  signal ctl0_efull  : std_logic;
  signal ctl0_elost  : std_logic;
  signal ctl0_eaddr  : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl0_ephase : std_logic_vector(2 downto 0);
  signal ctl0_ebits  : std_logic_vector(MEMORY_DATA_WIDTH - 1 downto 0);
  signal ctl0_edata  : std_logic_vector(MEMORY_DATA_WIDTH - 1 downto 0);

  signal c1_sys_rst : std_logic;
  signal c1_init_calib_complete : std_logic;
  signal c1_ui_clk : std_logic;
  signal c1_ui_clk_sync_rst : std_logic;
  signal c1_aresetn : std_logic;

  signal axi1_araddr : std_logic_vector(M_AXI_AXADDR_WIDTH - 1 downto 0);
  signal axi1_arburst : std_logic_vector(1 downto 0);
  signal axi1_arcache : std_logic_vector(3 downto 0);
  signal axi1_arid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi1_arlen : std_logic_vector(M_AXI_AXLEN_WIDTH - 1 downto 0);
  signal axi1_arlock : std_logic_vector(0 downto 0);
  signal axi1_arprot : std_logic_vector(2 downto 0);
  signal axi1_arqos : std_logic_vector(3 downto 0);
  signal axi1_arready : std_logic;
  signal axi1_arsize : std_logic_vector(2 downto 0);
  signal axi1_arvalid : std_logic;
  signal axi1_awaddr : std_logic_vector(M_AXI_AXADDR_WIDTH - 1 downto 0);
  signal axi1_awburst : std_logic_vector(1 downto 0);
  signal axi1_awcache : std_logic_vector(3 downto 0);
  signal axi1_awid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi1_awlen : std_logic_vector(M_AXI_AXLEN_WIDTH - 1 downto 0);
  signal axi1_awlock : std_logic_vector(0 downto 0);
  signal axi1_awprot : std_logic_vector(2 downto 0);
  signal axi1_awqos  : std_logic_vector(3 downto 0);
  signal axi1_awready : std_logic;
  signal axi1_awsize : std_logic_vector(2 downto 0);
  signal axi1_awvalid : std_logic;
  signal axi1_wdata : std_logic_vector(M_AXI_XDATA_WIDTH - 1 downto 0);
  signal axi1_wlast : std_logic;
  signal axi1_wstrb : std_logic_vector(M_AXI_XDATA_WIDTH / 8 - 1 downto 0);
  signal axi1_wready : std_logic;
  signal axi1_wvalid : std_logic;
  signal axi1_bid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi1_bready : std_logic;
  signal axi1_bresp : std_logic_vector(1 downto 0);
  signal axi1_bvalid : std_logic;
  signal axi1_rdata : std_logic_vector(M_AXI_XDATA_WIDTH - 1 downto 0);
  signal axi1_rid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi1_rlast : std_logic;
  signal axi1_rready : std_logic;
  signal axi1_rresp : std_logic_vector(1 downto 0);
  signal axi1_rvalid : std_logic;

  signal ctl1_go     : std_logic;
  signal ctl1_offset : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl1_length : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl1_ph_en  : std_logic_vector(7 downto 0);
  signal ctl1_done   : std_logic;
  signal ctl1_erd    : std_logic;
  signal ctl1_eindex : std_logic_vector(ERROR_INDEX_WIDTH - 1 downto 0);
  signal ctl1_error  : std_logic;
  signal ctl1_ecount : std_logic_vector(ERROR_COUNT_WIDTH - 1 downto 0);
  signal ctl1_efull  : std_logic;
  signal ctl1_elost  : std_logic;
  signal ctl1_eaddr  : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl1_ephase : std_logic_vector(2 downto 0);
  signal ctl1_ebits  : std_logic_vector(MEMORY_DATA_WIDTH - 1 downto 0);
  signal ctl1_edata  : std_logic_vector(MEMORY_DATA_WIDTH - 1 downto 0);

  signal c2_sys_rst : std_logic;
  signal c2_init_calib_complete : std_logic;
  signal c2_ui_clk : std_logic;
  signal c2_ui_clk_sync_rst : std_logic;
  signal c2_aresetn : std_logic;

  signal axi2_araddr : std_logic_vector(M_AXI_AXADDR_WIDTH - 1 downto 0);
  signal axi2_arburst : std_logic_vector(1 downto 0);
  signal axi2_arcache : std_logic_vector(3 downto 0);
  signal axi2_arid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi2_arlen : std_logic_vector(M_AXI_AXLEN_WIDTH - 1 downto 0);
  signal axi2_arlock : std_logic_vector(0 downto 0);
  signal axi2_arprot : std_logic_vector(2 downto 0);
  signal axi2_arqos : std_logic_vector(3 downto 0);
  signal axi2_arready : std_logic;
  signal axi2_arsize : std_logic_vector(2 downto 0);
  signal axi2_arvalid : std_logic;
  signal axi2_awaddr : std_logic_vector(M_AXI_AXADDR_WIDTH - 1 downto 0);
  signal axi2_awburst : std_logic_vector(1 downto 0);
  signal axi2_awcache : std_logic_vector(3 downto 0);
  signal axi2_awid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi2_awlen : std_logic_vector(M_AXI_AXLEN_WIDTH - 1 downto 0);
  signal axi2_awlock : std_logic_vector(0 downto 0);
  signal axi2_awprot : std_logic_vector(2 downto 0);
  signal axi2_awqos  : std_logic_vector(3 downto 0);
  signal axi2_awready : std_logic;
  signal axi2_awsize : std_logic_vector(2 downto 0);
  signal axi2_awvalid : std_logic;
  signal axi2_wdata : std_logic_vector(M_AXI_XDATA_WIDTH - 1 downto 0);
  signal axi2_wlast : std_logic;
  signal axi2_wstrb : std_logic_vector(M_AXI_XDATA_WIDTH / 8 - 1 downto 0);
  signal axi2_wready : std_logic;
  signal axi2_wvalid : std_logic;
  signal axi2_bid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi2_bready : std_logic;
  signal axi2_bresp : std_logic_vector(1 downto 0);
  signal axi2_bvalid : std_logic;
  signal axi2_rdata : std_logic_vector(M_AXI_XDATA_WIDTH - 1 downto 0);
  signal axi2_rid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi2_rlast : std_logic;
  signal axi2_rready : std_logic;
  signal axi2_rresp : std_logic_vector(1 downto 0);
  signal axi2_rvalid : std_logic;

  signal ctl2_go     : std_logic;
  signal ctl2_offset : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl2_length : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl2_ph_en  : std_logic_vector(7 downto 0);
  signal ctl2_done   : std_logic;
  signal ctl2_erd    : std_logic;
  signal ctl2_eindex : std_logic_vector(ERROR_INDEX_WIDTH - 1 downto 0);
  signal ctl2_error  : std_logic;
  signal ctl2_ecount : std_logic_vector(ERROR_COUNT_WIDTH - 1 downto 0);
  signal ctl2_efull  : std_logic;
  signal ctl2_elost  : std_logic;
  signal ctl2_eaddr  : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl2_ephase : std_logic_vector(2 downto 0);
  signal ctl2_ebits  : std_logic_vector(MEMORY_DATA_WIDTH - 1 downto 0);
  signal ctl2_edata  : std_logic_vector(MEMORY_DATA_WIDTH - 1 downto 0);

  signal c3_sys_rst : std_logic;
  signal c3_init_calib_complete : std_logic;
  signal c3_ui_clk : std_logic;
  signal c3_ui_clk_sync_rst : std_logic;
  signal c3_aresetn : std_logic;

  signal axi3_araddr : std_logic_vector(M_AXI_AXADDR_WIDTH - 1 downto 0);
  signal axi3_arburst : std_logic_vector(1 downto 0);
  signal axi3_arcache : std_logic_vector(3 downto 0);
  signal axi3_arid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi3_arlen : std_logic_vector(M_AXI_AXLEN_WIDTH - 1 downto 0);
  signal axi3_arlock : std_logic_vector(0 downto 0);
  signal axi3_arprot : std_logic_vector(2 downto 0);
  signal axi3_arqos : std_logic_vector(3 downto 0);
  signal axi3_arready : std_logic;
  signal axi3_arsize : std_logic_vector(2 downto 0);
  signal axi3_arvalid : std_logic;
  signal axi3_awaddr : std_logic_vector(M_AXI_AXADDR_WIDTH - 1 downto 0);
  signal axi3_awburst : std_logic_vector(1 downto 0);
  signal axi3_awcache : std_logic_vector(3 downto 0);
  signal axi3_awid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi3_awlen : std_logic_vector(M_AXI_AXLEN_WIDTH - 1 downto 0);
  signal axi3_awlock : std_logic_vector(0 downto 0);
  signal axi3_awprot : std_logic_vector(2 downto 0);
  signal axi3_awqos  : std_logic_vector(3 downto 0);
  signal axi3_awready : std_logic;
  signal axi3_awsize : std_logic_vector(2 downto 0);
  signal axi3_awvalid : std_logic;
  signal axi3_wdata : std_logic_vector(M_AXI_XDATA_WIDTH - 1 downto 0);
  signal axi3_wlast : std_logic;
  signal axi3_wstrb : std_logic_vector(M_AXI_XDATA_WIDTH / 8 - 1 downto 0);
  signal axi3_wready : std_logic;
  signal axi3_wvalid : std_logic;
  signal axi3_bid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi3_bready : std_logic;
  signal axi3_bresp : std_logic_vector(1 downto 0);
  signal axi3_bvalid : std_logic;
  signal axi3_rdata : std_logic_vector(M_AXI_XDATA_WIDTH - 1 downto 0);
  signal axi3_rid : std_logic_vector(M_AXI_XID_WIDTH - 1 downto 0);
  signal axi3_rlast : std_logic;
  signal axi3_rready : std_logic;
  signal axi3_rresp : std_logic_vector(1 downto 0);
  signal axi3_rvalid : std_logic;

  signal ctl3_go     : std_logic;
  signal ctl3_offset : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl3_length : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl3_ph_en  : std_logic_vector(7 downto 0);
  signal ctl3_done   : std_logic;
  signal ctl3_erd    : std_logic;
  signal ctl3_eindex : std_logic_vector(ERROR_INDEX_WIDTH - 1 downto 0);
  signal ctl3_error  : std_logic;
  signal ctl3_ecount : std_logic_vector(ERROR_COUNT_WIDTH - 1 downto 0);
  signal ctl3_efull  : std_logic;
  signal ctl3_elost  : std_logic;
  signal ctl3_eaddr  : std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
  signal ctl3_ephase : std_logic_vector(2 downto 0);
  signal ctl3_ebits  : std_logic_vector(MEMORY_DATA_WIDTH - 1 downto 0);
  signal ctl3_edata  : std_logic_vector(MEMORY_DATA_WIDTH - 1 downto 0);

  type ex_state_t is (ex_s_waiting, ex_s_go, ex_s_halt);
  signal ex0_state : ex_state_t;
  signal ex1_state : ex_state_t;
  signal ex2_state : ex_state_t;
  signal ex3_state : ex_state_t;
  signal ex0_ctl_go : std_logic;
  signal ex1_ctl_go : std_logic;
  signal ex2_ctl_go : std_logic;
  signal ex3_ctl_go : std_logic;

  signal i_stat_rst  : std_logic_vector(3 downto 0);
  signal i_stat_err  : std_logic_vector(3 downto 0);
  signal i_stat_done : std_logic_vector(3 downto 0);
  signal i_stat_init : std_logic_vector(3 downto 0);

begin

  -- VIO resets are DDR4 SDRAM controller resets (in pairs due to issues resetting controllers individually)
  c0_sys_rst <= vio0_reset or vio1_reset;
  c1_sys_rst <= vio0_reset or vio1_reset;
  c2_sys_rst <= vio2_reset or vio3_reset;
  c3_sys_rst <= vio2_reset or vio3_reset;

  c0_aresetn <= not c0_sys_rst;
  c1_aresetn <= not c1_sys_rst;
  c2_aresetn <= not c2_sys_rst;
  c3_aresetn <= not c3_sys_rst;


  -- Instantiate memory exercisers (with AXI4 interface).
  --
  -- Continually read the error log so that when user changes error index
  -- output by VIO core, error record is returned straight away.
  ctl0_erd <= '1';

  exerciser0_i : memory_exerciser_axi4
    generic map(
      memory_addr_width => MEMORY_ADDR_WIDTH,
      memory_data_width => MEMORY_DATA_WIDTH,
      m_axi_axaddr_width => M_AXI_AXADDR_WIDTH,
      m_axi_xdata_width => MEMORY_DATA_WIDTH,
      m_axi_xid_width => M_AXI_XID_WIDTH,
      m_axi_axlen_width => M_AXI_AXLEN_WIDTH,
      pack_factor => PACK_FACTOR,
      pack_rearranged => PACK_REARRANGED,
      partial_writes => PARTIAL_WRITES,
      error_count_width => ERROR_COUNT_WIDTH,
      error_index_width => ERROR_INDEX_WIDTH)
    port map(
      aresetn => c0_aresetn,
      aclk => c0_ui_clk,
      m_axi_araddr => axi0_araddr,
      m_axi_arburst => axi0_arburst,
      m_axi_arcache => axi0_arcache,
      m_axi_arid => axi0_arid,
      m_axi_arlen => axi0_arlen,
      m_axi_arlock => axi0_arlock,
      m_axi_arprot => axi0_arprot,
      m_axi_arqos => axi0_arqos,
      m_axi_arready => axi0_arready,
      m_axi_arregion => open, -- not used by MIG with AXI4 interface
      m_axi_arsize => axi0_arsize,
      m_axi_arvalid => axi0_arvalid,
      m_axi_awaddr => axi0_awaddr,
      m_axi_awburst => axi0_awburst,
      m_axi_awcache => axi0_awcache,
      m_axi_awid => axi0_awid,
      m_axi_awlen => axi0_awlen,
      m_axi_awlock => axi0_awlock,
      m_axi_awprot => axi0_awprot,
      m_axi_awqos => axi0_awqos,
      m_axi_awready => axi0_awready,
      m_axi_awregion => open, -- not used by MIG with AXI4 interface
      m_axi_awsize => axi0_awsize,
      m_axi_awvalid => axi0_awvalid,
      m_axi_wdata => axi0_wdata,
      m_axi_wlast => axi0_wlast,
      m_axi_wstrb => axi0_wstrb,
      m_axi_wready => axi0_wready,
      m_axi_wvalid => axi0_wvalid,
      m_axi_bid => axi0_bid,
      m_axi_bready => axi0_bready,
      m_axi_bresp => axi0_bresp,
      m_axi_bvalid => axi0_bvalid,
      m_axi_rdata => axi0_rdata,
      m_axi_rid => axi0_rid,
      m_axi_rlast => axi0_rlast,
      m_axi_rready => axi0_rready,
      m_axi_rresp => axi0_rresp,
      m_axi_rvalid => axi0_rvalid,
      ctl_go => ctl0_go,
      ctl_offset => ctl0_offset,
      ctl_length => ctl0_length,
      ctl_ph_en => ctl0_ph_en,
      ctl_done => ctl0_done,
      ctl_erd => ctl0_erd,
      ctl_eindex => ctl0_eindex,
      ctl_error => ctl0_error,
      ctl_ecount => ctl0_ecount,
      ctl_efull => ctl0_efull,
      ctl_elost => ctl0_elost,
      ctl_eaddr => ctl0_eaddr,
      ctl_ephase => ctl0_ephase,
      ctl_ebits => ctl0_ebits,
      ctl_edata => ctl0_edata);


  ctl1_erd <= '1';

  exerciser1_i : memory_exerciser_axi4
    generic map(
      memory_addr_width => MEMORY_ADDR_WIDTH,
      memory_data_width => MEMORY_DATA_WIDTH,
      m_axi_axaddr_width => M_AXI_AXADDR_WIDTH,
      m_axi_xdata_width => MEMORY_DATA_WIDTH,
      m_axi_xid_width => M_AXI_XID_WIDTH,
      m_axi_axlen_width => M_AXI_AXLEN_WIDTH,
      pack_factor => PACK_FACTOR,
      pack_rearranged => PACK_REARRANGED,
      partial_writes => PARTIAL_WRITES,
      error_count_width => ERROR_COUNT_WIDTH,
      error_index_width => ERROR_INDEX_WIDTH)
    port map(
      aresetn => c1_aresetn,
      aclk => c1_ui_clk,
      m_axi_araddr => axi1_araddr,
      m_axi_arburst => axi1_arburst,
      m_axi_arcache => axi1_arcache,
      m_axi_arid => axi1_arid,
      m_axi_arlen => axi1_arlen,
      m_axi_arlock => axi1_arlock,
      m_axi_arprot => axi1_arprot,
      m_axi_arqos => axi1_arqos,
      m_axi_arready => axi1_arready,
      m_axi_arregion => open, -- not used by MIG with AXI4 interface
      m_axi_arsize => axi1_arsize,
      m_axi_arvalid => axi1_arvalid,
      m_axi_awaddr => axi1_awaddr,
      m_axi_awburst => axi1_awburst,
      m_axi_awcache => axi1_awcache,
      m_axi_awid => axi1_awid,
      m_axi_awlen => axi1_awlen,
      m_axi_awlock => axi1_awlock,
      m_axi_awprot => axi1_awprot,
      m_axi_awqos => axi1_awqos,
      m_axi_awready => axi1_awready,
      m_axi_awregion => open, -- not used by MIG with AXI4 interface
      m_axi_awsize => axi1_awsize,
      m_axi_awvalid => axi1_awvalid,
      m_axi_wdata => axi1_wdata,
      m_axi_wlast => axi1_wlast,
      m_axi_wstrb => axi1_wstrb,
      m_axi_wready => axi1_wready,
      m_axi_wvalid => axi1_wvalid,
      m_axi_bid => axi1_bid,
      m_axi_bready => axi1_bready,
      m_axi_bresp => axi1_bresp,
      m_axi_bvalid => axi1_bvalid,
      m_axi_rdata => axi1_rdata,
      m_axi_rid => axi1_rid,
      m_axi_rlast => axi1_rlast,
      m_axi_rready => axi1_rready,
      m_axi_rresp => axi1_rresp,
      m_axi_rvalid => axi1_rvalid,
      ctl_go => ctl1_go,
      ctl_offset => ctl1_offset,
      ctl_length => ctl1_length,
      ctl_ph_en => ctl1_ph_en,
      ctl_done => ctl1_done,
      ctl_erd => ctl1_erd,
      ctl_eindex => ctl1_eindex,
      ctl_error => ctl1_error,
      ctl_ecount => ctl1_ecount,
      ctl_efull => ctl1_efull,
      ctl_elost => ctl1_elost,
      ctl_eaddr => ctl1_eaddr,
      ctl_ephase => ctl1_ephase,
      ctl_ebits => ctl1_ebits,
      ctl_edata => ctl1_edata);


  ctl2_erd <= '1';

  exerciser2_i : memory_exerciser_axi4
    generic map(
      memory_addr_width => MEMORY_ADDR_WIDTH,
      memory_data_width => MEMORY_DATA_WIDTH,
      m_axi_axaddr_width => M_AXI_AXADDR_WIDTH,
      m_axi_xdata_width => MEMORY_DATA_WIDTH,
      m_axi_xid_width => M_AXI_XID_WIDTH,
      m_axi_axlen_width => M_AXI_AXLEN_WIDTH,
      pack_factor => PACK_FACTOR,
      pack_rearranged => PACK_REARRANGED,
      partial_writes => PARTIAL_WRITES,
      error_count_width => ERROR_COUNT_WIDTH,
      error_index_width => ERROR_INDEX_WIDTH)
    port map(
      aresetn => c2_aresetn,
      aclk => c2_ui_clk,
      m_axi_araddr => axi2_araddr,
      m_axi_arburst => axi2_arburst,
      m_axi_arcache => axi2_arcache,
      m_axi_arid => axi2_arid,
      m_axi_arlen => axi2_arlen,
      m_axi_arlock => axi2_arlock,
      m_axi_arprot => axi2_arprot,
      m_axi_arqos => axi2_arqos,
      m_axi_arready => axi2_arready,
      m_axi_arregion => open, -- not used by MIG with AXI4 interface
      m_axi_arsize => axi2_arsize,
      m_axi_arvalid => axi2_arvalid,
      m_axi_awaddr => axi2_awaddr,
      m_axi_awburst => axi2_awburst,
      m_axi_awcache => axi2_awcache,
      m_axi_awid => axi2_awid,
      m_axi_awlen => axi2_awlen,
      m_axi_awlock => axi2_awlock,
      m_axi_awprot => axi2_awprot,
      m_axi_awqos => axi2_awqos,
      m_axi_awready => axi2_awready,
      m_axi_awregion => open, -- not used by MIG with AXI4 interface
      m_axi_awsize => axi2_awsize,
      m_axi_awvalid => axi2_awvalid,
      m_axi_wdata => axi2_wdata,
      m_axi_wlast => axi2_wlast,
      m_axi_wstrb => axi2_wstrb,
      m_axi_wready => axi2_wready,
      m_axi_wvalid => axi2_wvalid,
      m_axi_bid => axi2_bid,
      m_axi_bready => axi2_bready,
      m_axi_bresp => axi2_bresp,
      m_axi_bvalid => axi2_bvalid,
      m_axi_rdata => axi2_rdata,
      m_axi_rid => axi2_rid,
      m_axi_rlast => axi2_rlast,
      m_axi_rready => axi2_rready,
      m_axi_rresp => axi2_rresp,
      m_axi_rvalid => axi2_rvalid,
      ctl_go => ctl2_go,
      ctl_offset => ctl2_offset,
      ctl_length => ctl2_length,
      ctl_ph_en => ctl2_ph_en,
      ctl_done => ctl2_done,
      ctl_erd => ctl2_erd,
      ctl_eindex => ctl2_eindex,
      ctl_error => ctl2_error,
      ctl_ecount => ctl2_ecount,
      ctl_efull => ctl2_efull,
      ctl_elost => ctl2_elost,
      ctl_eaddr => ctl2_eaddr,
      ctl_ephase => ctl2_ephase,
      ctl_ebits => ctl2_ebits,
      ctl_edata => ctl2_edata);


  ctl3_erd <= '1';

  exerciser3_i : memory_exerciser_axi4
    generic map(
      memory_addr_width => MEMORY_ADDR_WIDTH,
      memory_data_width => MEMORY_DATA_WIDTH,
      m_axi_axaddr_width => M_AXI_AXADDR_WIDTH,
      m_axi_xdata_width => MEMORY_DATA_WIDTH,
      m_axi_xid_width => M_AXI_XID_WIDTH,
      m_axi_axlen_width => M_AXI_AXLEN_WIDTH,
      pack_factor => PACK_FACTOR,
      pack_rearranged => PACK_REARRANGED,
      partial_writes => PARTIAL_WRITES,
      error_count_width => ERROR_COUNT_WIDTH,
      error_index_width => ERROR_INDEX_WIDTH)
    port map(
      aresetn => c3_aresetn,
      aclk => c3_ui_clk,
      m_axi_araddr => axi3_araddr,
      m_axi_arburst => axi3_arburst,
      m_axi_arcache => axi3_arcache,
      m_axi_arid => axi3_arid,
      m_axi_arlen => axi3_arlen,
      m_axi_arlock => axi3_arlock,
      m_axi_arprot => axi3_arprot,
      m_axi_arqos => axi3_arqos,
      m_axi_arready => axi3_arready,
      m_axi_arregion => open, -- not used by MIG with AXI4 interface
      m_axi_arsize => axi3_arsize,
      m_axi_arvalid => axi3_arvalid,
      m_axi_awaddr => axi3_awaddr,
      m_axi_awburst => axi3_awburst,
      m_axi_awcache => axi3_awcache,
      m_axi_awid => axi3_awid,
      m_axi_awlen => axi3_awlen,
      m_axi_awlock => axi3_awlock,
      m_axi_awprot => axi3_awprot,
      m_axi_awqos => axi3_awqos,
      m_axi_awready => axi3_awready,
      m_axi_awregion => open, -- not used by MIG with AXI4 interface
      m_axi_awsize => axi3_awsize,
      m_axi_awvalid => axi3_awvalid,
      m_axi_wdata => axi3_wdata,
      m_axi_wlast => axi3_wlast,
      m_axi_wstrb => axi3_wstrb,
      m_axi_wready => axi3_wready,
      m_axi_wvalid => axi3_wvalid,
      m_axi_bid => axi3_bid,
      m_axi_bready => axi3_bready,
      m_axi_bresp => axi3_bresp,
      m_axi_bvalid => axi3_bvalid,
      m_axi_rdata => axi3_rdata,
      m_axi_rid => axi3_rid,
      m_axi_rlast => axi3_rlast,
      m_axi_rready => axi3_rready,
      m_axi_rresp => axi3_rresp,
      m_axi_rvalid => axi3_rvalid,
      ctl_go => ctl3_go,
      ctl_offset => ctl3_offset,
      ctl_length => ctl3_length,
      ctl_ph_en => ctl3_ph_en,
      ctl_done => ctl3_done,
      ctl_erd => ctl3_erd,
      ctl_eindex => ctl3_eindex,
      ctl_error => ctl3_error,
      ctl_ecount => ctl3_ecount,
      ctl_efull => ctl3_efull,
      ctl_elost => ctl3_elost,
      ctl_eaddr => ctl3_eaddr,
      ctl_ephase => ctl3_ephase,
      ctl_ebits => ctl3_ebits,
      ctl_edata => ctl3_edata);


  --
  -- Instantiate MIG IP with AXI4 interface
  --

  -- Extract DDR bank0/1 system clock
  ddr4_clk01_dif_i : IBUFGDS
  port map(
    I  => c01_sys_clk_p,
    IB => c01_sys_clk_n,
    O  => c01_sys_clk_i);

  -- Assigned to signal with 'DONT_TOUCH' attribute
  c0_ddr4_alert_n_nc <= c0_ddr4_alert_n;

  bank0_i : ddr4sdram
  port map(
    sys_rst                 => c0_sys_rst,             -- in    std_logic
    c0_sys_clk_i            => c01_sys_clk_i,          -- in    std_logic
    --
    c0_ddr4_act_n           => c0_ddr4_act_n,          -- out   std_logic
    c0_ddr4_adr             => c0_ddr4_adr,            -- out   std_logic_vector (16 downto 0)
    c0_ddr4_ba              => c0_ddr4_ba,             -- out   std_logic_vector (1 downto 0)
    c0_ddr4_bg              => c0_ddr4_bg,             -- out   std_logic_vector (0 to 0)
    c0_ddr4_cke             => c0_ddr4_cke,            -- out   std_logic_vector (0 to 0)
    c0_ddr4_odt             => c0_ddr4_odt,            -- out   std_logic_vector (0 to 0)
    c0_ddr4_cs_n            => c0_ddr4_cs_n,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_ck_t            => c0_ddr4_ck_t,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_ck_c            => c0_ddr4_ck_c,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_reset_n         => c0_ddr4_reset_n,        -- out   std_logic
    c0_ddr4_dm_dbi_n        => c0_ddr4_dm_dbi_n,       -- inout std_logic_vector (3 downto 0)
    c0_ddr4_dq              => c0_ddr4_dq,             -- inout std_logic_vector (31 downto 0)
    c0_ddr4_dqs_t           => c0_ddr4_dqs_t,          -- inout std_logic_vector (3 downto 0)
    c0_ddr4_dqs_c           => c0_ddr4_dqs_c,          -- inout std_logic_vector (3 downto 0)
    --
    c0_ddr4_aresetn         => c0_aresetn,             -- in    std_logic
    c0_ddr4_ui_clk          => c0_ui_clk,              -- out   std_logic
    c0_ddr4_ui_clk_sync_rst => c0_ui_clk_sync_rst,     -- out   std_logic
    c0_init_calib_complete  => c0_init_calib_complete, -- out   std_logic
    --
    c0_ddr4_s_axi_awid      => axi0_awid,              -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awaddr    => axi0_awaddr,            -- in    std_logic_vector(s_axi_addr_width - 1 downto 0)
    c0_ddr4_s_axi_awlen     => axi0_awlen,             -- in    std_logic_vector(7 downto 0)
    c0_ddr4_s_axi_awsize    => axi0_awsize,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_awburst   => axi0_awburst,           -- in    std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_awlock    => axi0_awlock,            -- in    std_logic_vector(0 downto 0)
    c0_ddr4_s_axi_awcache   => axi0_awcache,           -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awprot    => axi0_awprot,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_awqos     => axi0_awqos,             -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awvalid   => axi0_awvalid,           -- in    std_logic
    c0_ddr4_s_axi_awready   => axi0_awready,           -- out   std_logic
    c0_ddr4_s_axi_wdata     => axi0_wdata,             -- in    std_logic_vector(255 downto 0)
    c0_ddr4_s_axi_wstrb     => axi0_wstrb,             -- in    std_logic_vector(31 downto 0)
    c0_ddr4_s_axi_wlast     => axi0_wlast,             -- in    std_logic
    c0_ddr4_s_axi_wvalid    => axi0_wvalid,            -- in    std_logic
    c0_ddr4_s_axi_wready    => axi0_wready,            -- out   std_logic
    c0_ddr4_s_axi_bready    => axi0_bready,            -- in    std_logic
    c0_ddr4_s_axi_bid       => axi0_bid,               -- out   std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_bresp     => axi0_bresp,             -- out   std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_bvalid    => axi0_bvalid,            -- out   std_logic
    c0_ddr4_s_axi_arid      => axi0_arid,              -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_araddr    => axi0_araddr,            -- in    std_logic_vector(s_axi_addr_width - 1 downto 0)
    c0_ddr4_s_axi_arlen     => axi0_arlen,             -- in    std_logic_vector(7 downto 0)
    c0_ddr4_s_axi_arsize    => axi0_arsize,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_arburst   => axi0_arburst,           -- in    std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_arlock    => axi0_arlock,            -- in    std_logic_vector(0 downto 0)
    c0_ddr4_s_axi_arcache   => axi0_arcache,           -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_arprot    => axi0_arprot,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_arqos     => axi0_arqos,             -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_arvalid   => axi0_arvalid,           -- in    std_logic
    c0_ddr4_s_axi_arready   => axi0_arready,           -- out   std_logic
    c0_ddr4_s_axi_rready    => axi0_rready,            -- in    std_logic
    c0_ddr4_s_axi_rlast     => axi0_rlast,             -- out   std_logic
    c0_ddr4_s_axi_rvalid    => axi0_rvalid,            -- out   std_logic
    c0_ddr4_s_axi_rresp     => axi0_rresp,             -- out   std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_rid       => axi0_rid,               -- out   std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_rdata     => axi0_rdata);            -- out   std_logic_vector(255 downto 0)

  c0_ddr4_par <= '0';
  c0_ddr4_ten <= '0';


  -- Assigned to signal with 'DONT_TOUCH' attribute
  c1_ddr4_alert_n_nc <= c1_ddr4_alert_n;

  bank1_i : ddr4sdram
  port map(
    sys_rst                 => c1_sys_rst,             -- in    std_logic
    c0_sys_clk_i            => c01_sys_clk_i,          -- in    std_logic
    --
    c0_ddr4_act_n           => c1_ddr4_act_n,          -- out   std_logic
    c0_ddr4_adr             => c1_ddr4_adr,            -- out   std_logic_vector (16 downto 0)
    c0_ddr4_ba              => c1_ddr4_ba,             -- out   std_logic_vector (1 downto 0)
    c0_ddr4_bg              => c1_ddr4_bg,             -- out   std_logic_vector (0 to 0)
    c0_ddr4_cke             => c1_ddr4_cke,            -- out   std_logic_vector (0 to 0)
    c0_ddr4_odt             => c1_ddr4_odt,            -- out   std_logic_vector (0 to 0)
    c0_ddr4_cs_n            => c1_ddr4_cs_n,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_ck_t            => c1_ddr4_ck_t,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_ck_c            => c1_ddr4_ck_c,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_reset_n         => c1_ddr4_reset_n,        -- out   std_logic
    c0_ddr4_dm_dbi_n        => c1_ddr4_dm_dbi_n,       -- inout std_logic_vector (3 downto 0)
    c0_ddr4_dq              => c1_ddr4_dq,             -- inout std_logic_vector (31 downto 0)
    c0_ddr4_dqs_t           => c1_ddr4_dqs_t,          -- inout std_logic_vector (3 downto 0)
    c0_ddr4_dqs_c           => c1_ddr4_dqs_c,          -- inout std_logic_vector (3 downto 0)
    --
    c0_ddr4_aresetn         => c1_aresetn,             -- in    std_logic
    c0_ddr4_ui_clk          => c1_ui_clk,              -- out   std_logic
    c0_ddr4_ui_clk_sync_rst => c1_ui_clk_sync_rst,     -- out   std_logic
    c0_init_calib_complete  => c1_init_calib_complete, -- out   std_logic
    --
    c0_ddr4_s_axi_awid      => axi1_awid,              -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awaddr    => axi1_awaddr,            -- in    std_logic_vector(s_axi_addr_width - 1 downto 0)
    c0_ddr4_s_axi_awlen     => axi1_awlen,             -- in    std_logic_vector(7 downto 0)
    c0_ddr4_s_axi_awsize    => axi1_awsize,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_awburst   => axi1_awburst,           -- in    std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_awlock    => axi1_awlock,            -- in    std_logic_vector(0 downto 0)
    c0_ddr4_s_axi_awcache   => axi1_awcache,           -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awprot    => axi1_awprot,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_awqos     => axi1_awqos,             -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awvalid   => axi1_awvalid,           -- in    std_logic
    c0_ddr4_s_axi_awready   => axi1_awready,           -- out   std_logic
    c0_ddr4_s_axi_wdata     => axi1_wdata,             -- in    std_logic_vector(255 downto 0)
    c0_ddr4_s_axi_wstrb     => axi1_wstrb,             -- in    std_logic_vector(31 downto 0)
    c0_ddr4_s_axi_wlast     => axi1_wlast,             -- in    std_logic
    c0_ddr4_s_axi_wvalid    => axi1_wvalid,            -- in    std_logic
    c0_ddr4_s_axi_wready    => axi1_wready,            -- out   std_logic
    c0_ddr4_s_axi_bready    => axi1_bready,            -- in    std_logic
    c0_ddr4_s_axi_bid       => axi1_bid,               -- out   std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_bresp     => axi1_bresp,             -- out   std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_bvalid    => axi1_bvalid,            -- out   std_logic
    c0_ddr4_s_axi_arid      => axi1_arid,              -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_araddr    => axi1_araddr,            -- in    std_logic_vector(s_axi_addr_width - 1 downto 0)
    c0_ddr4_s_axi_arlen     => axi1_arlen,             -- in    std_logic_vector(7 downto 0)
    c0_ddr4_s_axi_arsize    => axi1_arsize,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_arburst   => axi1_arburst,           -- in    std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_arlock    => axi1_arlock,            -- in    std_logic_vector(0 downto 0)
    c0_ddr4_s_axi_arcache   => axi1_arcache,           -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_arprot    => axi1_arprot,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_arqos     => axi1_arqos,             -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_arvalid   => axi1_arvalid,           -- in    std_logic
    c0_ddr4_s_axi_arready   => axi1_arready,           -- out   std_logic
    c0_ddr4_s_axi_rready    => axi1_rready,            -- in    std_logic
    c0_ddr4_s_axi_rlast     => axi1_rlast,             -- out   std_logic
    c0_ddr4_s_axi_rvalid    => axi1_rvalid,            -- out   std_logic
    c0_ddr4_s_axi_rresp     => axi1_rresp,             -- out   std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_rid       => axi1_rid,               -- out   std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_rdata     => axi1_rdata);            -- out   std_logic_vector(255 downto 0)

  c1_ddr4_par <= '0';
  c1_ddr4_ten <= '0';


  -- Extract DDR bank2/3 system clock
  ddr4_clk23_dif_i : IBUFGDS
  port map(
    I  => c23_sys_clk_p,
    IB => c23_sys_clk_n,
    O  => c23_sys_clk_i);

  -- Assigned to signal with 'DONT_TOUCH' attribute
  c2_ddr4_alert_n_nc <= c2_ddr4_alert_n;

  bank2_i : ddr4sdram
  port map(
    sys_rst                 => c2_sys_rst,    -- in    std_logic;
    c0_sys_clk_i            => c23_sys_clk_i, -- in    std_logic;
    --
    c0_ddr4_act_n           => c2_ddr4_act_n,          -- out   std_logic
    c0_ddr4_adr             => c2_ddr4_adr,            -- out   std_logic_vector (16 downto 0)
    c0_ddr4_ba              => c2_ddr4_ba,             -- out   std_logic_vector (1 downto 0)
    c0_ddr4_bg              => c2_ddr4_bg,             -- out   std_logic_vector (0 to 0)
    c0_ddr4_cke             => c2_ddr4_cke,            -- out   std_logic_vector (0 to 0)
    c0_ddr4_odt             => c2_ddr4_odt,            -- out   std_logic_vector (0 to 0)
    c0_ddr4_cs_n            => c2_ddr4_cs_n,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_ck_t            => c2_ddr4_ck_t,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_ck_c            => c2_ddr4_ck_c,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_reset_n         => c2_ddr4_reset_n,        -- out   std_logic
    c0_ddr4_dm_dbi_n        => c2_ddr4_dm_dbi_n,       -- inout std_logic_vector (3 downto 0)
    c0_ddr4_dq              => c2_ddr4_dq,             -- inout std_logic_vector (31 downto 0)
    c0_ddr4_dqs_t           => c2_ddr4_dqs_t,          -- inout std_logic_vector (3 downto 0)
    c0_ddr4_dqs_c           => c2_ddr4_dqs_c,          -- inout std_logic_vector (3 downto 0)
    --
    c0_ddr4_aresetn         => c2_aresetn,             -- in    std_logic
    c0_ddr4_ui_clk          => c2_ui_clk,              -- out   std_logic
    c0_ddr4_ui_clk_sync_rst => c2_ui_clk_sync_rst,     -- out   std_logic
    c0_init_calib_complete  => c2_init_calib_complete, -- out   std_logic
    --
    c0_ddr4_s_axi_awid      => axi2_awid,              -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awaddr    => axi2_awaddr,            -- in    std_logic_vector(s_axi_addr_width - 1 downto 0)
    c0_ddr4_s_axi_awlen     => axi2_awlen,             -- in    std_logic_vector(7 downto 0)
    c0_ddr4_s_axi_awsize    => axi2_awsize,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_awburst   => axi2_awburst,           -- in    std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_awlock    => axi2_awlock,            -- in    std_logic_vector(0 downto 0)
    c0_ddr4_s_axi_awcache   => axi2_awcache,           -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awprot    => axi2_awprot,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_awqos     => axi2_awqos,             -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awvalid   => axi2_awvalid,           -- in    std_logic
    c0_ddr4_s_axi_awready   => axi2_awready,           -- out   std_logic
    c0_ddr4_s_axi_wdata     => axi2_wdata,             -- in    std_logic_vector(255 downto 0)
    c0_ddr4_s_axi_wstrb     => axi2_wstrb,             -- in    std_logic_vector(31 downto 0)
    c0_ddr4_s_axi_wlast     => axi2_wlast,             -- in    std_logic
    c0_ddr4_s_axi_wvalid    => axi2_wvalid,            -- in    std_logic
    c0_ddr4_s_axi_wready    => axi2_wready,            -- out   std_logic
    c0_ddr4_s_axi_bready    => axi2_bready,            -- in    std_logic
    c0_ddr4_s_axi_bid       => axi2_bid,               -- out   std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_bresp     => axi2_bresp,             -- out   std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_bvalid    => axi2_bvalid,            -- out   std_logic
    c0_ddr4_s_axi_arid      => axi2_arid,              -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_araddr    => axi2_araddr,            -- in    std_logic_vector(s_axi_addr_width - 1 downto 0)
    c0_ddr4_s_axi_arlen     => axi2_arlen,             -- in    std_logic_vector(7 downto 0)
    c0_ddr4_s_axi_arsize    => axi2_arsize,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_arburst   => axi2_arburst,           -- in    std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_arlock    => axi2_arlock,            -- in    std_logic_vector(0 downto 0)
    c0_ddr4_s_axi_arcache   => axi2_arcache,           -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_arprot    => axi2_arprot,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_arqos     => axi2_arqos,             -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_arvalid   => axi2_arvalid,           -- in    std_logic
    c0_ddr4_s_axi_arready   => axi2_arready,           -- out   std_logic
    c0_ddr4_s_axi_rready    => axi2_rready,            -- in    std_logic
    c0_ddr4_s_axi_rlast     => axi2_rlast,             -- out   std_logic
    c0_ddr4_s_axi_rvalid    => axi2_rvalid,            -- out   std_logic
    c0_ddr4_s_axi_rresp     => axi2_rresp,             -- out   std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_rid       => axi2_rid,               -- out   std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_rdata     => axi2_rdata);            -- out   std_logic_vector(255 downto 0)

  c2_ddr4_par <= '0';
  c2_ddr4_ten <= '0';


  -- Assigned to signal with 'DONT_TOUCH' attribute
  c3_ddr4_alert_n_nc <= c3_ddr4_alert_n;

  bank3_i : ddr4sdram
  port map(
    sys_rst                 => c3_sys_rst,    -- in    std_logic;
    c0_sys_clk_i            => c23_sys_clk_i, -- in    std_logic;
    --
    c0_ddr4_act_n           => c3_ddr4_act_n,          -- out   std_logic
    c0_ddr4_adr             => c3_ddr4_adr,            -- out   std_logic_vector (16 downto 0)
    c0_ddr4_ba              => c3_ddr4_ba,             -- out   std_logic_vector (1 downto 0)
    c0_ddr4_bg              => c3_ddr4_bg,             -- out   std_logic_vector (0 to 0)
    c0_ddr4_cke             => c3_ddr4_cke,            -- out   std_logic_vector (0 to 0)
    c0_ddr4_odt             => c3_ddr4_odt,            -- out   std_logic_vector (0 to 0)
    c0_ddr4_cs_n            => c3_ddr4_cs_n,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_ck_t            => c3_ddr4_ck_t,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_ck_c            => c3_ddr4_ck_c,           -- out   std_logic_vector (0 to 0)
    c0_ddr4_reset_n         => c3_ddr4_reset_n,        -- out   std_logic
    c0_ddr4_dm_dbi_n        => c3_ddr4_dm_dbi_n,       -- inout std_logic_vector (3 downto 0)
    c0_ddr4_dq              => c3_ddr4_dq,             -- inout std_logic_vector (31 downto 0)
    c0_ddr4_dqs_t           => c3_ddr4_dqs_t,          -- inout std_logic_vector (3 downto 0)
    c0_ddr4_dqs_c           => c3_ddr4_dqs_c,          -- inout std_logic_vector (3 downto 0)
    --
    c0_ddr4_aresetn         => c3_aresetn,             -- in    std_logic
    c0_ddr4_ui_clk          => c3_ui_clk,              -- out   std_logic
    c0_ddr4_ui_clk_sync_rst => c3_ui_clk_sync_rst,     -- out   std_logic
    c0_init_calib_complete  => c3_init_calib_complete, -- out   std_logic
    --
    c0_ddr4_s_axi_awid      => axi3_awid,              -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awaddr    => axi3_awaddr,            -- in    std_logic_vector(s_axi_addr_width - 1 downto 0)
    c0_ddr4_s_axi_awlen     => axi3_awlen,             -- in    std_logic_vector(7 downto 0)
    c0_ddr4_s_axi_awsize    => axi3_awsize,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_awburst   => axi3_awburst,           -- in    std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_awlock    => axi3_awlock,            -- in    std_logic_vector(0 downto 0)
    c0_ddr4_s_axi_awcache   => axi3_awcache,           -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awprot    => axi3_awprot,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_awqos     => axi3_awqos,             -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_awvalid   => axi3_awvalid,           -- in    std_logic
    c0_ddr4_s_axi_awready   => axi3_awready,           -- out   std_logic
    c0_ddr4_s_axi_wdata     => axi3_wdata,             -- in    std_logic_vector(255 downto 0)
    c0_ddr4_s_axi_wstrb     => axi3_wstrb,             -- in    std_logic_vector(31 downto 0)
    c0_ddr4_s_axi_wlast     => axi3_wlast,             -- in    std_logic
    c0_ddr4_s_axi_wvalid    => axi3_wvalid,            -- in    std_logic
    c0_ddr4_s_axi_wready    => axi3_wready,            -- out   std_logic
    c0_ddr4_s_axi_bready    => axi3_bready,            -- in    std_logic
    c0_ddr4_s_axi_bid       => axi3_bid,               -- out   std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_bresp     => axi3_bresp,             -- out   std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_bvalid    => axi3_bvalid,            -- out   std_logic
    c0_ddr4_s_axi_arid      => axi3_arid,              -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_araddr    => axi3_araddr,            -- in    std_logic_vector(s_axi_addr_width - 1 downto 0)
    c0_ddr4_s_axi_arlen     => axi3_arlen,             -- in    std_logic_vector(7 downto 0)
    c0_ddr4_s_axi_arsize    => axi3_arsize,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_arburst   => axi3_arburst,           -- in    std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_arlock    => axi3_arlock,            -- in    std_logic_vector(0 downto 0)
    c0_ddr4_s_axi_arcache   => axi3_arcache,           -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_arprot    => axi3_arprot,            -- in    std_logic_vector(2 downto 0)
    c0_ddr4_s_axi_arqos     => axi3_arqos,             -- in    std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_arvalid   => axi3_arvalid,           -- in    std_logic
    c0_ddr4_s_axi_arready   => axi3_arready,           -- out   std_logic
    c0_ddr4_s_axi_rready    => axi3_rready,            -- in    std_logic
    c0_ddr4_s_axi_rlast     => axi3_rlast,             -- out   std_logic
    c0_ddr4_s_axi_rvalid    => axi3_rvalid,            -- out   std_logic
    c0_ddr4_s_axi_rresp     => axi3_rresp,             -- out   std_logic_vector(1 downto 0)
    c0_ddr4_s_axi_rid       => axi3_rid,               -- out   std_logic_vector(3 downto 0)
    c0_ddr4_s_axi_rdata     => axi3_rdata);            -- out   std_logic_vector(255 downto 0)

  c3_ddr4_par <= '0';
  c3_ddr4_ten <= '0';


  --
  -- State machines wait for calibration to complete, and then
  -- initiate a memory test (once).
  --

  gen_state0_p : process(c0_sys_rst, c0_ui_clk)
  begin
    if c0_sys_rst = '1' then
      ex0_state <= ex_s_waiting;
      ex0_ctl_go <= '0';
    elsif rising_edge(c0_ui_clk) then
      case ex0_state is
        when ex_s_waiting =>
          ex0_ctl_go <= '0';
          if c0_ui_clk_sync_rst = '1' or c0_init_calib_complete = '0' then
            ex0_state <= ex_s_waiting;
          else
            ex0_state <= ex_s_go;
          end if;

        when ex_s_go =>
          ex0_state <= ex_s_halt;
          ex0_ctl_go <= '1';

        when ex_s_halt =>
          ex0_state <= ex_s_halt;
          ex0_ctl_go <= '0';
      end case;
    end if;
  end process;

  ctl0_go <= ex0_ctl_go;


  gen_state1_p : process(c1_sys_rst, c1_ui_clk)
  begin
    if c1_sys_rst = '1' then
      ex1_state <= ex_s_waiting;
      ex1_ctl_go <= '0';
    elsif rising_edge(c1_ui_clk) then
      case ex1_state is
        when ex_s_waiting =>
          ex1_ctl_go <= '0';
          if c1_ui_clk_sync_rst = '1' or c1_init_calib_complete = '0' then
            ex1_state <= ex_s_waiting;
          else
            ex1_state <= ex_s_go;
          end if;

        when ex_s_go =>
          ex1_state <= ex_s_halt;
          ex1_ctl_go <= '1';

        when ex_s_halt =>
          ex1_state <= ex_s_halt;
          ex1_ctl_go <= '0';
      end case;
    end if;
  end process;

  ctl1_go <= ex1_ctl_go;


  gen_state2_p : process(c2_sys_rst, c2_ui_clk)
  begin
    if c2_sys_rst = '1' then
      ex2_state <= ex_s_waiting;
      ex2_ctl_go <= '0';
    elsif rising_edge(c2_ui_clk) then
      case ex2_state is
        when ex_s_waiting =>
          ex2_ctl_go <= '0';
          if c2_ui_clk_sync_rst = '1' or c2_init_calib_complete = '0' then
            ex2_state <= ex_s_waiting;
          else
            ex2_state <= ex_s_go;
          end if;

        when ex_s_go =>
          ex2_state <= ex_s_halt;
          ex2_ctl_go <= '1';

        when ex_s_halt =>
          ex2_state <= ex_s_halt;
          ex2_ctl_go <= '0';
      end case;
    end if;
  end process;

  ctl2_go <= ex2_ctl_go;


  gen_state3_p : process(c3_sys_rst, c3_ui_clk)
  begin
    if c3_sys_rst = '1' then
      ex3_state <= ex_s_waiting;
      ex3_ctl_go <= '0';
    elsif rising_edge(c3_ui_clk) then
      case ex3_state is
        when ex_s_waiting =>
          ex3_ctl_go <= '0';
          if c3_ui_clk_sync_rst = '1' or c3_init_calib_complete = '0' then
            ex3_state <= ex_s_waiting;
          else
            ex3_state <= ex_s_go;
          end if;

        when ex_s_go =>
          ex3_state <= ex_s_halt;
          ex3_ctl_go <= '1';

        when ex_s_halt =>
          ex3_state <= ex_s_halt;
          ex3_ctl_go <= '0';
      end case;
    end if;
  end process;

  ctl3_go <= ex3_ctl_go;


  --
  -- VIO core
  --

  ibuf_refclk300_i : IBUFGDS
    port map(
      I  => refclk300_p,
      IB => refclk300_n,
      O  => refclk300_ibuf);

  -- Divide refclk300 by 3
  bufg_vio_clk_i : BUFGCE_DIV
  generic map(
    BUFGCE_DIVIDE => 3)
  port map(
    CE  => '1',
    CLR => '0',
    I   => refclk300_ibuf,
    O   => vio_clk);


  -- Simulation: don't instantiate VIO core
  no_vio_g : if simulation generate

    vio0_reset <= transport '1','0' after 200 ns;

    ctl0_offset <= std_logic_vector(to_unsigned(simulation_offset, MEMORY_ADDR_WIDTH));
    ctl0_length <= std_logic_vector(to_unsigned(simulation_length - 1, MEMORY_ADDR_WIDTH));
    ctl0_eindex <= (others => '0');
    ctl0_ph_en <= (others => '1');

    vio1_reset <= transport '1','0' after 200 ns;

    ctl1_offset <= std_logic_vector(to_unsigned(simulation_offset, MEMORY_ADDR_WIDTH));
    ctl1_length <= std_logic_vector(to_unsigned(simulation_length - 1, MEMORY_ADDR_WIDTH));
    ctl1_eindex <= (others => '0');
    ctl1_ph_en <= (others => '1');

    vio2_reset <= transport '1','0' after 200 ns;

    ctl2_offset <= std_logic_vector(to_unsigned(simulation_offset, MEMORY_ADDR_WIDTH));
    ctl2_length <= std_logic_vector(to_unsigned(simulation_length - 1, MEMORY_ADDR_WIDTH));
    ctl2_eindex <= (others => '0');
    ctl2_ph_en <= (others => '1');

    vio3_reset <= transport '1','0' after 200 ns;

    ctl3_offset <= std_logic_vector(to_unsigned(simulation_offset, MEMORY_ADDR_WIDTH));
    ctl3_length <= std_logic_vector(to_unsigned(simulation_length - 1, MEMORY_ADDR_WIDTH));
    ctl3_eindex <= (others => '0');
    ctl3_ph_en <= (others => '1');

  end generate;


  -- Synthesis: VIO cores for controlling via Hardware Manager
  gen_vio_g : if not simulation generate

    component vio_ip
    port(
      clk        : in    std_logic;
      probe_in0  : in    std_logic_vector(0 downto 0);  -- init_calib_complete
      probe_in1  : in    std_logic_vector(0 downto 0);  -- ctl_done
      probe_in2  : in    std_logic_vector(0 downto 0);  -- ctl_error
      probe_in3  : in    std_logic_vector(0 downto 0);  -- ctl_efull
      probe_in4  : in    std_logic_vector(0 downto 0);  -- ctl_elost
      probe_in5  : in    std_logic_vector(31 downto 0); -- ctl_ecount
      probe_in6  : in    std_logic_vector(2 downto 0);  -- ctl_ephase
      probe_in7  : in    std_logic_vector(31 downto 0); -- ctl_eaddr
      probe_in8  : in    std_logic_vector(31 downto 0); -- ctl_edata[31:0]
      probe_in9  : in    std_logic_vector(31 downto 0); -- ctl_ebits[31:0]
      probe_in10 : in    std_logic_vector(31 downto 0); -- ctl_edata[63:32]
      probe_in11 : in    std_logic_vector(31 downto 0); -- ctl_ebits[63:32]
      probe_in12 : in    std_logic_vector(31 downto 0); -- ctl_edata[95:64]
      probe_in13 : in    std_logic_vector(31 downto 0); -- ctl_ebits[95:64]
      probe_in14 : in    std_logic_vector(31 downto 0); -- ctl_edata[127:96]
      probe_in15 : in    std_logic_vector(31 downto 0); -- ctl_ebits[127:96]
      probe_in16 : in    std_logic_vector(31 downto 0); -- ctl_edata[159:128]
      probe_in17 : in    std_logic_vector(31 downto 0); -- ctl_ebits[159:128]
      probe_in18 : in    std_logic_vector(31 downto 0); -- ctl_edata[191:160]
      probe_in19 : in    std_logic_vector(31 downto 0); -- ctl_ebits[191:160]
      probe_in20 : in    std_logic_vector(31 downto 0); -- ctl_edata[223:192]
      probe_in21 : in    std_logic_vector(31 downto 0); -- ctl_ebits[223:192]
      probe_in22 : in    std_logic_vector(31 downto 0); -- ctl_edata[255:224]
      probe_in23 : in    std_logic_vector(31 downto 0); -- ctl_ebits[255:224]
      --
      probe_out0 : out   std_logic_vector(0 downto 0);  -- vioN_reset
      probe_out1 : out   std_logic_vector(31 downto 0); -- ctl_offset_vio
      probe_out2 : out   std_logic_vector(31 downto 0); -- ctl_length_vio
      probe_out3 : out   std_logic_vector(7 downto 0);  -- ctl_eindex
      probe_out4 : out   std_logic_vector(7 downto 0)); -- ctl_ph_en
    end component;

    signal ctl0_ecount_vio : std_logic_vector(31 downto 0);
    signal ctl0_ephase_vio : std_logic_vector(2 downto 0);
    signal ctl0_eaddr_vio  : std_logic_vector(31 downto 0);
    signal ctl0_edata_vio  : std_logic_vector(255 downto 0);
    signal ctl0_ebits_vio  : std_logic_vector(255 downto 0);
    signal ctl0_offset_vio : std_logic_vector(31 downto 0);
    signal ctl0_length_vio : std_logic_vector(31 downto 0);

    signal ctl1_ecount_vio : std_logic_vector(31 downto 0);
    signal ctl1_ephase_vio : std_logic_vector(2 downto 0);
    signal ctl1_eaddr_vio  : std_logic_vector(31 downto 0);
    signal ctl1_edata_vio  : std_logic_vector(255 downto 0);
    signal ctl1_ebits_vio  : std_logic_vector(255 downto 0);
    signal ctl1_offset_vio : std_logic_vector(31 downto 0);
    signal ctl1_length_vio : std_logic_vector(31 downto 0);

    signal ctl2_ecount_vio : std_logic_vector(31 downto 0);
    signal ctl2_ephase_vio : std_logic_vector(2 downto 0);
    signal ctl2_eaddr_vio  : std_logic_vector(31 downto 0);
    signal ctl2_edata_vio  : std_logic_vector(255 downto 0);
    signal ctl2_ebits_vio  : std_logic_vector(255 downto 0);
    signal ctl2_offset_vio : std_logic_vector(31 downto 0);
    signal ctl2_length_vio : std_logic_vector(31 downto 0);

    signal ctl3_ecount_vio : std_logic_vector(31 downto 0);
    signal ctl3_ephase_vio : std_logic_vector(2 downto 0);
    signal ctl3_eaddr_vio  : std_logic_vector(31 downto 0);
    signal ctl3_edata_vio  : std_logic_vector(255 downto 0);
    signal ctl3_ebits_vio  : std_logic_vector(255 downto 0);
    signal ctl3_offset_vio : std_logic_vector(31 downto 0);
    signal ctl3_length_vio : std_logic_vector(31 downto 0);

    attribute dont_touch : string;
    attribute dont_touch of ctl0_ecount_vio : signal is "true";
    attribute dont_touch of ctl0_ephase_vio : signal is "true";
    attribute dont_touch of ctl0_eaddr_vio  : signal is "true";
    attribute dont_touch of ctl0_edata_vio  : signal is "true";
    attribute dont_touch of ctl0_ebits_vio  : signal is "true";
    attribute dont_touch of ctl0_offset_vio : signal is "true";
    attribute dont_touch of ctl0_length_vio : signal is "true";

    attribute dont_touch of ctl1_ecount_vio : signal is "true";
    attribute dont_touch of ctl1_ephase_vio : signal is "true";
    attribute dont_touch of ctl1_eaddr_vio  : signal is "true";
    attribute dont_touch of ctl1_edata_vio  : signal is "true";
    attribute dont_touch of ctl1_ebits_vio  : signal is "true";
    attribute dont_touch of ctl1_offset_vio : signal is "true";
    attribute dont_touch of ctl1_length_vio : signal is "true";

    attribute dont_touch of ctl2_ecount_vio : signal is "true";
    attribute dont_touch of ctl2_ephase_vio : signal is "true";
    attribute dont_touch of ctl2_eaddr_vio  : signal is "true";
    attribute dont_touch of ctl2_edata_vio  : signal is "true";
    attribute dont_touch of ctl2_ebits_vio  : signal is "true";
    attribute dont_touch of ctl2_offset_vio : signal is "true";
    attribute dont_touch of ctl2_length_vio : signal is "true";

    attribute dont_touch of ctl3_ecount_vio : signal is "true";
    attribute dont_touch of ctl3_ephase_vio : signal is "true";
    attribute dont_touch of ctl3_eaddr_vio  : signal is "true";
    attribute dont_touch of ctl3_edata_vio  : signal is "true";
    attribute dont_touch of ctl3_ebits_vio  : signal is "true";
    attribute dont_touch of ctl3_offset_vio : signal is "true";
    attribute dont_touch of ctl3_length_vio : signal is "true";

  begin

    ctl0_ecount_vio <= ctl0_ecount;
    ctl0_ephase_vio <= ctl0_ephase;
    ctl0_eaddr_vio  <= std_logic_vector(resize(unsigned(ctl0_eaddr), 32));
    ctl0_edata_vio  <= ctl0_edata;
    ctl0_ebits_vio  <= ctl0_ebits;

    vio0_i : vio_ip
    port map(
      clk => vio_clk,
      probe_in0(0) => c0_init_calib_complete,
      probe_in1(0) => ctl0_done,
      probe_in2(0) => ctl0_error,
      probe_in3(0) => ctl0_efull,
      probe_in4(0) => ctl0_elost,
      probe_in5 => ctl0_ecount_vio,
      probe_in6 => ctl0_ephase_vio,
      probe_in7 => ctl0_eaddr_vio,
      probe_in8 => ctl0_edata_vio(31 downto 0),
      probe_in9 => ctl0_ebits_vio(31 downto 0),
      probe_in10 => ctl0_edata_vio(63 downto 32),
      probe_in11 => ctl0_ebits_vio(63 downto 32),
      probe_in12 => ctl0_edata_vio(95 downto 64),
      probe_in13 => ctl0_ebits_vio(95 downto 64),
      probe_in14 => ctl0_edata_vio(127 downto 96),
      probe_in15 => ctl0_ebits_vio(127 downto 96),
      probe_in16 => ctl0_edata_vio(159 downto 128),
      probe_in17 => ctl0_ebits_vio(159 downto 128),
      probe_in18 => ctl0_edata_vio(191 downto 160),
      probe_in19 => ctl0_ebits_vio(191 downto 160),
      probe_in20 => ctl0_edata_vio(223 downto 192),
      probe_in21 => ctl0_ebits_vio(223 downto 192),
      probe_in22 => ctl0_edata_vio(255 downto 224),
      probe_in23 => ctl0_ebits_vio(255 downto 224),
      --
      probe_out0(0) => vio0_reset,
      probe_out1    => ctl0_offset_vio,
      probe_out2    => ctl0_length_vio,
      probe_out3    => ctl0_eindex,
      probe_out4    => ctl0_ph_en);

    ctl0_offset <= ctl0_offset_vio(ctl0_offset'range);
    ctl0_length <= ctl0_length_vio(ctl0_length'range);


    ctl1_ecount_vio <= ctl1_ecount;
    ctl1_ephase_vio <= ctl1_ephase;
    ctl1_eaddr_vio  <= std_logic_vector(resize(unsigned(ctl1_eaddr), 32));
    ctl1_edata_vio  <= ctl1_edata;
    ctl1_ebits_vio  <= ctl1_ebits;

    vio1_i : vio_ip
    port map(
      clk => vio_clk,
      probe_in0(0) => c1_init_calib_complete,
      probe_in1(0) => ctl1_done,
      probe_in2(0) => ctl1_error,
      probe_in3(0) => ctl1_efull,
      probe_in4(0) => ctl1_elost,
      probe_in5 => ctl1_ecount_vio,
      probe_in6 => ctl1_ephase_vio,
      probe_in7 => ctl1_eaddr_vio,
      probe_in8 => ctl1_edata_vio(31 downto 0),
      probe_in9 => ctl1_ebits_vio(31 downto 0),
      probe_in10 => ctl1_edata_vio(63 downto 32),
      probe_in11 => ctl1_ebits_vio(63 downto 32),
      probe_in12 => ctl1_edata_vio(95 downto 64),
      probe_in13 => ctl1_ebits_vio(95 downto 64),
      probe_in14 => ctl1_edata_vio(127 downto 96),
      probe_in15 => ctl1_ebits_vio(127 downto 96),
      probe_in16 => ctl1_edata_vio(159 downto 128),
      probe_in17 => ctl1_ebits_vio(159 downto 128),
      probe_in18 => ctl1_edata_vio(191 downto 160),
      probe_in19 => ctl1_ebits_vio(191 downto 160),
      probe_in20 => ctl1_edata_vio(223 downto 192),
      probe_in21 => ctl1_ebits_vio(223 downto 192),
      probe_in22 => ctl1_edata_vio(255 downto 224),
      probe_in23 => ctl1_ebits_vio(255 downto 224),
      --
      probe_out0(0) => vio1_reset,
      probe_out1    => ctl1_offset_vio,
      probe_out2    => ctl1_length_vio,
      probe_out3    => ctl1_eindex,
      probe_out4    => ctl1_ph_en);

    ctl1_offset <= ctl1_offset_vio(ctl1_offset'range);
    ctl1_length <= ctl1_length_vio(ctl1_length'range);


    ctl2_ecount_vio <= ctl2_ecount;
    ctl2_ephase_vio <= ctl2_ephase;
    ctl2_eaddr_vio  <= std_logic_vector(resize(unsigned(ctl2_eaddr), 32));
    ctl2_edata_vio  <= ctl2_edata;
    ctl2_ebits_vio  <= ctl2_ebits;

    vio2_i : vio_ip
    port map(
      clk => vio_clk,
      probe_in0(0) => c2_init_calib_complete,
      probe_in1(0) => ctl2_done,
      probe_in2(0) => ctl2_error,
      probe_in3(0) => ctl2_efull,
      probe_in4(0) => ctl2_elost,
      probe_in5 => ctl2_ecount_vio,
      probe_in6 => ctl2_ephase_vio,
      probe_in7 => ctl2_eaddr_vio,
      probe_in8 => ctl2_edata_vio(31 downto 0),
      probe_in9 => ctl2_ebits_vio(31 downto 0),
      probe_in10 => ctl2_edata_vio(63 downto 32),
      probe_in11 => ctl2_ebits_vio(63 downto 32),
      probe_in12 => ctl2_edata_vio(95 downto 64),
      probe_in13 => ctl2_ebits_vio(95 downto 64),
      probe_in14 => ctl2_edata_vio(127 downto 96),
      probe_in15 => ctl2_ebits_vio(127 downto 96),
      probe_in16 => ctl2_edata_vio(159 downto 128),
      probe_in17 => ctl2_ebits_vio(159 downto 128),
      probe_in18 => ctl2_edata_vio(191 downto 160),
      probe_in19 => ctl2_ebits_vio(191 downto 160),
      probe_in20 => ctl2_edata_vio(223 downto 192),
      probe_in21 => ctl2_ebits_vio(223 downto 192),
      probe_in22 => ctl2_edata_vio(255 downto 224),
      probe_in23 => ctl2_ebits_vio(255 downto 224),
      --
      probe_out0(0) => vio2_reset,
      probe_out1    => ctl2_offset_vio,
      probe_out2    => ctl2_length_vio,
      probe_out3    => ctl2_eindex,
      probe_out4    => ctl2_ph_en);

    ctl2_offset <= ctl2_offset_vio(ctl2_offset'range);
    ctl2_length <= ctl2_length_vio(ctl2_length'range);


    ctl3_ecount_vio <= ctl3_ecount;
    ctl3_ephase_vio <= ctl3_ephase;
    ctl3_eaddr_vio  <= std_logic_vector(resize(unsigned(ctl3_eaddr), 32));
    ctl3_edata_vio  <= ctl3_edata;
    ctl3_ebits_vio  <= ctl3_ebits;

    vio3_i : vio_ip
    port map(
      clk => vio_clk,
      probe_in0(0) => c3_init_calib_complete,
      probe_in1(0) => ctl3_done,
      probe_in2(0) => ctl3_error,
      probe_in3(0) => ctl3_efull,
      probe_in4(0) => ctl3_elost,
      probe_in5 => ctl3_ecount_vio,
      probe_in6 => ctl3_ephase_vio,
      probe_in7 => ctl3_eaddr_vio,
      probe_in8 => ctl3_edata_vio(31 downto 0),
      probe_in9 => ctl3_ebits_vio(31 downto 0),
      probe_in10 => ctl3_edata_vio(63 downto 32),
      probe_in11 => ctl3_ebits_vio(63 downto 32),
      probe_in12 => ctl3_edata_vio(95 downto 64),
      probe_in13 => ctl3_ebits_vio(95 downto 64),
      probe_in14 => ctl3_edata_vio(127 downto 96),
      probe_in15 => ctl3_ebits_vio(127 downto 96),
      probe_in16 => ctl3_edata_vio(159 downto 128),
      probe_in17 => ctl3_ebits_vio(159 downto 128),
      probe_in18 => ctl3_edata_vio(191 downto 160),
      probe_in19 => ctl3_ebits_vio(191 downto 160),
      probe_in20 => ctl3_edata_vio(223 downto 192),
      probe_in21 => ctl3_ebits_vio(223 downto 192),
      probe_in22 => ctl3_edata_vio(255 downto 224),
      probe_in23 => ctl3_ebits_vio(255 downto 224),
      --
      probe_out0(0) => vio3_reset,
      probe_out1    => ctl3_offset_vio,
      probe_out2    => ctl3_length_vio,
      probe_out3    => ctl3_eindex,
      probe_out4    => ctl3_ph_en);

    ctl3_offset <= ctl3_offset_vio(ctl3_offset'range);
    ctl3_length <= ctl3_length_vio(ctl3_length'range);

  end generate;


  -- synthesis translate_off

  -- Status outputs used in simulation only.
  gen_stats_p : process(vio_clk)
  begin
    if rising_edge(vio_clk) then
      -- Synchronise into vio_clk domain
      i_stat_rst  <= vio3_reset & vio2_reset & vio1_reset & vio0_reset;
      i_stat_err  <= ctl3_error & ctl2_error & ctl1_error & ctl0_error;
      i_stat_done <= ctl3_done & ctl2_done & ctl1_done & ctl0_done;
      i_stat_init <= c3_init_calib_complete & c2_init_calib_complete & c1_init_calib_complete & c0_init_calib_complete;
    end if;
  end process;

  stat_rst  <= i_stat_rst;
  stat_err  <= i_stat_err;
  stat_done <= i_stat_done;
  stat_init <= i_stat_init;

  -- synthesis translate_on

end architecture;