library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;

library work;
use work.axi4_128.all;

entity tb_dma_demo is
  generic(
    sim_axi : boolean);
end;

architecture behav of tb_dma_demo is

  constant MODEL_NAME     : string(1 to 11) := "ADM-XRC-KU1";

  -- Clocks
  constant MPTL_CLK_PER   : time := 1 us/250;
  signal mptl_clk_p  : std_logic := '0';
  signal mptl_clk_n  : std_logic := '1';

  -- MPTL
  constant MPTL_SER_WIDTH : natural := 4;
  signal mptl_b2t_p : std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
  signal mptl_b2t_n : std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
  signal mptl_t2b_p : std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
  signal mptl_t2b_n : std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
  -- MPTL sideband
  signal mptl_bridge_gtp_online_l : std_logic := '1';
  signal mptl_target_configured_l : std_logic;
  signal mptl_target_gtp_online_l : std_logic;

  component dma_demo
  generic(
    sim_axi    : boolean := false);
  port(
    -- MPTL
    mptl_clk_p : in    std_logic;
    mptl_clk_n : in    std_logic;
    mptl_b2t_p : in    std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_b2t_n : in    std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_t2b_p : out   std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    mptl_t2b_n : out   std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    -- MPTL sideband
    mptl_bridge_gtp_online_l : in    std_logic;
    mptl_target_configured_l : out   std_logic;
    mptl_target_gtp_online_l : out   std_logic);
  end component;

begin

  -- MPTL reference clock
  mptl_clk_p <= not mptl_clk_p after MPTL_CLK_PER / 2;
  mptl_clk_n <= not mptl_clk_n after MPTL_CLK_PER / 2;

  
  -- In real hardware, this signal is driven by Artix Bridge FPGA.
  -- In simulation, we just assert it after a while.
  mptl_bridge_gtp_online_l <= '0' after 50 ns;
  

  uut : dma_demo
  generic map(
    sim_axi    => sim_axi)                                 -- in    boolean := false;
  port map(
    -- MPTL
    mptl_clk_p => mptl_clk_p,                              -- in    std_logic
    mptl_clk_n => mptl_clk_n,                              -- in    std_logic
    mptl_b2t_p => mptl_b2t_p,                              -- in    std_logic_vector(3 downto 0)
    mptl_b2t_n => mptl_b2t_n,                              -- in    std_logic_vector(3 downto 0)
    mptl_t2b_p => mptl_t2b_p,                              -- out   std_logic_vector(3 downto 0)
    mptl_t2b_n => mptl_t2b_n,                              -- out   std_logic_vector(3 downto 0)
    -- MPTL sideband
    mptl_bridge_gtp_online_l => mptl_bridge_gtp_online_l,  -- in    std_logic
    mptl_target_configured_l => mptl_target_configured_l,  -- out   std_logic
    mptl_target_gtp_online_l => mptl_target_gtp_online_l); -- out   std_logic


end;
