/*
** cmdline.cpp
**
** Test program for the Interrupt Test FPGA Design (itest)
**
** Notes
** =====
**
** This source file contains the main() entry point for Linux & Windows,
** and should not be compiled for VxWorks.
*/

#ifdef _WIN32

# include <windows.h>
# include <tchar.h>

typedef UINT32 uint32_t;

#else

# include <stdint.h>

# define _T(x) x
# define _tmain main
typedef char TCHAR;

#endif

#include <iostream>

#include <exappcmdline.h>

#include "itest.h"

using namespace AlphaData::AppFramework;

#ifdef _UNICODE
typedef std::wstringstream SStreamType;
#define SSEMPTY(ss) ((ss).str(std::basic_string<wchar_t>()))
#else
typedef std::stringstream SStreamType;
#define SSEMPTY(ss) ((ss).str(std::string()))
#endif

#ifdef _WIN32
const TCHAR g_bitPathDefault[] = _T("..\\..\\..\\..\\fpga\\bit");
#else
const TCHAR g_bitPathDefault[] = _T("../../../fpga/bit");
#endif

class MyCmdLineArguments : public CExAppCmdLineArgs {
public:
  MyCmdLineArguments(int argc, TCHAR* argv[]) :
    CExAppCmdLineArgs(argc, argv),
    m_index(0U, _T("index"), _T("Zero-based index of reconfigurable computing card to use. Mutually exclusive with '-sn'.")),
    m_serialNumber(_T("sn"), _T("Serial number of reconfigurable computing card to use. Mutually exclusive with '-index'.")),
    m_bitPath(g_bitPathDefault, _T("bitpath"), _T("Default path under which to look for itest.bit files.")),
    m_bitFile(_T("bitfile"), _T("The .bit file to use (overrides -bitpath)."))
  {
    AddOption(m_index);
    AddOption(m_serialNumber);
    AddOption(m_bitPath);
    AddOption(m_bitFile);
  }

  virtual void OnParseComplete() {
    if (m_index.IsSet() && m_serialNumber.IsSet()) {
      AddError(_T("'-index' and '-sn' options are mutually exclusive"));
      SetParseResult(ExAppParseResultMutuallyExclusive);
    }
  }
  
#ifdef _UNICODE
  virtual void DisplaySummary(OStreamType& ostream = std::wcout) {
#else
  virtual void DisplaySummary(OStreamType& ostream = std::cout) {
#endif
    m_bodyLayout.GetStream(0) << _T("This program tests the Interrupt Test FPGA Design (itest).") << std::endl << std::endl;
    m_bodyLayout.Flush(ostream);
    CExAppCmdLineArgs::DisplaySummary(ostream);
  }
  
public:
  CExAppOptionUInt m_index;
  CExAppOptionUInt32 m_serialNumber;
  CExAppOptionStringDef m_bitPath;
  CExAppOptionString m_bitFile;
};

int
_tmain(
  int argc,
  TCHAR* argv[])
{
  int exitCode = ExitCodeSuccess;

  // Parse the command-line arguments; performs most of the required validation.
  MyCmdLineArguments args(argc, argv);
  args.SetMessageDisplay(&g_itestMessageDisplay);
  ExAppParseResult result = args.Parse();
  args.DisplayMessages();
  if (ExAppParseResultOk != result) {
    exitCode = args.GetExitCode();
    goto done;
  }
  
  if (args.m_serialNumber.IsSet()) {
    exitCode = admxrc3ITestSN(
      args.m_bitPath.GetValueString().c_str(),
      args.m_bitFile.IsSet() ? args.m_bitFile.GetValueString().c_str() : NULL,
      args.m_serialNumber.GetValueUInt32());
  } else {
    exitCode = admxrc3ITestIndex(
      args.m_bitPath.GetValueString().c_str(),
      args.m_bitFile.IsSet() ? args.m_bitFile.GetValueString().c_str() : NULL,
      args.m_index.GetValueUInt());
  }

done:
  args.DisplayMessages();
  return (int)exitCode;
}
