library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;

entity mptl_tgt_ku_ocp_sim is
port(
  -- OCP
  ocp_ready                : in    std_logic;
  ocp_clk                  : in    std_logic;
  direct_slave_m2s         : out   adb3_ocp_m2sT;
  direct_slave_s2m         : in    adb3_ocp_s2mT;
  mptl_ready               : out   std_logic;
  -- MPTL
  mptl_tx_p                : out   std_logic_vector(3 downto 0);
  mptl_tx_n                : out   std_logic_vector(3 downto 0);
  mptl_rx_p                : in    std_logic_vector(3 downto 0);
  mptl_rx_n                : in    std_logic_vector(3 downto 0);
  mptl_clk_p               : in    std_logic;
  mptl_clk_n               : in    std_logic;
  mptl_clk_out             : out   std_logic;
  -- MPTL initialisation
  mptl_target_configured_n : out   std_logic;
  mptl_target_gtp_online_n : out   std_logic;
  mptl_bridge_gtp_online_n : in    std_logic;
  -- MPTL sideband
  gpio_t2b                 : in    std_logic_vector(15 downto 0);
  gpio_b2t                 : out   std_logic_vector(15 downto 0));
end;


architecture behav of mptl_tgt_ku_ocp_sim is

  signal mptl_clk_div2 : std_logic := '0';

  -- Configuration/online
  signal mptl_online : std_logic;
 
  -- Host application
  component tb_simple_ocp_stim
  port(
    -- OCP
    ocp_clk          : in    std_logic;
    direct_slave_m2s : out   adb3_ocp_m2sT;
    direct_slave_s2m : in    adb3_ocp_s2mT;
    -- Online
    mptl_online      : in    std_logic);
  end component;

begin
  
  mptl_ready <= ocp_ready;
  
  mptl_target_configured_n <= not ocp_ready;
  mptl_target_gtp_online_n <= mptl_bridge_gtp_online_n;
  
  -- Generate online signal
  mptl_online <= not mptl_bridge_gtp_online_n or not ocp_ready;

  mptl_tx_p <= (others => '-');
  mptl_tx_n <= (others => '-');

  -- MPTL clock div 2
  mptl_clk_out_p : process(mptl_clk_p)
  begin
    if rising_edge(mptl_clk_p) then
      mptl_clk_div2 <= not(mptl_clk_div2);
    end if;
  end process;

  -- mptl_clk_out = half mptl_clk frequency
  mptl_clk_out <= mptl_clk_div2;
  
  gpio_b2t <= (others => '-');


  -- Simulation of host application using OCP
  tb_simple_ocp_stim_i : tb_simple_ocp_stim
  port map(
    -- OCP
    ocp_clk          => ocp_clk,          -- in    std_logic
    direct_slave_m2s => direct_slave_m2s, -- out   adb3_ocp_m2sT
    direct_slave_s2m => direct_slave_s2m, -- in    adb3_ocp_s2mT
    -- Online
    mptl_online      => mptl_online);     -- in    std_logic
  
end;