library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_tb_pkg.all;

entity tb_simple_ocp_stim is
port(
  -- OCP
  ocp_clk          : in    std_logic;
  direct_slave_m2s : out   adb3_ocp_m2sT;
  direct_slave_s2m : in    adb3_ocp_s2mT;
  -- Online
  mptl_online      : in    std_logic);
end;

architecture behav of tb_simple_ocp_stim is

  signal ds_m2s : adb3_ocp_m2sT := ADB3_OCP_DEFAULT_M2S;

begin


  ----------------------------
  -- Direct Slave interface --
  ----------------------------

  direct_slave_p : process
    variable reg_data_wr : byte_vector_t(0 to ADB3_OCP_BE_WIDTH-1);
    variable reg_data_rd : byte_vector_t(0 to ADB3_OCP_BE_WIDTH-1);
    variable reg_be      : byte_enable_t(0 to ADB3_OCP_BE_WIDTH-1);
    variable reg_addr    : adb3_ocp_addr_s;
    variable reg_xfer    : natural;
    variable passed      : boolean := true; -- Assume all OK, falsify later if necessary

  begin
    wait until (mptl_online = '1');  -- OCP commands ignored until MPTL online
    report "MPTL now online";
        
    --------------------------
    -- Simple Register Test --
    --------------------------
    reg_be(0 to 3) := conv_byte_enable("1111");

    -- Write a pattern to 32-bit register
    -- Uses adb3_ocp_sim_write_reg32 procedure in adb3_ocp_tb_pkg
    reg_addr := (others => '0');
    reg_data_wr(0 to 3) := conv_byte_vector(X"cafeface");

    adb3_ocp_sim_write_reg32(reg_addr,10,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,direct_slave_s2m);
    report "Wrote simple DATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string_bin(reg_be(0 to 3)) &
           " to byte address 0x" & conv_string_hex(reg_addr);


    -- Read back nibble reversed pattern from 32-bit register
    -- Uses adb3_ocp_sim_read_reg32 procedure in adb3_ocp_tb_pkg
    reg_addr := (others => '0');
    adb3_ocp_sim_read_reg32(reg_addr,11,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,direct_slave_s2m);
    report "Read simple DATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address 0x" & conv_string_hex(reg_addr);

    if (reg_data_rd(0 to 3) = conv_byte_vector(X"ecafefac")) then
      assert false report "Write & read back register test completed: PASSED." severity note;
    else
      assert false report "Write & read back register test completed: FAILED." severity error;
      passed := false;
    end if;

    if passed then
      assert false report "Simulation of design Simple (OCP) completed: PASSED." severity failure;
    else
      assert false report "Simulation of design Simple (OCP) completed: FAILED." severity failure;
    end if;
    
    wait;
  end process;

  direct_slave_m2s <= ds_m2s;


end;