library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;

library std;
use std.textio.all;

library work;
use work.ddr4_sdram_model.all;
use work.ddr4_sdram_model_int.all;

entity ddr4_sdram_refresh_monitor is
  generic(
    model_name          : in    string;
    part                : in    part_t;
    num_bank_total      : in    natural
  );
  port(
    reset_n             : in    std_logic; -- Should be _X01 version of signal
    ck_t                : in    std_logic;
    cke                 : in    std_logic; -- Should be _X01 version of signal
    cs_n                : in    std_logic; -- Should be _X01 version of signal
    act_n               : in    std_logic; -- Should be _X01 version of signal
    ras_n               : in    std_logic; -- Should be _X01 version of signal
    cas_n               : in    std_logic; -- Should be _X01 version of signal
    we_n                : in    std_logic; -- Should be _X01 version of signal
    bank_states         : in    bank_state_array_t(0 to num_bank_total - 1)
  );
end entity;

architecture behav of ddr4_sdram_refresh_monitor is

  
begin

  monitor_refresh_cycles : process(reset_n, ck_t)
    variable refresh_complete_time : time := 0 ns;
  begin
    if reset_n = '0' then
      refresh_complete_time := 0 ns;
    elsif rising_edge(ck_t) then
      if cke = '1' then
        if cs_n = '0' then
          if now < refresh_complete_time then
            report model_name & ": Illegal non-DES command (" &
                   classify_command('1', cke, cs_n, act_n, ras_n, cas_n, we_n) &
                   ") during refresh cycle"
            severity warning;
          end if;
          
          if act_n = '1' and ras_n = '0' and cas_n = '0' and we_n = '1' then
            refresh_complete_time := now + part.timing.refresh.t_rfc;
          end if;
        end if;
      end if;
    end if;
  end process;
  
  
end architecture;
