
# Loading additional proc with user specified bodies to compute parameter values.
source [file join [file dirname [file dirname [info script]]] gui/ram_interface_axi4_v1_4.gtcl]

# Definitional proc to organize widgets for parameters.
proc init_gui { IPINST } {
  ipgui::add_param $IPINST -name "Component_Name"
  #Adding Page
  set axi4_param [ipgui::add_page $IPINST -name "axi4_param" -display_name {AXI4 Parameters}]
  set_property tooltip {AXI4 Memory-mapped Parameters} ${axi4_param}
  set axi4_interface_type [ipgui::add_param $IPINST -name "axi4_interface_type" -parent ${axi4_param} -widget comboBox]
  set_property tooltip {Specifies the type of AXI4 interface.} ${axi4_interface_type}
  set s_axi_axaddr_width [ipgui::add_param $IPINST -name "s_axi_axaddr_width" -parent ${axi4_param}]
  set_property tooltip {Specifies the width of AXI4 addresses.} ${s_axi_axaddr_width}
  set s_axi_xdata_width [ipgui::add_param $IPINST -name "s_axi_xdata_width" -parent ${axi4_param} -widget comboBox]
  set_property tooltip {Specifies the width of AXI4 data.} ${s_axi_xdata_width}
  set s_axi_xid_width_user [ipgui::add_param $IPINST -name "s_axi_xid_width_user" -parent ${axi4_param}]
  set_property tooltip {Specifies the width of AXI4 transaction IDs.} ${s_axi_xid_width_user}
  set s_axi_axlen_width_user [ipgui::add_param $IPINST -name "s_axi_axlen_width_user" -parent ${axi4_param}]
  set_property tooltip {Specifies the width of AXI4 burst lengths.} ${s_axi_axlen_width_user}
  set no_x [ipgui::add_param $IPINST -name "no_x" -parent ${axi4_param}]
  set_property tooltip {If true, never drives Xs on AXI signals even when not qualified by VALID. May be required to work around bugs in other AXI IP when simulating.} ${no_x}

  #Adding Page
  set ram_param [ipgui::add_page $IPINST -name "ram_param" -display_name {RAM Parameters}]
  set_property tooltip {RAM Parameters} ${ram_param}
  set ram_read_latency [ipgui::add_param $IPINST -name "ram_read_latency" -parent ${ram_param}]
  set_property tooltip {Specifies the read latency of the RAM, in clock cycles, where 1 means that read data is available in the next cycle after read enable is asserted.} ${ram_read_latency}
  set ram_order [ipgui::add_param $IPINST -name "ram_order" -parent ${ram_param} -widget comboBox]
  set_property tooltip {Specifies the depth of the RAM, in words.} ${ram_order}

  #Adding Page
  set arb_param [ipgui::add_page $IPINST -name "arb_param" -display_name {Arbitration Parameters}]
  set_property tooltip {Arbitration Parameters} ${arb_param}
  set arbitration_mode [ipgui::add_param $IPINST -name "arbitration_mode" -parent ${arb_param}]
  set_property tooltip {Specifies the arbitration mode for the read and write subchannels.} ${arbitration_mode}

  #Adding Page
  set debug [ipgui::add_page $IPINST -name "debug" -display_name {Debug}]
  set_property tooltip {These values are for debugging the IP-XACT definition itself, and can safely be ignored by users of the IP.} ${debug}
  ipgui::add_param $IPINST -name "log2_num_byte_lane" -parent ${debug}
  ipgui::add_param $IPINST -name "s_axi_axlen_width" -parent ${debug}
  ipgui::add_param $IPINST -name "s_axi_xid_width" -parent ${debug}


}

proc update_PARAM_VALUE.log2_num_byte_lane { PARAM_VALUE.log2_num_byte_lane PARAM_VALUE.s_axi_xdata_width } {
	# Procedure called to update log2_num_byte_lane when any of the dependent parameters in the arguments change
	
	set log2_num_byte_lane ${PARAM_VALUE.log2_num_byte_lane}
	set s_axi_xdata_width ${PARAM_VALUE.s_axi_xdata_width}
	set values(s_axi_xdata_width) [get_property value $s_axi_xdata_width]
	set_property value [gen_USERPARAMETER_log2_num_byte_lane_VALUE $values(s_axi_xdata_width)] $log2_num_byte_lane
}

proc validate_PARAM_VALUE.log2_num_byte_lane { PARAM_VALUE.log2_num_byte_lane } {
	# Procedure called to validate log2_num_byte_lane
	return true
}

proc update_PARAM_VALUE.s_axi_axlen_width { PARAM_VALUE.s_axi_axlen_width PARAM_VALUE.s_axi_axlen_width_user } {
	# Procedure called to update s_axi_axlen_width when any of the dependent parameters in the arguments change
	
	set s_axi_axlen_width ${PARAM_VALUE.s_axi_axlen_width}
	set s_axi_axlen_width_user ${PARAM_VALUE.s_axi_axlen_width_user}
	set values(s_axi_axlen_width_user) [get_property value $s_axi_axlen_width_user]
	set_property value [gen_USERPARAMETER_s_axi_axlen_width_VALUE $values(s_axi_axlen_width_user)] $s_axi_axlen_width
}

proc validate_PARAM_VALUE.s_axi_axlen_width { PARAM_VALUE.s_axi_axlen_width } {
	# Procedure called to validate s_axi_axlen_width
	return true
}

proc update_PARAM_VALUE.s_axi_axlen_width_user { PARAM_VALUE.s_axi_axlen_width_user PARAM_VALUE.axi4_interface_type } {
	# Procedure called to update s_axi_axlen_width_user when any of the dependent parameters in the arguments change
	
	set s_axi_axlen_width_user ${PARAM_VALUE.s_axi_axlen_width_user}
	set axi4_interface_type ${PARAM_VALUE.axi4_interface_type}
	set values(axi4_interface_type) [get_property value $axi4_interface_type]
	if { [gen_USERPARAMETER_s_axi_axlen_width_user_ENABLEMENT $values(axi4_interface_type)] } {
		set_property enabled true $s_axi_axlen_width_user
	} else {
		set_property enabled false $s_axi_axlen_width_user
	}
}

proc validate_PARAM_VALUE.s_axi_axlen_width_user { PARAM_VALUE.s_axi_axlen_width_user } {
	# Procedure called to validate s_axi_axlen_width_user
	return true
}

proc update_PARAM_VALUE.s_axi_xid_width { PARAM_VALUE.s_axi_xid_width PARAM_VALUE.s_axi_xid_width_user } {
	# Procedure called to update s_axi_xid_width when any of the dependent parameters in the arguments change
	
	set s_axi_xid_width ${PARAM_VALUE.s_axi_xid_width}
	set s_axi_xid_width_user ${PARAM_VALUE.s_axi_xid_width_user}
	set values(s_axi_xid_width_user) [get_property value $s_axi_xid_width_user]
	set_property value [gen_USERPARAMETER_s_axi_xid_width_VALUE $values(s_axi_xid_width_user)] $s_axi_xid_width
}

proc validate_PARAM_VALUE.s_axi_xid_width { PARAM_VALUE.s_axi_xid_width } {
	# Procedure called to validate s_axi_xid_width
	return true
}

proc update_PARAM_VALUE.s_axi_xid_width_user { PARAM_VALUE.s_axi_xid_width_user PARAM_VALUE.axi4_interface_type } {
	# Procedure called to update s_axi_xid_width_user when any of the dependent parameters in the arguments change
	
	set s_axi_xid_width_user ${PARAM_VALUE.s_axi_xid_width_user}
	set axi4_interface_type ${PARAM_VALUE.axi4_interface_type}
	set values(axi4_interface_type) [get_property value $axi4_interface_type]
	if { [gen_USERPARAMETER_s_axi_xid_width_user_ENABLEMENT $values(axi4_interface_type)] } {
		set_property enabled true $s_axi_xid_width_user
	} else {
		set_property enabled false $s_axi_xid_width_user
	}
}

proc validate_PARAM_VALUE.s_axi_xid_width_user { PARAM_VALUE.s_axi_xid_width_user } {
	# Procedure called to validate s_axi_xid_width_user
	return true
}

proc update_PARAM_VALUE.arbitration_mode { PARAM_VALUE.arbitration_mode } {
	# Procedure called to update arbitration_mode when any of the dependent parameters in the arguments change
}

proc validate_PARAM_VALUE.arbitration_mode { PARAM_VALUE.arbitration_mode } {
	# Procedure called to validate arbitration_mode
	return true
}

proc update_PARAM_VALUE.axi4_interface_type { PARAM_VALUE.axi4_interface_type } {
	# Procedure called to update axi4_interface_type when any of the dependent parameters in the arguments change
}

proc validate_PARAM_VALUE.axi4_interface_type { PARAM_VALUE.axi4_interface_type } {
	# Procedure called to validate axi4_interface_type
	return true
}

proc update_PARAM_VALUE.no_x { PARAM_VALUE.no_x } {
	# Procedure called to update no_x when any of the dependent parameters in the arguments change
}

proc validate_PARAM_VALUE.no_x { PARAM_VALUE.no_x } {
	# Procedure called to validate no_x
	return true
}

proc update_PARAM_VALUE.ram_order { PARAM_VALUE.ram_order } {
	# Procedure called to update ram_order when any of the dependent parameters in the arguments change
}

proc validate_PARAM_VALUE.ram_order { PARAM_VALUE.ram_order } {
	# Procedure called to validate ram_order
	return true
}

proc update_PARAM_VALUE.ram_read_latency { PARAM_VALUE.ram_read_latency } {
	# Procedure called to update ram_read_latency when any of the dependent parameters in the arguments change
}

proc validate_PARAM_VALUE.ram_read_latency { PARAM_VALUE.ram_read_latency } {
	# Procedure called to validate ram_read_latency
	return true
}

proc update_PARAM_VALUE.s_axi_axaddr_width { PARAM_VALUE.s_axi_axaddr_width } {
	# Procedure called to update s_axi_axaddr_width when any of the dependent parameters in the arguments change
}

proc validate_PARAM_VALUE.s_axi_axaddr_width { PARAM_VALUE.s_axi_axaddr_width } {
	# Procedure called to validate s_axi_axaddr_width
	return true
}

proc update_PARAM_VALUE.s_axi_xdata_width { PARAM_VALUE.s_axi_xdata_width } {
	# Procedure called to update s_axi_xdata_width when any of the dependent parameters in the arguments change
}

proc validate_PARAM_VALUE.s_axi_xdata_width { PARAM_VALUE.s_axi_xdata_width } {
	# Procedure called to validate s_axi_xdata_width
	return true
}


proc update_MODELPARAM_VALUE.s_axi_axaddr_width { MODELPARAM_VALUE.s_axi_axaddr_width PARAM_VALUE.s_axi_axaddr_width } {
	# Procedure called to set VHDL generic/Verilog parameter value(s) based on TCL parameter value
	set_property value [get_property value ${PARAM_VALUE.s_axi_axaddr_width}] ${MODELPARAM_VALUE.s_axi_axaddr_width}
}

proc update_MODELPARAM_VALUE.s_axi_xdata_width { MODELPARAM_VALUE.s_axi_xdata_width PARAM_VALUE.s_axi_xdata_width } {
	# Procedure called to set VHDL generic/Verilog parameter value(s) based on TCL parameter value
	set_property value [get_property value ${PARAM_VALUE.s_axi_xdata_width}] ${MODELPARAM_VALUE.s_axi_xdata_width}
}

proc update_MODELPARAM_VALUE.s_axi_xid_width { MODELPARAM_VALUE.s_axi_xid_width PARAM_VALUE.s_axi_xid_width } {
	# Procedure called to set VHDL generic/Verilog parameter value(s) based on TCL parameter value
	set_property value [get_property value ${PARAM_VALUE.s_axi_xid_width}] ${MODELPARAM_VALUE.s_axi_xid_width}
}

proc update_MODELPARAM_VALUE.s_axi_axlen_width { MODELPARAM_VALUE.s_axi_axlen_width PARAM_VALUE.s_axi_axlen_width } {
	# Procedure called to set VHDL generic/Verilog parameter value(s) based on TCL parameter value
	set_property value [get_property value ${PARAM_VALUE.s_axi_axlen_width}] ${MODELPARAM_VALUE.s_axi_axlen_width}
}

proc update_MODELPARAM_VALUE.no_x { MODELPARAM_VALUE.no_x PARAM_VALUE.no_x } {
	# Procedure called to set VHDL generic/Verilog parameter value(s) based on TCL parameter value
	set_property value [get_property value ${PARAM_VALUE.no_x}] ${MODELPARAM_VALUE.no_x}
}

proc update_MODELPARAM_VALUE.ram_read_latency { MODELPARAM_VALUE.ram_read_latency PARAM_VALUE.ram_read_latency } {
	# Procedure called to set VHDL generic/Verilog parameter value(s) based on TCL parameter value
	set_property value [get_property value ${PARAM_VALUE.ram_read_latency}] ${MODELPARAM_VALUE.ram_read_latency}
}

proc update_MODELPARAM_VALUE.ram_order { MODELPARAM_VALUE.ram_order PARAM_VALUE.ram_order } {
	# Procedure called to set VHDL generic/Verilog parameter value(s) based on TCL parameter value
	set_property value [get_property value ${PARAM_VALUE.ram_order}] ${MODELPARAM_VALUE.ram_order}
}

proc update_MODELPARAM_VALUE.arbitration_mode { MODELPARAM_VALUE.arbitration_mode PARAM_VALUE.arbitration_mode } {
	# Procedure called to set VHDL generic/Verilog parameter value(s) based on TCL parameter value
	set_property value [get_property value ${PARAM_VALUE.arbitration_mode}] ${MODELPARAM_VALUE.arbitration_mode}
}

