--==============================================================================
--
--     File Name: adb3_ocp_dm_adapter_dwfifo.vhd
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: adb3_ocp_dm_adapter
--
--   Description: Submodule of adb3_ocp_dm_adapter responsible for buffering
--                write data and byte enables to the Direct Master OCP channel.
--
--   Limitations: 
--
--         Notes: 
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2013 Alpha Data, All rights reserved
--
--     Revisions: 0.1, TW, 9 Dec 2013: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_dm_int.all;

entity adb3_ocp_dm_adapter_dwfifo is
generic(
  ORDER           : in    natural;
  DATA_WIDTH      : in    natural);
port(
  ocp_rst         : in    std_logic;
  ocp_clk         : in    std_logic;
  -- Data & byte enables from chopper (writes only)
  chop_data       : in    std_logic_vector(DATA_WIDTH - 1 downto 0);
  chop_be         : in    std_logic_vector(DATA_WIDTH / 8 - 1 downto 0);
  chop_data_valid : in    std_logic;
  -- To DM OCP port
  dm_data_acc     : in    std_logic;
  dm_data         : out   std_logic_vector(DATA_WIDTH - 1 downto 0);
  dm_be           : out   std_logic_vector(DATA_WIDTH / 8 - 1 downto 0);
  dm_data_valid   : out   std_logic;
  -- Flow control
  nafull          : out   std_logic;
  -- Debug
  debug_info      : out   std_logic_vector(11 downto 0));
end;


architecture syn of adb3_ocp_dm_adapter_dwfifo is

  constant BE_WIDTH : natural := DATA_WIDTH / 8;
  constant WIDTH : natural := BE_WIDTH + DATA_WIDTH;
  
  signal wd : std_logic_vector(WIDTH - 1 downto 0);
  signal rq : std_logic_vector(WIDTH - 1 downto 0);
  signal radv : std_logic;
  signal rvalid : std_logic;
  
  signal logic0 : std_logic;

  
begin
  
  logic0 <= '0';
  
  wd <= chop_be & chop_data;
         
  U0 : adb3_ocp_dm_adapter_sfifo
  generic map(
    ORDER => ORDER,
    WIDTH => WIDTH)
  port map(
    rst => ocp_rst,
    sr => logic0,
    clk => ocp_clk,
    wen => chop_data_valid,
    wd => wd,
    wafull => open,
    wnafull => nafull,
    radv => radv,
    rvalid => rvalid,
    rq => rq,
    debug_info => debug_info);
  
  radv <= dm_data_acc AND rvalid;
  
  dm_data <= rq(DATA_WIDTH - 1 downto 0);
  dm_be <= rq(BE_WIDTH + DATA_WIDTH - 1 downto DATA_WIDTH);
  dm_data_valid <= rvalid;
  
end;
