--==============================================================================
--
--     File Name: adb3_ocp_dm_adapter_owfifo.vhd
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: adb3_ocp_dm_adapter
--
--   Description: Submodule of adb3_ocp_dm_adapter responsible for buffering
--                write data and byte enables from the general OCP port.
--
--   Limitations: 
--
--         Notes: 
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2013 Alpha Data, All rights reserved
--
--     Revisions: 0.1, TW, 9 Dec 2013: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_dm_int.all;

entity adb3_ocp_dm_adapter_owfifo is
generic(
  ORDER           : in    natural;
  DATA_WIDTH      : in    natural);
port(
  ocp_rst         : in    std_logic;
  ocp_clk         : in    std_logic;
  -- Data & byte enables from general OCP port
  ocp_data        : in    std_logic_vector(DATA_WIDTH - 1 downto 0);
  ocp_be          : in    std_logic_vector(DATA_WIDTH / 8 - 1 downto 0);
  ocp_data_valid  : in    std_logic;
  ocp_data_acc    : out   std_logic;
  -- Data & byte enables to Chopper
  owf_data_adv    : in    std_logic;
  owf_data        : out   std_logic_vector(DATA_WIDTH - 1 downto 0);
  owf_be          : out   std_logic_vector(DATA_WIDTH / 8 - 1 downto 0);
  owf_data_valid  : out   std_logic;
  -- Debug
  debug_info      : out   std_logic_vector(11 downto 0));
end;


architecture syn of adb3_ocp_dm_adapter_owfifo is

  constant BE_WIDTH : natural := DATA_WIDTH / 8;
  constant WIDTH : natural := BE_WIDTH + DATA_WIDTH;
  
  signal nafull : std_logic;
  signal wen : std_logic;
  signal wd : std_logic_vector(WIDTH - 1 downto 0);
  signal rq : std_logic_vector(WIDTH - 1 downto 0);
  
  signal logic0 : std_logic;

  
begin
  
  logic0 <= '0';
  
  wen <= ocp_data_valid AND nafull;
  wd <= ocp_be & ocp_data;
  ocp_data_acc <= nafull;
         
  U0 : adb3_ocp_dm_adapter_sfifo
  generic map(
    ORDER => ORDER,
    WIDTH => WIDTH)
  port map(
    rst => ocp_rst,
    sr => logic0,
    clk => ocp_clk,
    wen => wen,
    wd => wd,
    wafull => open,
    wnafull => nafull,
    radv => owf_data_adv,
    rvalid => owf_data_valid,
    rq => rq,
    debug_info => debug_info);
  
  owf_data <= rq(DATA_WIDTH - 1 downto 0);
  owf_be <= rq(BE_WIDTH + DATA_WIDTH - 1 downto DATA_WIDTH);
  
end;
