--==============================================================================
--
--     File Name: adb3_ocp_dm_adapter_rfifo.vhd
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: adb3_ocp_dm_adapter
--
--   Description: Submodule of adb3_ocp_dm_adapter responsible for buffering
--                tag-mapped responses from Direct Master OCP channel.
--
--   Limitations: 
--
--         Notes: 
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2013 Alpha Data, All rights reserved
--
--     Revisions: 0.1, TW, 26 Nov 2013: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_dm_int.all;

entity adb3_ocp_dm_adapter_rfifo is
generic(
  DATA_WIDTH      : in    natural);
port(
  ocp_rst         : in    std_logic;
  ocp_clk         : in    std_logic;
  -- Tag-mapped response to response FIFO
  tm_resp_valid   : in    std_logic;
  tm_resp         : in    ocp_RespT;
  tm_data         : in    std_logic_vector(DATA_WIDTH - 1 downto 0);
  tm_tag          : in    adb3_ocp_tag_s;
  tm_resp_acc     : out   std_logic;
  -- Response to OCP master
  ocp_resp_acc    : in    std_logic;
  ocp_resp        : out   ocp_RespT;
  ocp_data        : out   std_logic_vector(DATA_WIDTH - 1 downto 0);
  ocp_tag         : out   adb3_ocp_tag_s;
  -- Debug
  debug_info      : out   std_logic_vector(11 downto 0));
end;


architecture syn of adb3_ocp_dm_adapter_rfifo is

  constant ORDER : natural := 4;
  constant WIDTH : natural := DATA_WIDTH + ADB3_OCP_TAG_WIDTH + ADB3_OCP_RESP_WIDTH;
  constant EMPTY_LO : natural := DATA_WIDTH + ADB3_OCP_TAG_WIDTH;
  constant EMPTY_HI : natural := WIDTH;
  
  signal wd : std_logic_vector(WIDTH - 1 downto 0);
  signal rq : std_logic_vector(WIDTH - 1 downto 0);
  signal radv : std_logic;
  signal rvalid : std_logic;
  
  signal logic0 : std_logic;

  
begin
  
  logic0 <= '0';
  
  wd <= tm_resp & tm_tag & tm_data;
  
  U0 : adb3_ocp_dm_adapter_sfifo
  generic map(
    ORDER => ORDER,
    WIDTH => WIDTH,
    EMPTY_LO => EMPTY_LO,
    EMPTY_HI => EMPTY_HI)
  port map(
    rst => ocp_rst,
    sr => logic0,
    clk => ocp_clk,
    wen => tm_resp_valid,
    wd => wd,
    wafull => open,
    wnafull => tm_resp_acc,
    radv => radv,
    rvalid => rvalid,
    rq => rq,
    debug_info => debug_info);
  
  radv <= ocp_resp_acc AND rvalid;
  
  ocp_data <= rq(DATA_WIDTH - 1 downto 0);
  ocp_tag <= rq(DATA_WIDTH + ADB3_OCP_TAG_WIDTH - 1 downto DATA_WIDTH);
  ocp_resp <= rq(DATA_WIDTH + ADB3_OCP_TAG_WIDTH + ADB3_OCP_RESP_WIDTH - 1 downto DATA_WIDTH + ADB3_OCP_TAG_WIDTH);
  
end;
