--
-- ADB3 Target PCIe Bridge
--
-- Embeddable Target PCIe Bridge
-- Supports Direct Slave and 4 channel DMA transfers only 
--

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.STD_LOGIC_MISC.all;

library work;
use work.adb3_ocp.all;
use work.adb3_model_defs.all;
use work.adb3w_ocp_axi4.all;
use work.adb3_admxrcku1_bm.all;


library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity adb3_admxrcku1_x8_axi4 is
  generic (
    number_of_dma_engines    : natural range 0 to 4   := 4;
    number_of_pcie_dm_ports  : natural range 0 to 1   := 0;
    number_of_bar01_dm_ports : natural range 0 to 1   := 0;
    dma_engine0_config       : natural range 0 to 4   := 0;
    dma_engine1_config       : natural range 0 to 4   := 0;
    dma_engine2_config       : natural range 0 to 4   := 0;
    dma_engine3_config       : natural range 0 to 4   := 0;
    use_axi4_pcie_legalizer  : boolean                := false;
    bar2_size                : natural range 15 to 21 := 15;
    enable_dma_abort         : boolean                := false;
    enable_interrupt         : boolean                := false;
    use_diff_ref_clock : boolean := true;
    pcie_dm_port_id_width : natural range 1 to 4 := 1;
    behav_sim_axi4_over_pcie : boolean := false;
    abmi_mode    : natural range 0 to 2 := 0
    );

  port (
    -- PCIE
    perst_n            : in  std_logic;
    pcie100_p          : in  std_logic;
    pcie100_n          : in  std_logic;
    refclk300_p        : in  std_logic;
    refclk300_n        : in  std_logic;
    refclk300_in       : in  std_logic;
    refclk             : out std_logic;
    aclk               : out std_logic;
    aresetn            : out std_logic;
    pci_exp_txn        : out std_logic_vector(7 downto 0);
    pci_exp_txp        : out std_logic_vector(7 downto 0);
    pci_exp_rxn        : in  std_logic_vector(7 downto 0);
    pci_exp_rxp        : in  std_logic_vector(7 downto 0);
    -- Model Specific IO
    vpd_out            : out std_logic_vector(47 downto 0);
    avr_clk            : in  std_logic;
    avr_b2t            : in  std_logic;
    avr_t2b            : out std_logic;
    --
    
    -- AXI4 Interfaces
    -- Direct Slave
    ds_axi_awaddr      : out std_logic_vector(63 downto 0);
    ds_axi_awlen       : out std_logic_vector(7 downto 0);
    ds_axi_awsize      : out std_logic_vector(2 downto 0);
    ds_axi_awburst     : out std_logic_vector(1 downto 0);
    ds_axi_awcache     : out std_logic_vector(3 downto 0);
    ds_axi_awprot      : out std_logic_vector(2 downto 0);
    ds_axi_awvalid     : out std_logic;
    ds_axi_wdata       : out std_logic_vector(255 downto 0);
    ds_axi_wstrb       : out std_logic_vector(31 downto 0);
    ds_axi_wlast       : out std_logic;
    ds_axi_wvalid      : out std_logic;
    ds_axi_bready      : out std_logic;
    ds_axi_araddr      : out std_logic_vector(63 downto 0);
    ds_axi_arlen       : out std_logic_vector(7 downto 0);
    ds_axi_arsize      : out std_logic_vector(2 downto 0);
    ds_axi_arburst     : out std_logic_vector(1 downto 0);
    ds_axi_arcache     : out std_logic_vector(3 downto 0);
    ds_axi_arprot      : out std_logic_vector(2 downto 0);
    ds_axi_arvalid     : out std_logic;
    ds_axi_rready      : out std_logic;
    ds_axi_awready     : in  std_logic;
    ds_axi_wready      : in  std_logic;
    ds_axi_bresp       : in  std_logic_vector(1 downto 0);
    ds_axi_bvalid      : in  std_logic;
    ds_axi_arready     : in  std_logic;
    ds_axi_rdata       : in  std_logic_vector(255 downto 0);
    ds_axi_rresp       : in  std_logic_vector(1 downto 0);
    ds_axi_rlast       : in  std_logic;
    ds_axi_rvalid      : in  std_logic;
    ds_axi_awlock      : out std_logic_vector(0 downto 0);
    ds_axi_awqos       : out std_logic_vector (3 downto 0);
    ds_axi_arlock      : out std_logic_vector(0 downto 0);
    ds_axi_arqos       : out std_logic_vector (3 downto 0);
    ds_axi_awregion    : out std_logic_vector (3 downto 0);
    ds_axi_arregion    : out std_logic_vector (3 downto 0);
    -- DMA Channel 0
    dma0_axi_awaddr    : out std_logic_vector(38 downto 0);
    dma0_axi_awlen     : out std_logic_vector(7 downto 0);
    dma0_axi_awsize    : out std_logic_vector(2 downto 0);
    dma0_axi_awburst   : out std_logic_vector(1 downto 0);
    dma0_axi_awcache   : out std_logic_vector(3 downto 0);
    dma0_axi_awprot    : out std_logic_vector(2 downto 0);
    dma0_axi_awvalid   : out std_logic;
    dma0_axi_wdata     : out std_logic_vector(255 downto 0);
    dma0_axi_wstrb     : out std_logic_vector(31 downto 0);
    dma0_axi_wlast     : out std_logic;
    dma0_axi_wvalid    : out std_logic;
    dma0_axi_bready    : out std_logic;
    dma0_axi_araddr    : out std_logic_vector(38 downto 0);
    dma0_axi_arlen     : out std_logic_vector(7 downto 0);
    dma0_axi_arsize    : out std_logic_vector(2 downto 0);
    dma0_axi_arburst   : out std_logic_vector(1 downto 0);
    dma0_axi_arcache   : out std_logic_vector(3 downto 0);
    dma0_axi_arprot    : out std_logic_vector(2 downto 0);
    dma0_axi_arvalid   : out std_logic;
    dma0_axi_rready    : out std_logic;
    dma0_axi_awready   : in  std_logic;
    dma0_axi_wready    : in  std_logic;
    dma0_axi_bresp     : in  std_logic_vector(1 downto 0);
    dma0_axi_bvalid    : in  std_logic;
    dma0_axi_arready   : in  std_logic;
    dma0_axi_rdata     : in  std_logic_vector(255 downto 0);
    dma0_axi_rresp     : in  std_logic_vector(1 downto 0);
    dma0_axi_rlast     : in  std_logic;
    dma0_axi_rvalid    : in  std_logic;
    dma0_axi_awlock    : out std_logic_vector(0 downto 0);
    dma0_axi_awqos     : out std_logic_vector (3 downto 0);
    dma0_axi_arlock    : out std_logic_vector(0 downto 0);
    dma0_axi_arqos     : out std_logic_vector (3 downto 0);
    dma0_axi_awregion  : out std_logic_vector (3 downto 0);
    dma0_axi_arregion  : out std_logic_vector (3 downto 0);
    -- DMA Channel 1
    dma1_axi_awaddr    : out std_logic_vector(38 downto 0);
    dma1_axi_awlen     : out std_logic_vector(7 downto 0);
    dma1_axi_awsize    : out std_logic_vector(2 downto 0);
    dma1_axi_awburst   : out std_logic_vector(1 downto 0);
    dma1_axi_awcache   : out std_logic_vector(3 downto 0);
    dma1_axi_awprot    : out std_logic_vector(2 downto 0);
    dma1_axi_awvalid   : out std_logic;
    dma1_axi_wdata     : out std_logic_vector(255 downto 0);
    dma1_axi_wstrb     : out std_logic_vector(31 downto 0);
    dma1_axi_wlast     : out std_logic;
    dma1_axi_wvalid    : out std_logic;
    dma1_axi_bready    : out std_logic;
    dma1_axi_araddr    : out std_logic_vector(38 downto 0);
    dma1_axi_arlen     : out std_logic_vector(7 downto 0);
    dma1_axi_arsize    : out std_logic_vector(2 downto 0);
    dma1_axi_arburst   : out std_logic_vector(1 downto 0);
    dma1_axi_arcache   : out std_logic_vector(3 downto 0);
    dma1_axi_arprot    : out std_logic_vector(2 downto 0);
    dma1_axi_arvalid   : out std_logic;
    dma1_axi_rready    : out std_logic;
    dma1_axi_awready   : in  std_logic;
    dma1_axi_wready    : in  std_logic;
    dma1_axi_bresp     : in  std_logic_vector(1 downto 0);
    dma1_axi_bvalid    : in  std_logic;
    dma1_axi_arready   : in  std_logic;
    dma1_axi_rdata     : in  std_logic_vector(255 downto 0);
    dma1_axi_rresp     : in  std_logic_vector(1 downto 0);
    dma1_axi_rlast     : in  std_logic;
    dma1_axi_rvalid    : in  std_logic;
    dma1_axi_awlock    : out std_logic_vector(0 downto 0);
    dma1_axi_awqos     : out std_logic_vector (3 downto 0);
    dma1_axi_arlock    : out std_logic_vector(0 downto 0);
    dma1_axi_arqos     : out std_logic_vector (3 downto 0);
    dma1_axi_awregion  : out std_logic_vector (3 downto 0);
    dma1_axi_arregion  : out std_logic_vector (3 downto 0);
    -- DMA Channel 2
    dma2_axi_awaddr    : out std_logic_vector(38 downto 0);
    dma2_axi_awlen     : out std_logic_vector(7 downto 0);
    dma2_axi_awsize    : out std_logic_vector(2 downto 0);
    dma2_axi_awburst   : out std_logic_vector(1 downto 0);
    dma2_axi_awcache   : out std_logic_vector(3 downto 0);
    dma2_axi_awprot    : out std_logic_vector(2 downto 0);
    dma2_axi_awvalid   : out std_logic;
    dma2_axi_wdata     : out std_logic_vector(255 downto 0);
    dma2_axi_wstrb     : out std_logic_vector(31 downto 0);
    dma2_axi_wlast     : out std_logic;
    dma2_axi_wvalid    : out std_logic;
    dma2_axi_bready    : out std_logic;
    dma2_axi_araddr    : out std_logic_vector(38 downto 0);
    dma2_axi_arlen     : out std_logic_vector(7 downto 0);
    dma2_axi_arsize    : out std_logic_vector(2 downto 0);
    dma2_axi_arburst   : out std_logic_vector(1 downto 0);
    dma2_axi_arcache   : out std_logic_vector(3 downto 0);
    dma2_axi_arprot    : out std_logic_vector(2 downto 0);
    dma2_axi_arvalid   : out std_logic;
    dma2_axi_rready    : out std_logic;
    dma2_axi_awready   : in  std_logic;
    dma2_axi_wready    : in  std_logic;
    dma2_axi_bresp     : in  std_logic_vector(1 downto 0);
    dma2_axi_bvalid    : in  std_logic;
    dma2_axi_arready   : in  std_logic;
    dma2_axi_rdata     : in  std_logic_vector(255 downto 0);
    dma2_axi_rresp     : in  std_logic_vector(1 downto 0);
    dma2_axi_rlast     : in  std_logic;
    dma2_axi_rvalid    : in  std_logic;
    dma2_axi_awlock    : out std_logic_vector(0 downto 0);
    dma2_axi_awqos     : out std_logic_vector (3 downto 0);
    dma2_axi_arlock    : out std_logic_vector(0 downto 0);
    dma2_axi_arqos     : out std_logic_vector (3 downto 0);
    dma2_axi_awregion  : out std_logic_vector (3 downto 0);
    dma2_axi_arregion  : out std_logic_vector (3 downto 0);
    -- DMA Channel 3
    dma3_axi_awaddr    : out std_logic_vector(38 downto 0);
    dma3_axi_awlen     : out std_logic_vector(7 downto 0);
    dma3_axi_awsize    : out std_logic_vector(2 downto 0);
    dma3_axi_awburst   : out std_logic_vector(1 downto 0);
    dma3_axi_awcache   : out std_logic_vector(3 downto 0);
    dma3_axi_awprot    : out std_logic_vector(2 downto 0);
    dma3_axi_awvalid   : out std_logic;
    dma3_axi_wdata     : out std_logic_vector(255 downto 0);
    dma3_axi_wstrb     : out std_logic_vector(31 downto 0);
    dma3_axi_wlast     : out std_logic;
    dma3_axi_wvalid    : out std_logic;
    dma3_axi_bready    : out std_logic;
    dma3_axi_araddr    : out std_logic_vector(38 downto 0);
    dma3_axi_arlen     : out std_logic_vector(7 downto 0);
    dma3_axi_arsize    : out std_logic_vector(2 downto 0);
    dma3_axi_arburst   : out std_logic_vector(1 downto 0);
    dma3_axi_arcache   : out std_logic_vector(3 downto 0);
    dma3_axi_arprot    : out std_logic_vector(2 downto 0);
    dma3_axi_arvalid   : out std_logic;
    dma3_axi_rready    : out std_logic;
    dma3_axi_awready   : in  std_logic;
    dma3_axi_wready    : in  std_logic;
    dma3_axi_bresp     : in  std_logic_vector(1 downto 0);
    dma3_axi_bvalid    : in  std_logic;
    dma3_axi_arready   : in  std_logic;
    dma3_axi_rdata     : in  std_logic_vector(255 downto 0);
    dma3_axi_rresp     : in  std_logic_vector(1 downto 0);
    dma3_axi_rlast     : in  std_logic;
    dma3_axi_rvalid    : in  std_logic;
    dma3_axi_awlock    : out std_logic_vector(0 downto 0);
    dma3_axi_awqos     : out std_logic_vector (3 downto 0);
    dma3_axi_arlock    : out std_logic_vector(0 downto 0);
    dma3_axi_arqos     : out std_logic_vector (3 downto 0);
    dma3_axi_awregion  : out std_logic_vector (3 downto 0);
    dma3_axi_arregion  : out std_logic_vector (3 downto 0);
    -- PCIE Direct Master Channel
    dm_axi_awaddr      : in  std_logic_vector(63 downto 0);
    dm_axi_awlen       : in  std_logic_vector(7 downto 0);
    dm_axi_awsize      : in  std_logic_vector(2 downto 0);
    dm_axi_awburst     : in  std_logic_vector(1 downto 0);
    dm_axi_awcache     : in  std_logic_vector(3 downto 0);
    dm_axi_awprot      : in  std_logic_vector(2 downto 0);
    dm_axi_awvalid     : in  std_logic;
    dm_axi_awid    : in std_Logic_vector(pcie_dm_port_id_width-1 downto 0);
    dm_axi_wdata       : in  std_logic_vector(255 downto 0);
    dm_axi_wstrb       : in  std_logic_vector(31 downto 0);
    dm_axi_wlast       : in  std_logic;
    dm_axi_wvalid      : in  std_logic;
    --dm_axi_wid    : in std_Logic_vector(pcie_dm_port_id_width-1 downto 0);
    dm_axi_bready      : in  std_logic;
    dm_axi_araddr      : in  std_logic_vector(63 downto 0);
    dm_axi_arlen       : in  std_logic_vector(7 downto 0);
    dm_axi_arsize      : in  std_logic_vector(2 downto 0);
    dm_axi_arburst     : in  std_logic_vector(1 downto 0);
    dm_axi_arcache     : in  std_logic_vector(3 downto 0);
    dm_axi_arprot      : in  std_logic_vector(2 downto 0);
    dm_axi_arvalid     : in  std_logic;
    dm_axi_arid    : in std_Logic_vector(pcie_dm_port_id_width-1 downto 0);
    dm_axi_rready      : in  std_logic;
    dm_axi_awready     : out std_logic;
    dm_axi_wready      : out std_logic;
    dm_axi_bresp       : out std_logic_vector(1 downto 0);
    dm_axi_bvalid      : out std_logic;
    dm_axi_bid    : out std_Logic_vector(pcie_dm_port_id_width-1 downto 0);
    dm_axi_arready     : out std_logic;
    dm_axi_rdata       : out std_logic_vector(255 downto 0);
    dm_axi_rresp       : out std_logic_vector(1 downto 0);
    dm_axi_rlast       : out std_logic;
    dm_axi_rvalid      : out std_logic;
    dm_axi_rid    : out std_Logic_vector(pcie_dm_port_id_width-1 downto 0);
    dm_axi_awlock      : in  std_logic_vector(0 downto 0);
    dm_axi_awqos       : in  std_logic_vector (3 downto 0);
    dm_axi_arlock      : in  std_logic_vector(0 downto 0);
    dm_axi_arqos       : in  std_logic_vector (3 downto 0);
    dm_axi_awregion    : in  std_logic_vector (3 downto 0);
    dm_axi_arregion    : in  std_logic_vector (3 downto 0);
    -- BAR Direct Master Channel
    dmb_axi_awaddr     : in  std_logic_vector(13 downto 0);
    dmb_axi_awvalid    : in  std_logic;
    dmb_axi_wdata      : in  std_logic_vector(31 downto 0);
    dmb_axi_wstrb      : in  std_logic_vector(3 downto 0);
    dmb_axi_wvalid     : in  std_logic;
    dmb_axi_bready     : in  std_logic;
    dmb_axi_araddr     : in  std_logic_vector(13 downto 0);
    dmb_axi_arvalid    : in  std_logic;
    dmb_axi_rready     : in  std_logic;
    dmb_axi_awready    : out std_logic;
    dmb_axi_wready     : out std_logic;
    dmb_axi_bresp      : out std_logic_vector(1 downto 0);
    dmb_axi_bvalid     : out std_logic;
    dmb_axi_arready    : out std_logic;
    dmb_axi_rdata      : out std_logic_vector(31 downto 0);
    dmb_axi_rresp      : out std_logic_vector(1 downto 0);
    dmb_axi_rvalid     : out std_logic;
    -- ABMI AXI4 Lite Master Channel
    abmi_axi_awaddr     : out  std_logic_vector(11 downto 0);
    abmi_axi_awvalid    : out  std_logic;
    abmi_axi_wdata      : out  std_logic_vector(31 downto 0);
    abmi_axi_wstrb      : out  std_logic_vector(3 downto 0);
    abmi_axi_wvalid     : out  std_logic;
    abmi_axi_bready     : out  std_logic;
    abmi_axi_araddr     : out  std_logic_vector(11 downto 0);
    abmi_axi_arvalid    : out  std_logic;
    abmi_axi_rready     : out  std_logic;  
    abmi_axi_awready    : in std_logic;
    abmi_axi_wready     : in std_logic;
    abmi_axi_bresp      : in std_logic_vector(1 downto 0);
    abmi_axi_bvalid     : in std_logic;
    abmi_axi_arready    : in std_logic;
    abmi_axi_rdata      : in std_logic_vector(31 downto 0);
    abmi_axi_rresp      : in std_logic_vector(1 downto 0);   
    abmi_axi_rvalid     : in std_logic;
    -- Flattened OCP Master ABMI
    abmi_m2s_slv        : out std_Logic_vector(232 downto 0);
    abmi_s2m_slv        : in  std_logic_vector(139 downto 0);
    
    -- AXIS Ports for optional AXI4 Stream configuration of DMA Channels
    dma0_m_axis_tdata  : out std_logic_vector(255 downto 0);
    dma0_m_axis_tready : in  std_logic;
    dma0_m_axis_tvalid : out std_logic;
    dma0_s_axis_tdata  : in  std_logic_vector(255 downto 0);
    dma0_s_axis_tready : out std_logic;
    dma0_s_axis_tvalid : in  std_logic;
    dma1_m_axis_tdata  : out std_logic_vector(255 downto 0);
    dma1_m_axis_tready : in  std_logic;
    dma1_m_axis_tvalid : out std_logic;
    dma1_s_axis_tdata  : in  std_logic_vector(255 downto 0);
    dma1_s_axis_tready : out std_logic;
    dma1_s_axis_tvalid : in  std_logic;
    dma2_m_axis_tdata  : out std_logic_vector(255 downto 0);
    dma2_m_axis_tready : in  std_logic;
    dma2_m_axis_tvalid : out std_logic;
    dma2_s_axis_tdata  : in  std_logic_vector(255 downto 0);
    dma2_s_axis_tready : out std_logic;
    dma2_s_axis_tvalid : in  std_logic;
    dma3_m_axis_tdata  : out std_logic_vector(255 downto 0);
    dma3_m_axis_tready : in  std_logic;
    dma3_m_axis_tvalid : out std_logic;
    dma3_s_axis_tdata  : in  std_logic_vector(255 downto 0);
    dma3_s_axis_tready : out std_logic;
    dma3_s_axis_tvalid : in  std_logic;
    interrupt          : in  std_logic;
    dma0_abort         : in  std_logic;
    dma1_abort         : in  std_logic;
    dma2_abort         : in  std_logic;
    dma3_abort         : in  std_logic;
    core_status        : out std_logic_vector(63 downto 0));

end adb3_admxrcku1_x8_axi4;


architecture synth of adb3_admxrcku1_x8_axi4 is

  component adb3_admxrcku1_tri is
    generic (
      no_dma_engines      : integer                := 4;
      pcie_direct_masters : integer                := 1;
      reg_direct_masters  : integer                := 0;
      dma_engine_config   : std_logic_vector(15 downto 0);
      bar2_size           : natural range 15 to 21 := 15);
    port (
      sys_reset_n            : in  std_logic;
      sys_clk                : in  std_logic;
      sys_clk_gt             : in  std_logic;
      ocpclkout              : out std_logic;
      rclkin                 : in  std_logic;
      pci_exp_txn            : out std_logic_vector(7 downto 0);
      pci_exp_txp            : out std_logic_vector(7 downto 0);
      pci_exp_rxn            : in  std_logic_vector(7 downto 0);
      pci_exp_rxp            : in  std_logic_vector(7 downto 0);
      avr_clk              : in std_logic;
      avr_b2t              : in std_logic;
      avr_t2b              : out std_logic;
      -- Advanced Board management interface (to bridge)
      abmi_m2s             : out   adb3_ocp_m2sT;
      abmi_s2m             : in    adb3_ocp_s2mT;    
      direct_slave_m2s       : out adb3w_ocp_m2sT;
      direct_slave_s2m       : in  adb3w_ocp_s2mT;
      dma_channels_m2s       : out adb3w_ocp_m2sT_array(0 to no_dma_engines-1);
      dma_channels_s2m       : in  adb3w_ocp_s2mT_array(0 to no_dma_engines-1);
      direct_master_pcie_m2s : in  adb3w_ocp_m2sT;
      direct_master_pcie_s2m : out adb3w_ocp_s2mT;
      direct_master_regs_m2s : in  adb3l_ocp_m2sT;
      direct_master_regs_s2m : out adb3l_ocp_s2mT;
      interrupt              : in  std_logic;
      dma_abort              : in  std_logic_vector(no_dma_engines-1 downto 0);
      vpd_out                : out std_logic_vector(47 downto 0);
      core_status            : out std_logic_vector(63 downto 0);
      core_rst_out           : out std_logic);
  end component adb3_admxrcku1_tri;


  function behav_sim_decision_func(
    constant behav_sim : boolean := false)
    return boolean is
  begin
    return behav_sim;
  end;

    constant behav_sim : boolean := behav_sim_decision_func
    -- synthesis translate_off
    (behav_sim => behav_sim_axi4_over_pcie)
    -- synthesis translate_on
    ;

  
  function resolve_dma_engine_config (
    constant number_of_dma_engines : in natural;
    constant dma_engine0_config    : in natural;
    constant dma_engine1_config    : in natural;
    constant dma_engine2_config    : in natural;
    constant dma_engine3_config    : in natural)
    return integer is
    variable tmp : std_logic_vector(15 downto 0);
  begin
    if number_of_dma_engines = 0 then
      tmp := X"0003";                   -- Minimise logic for DMA engine 0
    else
      tmp(15 downto 8) := X"00";
      if dma_engine0_config = 0 then
        tmp(1 downto 0) := "00";
      elsif dma_engine0_config = 1 or dma_engine0_config = 3 then
        tmp(1 downto 0) := "01";
      elsif dma_engine0_config = 2 or dma_engine0_config = 4 then
        tmp(1 downto 0) := "10";
      else
        tmp(1 downto 0) := "11";
      end if;
      if dma_engine1_config = 0 then
        tmp(3 downto 2) := "00";
      elsif dma_engine1_config = 1 or dma_engine1_config = 3 then
        tmp(3 downto 2) := "01";
      elsif dma_engine1_config = 2 or dma_engine1_config = 4 then
        tmp(3 downto 2) := "10";
      else
        tmp(3 downto 2) := "11";
      end if;
      if dma_engine2_config = 0 then
        tmp(5 downto 4) := "00";
      elsif dma_engine2_config = 1 or dma_engine2_config = 3 then
        tmp(5 downto 4) := "01";
      elsif dma_engine2_config = 2 or dma_engine2_config = 4 then
        tmp(5 downto 4) := "10";
      else
        tmp(5 downto 4) := "11";
      end if;
      if dma_engine3_config = 0 then
        tmp(7 downto 6) := "00";
      elsif dma_engine3_config = 1 or dma_engine3_config = 3 then
        tmp(7 downto 6) := "01";
      elsif dma_engine3_config = 2 or dma_engine3_config = 4 then
        tmp(7 downto 6) := "10";
      else
        tmp(7 downto 6) := "11";
      end if;
    end if;
    return CONV_INTEGER(tmp);
  end;

  function max (
    x : natural;
    y : natural)
    return natural is
  begin
    if x > y then
      return x;
    else
      return y;
    end if;
  end;

  constant no_dma_engines : natural := max(1, number_of_dma_engines);
  constant dma_engine_config : std_logic_vector(15 downto 0) := CONV_STD_LOGIC_VECTOR(resolve_dma_engine_config(number_of_dma_engines,
                                                                                                               dma_engine0_config,
                                                                                                               dma_engine1_config,
                                                                                                               dma_engine2_config,
                                                                                                               dma_engine3_config),16);
  type dma_enginex_config_type is array (0 to 3) of natural;
  constant dma_enginex_config : dma_enginex_config_type := (dma_engine0_config,
                                                            dma_engine1_config,
                                                            dma_engine2_config,
                                                            dma_engine3_config);

  signal direct_slave_m2s : adb3w_ocp_m2sT                              := ADB3W_OCP_DEFAULT_M2S;
  signal direct_slave_s2m : adb3w_ocp_s2mT                              := ADB3W_OCP_DEFAULT_S2M;
  signal dma_channels_m2s : adb3w_ocp_m2sT_array(0 to no_dma_engines-1) := (others => ADB3W_OCP_DEFAULT_M2S);
  signal dma_channels_s2m : adb3w_ocp_s2mT_array(0 to no_dma_engines-1) := (others => ADB3W_OCP_DEFAULT_S2M);

  signal direct_master_regs_m2s : adb3l_ocp_m2sT := ADB3L_OCP_DEFAULT_M2S;
  signal direct_master_regs_s2m : adb3l_ocp_s2mT := ADB3L_OCP_DEFAULT_S2M;
  signal direct_master_pcie_m2s : adb3w_ocp_m2sT := ADB3W_OCP_DEFAULT_M2S;
  signal direct_master_pcie_s2m : adb3w_ocp_s2mT := ADB3W_OCP_DEFAULT_S2M;
  signal    direct_master_pcie_m2s_1 : adb3w_ocp_m2sT := ADB3W_OCP_DEFAULT_M2S;
  signal    direct_master_pcie_s2m_1 : adb3w_ocp_s2mT := ADB3W_OCP_DEFAULT_S2M;
  
  signal ds_axi4_m2s : axi4_256_m2sT := AXI4_256_M2ST_ZEROS;
  signal ds_axi4_s2m : axi4_256_s2mT := AXI4_256_S2MT_ZEROS;

  signal dma_axi4_m2s : axi4_256_m2sT_array(0 to 3) := (others => AXI4_256_M2ST_ZEROS);
  signal dma_axi4_s2m : axi4_256_s2mT_array(0 to 3) := (others => AXI4_256_S2MT_ZEROS);

  signal dm_axi4_m2s : axi4_256_m2sT := AXI4_256_M2ST_ZEROS;
  signal dm_axi4_s2m : axi4_256_s2mT := AXI4_256_S2MT_ZEROS;

  signal reset_counter : std_logic_vector(5 downto 0) := (others => '1');
  signal srst          : std_logic;
  signal srst_n        : std_logic;
  signal ocpclk        : std_logic;

  signal sys_clk, rclkin, sys_clk_gt : std_logic;



  type   stream_data_array is array (0 to 3) of std_logic_vector(255 downto 0);
  signal ss_data  : stream_data_array;
  signal ss_adv   : std_logic_vector(3 downto 0);
  signal ss_nfull : std_logic_vector(3 downto 0);

  signal sm_data  : stream_data_array;
  signal sm_adv   : std_logic_vector(3 downto 0);
  signal sm_valid : std_logic_vector(3 downto 0);

  signal s_data  : stream_data_array;
  signal s_adv   : std_logic_vector(3 downto 0);
  signal s_valid : std_logic_vector(3 downto 0);
  signal s_empty : std_logic_vector(3 downto 0);

  signal dma_abort : std_logic_vector(3 downto 0);
  
     -- Advanced Board management interface (to bridge)
  signal  abmi_m2s             : adb3_ocp_m2sT;
  signal  abmi_s2m             : adb3_ocp_s2mT;
  signal  abmi_l_m2s : adb3l_ocp_m2sT := ADB3L_OCP_DEFAULT_M2S;
  signal  abmi_l_s2m : adb3l_ocp_s2mT := ADB3L_OCP_DEFAULT_S2M;

  signal open_signal1,open_signal2 : std_logic_vector(31 downto 12);

  signal refclk300_i : std_logic;
  
    signal core_status_i : std_logic_vector(63 downto 0);


begin  -- synth


   gen_synth: if behav_sim = false generate

  
  aclk      <= ocpclk;
  aresetn   <= srst_n;
  dma_abort <= dma3_abort & dma2_abort & dma1_abort & dma0_abort;


  srst_n <= not srst;


  refclk_ibuf : IBUFDS_GTE3
    port map(
      O     => sys_clk_gt,
      ODIV2 => sys_clk,
      I     => pcie100_p,
      IB    => pcie100_n,
      CEB   => '0');

  use_diff_ref_clock0: if use_diff_ref_clock generate
    rclk_ibuf : IBUFDS
      port map(
        O     => refclk300_i,
        I     => refclk300_p,
        IB    => refclk300_n);

    bg0: BUFG
      port map (
        I => refclk300_i,
        O => rclkin);
    refclk <= rclkin;
  end generate use_diff_ref_clock0;

  not_udrc: if not use_diff_ref_clock generate
    rclkin <= refclk300_in;
  end generate not_udrc;



  ds_ocp2axi : adb3w_ocp_to_axi4
    port map (
      clk             => ocpclk,
      rst             => srst,
      ocp_m2s         => direct_slave_m2s,
      ocp_s2m         => direct_slave_s2m,
      axi_m2s         => ds_axi4_m2s,
      axi_s2m         => ds_axi4_s2m,
      illegal_ocp_cmd => open);

  gen_dma_ocp2axi : for i in 0 to number_of_dma_engines-1 generate
    gen_axi_mmap_dma : if dma_enginex_config(i) < 3 generate
      dma_ocp2axi : adb3w_ocp_to_axi4
        port map (
          clk             => ocpclk,
          rst             => srst,
          ocp_m2s         => dma_channels_m2s(i),
          ocp_s2m         => dma_channels_s2m(i),
          axi_m2s         => dma_axi4_m2s(i),
          axi_s2m         => dma_axi4_s2m(i),
          illegal_ocp_cmd => open);
    end generate gen_axi_mmap_dma;
    gen_axi_f2h_stream_dma : if dma_enginex_config(i) = 3 generate
      adb3_ocp_w_readen_read_fifo_interface256_1 : entity work.adb3_ocp_w_readen_read_fifo_interface256
        port map (
          ocp_clk => ocpclk,
          ocp_m2s => dma_channels_m2s(i),
          ocp_s2m => dma_channels_s2m(i),
          q       => s_data(i),
          qv      => s_valid(i),
          radv    => s_adv(i));    
      dpram_sfifo_1 : entity work.dpram_sfifo
        generic map (
          width => 256,
          order => 9)
        port map (
          clk    => ocpclk,
          rst    => srst,
          wadv   => ss_adv(i),
          wdata  => ss_data(i),
          nfull  => ss_nfull(i),
          radv   => s_adv(i),
          rempty => s_empty(i),
          rdata  => s_data(i));
      s_valid(i) <= not s_empty(i);
    end generate gen_axi_f2h_stream_dma;


    gen_axi_h2f_stream_dma : if dma_enginex_config(i) = 4 generate
      adb3_ocp_w_byteen_write_fifo_interface256_1 : entity work.adb3_ocp_w_byteen_write_fifo_interface256
        port map (
          ocp_clk => ocpclk,
          ocp_m2s => dma_channels_m2s(i),
          ocp_s2m => dma_channels_s2m(i),
          d       => sm_data(i),
          dv      => sm_valid(i),
          pop     => sm_adv(i));
    end generate gen_axi_h2f_stream_dma;
  end generate gen_dma_ocp2axi;

  gen_dm: if number_of_pcie_dm_ports>0 generate

    gen_axipcie_raw: if not use_axi4_pcie_legalizer generate
      dm_axi2ocp: entity work.axi4_to_adb3w_ocp_dm
        port map (
          clk             => ocpclk,
          rst             => srst,
          ocp_m2s         => direct_master_pcie_m2s,
          ocp_s2m         => direct_master_pcie_s2m,
          axi_m2s         => dm_axi4_m2s,
          axi_s2m         => dm_axi4_s2m);
    end generate gen_axipcie_raw;

    gen_axipcie_adapt: if use_axi4_pcie_legalizer generate
       dm_axi2ocp: entity work.axi4_to_adb3w_ocp_dm
        port map (
          clk             => ocpclk,
          rst             => srst,
          ocp_m2s         => direct_master_pcie_m2s_1,
          ocp_s2m         => direct_master_pcie_s2m_1,
          axi_m2s         => dm_axi4_m2s,
          axi_s2m         => dm_axi4_s2m);

       adb3_ocp_w_dm_adapter_1: entity work.adb3_ocp_w_dm_adapter
         generic map (
           MPTL_COMPATIBLE => false,
           PCIE_HINTS => true)
         port map (
           ocp_rst         => srst,
           ocp_clk         => ocpclk,
           slave_m2s       => direct_master_pcie_m2s_1,
           slave_s2m       => direct_master_pcie_s2m_1,
           master_m2s      => direct_master_pcie_m2s,
           master_s2m      => direct_master_pcie_s2m,
           ctl_max_rd_req  => core_status_i(15 downto 13),
           ctl_max_payload => core_status_i(12 downto 10),
           debug_info      => open);
       
      
    end generate gen_axipcie_adapt;
    
  end generate gen_dm;

  gen_nodm : if number_of_bar01_dm_ports = 0 generate
    direct_master_pcie_m2s <= ADB3W_OCP_DEFAULT_M2S;
  end generate gen_nodm;

  gen_dmb : if number_of_bar01_dm_ports > 0 generate
    axi4_lite_to_adb3_ocpl_1 : entity work.axi4_lite_to_adb3_ocpl
      port map (
        S_AXI_ACLK                 => ocpclk,
        S_AXI_ARESETN              => srst_n,
        S_AXI_AWADDR(13 downto 0)  => DMB_AXI_AWADDR,
        S_AXI_AWADDR(31 downto 14) => "000000000000000000",
        S_AXI_AWVALID              => DMB_AXI_AWVALID,
        S_AXI_WDATA                => DMB_AXI_WDATA,
        S_AXI_WSTRB                => DMB_AXI_WSTRB,
        S_AXI_WVALID               => DMB_AXI_WVALID,
        S_AXI_BREADY               => DMB_AXI_BREADY,
        S_AXI_ARADDR(13 downto 0)  => DMB_AXI_ARADDR,
        S_AXI_ARADDR(31 downto 14) => "000000000000000000",
        S_AXI_ARVALID              => DMB_AXI_ARVALID,
        S_AXI_RREADY               => DMB_AXI_RREADY,
        S_AXI_ARREADY              => DMB_AXI_ARREADY,
        S_AXI_RDATA                => DMB_AXI_RDATA,
        S_AXI_RRESP                => DMB_AXI_RRESP,
        S_AXI_RVALID               => DMB_AXI_RVALID,
        S_AXI_WREADY               => DMB_AXI_WREADY,
        S_AXI_BRESP                => DMB_AXI_BRESP,
        S_AXI_BVALID               => DMB_AXI_BVALID,
        S_AXI_AWREADY              => DMB_AXI_AWREADY,
        ocpl_cmd                   => direct_master_regs_m2s.cmd,
        ocpl_addr                  => direct_master_regs_m2s.addr,
        ocpl_m2sdata               => direct_master_regs_m2s.data,
        ocpl_databyteen            => direct_master_regs_m2s.databyteen,
        ocpl_datavalid          => direct_master_regs_m2s.datavalid,
        ocpl_respaccept            => direct_master_regs_m2s.respaccept,
        ocpl_cmdaccept             => direct_master_regs_s2m.cmdaccept,
        ocpl_dataaccept             => direct_master_regs_s2m.dataaccept,
        ocpl_resp                  => direct_master_regs_s2m.resp,
        ocpl_s2mdata               => direct_master_regs_s2m.data);
  end generate gen_dmb;

   gen_nodmb : if number_of_bar01_dm_ports = 0 generate
    direct_master_regs_m2s <= ADB3L_OCP_DEFAULT_M2S;
   end generate gen_nodmb;
   
    adm_xrc_ku1_i : adb3_admxrcku1_tri
      generic map (
        no_dma_engines      => no_dma_engines,
        pcie_direct_masters => number_of_pcie_dm_ports,
        reg_direct_masters  => number_of_bar01_dm_ports,
        dma_engine_config   => dma_engine_config,
        bar2_size           => bar2_size)
      port map (
        sys_reset_n             => perst_n,
        sys_clk                 => sys_clk,
        sys_clk_gt              => sys_clk_gt,
        ocpclkout               => ocpclk,
        rclkin                  => rclkin,
        pci_exp_txn             => pci_exp_txn,
        pci_exp_txp             => pci_exp_txp,
        pci_exp_rxn             => pci_exp_rxn,
        pci_exp_rxp             => pci_exp_rxp,
        avr_clk          => avr_clk,
        avr_t2b          => avr_t2b,
        avr_b2t          => avr_b2t,
        abmi_m2s         => abmi_m2s,
        abmi_s2m         => abmi_s2m,
        direct_slave_m2s       => direct_slave_m2s,
        direct_slave_s2m       => direct_slave_s2m,
        dma_channels_m2s       => dma_channels_m2s,
        dma_channels_s2m       => dma_channels_s2m,
        direct_master_pcie_m2s => direct_master_pcie_m2s,
        direct_master_pcie_s2m => direct_master_pcie_s2m,
        direct_master_regs_m2s => direct_master_regs_m2s,
        direct_master_regs_s2m => direct_master_regs_s2m,
        interrupt              => interrupt,
        dma_abort              => dma_abort(no_dma_engines-1 downto 0),
        vpd_out                => vpd_out,
        core_status            => core_status_i,
        core_rst_out           => srst);

  core_status <= core_status_i;

  ds_axi_awaddr       <= ds_axi4_m2s.awaddr(63 downto 0);
  ds_axi_awlen        <= ds_axi4_m2s.awlen;
  ds_axi_awsize       <= ds_axi4_m2s.awsize;
  ds_axi_awburst      <= ds_axi4_m2s.awburst;
  ds_axi_awcache      <= ds_axi4_m2s.awcache;
  ds_axi_awprot       <= ds_axi4_m2s.awprot;
  ds_axi_awvalid      <= ds_axi4_m2s.awvalid;
  ds_axi_wdata        <= ds_axi4_m2s.wdata;
  ds_axi_wstrb        <= ds_axi4_m2s.wstrb;
  ds_axi_wlast        <= ds_axi4_m2s.wlast;
  ds_axi_wvalid       <= ds_axi4_m2s.wvalid;
  ds_axi_bready       <= ds_axi4_m2s.bready;
  ds_axi_araddr       <= ds_axi4_m2s.araddr(63 downto 0);
  ds_axi_arlen        <= ds_axi4_m2s.arlen;
  ds_axi_arsize       <= ds_axi4_m2s.arsize;
  ds_axi_arburst      <= ds_axi4_m2s.arburst;
  ds_axi_arcache      <= ds_axi4_m2s.arcache;
  ds_axi_arprot       <= ds_axi4_m2s.arprot;
  ds_axi_arvalid      <= ds_axi4_m2s.arvalid;
  ds_axi_rready       <= ds_axi4_m2s.rready;
  ds_axi4_s2m.awready <= ds_axi_awready;
  ds_axi4_s2m.wready  <= ds_axi_wready;
  ds_axi4_s2m.bresp   <= ds_axi_bresp;
  ds_axi4_s2m.bvalid  <= ds_axi_bvalid;
  ds_axi4_s2m.arready <= ds_axi_arready;
  ds_axi4_s2m.rdata   <= ds_axi_rdata;
  ds_axi4_s2m.rresp   <= ds_axi_rresp;
  ds_axi4_s2m.rlast   <= ds_axi_rlast;
  ds_axi4_s2m.rvalid  <= ds_axi_rvalid;
  ds_axi_awlock       <= (others => '0');
  ds_axi_awqos        <= (others => '0');
  ds_axi_arlock       <= (others => '0');
  ds_axi_arqos        <= (others => '0');
  ds_axi_arregion     <= (others => '0');
  ds_axi_awregion     <= (others => '0');

  dma0_axi_awaddr         <= dma_axi4_m2s(0).awaddr(38 downto 0);
  dma0_axi_awlen          <= dma_axi4_m2s(0).awlen;
  dma0_axi_awsize         <= dma_axi4_m2s(0).awsize;
  dma0_axi_awburst        <= dma_axi4_m2s(0).awburst;
  dma0_axi_awcache        <= dma_axi4_m2s(0).awcache;
  dma0_axi_awprot         <= dma_axi4_m2s(0).awprot;
  dma0_axi_awvalid        <= dma_axi4_m2s(0).awvalid;
  dma0_axi_wdata          <= dma_axi4_m2s(0).wdata;
  dma0_axi_wstrb          <= dma_axi4_m2s(0).wstrb;
  dma0_axi_wlast          <= dma_axi4_m2s(0).wlast;
  dma0_axi_wvalid         <= dma_axi4_m2s(0).wvalid;
  dma0_axi_bready         <= dma_axi4_m2s(0).bready;
  dma0_axi_araddr         <= dma_axi4_m2s(0).araddr(38 downto 0);
  dma0_axi_arlen          <= dma_axi4_m2s(0).arlen;
  dma0_axi_arsize         <= dma_axi4_m2s(0).arsize;
  dma0_axi_arburst        <= dma_axi4_m2s(0).arburst;
  dma0_axi_arcache        <= dma_axi4_m2s(0).arcache;
  dma0_axi_arprot         <= dma_axi4_m2s(0).arprot;
  dma0_axi_arvalid        <= dma_axi4_m2s(0).arvalid;
  dma0_axi_rready         <= dma_axi4_m2s(0).rready;
  dma_axi4_s2m(0).awready <= dma0_axi_awready;
  dma_axi4_s2m(0).wready  <= dma0_axi_wready;
  dma_axi4_s2m(0).bresp   <= dma0_axi_bresp;
  dma_axi4_s2m(0).bvalid  <= dma0_axi_bvalid;
  dma_axi4_s2m(0).arready <= dma0_axi_arready;
  dma_axi4_s2m(0).rdata   <= dma0_axi_rdata;
  dma_axi4_s2m(0).rresp   <= dma0_axi_rresp;
  dma_axi4_s2m(0).rlast   <= dma0_axi_rlast;
  dma_axi4_s2m(0).rvalid  <= dma0_axi_rvalid;
  dma0_axi_awlock         <= (others => '0');
  dma0_axi_awqos          <= (others => '0');
  dma0_axi_arlock         <= (others => '0');
  dma0_axi_arqos          <= (others => '0');
  dma0_axi_arregion       <= (others => '0');
  dma0_axi_awregion       <= (others => '0');

  dma1_axi_awaddr         <= dma_axi4_m2s(1).awaddr(38 downto 0);
  dma1_axi_awlen          <= dma_axi4_m2s(1).awlen;
  dma1_axi_awsize         <= dma_axi4_m2s(1).awsize;
  dma1_axi_awburst        <= dma_axi4_m2s(1).awburst;
  dma1_axi_awcache        <= dma_axi4_m2s(1).awcache;
  dma1_axi_awprot         <= dma_axi4_m2s(1).awprot;
  dma1_axi_awvalid        <= dma_axi4_m2s(1).awvalid;
  dma1_axi_wdata          <= dma_axi4_m2s(1).wdata;
  dma1_axi_wstrb          <= dma_axi4_m2s(1).wstrb;
  dma1_axi_wlast          <= dma_axi4_m2s(1).wlast;
  dma1_axi_wvalid         <= dma_axi4_m2s(1).wvalid;
  dma1_axi_bready         <= dma_axi4_m2s(1).bready;
  dma1_axi_araddr         <= dma_axi4_m2s(1).araddr(38 downto 0);
  dma1_axi_arlen          <= dma_axi4_m2s(1).arlen;
  dma1_axi_arsize         <= dma_axi4_m2s(1).arsize;
  dma1_axi_arburst        <= dma_axi4_m2s(1).arburst;
  dma1_axi_arcache        <= dma_axi4_m2s(1).arcache;
  dma1_axi_arprot         <= dma_axi4_m2s(1).arprot;
  dma1_axi_arvalid        <= dma_axi4_m2s(1).arvalid;
  dma1_axi_rready         <= dma_axi4_m2s(1).rready;
  dma_axi4_s2m(1).awready <= dma1_axi_awready;
  dma_axi4_s2m(1).wready  <= dma1_axi_wready;
  dma_axi4_s2m(1).bresp   <= dma1_axi_bresp;
  dma_axi4_s2m(1).bvalid  <= dma1_axi_bvalid;
  dma_axi4_s2m(1).arready <= dma1_axi_arready;
  dma_axi4_s2m(1).rdata   <= dma1_axi_rdata;
  dma_axi4_s2m(1).rresp   <= dma1_axi_rresp;
  dma_axi4_s2m(1).rlast   <= dma1_axi_rlast;
  dma_axi4_s2m(1).rvalid  <= dma1_axi_rvalid;
  dma1_axi_awlock         <= (others => '0');
  dma1_axi_awqos          <= (others => '0');
  dma1_axi_arlock         <= (others => '0');
  dma1_axi_arqos          <= (others => '0');
  dma1_axi_arregion       <= (others => '0');
  dma1_axi_awregion       <= (others => '0');

  dma2_axi_awaddr         <= dma_axi4_m2s(2).awaddr(38 downto 0);
  dma2_axi_awlen          <= dma_axi4_m2s(2).awlen;
  dma2_axi_awsize         <= dma_axi4_m2s(2).awsize;
  dma2_axi_awburst        <= dma_axi4_m2s(2).awburst;
  dma2_axi_awcache        <= dma_axi4_m2s(2).awcache;
  dma2_axi_awprot         <= dma_axi4_m2s(2).awprot;
  dma2_axi_awvalid        <= dma_axi4_m2s(2).awvalid;
  dma2_axi_wdata          <= dma_axi4_m2s(2).wdata;
  dma2_axi_wstrb          <= dma_axi4_m2s(2).wstrb;
  dma2_axi_wlast          <= dma_axi4_m2s(2).wlast;
  dma2_axi_wvalid         <= dma_axi4_m2s(2).wvalid;
  dma2_axi_bready         <= dma_axi4_m2s(2).bready;
  dma2_axi_araddr         <= dma_axi4_m2s(2).araddr(38 downto 0);
  dma2_axi_arlen          <= dma_axi4_m2s(2).arlen;
  dma2_axi_arsize         <= dma_axi4_m2s(2).arsize;
  dma2_axi_arburst        <= dma_axi4_m2s(2).arburst;
  dma2_axi_arcache        <= dma_axi4_m2s(2).arcache;
  dma2_axi_arprot         <= dma_axi4_m2s(2).arprot;
  dma2_axi_arvalid        <= dma_axi4_m2s(2).arvalid;
  dma2_axi_rready         <= dma_axi4_m2s(2).rready;
  dma_axi4_s2m(2).awready <= dma2_axi_awready;
  dma_axi4_s2m(2).wready  <= dma2_axi_wready;
  dma_axi4_s2m(2).bresp   <= dma2_axi_bresp;
  dma_axi4_s2m(2).bvalid  <= dma2_axi_bvalid;
  dma_axi4_s2m(2).arready <= dma2_axi_arready;
  dma_axi4_s2m(2).rdata   <= dma2_axi_rdata;
  dma_axi4_s2m(2).rresp   <= dma2_axi_rresp;
  dma_axi4_s2m(2).rlast   <= dma2_axi_rlast;
  dma_axi4_s2m(2).rvalid  <= dma2_axi_rvalid;
  dma2_axi_awlock         <= (others => '0');
  dma2_axi_awqos          <= (others => '0');
  dma2_axi_arlock         <= (others => '0');
  dma2_axi_arqos          <= (others => '0');
  dma2_axi_arregion       <= (others => '0');
  dma2_axi_awregion       <= (others => '0');

  dma3_axi_awaddr         <= dma_axi4_m2s(3).awaddr(38 downto 0);
  dma3_axi_awlen          <= dma_axi4_m2s(3).awlen;
  dma3_axi_awsize         <= dma_axi4_m2s(3).awsize;
  dma3_axi_awburst        <= dma_axi4_m2s(3).awburst;
  dma3_axi_awcache        <= dma_axi4_m2s(3).awcache;
  dma3_axi_awprot         <= dma_axi4_m2s(3).awprot;
  dma3_axi_awvalid        <= dma_axi4_m2s(3).awvalid;
  dma3_axi_wdata          <= dma_axi4_m2s(3).wdata;
  dma3_axi_wstrb          <= dma_axi4_m2s(3).wstrb;
  dma3_axi_wlast          <= dma_axi4_m2s(3).wlast;
  dma3_axi_wvalid         <= dma_axi4_m2s(3).wvalid;
  dma3_axi_bready         <= dma_axi4_m2s(3).bready;
  dma3_axi_araddr         <= dma_axi4_m2s(3).araddr(38 downto 0);
  dma3_axi_arlen          <= dma_axi4_m2s(3).arlen;
  dma3_axi_arsize         <= dma_axi4_m2s(3).arsize;
  dma3_axi_arburst        <= dma_axi4_m2s(3).arburst;
  dma3_axi_arcache        <= dma_axi4_m2s(3).arcache;
  dma3_axi_arprot         <= dma_axi4_m2s(3).arprot;
  dma3_axi_arvalid        <= dma_axi4_m2s(3).arvalid;
  dma3_axi_rready         <= dma_axi4_m2s(3).rready;
  dma_axi4_s2m(3).awready <= dma3_axi_awready;
  dma_axi4_s2m(3).wready  <= dma3_axi_wready;
  dma_axi4_s2m(3).bresp   <= dma3_axi_bresp;
  dma_axi4_s2m(3).bvalid  <= dma3_axi_bvalid;
  dma_axi4_s2m(3).arready <= dma3_axi_arready;
  dma_axi4_s2m(3).rdata   <= dma3_axi_rdata;
  dma_axi4_s2m(3).rresp   <= dma3_axi_rresp;
  dma_axi4_s2m(3).rlast   <= dma3_axi_rlast;
  dma_axi4_s2m(3).rvalid  <= dma3_axi_rvalid;
  dma3_axi_awlock         <= (others => '0');
  dma3_axi_awqos          <= (others => '0');
  dma3_axi_arlock         <= (others => '0');
  dma3_axi_arqos          <= (others => '0');
  dma3_axi_arregion       <= (others => '0');
  dma3_axi_awregion       <= (others => '0');


       dm_axi4_m2s.awaddr <= dm_axi_awaddr;
  dm_axi4_m2s.awlen  <= dm_axi_awlen;
  dm_axi4_m2s.awsize <= dm_axi_awsize;
  dm_axi4_m2s.awburst <= dm_axi_awburst;
  dm_axi4_m2s.awcache <= dm_axi_awcache;
  dm_axi4_m2s.awprot <= dm_axi_awprot;
  dm_axi4_m2s.awvalid <= dm_axi_awvalid;
  dm_axi4_m2s.wdata <= dm_axi_wdata;
  dm_axi4_m2s.wstrb <= dm_axi_wstrb;
  dm_axi4_m2s.wlast <= dm_axi_wlast;
  dm_axi4_m2s.wvalid <= dm_axi_wvalid;
  dm_axi4_m2s.bready <= dm_axi_bready;
  dm_axi4_m2s.araddr <= dm_axi_araddr;
  dm_axi4_m2s.arlen  <= dm_axi_arlen;
  dm_axi4_m2s.arsize<= dm_axi_arsize;
  dm_axi4_m2s.arburst<= dm_axi_arburst;
  dm_axi4_m2s.arcache<= dm_axi_arcache;
  dm_axi4_m2s.arprot <= dm_axi_arprot;
  dm_axi4_m2s.arvalid<= dm_axi_arvalid;
  dm_axi4_m2s.rready<= dm_axi_rready;

  dm_axi4_m2s.arid(pcie_dm_port_id_width-1 downto 0)<= dm_axi_arid;
  dm_axi4_m2s.awid(pcie_dm_port_id_width-1 downto 0)<= dm_axi_awid;
  --dm_axi4_m2s.wid(pcie_dm_port_id_width-1 downto 0)<= dm_axi_wid;

  dm_axi_awready <= dm_axi4_s2m.awready;
  dm_axi_wready <= dm_axi4_s2m.wready;
  dm_axi_bresp <= dm_axi4_s2m.bresp;
  dm_axi_bvalid <= dm_axi4_s2m.bvalid;
  dm_axi_arready <= dm_axi4_s2m.arready;
  dm_axi_rdata <= dm_axi4_s2m.rdata;
  dm_axi_rresp <= dm_axi4_s2m.rresp;
  dm_axi_rlast <= dm_axi4_s2m.rlast;
  dm_axi_rvalid <= dm_axi4_s2m.rvalid;

  dm_axi_rid <= dm_axi4_s2m.rid(pcie_dm_port_id_width-1 downto 0);
  dm_axi_bid <= dm_axi4_s2m.bid(pcie_dm_port_id_width-1 downto 0);
  
 
  
  dma0_m_axis_tdata  <= sm_data(0);
  sm_adv(0)          <= dma0_m_axis_tready and sm_valid(0);
  dma0_m_axis_tvalid <= sm_valid(0);

  ss_data(0)         <= dma0_s_axis_tdata;
  ss_adv(0)          <= dma0_s_axis_tvalid;
  dma0_s_axis_tready <= not ss_nfull(0);

  dma1_m_axis_tdata  <= sm_data(1);
  sm_adv(1)          <= dma1_m_axis_tready and sm_valid(1);
  dma1_m_axis_tvalid <= sm_valid(1);

  ss_data(1)         <= dma1_s_axis_tdata;
  ss_adv(1)          <= dma1_s_axis_tvalid;
  dma1_s_axis_tready <= not ss_nfull(1);

  dma2_m_axis_tdata  <= sm_data(2);
  sm_adv(2)          <= dma2_m_axis_tready and sm_valid(2);
  dma2_m_axis_tvalid <= sm_valid(2);

  ss_data(2)         <= dma2_s_axis_tdata;
  ss_adv(2)          <= dma2_s_axis_tvalid;
  dma2_s_axis_tready <= not ss_nfull(2);

  dma3_m_axis_tdata  <= sm_data(3);
  sm_adv(3)          <= dma3_m_axis_tready and sm_valid(3);
  dma3_m_axis_tvalid <= sm_valid(3);

  ss_data(3)         <= dma3_s_axis_tdata;
  ss_adv(3)          <= dma3_s_axis_tvalid;
  dma3_s_axis_tready <= not ss_nfull(3);


  gen_noabmi_ocp: if abmi_mode = 0 generate
     abmi_s2m <= ADB3_OCP_DEFAULT_S2M;
  end generate gen_noabmi_ocp;
  
  gen_abmi_ocp: if abmi_mode = 2 generate
     abmi_s2m <= to_adb3_ocp_s2mT(abmi_s2m_slv);
     abmi_m2s_slv <= to_std_logic_vector(abmi_m2s);
  end generate gen_abmi_ocp;

  gen_abmi_axi4lite: if abmi_mode = 1 generate
    -- Convert OCP abmi_m2s/s2m to AXI4 Lite
    adb3_to_adb3l0: entity work.adb3_to_adb3l 
      port map(
        clk  => ocpclk,
        pri_m2s => abmi_m2s,
        pri_s2m => abmi_s2m,
        sec_m2s => abmi_l_m2s,
        sec_s2m => abmi_l_s2m);

    adb3_ocpl_to_axi4_lite_1 : entity work.adb3_ocpl_to_axi4_lite
      port map (
        M_AXI_ACLK                 => ocpclk,
        M_AXI_ARESETN              => srst_n,
        M_AXI_AWADDR(11 downto 0)  => ABMI_AXI_AWADDR,
        M_AXI_AWADDR(31 downto 12) => open_signal1, 
        M_AXI_AWVALID              => ABMI_AXI_AWVALID,
        M_AXI_WDATA                => ABMI_AXI_WDATA,
        M_AXI_WSTRB                => ABMI_AXI_WSTRB,
        M_AXI_WVALID               => ABMI_AXI_WVALID,
        M_AXI_BREADY               => ABMI_AXI_BREADY,
        M_AXI_ARADDR(11 downto 0)  => ABMI_AXI_ARADDR,
        M_AXI_ARADDR(31 downto 12) => open_signal2, 
        M_AXI_ARVALID              => ABMI_AXI_ARVALID,
        M_AXI_RREADY               => ABMI_AXI_RREADY,
        M_AXI_ARREADY              => ABMI_AXI_ARREADY,
        M_AXI_RDATA                => ABMI_AXI_RDATA,
        M_AXI_RRESP                => ABMI_AXI_RRESP,
        M_AXI_RVALID               => ABMI_AXI_RVALID,
        M_AXI_WREADY               => ABMI_AXI_WREADY,
        M_AXI_BRESP                => ABMI_AXI_BRESP,
        M_AXI_BVALID               => ABMI_AXI_BVALID,
        M_AXI_AWREADY              => ABMI_AXI_AWREADY,
        ocpl_cmd                   => abmi_l_m2s.cmd,
        ocpl_addr                  => abmi_l_m2s.addr,
        ocpl_m2sdata               => abmi_l_m2s.data,
        ocpl_databyteen            => abmi_l_m2s.databyteen,
        ocpl_datavalid          => abmi_l_m2s.datavalid,
        ocpl_respaccept            => abmi_l_m2s.respaccept,
        ocpl_cmdaccept             => abmi_l_s2m.cmdaccept,
        ocpl_dataaccept             => abmi_l_s2m.dataaccept,
        ocpl_resp                  => abmi_l_s2m.resp,
        ocpl_s2mdata               => abmi_l_s2m.data);
  end generate gen_abmi_axi4lite;
  

end generate gen_synth;


  --synthesis translate_off
gen_fast_sim: if behav_sim = true generate
  
  adb3_admpcieku3_x8_axi4_behav_1: adb3_admxrcku1_bm_x8_axi4
    generic map (
      number_of_dma_engines    => number_of_dma_engines,
      number_of_pcie_dm_ports  => number_of_pcie_dm_ports,
      number_of_bar01_dm_ports => number_of_bar01_dm_ports,
      dma_engine0_config       => dma_engine0_config,
      dma_engine1_config       => dma_engine1_config,
      dma_engine2_config       => dma_engine2_config,
      dma_engine3_config       => dma_engine3_config,
      disable_model_io         => true,
      use_axi4_pcie_legalizer  => use_axi4_pcie_legalizer,
      bar2_size                => bar2_size,
      enable_dma_abort         => enable_dma_abort,
      enable_interrupt         => enable_interrupt,
      use_icap                 => false,
      use_diff_ref_clock       => use_diff_ref_clock,
      pcie_dm_port_id_width    => pcie_dm_port_id_width)
    port map (
      perst_n            => perst_n,
      pcie100_p          => pcie100_p,
      pcie100_n          => pcie100_n,
      refclk200_p        => refclk300_p,
      refclk200_n        => refclk300_n,
      refclk200_in       => refclk300_in,
      refclk             => refclk,
      aclk               => aclk,
      aresetn            => aresetn,
      pci_exp_txn        => pci_exp_txn,
      pci_exp_txp        => pci_exp_txp,
      pci_exp_rxn        => pci_exp_rxn,
      pci_exp_rxp        => pci_exp_rxp,
      vpd_out            => vpd_out,
      model_inout_i      => X"000000_00000000",
      model_inout_o      => open,
      model_inout_t      => open,
      ds_axi_awaddr      => ds_axi_awaddr,
      ds_axi_awlen       => ds_axi_awlen,
      ds_axi_awsize      => ds_axi_awsize,
      ds_axi_awburst     => ds_axi_awburst,
      ds_axi_awcache     => ds_axi_awcache,
      ds_axi_awprot      => ds_axi_awprot,
      ds_axi_awvalid     => ds_axi_awvalid,
      ds_axi_wdata       => ds_axi_wdata,
      ds_axi_wstrb       => ds_axi_wstrb,
      ds_axi_wlast       => ds_axi_wlast,
      ds_axi_wvalid      => ds_axi_wvalid,
      ds_axi_bready      => ds_axi_bready,
      ds_axi_araddr      => ds_axi_araddr,
      ds_axi_arlen       => ds_axi_arlen,
      ds_axi_arsize      => ds_axi_arsize,
      ds_axi_arburst     => ds_axi_arburst,
      ds_axi_arcache     => ds_axi_arcache,
      ds_axi_arprot      => ds_axi_arprot,
      ds_axi_arvalid     => ds_axi_arvalid,
      ds_axi_rready      => ds_axi_rready,
      ds_axi_awready     => ds_axi_awready,
      ds_axi_wready      => ds_axi_wready,
      ds_axi_bresp       => ds_axi_bresp,
      ds_axi_bvalid      => ds_axi_bvalid,
      ds_axi_arready     => ds_axi_arready,
      ds_axi_rdata       => ds_axi_rdata,
      ds_axi_rresp       => ds_axi_rresp,
      ds_axi_rlast       => ds_axi_rlast,
      ds_axi_rvalid      => ds_axi_rvalid,
      ds_axi_awlock      => ds_axi_awlock,
      ds_axi_awqos       => ds_axi_awqos,
      ds_axi_arlock      => ds_axi_arlock,
      ds_axi_arqos       => ds_axi_arqos,
      ds_axi_awregion    => ds_axi_awregion,
      ds_axi_arregion    => ds_axi_arregion,
      dma0_axi_awaddr    => dma0_axi_awaddr,
      dma0_axi_awlen     => dma0_axi_awlen,
      dma0_axi_awsize    => dma0_axi_awsize,
      dma0_axi_awburst   => dma0_axi_awburst,
      dma0_axi_awcache   => dma0_axi_awcache,
      dma0_axi_awprot    => dma0_axi_awprot,
      dma0_axi_awvalid   => dma0_axi_awvalid,
      dma0_axi_wdata     => dma0_axi_wdata,
      dma0_axi_wstrb     => dma0_axi_wstrb,
      dma0_axi_wlast     => dma0_axi_wlast,
      dma0_axi_wvalid    => dma0_axi_wvalid,
      dma0_axi_bready    => dma0_axi_bready,
      dma0_axi_araddr    => dma0_axi_araddr,
      dma0_axi_arlen     => dma0_axi_arlen,
      dma0_axi_arsize    => dma0_axi_arsize,
      dma0_axi_arburst   => dma0_axi_arburst,
      dma0_axi_arcache   => dma0_axi_arcache,
      dma0_axi_arprot    => dma0_axi_arprot,
      dma0_axi_arvalid   => dma0_axi_arvalid,
      dma0_axi_rready    => dma0_axi_rready,
      dma0_axi_awready   => dma0_axi_awready,
      dma0_axi_wready    => dma0_axi_wready,
      dma0_axi_bresp     => dma0_axi_bresp,
      dma0_axi_bvalid    => dma0_axi_bvalid,
      dma0_axi_arready   => dma0_axi_arready,
      dma0_axi_rdata     => dma0_axi_rdata,
      dma0_axi_rresp     => dma0_axi_rresp,
      dma0_axi_rlast     => dma0_axi_rlast,
      dma0_axi_rvalid    => dma0_axi_rvalid,
      dma0_axi_awlock    => dma0_axi_awlock,
      dma0_axi_awqos     => dma0_axi_awqos,
      dma0_axi_arlock    => dma0_axi_arlock,
      dma0_axi_arqos     => dma0_axi_arqos,
      dma0_axi_awregion  => dma0_axi_awregion,
      dma0_axi_arregion  => dma0_axi_arregion,
      dma1_axi_awaddr    => dma1_axi_awaddr,
      dma1_axi_awlen     => dma1_axi_awlen,
      dma1_axi_awsize    => dma1_axi_awsize,
      dma1_axi_awburst   => dma1_axi_awburst,
      dma1_axi_awcache   => dma1_axi_awcache,
      dma1_axi_awprot    => dma1_axi_awprot,
      dma1_axi_awvalid   => dma1_axi_awvalid,
      dma1_axi_wdata     => dma1_axi_wdata,
      dma1_axi_wstrb     => dma1_axi_wstrb,
      dma1_axi_wlast     => dma1_axi_wlast,
      dma1_axi_wvalid    => dma1_axi_wvalid,
      dma1_axi_bready    => dma1_axi_bready,
      dma1_axi_araddr    => dma1_axi_araddr,
      dma1_axi_arlen     => dma1_axi_arlen,
      dma1_axi_arsize    => dma1_axi_arsize,
      dma1_axi_arburst   => dma1_axi_arburst,
      dma1_axi_arcache   => dma1_axi_arcache,
      dma1_axi_arprot    => dma1_axi_arprot,
      dma1_axi_arvalid   => dma1_axi_arvalid,
      dma1_axi_rready    => dma1_axi_rready,
      dma1_axi_awready   => dma1_axi_awready,
      dma1_axi_wready    => dma1_axi_wready,
      dma1_axi_bresp     => dma1_axi_bresp,
      dma1_axi_bvalid    => dma1_axi_bvalid,
      dma1_axi_arready   => dma1_axi_arready,
      dma1_axi_rdata     => dma1_axi_rdata,
      dma1_axi_rresp     => dma1_axi_rresp,
      dma1_axi_rlast     => dma1_axi_rlast,
      dma1_axi_rvalid    => dma1_axi_rvalid,
      dma1_axi_awlock    => dma1_axi_awlock,
      dma1_axi_awqos     => dma1_axi_awqos,
      dma1_axi_arlock    => dma1_axi_arlock,
      dma1_axi_arqos     => dma1_axi_arqos,
      dma1_axi_awregion  => dma1_axi_awregion,
      dma1_axi_arregion  => dma1_axi_arregion,
      dma2_axi_awaddr    => dma2_axi_awaddr,
      dma2_axi_awlen     => dma2_axi_awlen,
      dma2_axi_awsize    => dma2_axi_awsize,
      dma2_axi_awburst   => dma2_axi_awburst,
      dma2_axi_awcache   => dma2_axi_awcache,
      dma2_axi_awprot    => dma2_axi_awprot,
      dma2_axi_awvalid   => dma2_axi_awvalid,
      dma2_axi_wdata     => dma2_axi_wdata,
      dma2_axi_wstrb     => dma2_axi_wstrb,
      dma2_axi_wlast     => dma2_axi_wlast,
      dma2_axi_wvalid    => dma2_axi_wvalid,
      dma2_axi_bready    => dma2_axi_bready,
      dma2_axi_araddr    => dma2_axi_araddr,
      dma2_axi_arlen     => dma2_axi_arlen,
      dma2_axi_arsize    => dma2_axi_arsize,
      dma2_axi_arburst   => dma2_axi_arburst,
      dma2_axi_arcache   => dma2_axi_arcache,
      dma2_axi_arprot    => dma2_axi_arprot,
      dma2_axi_arvalid   => dma2_axi_arvalid,
      dma2_axi_rready    => dma2_axi_rready,
      dma2_axi_awready   => dma2_axi_awready,
      dma2_axi_wready    => dma2_axi_wready,
      dma2_axi_bresp     => dma2_axi_bresp,
      dma2_axi_bvalid    => dma2_axi_bvalid,
      dma2_axi_arready   => dma2_axi_arready,
      dma2_axi_rdata     => dma2_axi_rdata,
      dma2_axi_rresp     => dma2_axi_rresp,
      dma2_axi_rlast     => dma2_axi_rlast,
      dma2_axi_rvalid    => dma2_axi_rvalid,
      dma2_axi_awlock    => dma2_axi_awlock,
      dma2_axi_awqos     => dma2_axi_awqos,
      dma2_axi_arlock    => dma2_axi_arlock,
      dma2_axi_arqos     => dma2_axi_arqos,
      dma2_axi_awregion  => dma2_axi_awregion,
      dma2_axi_arregion  => dma2_axi_arregion,
      dma3_axi_awaddr    => dma3_axi_awaddr,
      dma3_axi_awlen     => dma3_axi_awlen,
      dma3_axi_awsize    => dma3_axi_awsize,
      dma3_axi_awburst   => dma3_axi_awburst,
      dma3_axi_awcache   => dma3_axi_awcache,
      dma3_axi_awprot    => dma3_axi_awprot,
      dma3_axi_awvalid   => dma3_axi_awvalid,
      dma3_axi_wdata     => dma3_axi_wdata,
      dma3_axi_wstrb     => dma3_axi_wstrb,
      dma3_axi_wlast     => dma3_axi_wlast,
      dma3_axi_wvalid    => dma3_axi_wvalid,
      dma3_axi_bready    => dma3_axi_bready,
      dma3_axi_araddr    => dma3_axi_araddr,
      dma3_axi_arlen     => dma3_axi_arlen,
      dma3_axi_arsize    => dma3_axi_arsize,
      dma3_axi_arburst   => dma3_axi_arburst,
      dma3_axi_arcache   => dma3_axi_arcache,
      dma3_axi_arprot    => dma3_axi_arprot,
      dma3_axi_arvalid   => dma3_axi_arvalid,
      dma3_axi_rready    => dma3_axi_rready,
      dma3_axi_awready   => dma3_axi_awready,
      dma3_axi_wready    => dma3_axi_wready,
      dma3_axi_bresp     => dma3_axi_bresp,
      dma3_axi_bvalid    => dma3_axi_bvalid,
      dma3_axi_arready   => dma3_axi_arready,
      dma3_axi_rdata     => dma3_axi_rdata,
      dma3_axi_rresp     => dma3_axi_rresp,
      dma3_axi_rlast     => dma3_axi_rlast,
      dma3_axi_rvalid    => dma3_axi_rvalid,
      dma3_axi_awlock    => dma3_axi_awlock,
      dma3_axi_awqos     => dma3_axi_awqos,
      dma3_axi_arlock    => dma3_axi_arlock,
      dma3_axi_arqos     => dma3_axi_arqos,
      dma3_axi_awregion  => dma3_axi_awregion,
      dma3_axi_arregion  => dma3_axi_arregion,
      dm_axi_awaddr      => dm_axi_awaddr,
      dm_axi_awlen       => dm_axi_awlen,
      dm_axi_awsize      => dm_axi_awsize,
      dm_axi_awburst     => dm_axi_awburst,
      dm_axi_awcache     => dm_axi_awcache,
      dm_axi_awprot      => dm_axi_awprot,
      dm_axi_awvalid     => dm_axi_awvalid,
      dm_axi_awid        => dm_axi_awid,
      dm_axi_wdata       => dm_axi_wdata,
      dm_axi_wstrb       => dm_axi_wstrb,
      dm_axi_wlast       => dm_axi_wlast,
      dm_axi_wvalid      => dm_axi_wvalid,
      dm_axi_bready      => dm_axi_bready,
      dm_axi_araddr      => dm_axi_araddr,
      dm_axi_arlen       => dm_axi_arlen,
      dm_axi_arsize      => dm_axi_arsize,
      dm_axi_arburst     => dm_axi_arburst,
      dm_axi_arcache     => dm_axi_arcache,
      dm_axi_arprot      => dm_axi_arprot,
      dm_axi_arvalid     => dm_axi_arvalid,
      dm_axi_arid        => dm_axi_arid,
      dm_axi_rready      => dm_axi_rready,
      dm_axi_awready     => dm_axi_awready,
      dm_axi_wready      => dm_axi_wready,
      dm_axi_bresp       => dm_axi_bresp,
      dm_axi_bvalid      => dm_axi_bvalid,
      dm_axi_bid         => dm_axi_bid,
      dm_axi_arready     => dm_axi_arready,
      dm_axi_rdata       => dm_axi_rdata,
      dm_axi_rresp       => dm_axi_rresp,
      dm_axi_rlast       => dm_axi_rlast,
      dm_axi_rvalid      => dm_axi_rvalid,
      dm_axi_rid         => dm_axi_rid,
      dm_axi_awlock      => dm_axi_awlock,
      dm_axi_awqos       => dm_axi_awqos,
      dm_axi_arlock      => dm_axi_arlock,
      dm_axi_arqos       => dm_axi_arqos,
      dm_axi_awregion    => dm_axi_awregion,
      dm_axi_arregion    => dm_axi_arregion,
      dmb_axi_awaddr(12)     => dmb_axi_awaddr(13),
      dmb_axi_awaddr(11 downto 0) => dmb_axi_awaddr(11 downto 0),
      dmb_axi_awvalid    => dmb_axi_awvalid,
      dmb_axi_wdata      => dmb_axi_wdata,
      dmb_axi_wstrb      => dmb_axi_wstrb,
      dmb_axi_wvalid     => dmb_axi_wvalid,
      dmb_axi_bready     => dmb_axi_bready,
      dmb_axi_araddr(12)     => dmb_axi_araddr(13),
      dmb_axi_araddr(11 downto 0)     => dmb_axi_araddr(11 downto 0),
      dmb_axi_arvalid    => dmb_axi_arvalid,
      dmb_axi_rready     => dmb_axi_rready,
      dmb_axi_awready    => dmb_axi_awready,
      dmb_axi_wready     => dmb_axi_wready,
      dmb_axi_bresp      => dmb_axi_bresp,
      dmb_axi_bvalid     => dmb_axi_bvalid,
      dmb_axi_arready    => dmb_axi_arready,
      dmb_axi_rdata      => dmb_axi_rdata,
      dmb_axi_rresp      => dmb_axi_rresp,
      dmb_axi_rvalid     => dmb_axi_rvalid,
      dma0_m_axis_tdata  => dma0_m_axis_tdata,
      dma0_m_axis_tready => dma0_m_axis_tready,
      dma0_m_axis_tvalid => dma0_m_axis_tvalid,
      dma0_s_axis_tdata  => dma0_s_axis_tdata,
      dma0_s_axis_tready => dma0_s_axis_tready,
      dma0_s_axis_tvalid => dma0_s_axis_tvalid,
      dma1_m_axis_tdata  => dma1_m_axis_tdata,
      dma1_m_axis_tready => dma1_m_axis_tready,
      dma1_m_axis_tvalid => dma1_m_axis_tvalid,
      dma1_s_axis_tdata  => dma1_s_axis_tdata,
      dma1_s_axis_tready => dma1_s_axis_tready,
      dma1_s_axis_tvalid => dma1_s_axis_tvalid,
      dma2_m_axis_tdata  => dma2_m_axis_tdata,
      dma2_m_axis_tready => dma2_m_axis_tready,
      dma2_m_axis_tvalid => dma2_m_axis_tvalid,
      dma2_s_axis_tdata  => dma2_s_axis_tdata,
      dma2_s_axis_tready => dma2_s_axis_tready,
      dma2_s_axis_tvalid => dma2_s_axis_tvalid,
      dma3_m_axis_tdata  => dma3_m_axis_tdata,
      dma3_m_axis_tready => dma3_m_axis_tready,
      dma3_m_axis_tvalid => dma3_m_axis_tvalid,
      dma3_s_axis_tdata  => dma3_s_axis_tdata,
      dma3_s_axis_tready => dma3_s_axis_tready,
      dma3_s_axis_tvalid => dma3_s_axis_tvalid,
      interrupt          => interrupt,
      dma0_abort         => dma0_abort,
      dma1_abort         => dma1_abort,
      dma2_abort         => dma2_abort,
      dma3_abort         => dma3_abort,
      core_status        => core_status);
  
  
end generate gen_fast_sim;
--synthesis translate_on
  
end synth;
