--------------------------------------------------------------------------------
--
-- Module Name      : adb3_ocp_to_axi4_lite
--
--                    Copyright 2015 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : Package For Interfacing ADB3 OCP Lite Type Logic With
--                    AXI-4 Lite Compatible IP
--
--
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity adb3_ocpl_to_axi4_lite is
  port (
    -- Slave AXI Port
    M_AXI_ACLK                     : in  std_logic;
    M_AXI_ARESETN                  : in  std_logic;
    M_AXI_AWADDR                   : out  std_logic_vector(31 downto 0);
    M_AXI_AWVALID                  : out  std_logic;
    M_AXI_WDATA                    : out  std_logic_vector(31 downto 0);
    M_AXI_WSTRB                    : out  std_logic_vector(3 downto 0);
    M_AXI_WVALID                   : out  std_logic;
    M_AXI_BREADY                   : out  std_logic;
    M_AXI_ARADDR                   : out  std_logic_vector(31 downto 0);
    M_AXI_ARVALID                  : out  std_logic;
    M_AXI_RREADY                   : out  std_logic;
    M_AXI_ARREADY                  : in std_logic;
    M_AXI_RDATA                    : in std_logic_vector(31 downto 0);
    M_AXI_RRESP                    : in std_logic_vector(1 downto 0);
    M_AXI_RVALID                   : in std_logic;
    M_AXI_WREADY                   : in std_logic;
    M_AXI_BRESP                    : in std_logic_vector(1 downto 0);
    M_AXI_BVALID                   : in std_logic;
    M_AXI_AWREADY                  : in std_logic;
    -- OCP Lite Port (as std_logic_1164)
    ocpl_cmd                       : in std_logic_vector(2 downto 0);
    ocpl_addr                      : in std_logic_vector(31 downto 0);
    ocpl_m2sdata                   : in std_logic_vector(31 downto 0);
    ocpl_databyteen                : in std_logic_vector(3 downto 0);
    ocpl_datavalid                 : in std_logic;
    ocpl_respaccept                : in std_logic;
    ocpl_cmdaccept                 : out std_logic;
    ocpl_dataaccept                : out std_logic;
    ocpl_resp                      : out std_logic_vector(1 downto 0);
    ocpl_s2mdata                   : out std_logic_vector(31 downto 0));
end entity adb3_ocpl_to_axi4_lite;

architecture rtl of adb3_ocpl_to_axi4_lite is

  
  
begin  -- architecture rtl

 M_AXI_AWADDR <= ocpl_addr;
 M_AXI_AWVALID <= ocpl_cmd(0);
 M_AXI_WDATA <= ocpl_m2sdata;
 M_AXI_WSTRB <= ocpl_databyteen;
 M_AXI_WVALID <= ocpl_datavalid;
 M_AXI_BREADY <= '1';
 M_AXI_ARADDR <= ocpl_addr;
 M_AXI_ARVALID <= ocpl_cmd(1);
 M_AXI_RREADY <= ocpl_respaccept;

 ocpl_cmdaccept <= M_AXI_ARREADY and M_AXI_AWREADY;
 ocpl_dataaccept <= M_AXI_WREADY;
 ocpl_resp <= "01" when M_AXI_RVALID = '1' else "00";
 ocpl_s2mdata <= M_AXI_RDATA;
   
  

end architecture rtl;
