--
-- ADB3 ADM-XRC6T1-specific model definitions package
--

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
-- use IEEE.STD_LOGIC_MISC.all;

library work;

------------------------------------------------------------------
package adb3_model_defs is


   -- PCB Drawing Number
   constant adb3_pcb_drgnum : std_logic_vector(19 downto 0) := x"01278";
   
   -- FPGA Family
   constant adb3_fpga_family : string := "virtex7";
   
   -- Number of FPGA targets
   constant adb3_num_tgts : natural := 0;

   -- Power Status bus width
   constant adb3_pwrstat_width : natural := 10;
   
   -- 40MHz with 100MHz ref input to DCM  (Virtex5)
   constant adb3_vclk_default  : std_logic_vector(31 downto 0) := x"00040001";
   
   --
   -- Flash data & address bus width
   --
   constant adb3_flash_data_width    : natural := 16;
   constant adb3_flash_address_width : natural := 26;
	constant adb3_flash_latency_count : std_logic_vector(3 downto 0) := "0100";

   -- Number of lclk outputs
   constant adb3_num_vclk_o          : natural := 2;

   -- target FPGA readback latency (wrt BUSY)
   constant adb3_readback_latency : std_logic_vector(1 downto 0) := "01";
          
   -- Clock divide factor for 400kHz for IIC interfaces
   -- 25 = 400kHz, 4 = 2.5MHz
   constant adb3_400kHz_div : integer := 25; 
   constant adb3_1pps_div   : std_logic_vector(18 downto 0) := "110" & x"1A80";  
   
   --
   -- (Byte) Address Offset of model specific registers
   --    
   constant adb3_mregs_pcbrev         : std_logic_vector(11 downto 0) := x"000";
   --
   constant adb3_mregs_lclkset        : std_logic_vector(11 downto 0) := x"010";
   constant adb3_mregs_lclkd1stat     : std_logic_vector(11 downto 0) := x"014";
   constant adb3_mregs_lclkd2stat     : std_logic_vector(11 downto 0) := x"018";
   constant adb3_mregs_lclkstat       : std_logic_vector(11 downto 0) := x"01C";
   --
   constant adb3_mregs_fp0smap        : std_logic_vector(11 downto 0) := x"020";
   constant adb3_mregs_fp0ctl         : std_logic_vector(11 downto 0) := x"024";
   constant adb3_mregs_fp0rdctl       : std_logic_vector(11 downto 0) := x"028";
   --
   constant adb3_mregs_ictl           : std_logic_vector(11 downto 0) := x"030";
   constant adb3_mregs_istat          : std_logic_vector(11 downto 0) := x"034";
   --
   constant adb3_mregs_gpio_out       : std_logic_vector(11 downto 0) := x"040";
   constant adb3_mregs_gpio_in        : std_logic_vector(11 downto 0) := x"044";
   constant adb3_mregs_pwrstat        : std_logic_vector(11 downto 0) := x"048";
   --
   constant adb3_mregs_flctl          : std_logic_vector(11 downto 0) := x"050";
   constant adb3_mregs_flpage         : std_logic_vector(11 downto 0) := x"054";
   constant adb3_mregs_fldata         : std_logic_vector(11 downto 0) := x"058";
   --
   constant adb3_mregs_iic0ctl        : std_logic_vector(11 downto 0) := x"400";
   constant adb3_mregs_iic0stat       : std_logic_vector(11 downto 0) := x"404";
   constant adb3_mregs_sysmonstat     : std_logic_vector(11 downto 0) := x"408";
   constant adb3_mregs_xrmstat        : std_logic_vector(11 downto 0) := x"40C";
   --
   constant adb3_mregs_brdTempCtl     : std_logic_vector(11 downto 0) := x"410";
   constant adb3_mregs_brdTempStat    : std_logic_vector(11 downto 0) := x"414";
   constant adb3_mregs_tgtTempCtl     : std_logic_vector(11 downto 0) := x"418";
   constant adb3_mregs_tgtTempStat    : std_logic_vector(11 downto 0) := x"41C";
   --
   constant adb3_mregs_sysmonbuf_base : std_logic_vector(11 downto 0) := x"500";
   constant adb3_mregs_xrmbuf_base    : std_logic_vector(11 downto 0) := x"600";
   constant adb3_mregs_iicbuf_base    : std_logic_vector(11 downto 0) := x"700";
   
   
   -- VPD Start Address
   constant adb3_vpd_start_adr  : 
         std_logic_vector(adb3_flash_address_width downto 0) := 
         EXT(X"1000000", (adb3_flash_address_width+1));
                
   -- VPD (byte) offset in flash for Temperature grade information
   constant adb3_tgrade_adr : 
        std_logic_vector(adb3_flash_address_width downto 0) := 
        EXT(X"3E", (adb3_flash_address_width +1));
                                        
   -- VPD (byte) offeset in flash for lclk default value
   constant adb3_clkdef_adr : 
        std_logic_vector(adb3_flash_address_width downto 0) := 
        EXT(X"400", (adb3_flash_address_width +1));
                                        
   --
   -- Target FPGA bitstream (byte) start addresses
   --
   constant adb31_num_tgts        : natural := 0;
   
   constant adb3_bits0_start_adr  : 
         std_logic_vector(adb3_flash_address_width downto 0) := 
         EXT(X"1200000", (adb3_flash_address_width +1));
                
   constant adb3_bits1_start_adr  : 
          std_logic_vector(adb3_flash_address_width downto 0) := 
          EXT(X"2900000", (adb3_flash_address_width +1));

   constant adb3_bits2_start_adr  : 
         std_logic_vector(adb3_flash_address_width downto 0) := 
         EXT(X"1200000", (adb3_flash_address_width +1));
                
   constant adb3_bits3_start_adr  : 
          std_logic_vector(adb3_flash_address_width downto 0) := 
          EXT(X"2900000", (adb3_flash_address_width +1));

   --
   -- Temperature sensor limits
   --
   constant adb3_com_int_highlim : std_logic_vector(7 downto 0) := x"4B"; -- +75 
   constant adb3_com_int_lowlim  : std_logic_vector(7 downto 0) := x"FB"; -- -5
   constant adb3_com_ext_highlim : std_logic_vector(7 downto 0) := x"5A"; -- +90
   constant adb3_com_ext_lowlim  : std_logic_vector(7 downto 0) := x"FB"; -- -5
   constant adb3_ind_int_highlim : std_logic_vector(7 downto 0) := x"5A"; -- +90
   constant adb3_ind_int_lowlim  : std_logic_vector(7 downto 0) := x"D3"; -- -45
   constant adb3_ind_ext_highlim : std_logic_vector(7 downto 0) := x"69"; -- +105
   constant adb3_ind_ext_lowlim  : std_logic_vector(7 downto 0) := x"D3"; -- -45  

   
   ---------------------------------
   type adb3_model_in_t is
   ---------------------------------
   record
   --   -- Clocking
      avr_b2t       : std_logic;
   end record;  -- adb3_model_in_t

   
   ---------------------------------
   --type adb3_model_inout_t is
   ---------------------------------
   --record
   --   -- Flash Interface
   --   fl_a        : std_logic_vector(adb3_flash_address_width downto 1);
   --   fl_sel_n    : std_logic;
   --   fl_oe_n     : std_logic;
   --   fl_we_n     : std_logic;
   --   fl_rst_n    : std_logic;
   --   fl_d        : std_logic_vector(15 downto 0);
   --   fl_adv_n    : std_logic;
   --   --fl_clk      : std_logic;
   --   fl_wait_n   : std_logic;
   --   fl_rs1      : std_logic;
   --   -- Power
   --   -- IIC0 / System Monitor
   --   iic2_scl     : std_logic;
   --   iic2_sda     : std_logic;
   --   -- IIC1
   --   iic4_scl     : std_logic;
   --   iic4_sda     : std_logic;
   --   -- IIC1
   --   iic5_scl     : std_logic;
   --   iic5_sda     : std_logic;
   --   cable_present_l : std_logic;
   --end record;  -- adb3_model_inout_t

   
   ---------------------------------
   type adb3_model_out_t is
   ---------------------------------
   record
      -- Power
      avr_t2b   : std_logic;
   end record;  -- adb3_model_out_t

end;

package body adb3_model_defs is

end;
