--------------------------------------------------------------------------------
--
-- Module Name      : adm_pcieku3_model
--
--                    Copyright 2014 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : 
--
--
-- Dependencies     : adb3_ocp, adb3_vpd, adb3_model_defs, adb3_model, today
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.STD_LOGIC_MISC.all;

library work;
use work.adb3_ocp.all;
use work.adb3_board_ctrl_defs.all;
use work.adb3_board_ctrl.all;
use work.adb3_model_defs.all;
use work.today.all;

library unisim;
use unisim.vcomponents.all;

---------------------------------------------------------------
entity adm_xrc_ku1_tgt_model_tri is
  port (
    ocp_clk              : in    std_logic;
    rclk                 : in    std_logic;
    model_regs_m2s       : in    adb3_ocp_m2sT;
    model_regs_s2m       : out   adb3_ocp_s2mT;
    model_regs_irq       : out   std_logic;
    model_regs_reset     : in    std_logic;
    model_clk_out        : out   std_logic;
    model_clk_locked     : out   std_logic;
    linti_n              : in    std_logic;
    vpd_valid            : out   std_logic;
    vpd_out              : out   std_Logic_vector(47 downto 0);
    avr_clk              : in    std_Logic;
    avr_b2t              : in    std_Logic;
    avr_t2b              : out   std_logic;
    abmi_m2s             : out   adb3_ocp_m2sT;
    abmi_s2m             : in    adb3_ocp_s2mT);
end adm_xrc_ku1_tgt_model_tri;


---------------------------------------------------------------
architecture rtl of adm_xrc_ku1_tgt_model_tri is

  constant gen_iobs   : boolean := true;
  constant dont_care  : std_logic := '0';
  constant x32 : std_logic_vector(31 downto 0) := (others => dont_care );
  --
  signal lclk_locked      : std_logic;
  signal ocp_ref_clk_int  : std_logic;
  signal lclk             : std_logic;
  signal lclk270          : std_logic;
  signal lclk2x           : std_logic;
  signal lclkEn400k       : std_logic;
  signal lclkEn1pps       : std_logic;
  --                      
  signal rst_lclk         : std_logic := '1';
  --
  signal model_regs_m2s_lclk : adb3_ocp_m2sT := ADB3_OCP_DEFAULT_M2S;
  signal model_regs_m2s_throttled : adb3_ocp_m2sT := ADB3_OCP_DEFAULT_M2S;
  signal model_regs_s2m_lclk : adb3_ocp_s2mT := ADB3_OCP_DEFAULT_S2M;
  signal model_regs_s2m_throttled : adb3_ocp_s2mT := ADB3_OCP_DEFAULT_S2M;
  signal throttle_count : std_logic_vector(3 downto 0) := (others => '0');
  signal throttle : std_logic := '0';
  
  --
  signal model_regs_s2m_i    : adb3_ocp_s2mT := ADB3_OCP_DEFAULT_S2M;
  --
  signal addr           : std_logic_vector( 7 downto 4)  := (others => '0');
  signal bank           : std_logic_vector(11 downto 8)  := (others => '0');
  signal re             : std_logic                      := '0';
  signal re_1d          : std_logic                      := '0';
  signal re_2d          : std_logic                      := '0';
  signal tag            : std_logic_vector(7 downto 0)   := (others => '0');
  --
  signal we             : std_logic                      := '0';
  --
  signal write          : std_logic                      := '0';
  signal m2sData        : std_logic_vector(127 downto 0) := (others => '0');
  signal m2sBE          : std_logic_vector( 15 downto 0) := (others => '0');
  signal m2sadr : std_logic_vector(11 downto 4);
  signal avr_service : std_logic;
  
  --
  
  
  signal write_tgt      : std_logic := '0';
  signal write_lint     : std_logic := '0';
  signal write_flash    : std_logic := '0';
  signal write_gpio     : std_logic := '0';
  signal write_mptl     : std_logic := '0';
  signal write_sideser  : std_logic := '0';
  signal write_xsmon    : std_logic := '0';
  signal write_iic      : std_logic_vector(1 downto 0) := (others => '0');
  signal write_avr      : std_logic := '0';
  --
  --
  signal status_power      : std_logic_vector(31 downto 0);
  --
  signal xrm_vio_en_i      : std_logic;
  signal xrm_vio_en_o      : std_logic;
  signal xrm_vio_en_oe_n   : std_logic;
  signal xrm_vio_vsel_i    : std_logic_vector(2 downto 0);
  signal xrm_vio_vsel_o    : std_logic_vector(2 downto 0);
  signal xrm_vio_vsel_oe_n : std_logic_vector(2 downto 0);
  --
  signal hw_fault_n        : std_logic;
  --
  signal status_tgt        : std_logic_vector(95 downto 0);
  signal smap_full         : std_logic;
  signal TgtClrLatch       : std_logic;
  signal reload            : std_logic_vector(1 downto 0);
  signal reload_tgt0       : std_logic;
  signal aux_done          : std_logic;
  signal aux_init          : std_logic;
  --
  signal tgt_prog_i_n      : std_logic;
  signal tgt_prog_o_n      : std_logic := '1';
  signal tgt_prog_oe_n     : std_logic := '0';
  signal tgt_init_i_n      : std_logic;
  signal tgt_init_o_n      : std_logic := '0';
  signal tgt_init_oe_n     : std_logic := '1';
  signal tgt_smap_i        : std_logic_vector(7 downto 0);
  signal tgt_smap_o        : std_logic_vector(7 downto 0) := (others => '0');
  signal tgt_smap_oe_n     : std_logic_vector(7 downto 0) := (others => '1');
  signal tgt_cs_o_n        : std_logic := '1';
  signal tgt_we_o_n        : std_logic := '1';
  signal tgt_hswapen_o     : std_logic := '0';
  signal tgt_mode_i        : std_logic_vector(2 downto 0);
  signal tgt_mode_o        : std_logic_vector(2 downto 0);
  signal tgt_mode_oe_n     : std_logic_vector(2 downto 0);
  signal tgt_done_i        : std_logic;
  signal tgt_cclk_o        : std_logic;
  signal tgt_din_i         : std_logic;
  signal tgt_din_o         : std_logic := '0';
  signal tgt_din_oe_n      : std_logic := '1';
  signal tgt_busy_i        : std_logic;
  --
  signal status_lint       : std_logic_vector(63 downto 0);
  
  signal linti_i_n         : std_logic_vector(1 downto 0);
  
  signal status_flash      : std_logic_vector(95 downto 0);
  signal aux_override      : std_logic;
  signal aux_prog          : std_logic;
  signal aux_write         : std_logic;
  signal aux_data          : std_logic_vector(7 downto 0);
  signal TgtCleared        : std_logic;
  signal IndGrade          : std_logic;
  signal TempFlagVd        : std_logic;
  signal Clk0BootVal       : std_logic_vector(31 downto 0);
  signal Clk0BootVd        : std_logic;
  --
 
  signal fbs_i_n           : std_logic;
  signal reboot_en_i       : std_logic;
  signal pwrstat_i         : std_logic_vector(adb3_pwrstat_width -1 downto 0);
  signal xrm_presence_i_n  : std_logic;
  signal force_xrm2v5_i_n  : std_logic;
  signal factory_test_i_n  : std_logic;
  signal mvmro_mp_i        : std_logic;
                                           

  signal avr_rx           : std_logic;
  signal avr_tx           : std_logic;
  signal avr_rx_int           : std_logic;
  signal avr_tx_int           : std_logic;
  signal avr_mp_sgnl      : std_logic;
  signal avr_fpga_sgnl    : std_logic;
  signal avr_alarm_l      : std_logic;

  signal avr_alert        : std_logic_vector(3 downto 0);

  
  signal avr_sgnl         : std_logic;
  signal avr_coms_rx      : std_logic;
  signal avr_coms_tx      : std_logic;
  signal avr_bram_q : std_logic_vector(127 downto 0);
  
  signal status_avr           : std_logic_vector( 127 downto 0 );
  signal status_avr_vpd       : std_logic_vector( 127 downto 0 );
  signal status_avr_sns       : std_logic_vector( 127 downto 0 );
  signal status_avr_xrm       : std_logic_vector( 127 downto 0 );
  signal status_icap      : std_logic_vector(255 downto 0);
  signal icap_full : std_logic;
  signal write_icap      : std_logic := '0';
  
  ---------------------------------------
  component adb3_ocp_multiport2_sc
  ---------------------------------------
    generic (
      round_robin_arb : boolean;
      use_timeout     : boolean;
      timeout_bits    : integer);
    port (
      ocp_m2s_memif : out adb3_ocp_m2sT;
      ocp_s2m_memif : in  adb3_ocp_s2mT;
      ocp_m2s0      : in  adb3_ocp_m2sT;
      ocp_s2m0      : out adb3_ocp_s2mT;
      ocp_m2s1      : in  adb3_ocp_m2sT;
      ocp_s2m1      : out adb3_ocp_s2mT;
      status        : out std_logic_vector(3 downto 0);
      Clk           : in  std_logic);
  end component;
  
  signal slaves_m2s : adb3_ocp_m2sT_array(0 to 1);
  signal slaves_s2m : adb3_ocp_s2mT_array(0 to 1); 

    
  signal address  : std_logic_vector( 13 downto 4 );

  signal cable_present : std_logic := '0';

  signal vpd_addr : std_logic_vector(4 downto 0) := (others => '0');
  signal vpd_sel : std_logic_vector(15 downto 0) := (others => '0');
  signal vpd_data : std_logic_vector(7 downto 0) := (others => '0');
  
begin  -- rtl

  model_clk_out <= lclk;
  model_clk_locked <= lclk_locked;

    -------------------------------------------
   model_clocks_ku1_0 : entity work.model_clocks_ku1tgt
   -------------------------------------------
   generic map (
      OCPCLKDIV       => 4   
   )
   port map (
     rclk_bufg       => rclk,
      --
      lclk_locked     => lclk_locked,
      --
      rclk            => open,
      ocp_ref_clk     => open,
      lclk            => lclk,
      lclk270         => lclk270,
      lclk2x          => lclk2x,
      lclkEn400k      => lclkEn400k,
      lclkEn1pps      => lclkEn1pps
   ); -- model_clocks_0


   
   -------------------------------------------
   -- hold model registers in reset until 
   -- the PLL has locked
   -------------------------------------------
   reset_logic : process (lclk_locked, lclk)
   -------------------------------------------
   begin  -- process reset_logic
      if lclk_locked = '0' then
         rst_lclk <= '1';
      elsif rising_edge(lclk) then
         rst_lclk <= '0';
      end if;  -- lclk_locked / lclk
   end process reset_logic;
      

   adb3_ocp_demux_1: entity work.adb3_ocp_demux
    generic map (
      number_of_slaves  => 2,
      demux_address_lsb => 12)
    port map (
      clk        => ocp_clk,
      master_m2s => model_regs_m2s,
      master_s2m => model_regs_s2m,
      slaves_m2s => slaves_m2s,
      slaves_s2m => slaves_s2m);
  
  
   -------------------------------------------
   adb3_ocp_cross_clock_dom_0 : adb3_ocp_cross_clock_dom
   -------------------------------------------
   port map (
      master_clk => ocp_clk,
      slave_clk  => lclk,
      master_m2s => slaves_m2s(0),
      master_s2m => slaves_s2m(0),
      slave_m2s  => model_regs_m2s_lclk,
      slave_s2m  => model_regs_s2m_lclk
   );

  
    icap_full <= '0';
    status_icap <= (others => '0');
 
   
   -------------------------------------------------------------------------
   -- register interface for model_regs_m2s
   -------------------------------------------------------------------------

  -- can use smap_full as its "nearly" full (2 spaces) flag on SMAP FIFO
  -- Only 1 register level between
  model_regs_s2m_lclk.CmdAccept  <= model_regs_s2m_i.CmdAccept;
  model_regs_s2m_lclk.DataAccept <= model_regs_s2m_i.DataAccept;
  model_regs_s2m_lclk.Data       <= model_regs_s2m_i.Data;
  model_regs_s2m_lclk.Resp       <= model_regs_s2m_i.Resp;
  model_regs_s2m_lclk.Tag        <= model_regs_s2m_i.Tag;
  
  
  

   -------------------------------------------
   regs_p0 : process (lclk)
   -------------------------------------------
      variable active_addr  : std_logic_vector(3 downto 0);
      variable active_bank  : std_logic_vector(3 downto 0);
      variable write        : std_logic;
      
   begin  -- process general_regs_p0
      if rising_edge(lclk) then
      
         re_2d <= re_1d;
         re_1d <= re;    -- delay to allow BRAM reads
         
         if (model_regs_m2s_lclk.Cmd = OCP_CMD_WRITE) and 
               (model_regs_s2m_i.CmdAccept = '0') and 
               (icap_full = '0') and (we = '0') then
            active_addr                := model_regs_m2s_lclk.Addr(7 downto 4);
            addr                       <= model_regs_m2s_lclk.Addr(7 downto 4);
            active_bank                := model_regs_m2s_lclk.Addr(11 downto 8);
            bank                       <= model_regs_m2s_lclk.Addr(11 downto 8);
            address                    <= model_regs_m2s_lclk.Addr(13 downto 4);
            model_regs_s2m_i.CmdAccept <= '1';
            re                         <= '0';
            tag                        <= tag;
            
         elsif (model_regs_m2s_lclk.Cmd = OCP_CMD_READ) and 
                  (model_regs_s2m_i.CmdAccept = '0') and (we = '0') then
            active_addr                := model_regs_m2s_lclk.Addr(7 downto 4);
            addr                       <= model_regs_m2s_lclk.Addr(7 downto 4);
            active_bank                := model_regs_m2s_lclk.Addr(11 downto 8);
            bank                       <= model_regs_m2s_lclk.Addr(11 downto 8);
            address                    <= model_regs_m2s_lclk.Addr(13 downto 4);
            model_regs_s2m_i.CmdAccept <= '1';
            re                         <= '1';
            tag                        <= model_regs_m2s_lclk.Tag;
            
         else
            address                    <= address;
            active_addr                := addr;
            addr                       <= addr;
            active_bank                := bank;
            bank                       <= bank;
            model_regs_s2m_i.CmdAccept <= '0';
            re                         <= '0';
            tag                        <= tag;
         end if;
         
      
         if (model_regs_m2s_lclk.Cmd = OCP_CMD_WRITE) and 
               (model_regs_s2m_i.CmdAccept = '0') and 
               (icap_full = '0') and (we = '0') then
            we <= '1';
         elsif (model_regs_m2s_lclk.DataValid = '1') and (we = '1') then
            we <= '0';
         else
            we <= we;
         end if;
         
         
         if (model_regs_m2s_lclk.DataValid = '1') and (we = '1') then
            model_regs_s2m_i.DataAccept <= '1';
            if ( model_regs_m2s_lclk.Addr( 13 downto 12 ) = "00" ) then
              write       := '1';
            else
              write       := '0';
            end if;
            m2sData     <= model_regs_m2s_lclk.Data(127 downto 0);
            m2sBE       <= model_regs_m2s_lclk.DataByteEn(15 downto 0);
         else
            model_regs_s2m_i.DataAccept <= '0';
            write       := '0';
            m2sData     <= m2sData;
            m2sBE       <= m2sBE;
         end if;


         -- Map Register Write enables
         -- Bank 0
         
         -- defaults       
         write_tgt     <= '0';
         write_lint    <= '0';
         write_flash   <= '0';
         write_gpio    <= '0';
         write_mptl    <= '0';
         write_sideser <= '0';
         write_xsmon   <= '0';
         write_iic     <= (others => '0');
         write_avr     <= '0';
         write_icap     <= '0';
         
         if ( write = '1' ) then
            case active_bank is
               when x"0"   => case active_addr is
                                 when x"0"   => null;
                                 when x"1"   => null; -- was write_vclk
                                 when x"2"   => write_tgt     <= '1';
                                 when x"3"   => write_lint    <= '1';
                                 when x"4"   => write_flash   <= '1';
                                 when x"5"   => write_gpio    <= '1';
                                 when x"6"   => write_mptl    <= '1';
                                 when x"7"   => write_sideser <= '1';
                                 when x"8"   => write_xsmon   <= '1';
                                 when x"B"   => write_avr     <= '1';
                                 when x"C"   => write_icap     <= '1';
                                 when x"D"   => write_icap     <= '1';
                                 when others => null;
                              end case;   -- active_addr
                              
               when x"4"   => case active_addr is
                                 when x"0"   => write_iic(0)  <= '1';
                                 when x"1"   => write_iic(1)  <= '1';
                                 when others => null;
                              end case;   -- active_addr
                              
               when others => null;                          
            end case;   -- active_bank
               
         end if;  -- write
         
      end if;  -- lclk
   end process regs_p0;
         
      
 -------------------------------------------
   read_regs_proc : process(lclk)
   -------------------------------------------
      variable regs  : std_logic_vector(127 downto 0);
      variable regs0 : std_logic_vector(127 downto 0);
      variable regs1 : std_logic_vector(127 downto 0);
      variable regs2 : std_logic_vector(127 downto 0);
      variable regs3 : std_logic_vector(127 downto 0);
      variable regs4 : std_logic_vector(127 downto 0);
      variable regs5 : std_logic_vector(127 downto 0);
      variable regs6 : std_logic_vector(127 downto 0);
      variable regs7 : std_logic_vector(127 downto 0);
  begin  -- process read_regs_proc
    if rising_edge(lclk) then
      case address(13 downto 12) is
        when "00" =>    -- Standard registers  ( 0x0000-0x0FFF )
          case address(11 downto 8) is
            when x"0" =>                -- ( 0x0000-0x00FF )
              case address(7 downto 4) is
                when x"0" =>
                  regs := TODAYS_DATE & x32 & x32;
                when x"1" =>
                  regs := x32 & x32 & x32 & x32;
                when x"2" =>
                  regs := x32 & x32 & x32 & x32;
                when x"3" =>
                  regs := x32 & x32 & status_lint;
                when x"4" =>
                  regs := x32 & x32 & x32 & x32;
                when x"B" =>
                  regs := status_avr;
                when others =>
                  regs := (others => dont_care);
              end case;
            when x"1" =>                -- ( 0x0100-0x01FF )
              regs := (others => dont_care);
            when x"2" =>                -- ( 0x0200-0x01FF )
              regs := (others => dont_care);
            when x"3" =>                -- ( 0x0300-0x03FF )
              regs := (others => dont_care);
            when x"4" =>                -- ( 0x0400-0x04FF )
              regs := (others => dont_care);
            when x"5" =>                -- ( 0x0500-0x05FF )
              regs := (others => dont_care);  --iicbuf_q;
            when x"6" =>                -- ( 0x0600-0x06FF )
              regs := status_avr_xrm;
            when x"7" =>                -- ( 0x0700-0x07FF )
              regs := status_avr_xrm;
            when x"8" =>                -- ( 0x0800-0x08FF )
              regs := status_avr_vpd;
            when x"9" =>                -- ( 0x0900-0x09FF )
              regs := status_avr_vpd;
            when x"A" =>                -- ( 0x0A00-0x0AFF )
              regs := status_avr_sns;
            when x"B" =>                -- ( 0x0B00-0x0BFF )
              regs := status_avr_sns;
            when x"C" =>                -- ( 0x0C00-0x0CFF )
              regs := (others => dont_care);
            when x"D" =>                -- ( 0x0D00-0x0DFF )
              regs := (others => dont_care);
            when x"E" =>                -- ( 0x0E00-0x0EFF )
              regs := (others => dont_care);
            when x"F" =>                -- ( 0x0F00-0x0FFF )
              regs := (others => dont_care);
            when others =>
              regs := (others => dont_care);
          end case;
        when "01" =>                    -- Unused ( 0x1000-0x1FFF )          
          regs := (others => dont_care);
        when "10" =>                    -- Unused ( 0x2000-0x2FFF )
          regs := (others => dont_care);
        when others =>                  -- Unused ( 0x3000-0x3FFF )          
          regs := (others => dont_care);
        end case;
        if (re_1d = '1') and (re_2d = '0') then
           model_regs_s2m_i.Resp <= "01";
           model_regs_s2m_i.Data <= regs; 
           model_regs_s2m_i.Tag  <= tag;
        elsif model_regs_m2s_lclk.RespAccept = '1' then
           model_regs_s2m_i.Resp <= "00";
        end if;
      end if;  -- lclk
   end process read_regs_proc;

 
     -------------------------------------------
   fpga_int_0 : fpga_int_adb3
   -------------------------------------------
    generic map (
      NUMBER_OF_INTERRUPTS => 7,
      LEVEL_SENSITIVE_DEFAULTS => X"007E",
      ACTIVE_LOW => X"0001")
    port map(
      rst            => rst_lclk,
      lclk           => lclk,
      write          => write_lint,
      be             => m2sBE(7 downto 0),
      data           => m2sData(63 downto 0),
      status         => status_lint,
      int_local      => model_regs_irq,
      interrupts(0)  => linti_n,
      interrupts(1)  => avr_alert(0),
      interrupts(2)  => avr_alert(1),
      interrupts(3)  => avr_alert(2),
      interrupts(4)  => avr_alert(3),
      interrupts(5)  => avr_coms_rx,
      interrupts(6)  => avr_coms_tx);
   
   
   
   m2sAdr <= bank & addr;

    -----------------------------------------
  i_avr_proc_interface : entity work.avr2_proc_interface_v1
  -----------------------------------------
    port  map(
      -- Physical ports
      rx_ser_data       => avr_tx_int,      -- Connects to AVR TX Pin
      tx_ser_data       => avr_rx_int,      -- Connects to AVR RX Pin
      ck_ser_data       => avr_clk,
      -- Clock, Reset, Interrupt
      rst               => rst_lclk,
      reg_clk           => lclk,
      irq_coms_rx       => avr_coms_rx,
      irq_coms_tx       => avr_coms_tx,
      irq_alarm_thrm_crit => avr_alert(0),
      irq_alarm_thrm_warn => avr_alert(1),
      irq_alarm_sppl_crit => avr_alert(2),
      irq_alarm_sppl_warn => avr_alert(3),
      -- Serial Number Output
      serial_no           => open,
      serial_no_valid     => open,
      -- VPD Forwarding
      vpd_forward_data    => vpd_data,
      vpd_forward_addr    => vpd_addr,
      vpd_forward_be      => vpd_sel,
      -- Register interface
      write             => write_avr,
      be                => m2sBE( 15 downto 0 ),
      data              => m2sData( 127 downto 0 ),
      status            => status_avr,
      -- BRAM Interface (VPD) 512 bytes
      avr_bram_vpd_a    => m2sAdr( 8 downto 4 ),
      avr_bram_vpd_q    => status_avr_vpd,
      -- BRAM Interface (VPD) 512 bytes
      avr_bram_mez_a    => m2sAdr( 8 downto 4 ),
      avr_bram_mez_q    => status_avr_xrm,
      -- BRAM Interface (Sensors) 64 bytes
      avr_bram_sns_a    => m2sAdr( 8 downto 4 ),
      avr_bram_sns_q    => status_avr_sns
    );

  vpd_out(0) <= lclk;
  vpd_out(15 downto 8) <= vpd_data;
  vpd_out(31 downto 21) <= X"00" & "000";
  vpd_out(20 downto 16) <= vpd_addr;
  vpd_out(47 downto 32) <= vpd_sel;

  avr_t2b <= avr_rx_int;
  avr_tx_int <= avr_b2t;
  avr_alarm_l <= '1';
  avr_service <= '0';
  
 
  vpd_valid <= status_avr(30);

  abmi_m2s <= slaves_m2s(1);
  slaves_s2m(1) <= abmi_s2m;
  
  
end rtl;    -- adm_xrc6t1_model
