-------------------------------------------------------------------------------
-- Module Name      : dma_ocp_mux256
--
--                    Copyright 2013 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : 
--
--
-- Dependencies     : adb3_ocp
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;

entity dma_ocp_mux256 is
  
  port (
    clk          : in  std_logic;
    enable       : in  std_logic;
    external_m2s : out adb3w_ocp_m2sT;
    external_s2m : in  adb3w_ocp_s2mT;
    desc_fetch_m2s : in adb3w_ocp_m2sT;
    desc_fetch_s2m : out  adb3w_ocp_s2mT;
    reader_m2s : in adb3w_ocp_m2sT;
    reader_s2m : out  adb3w_ocp_s2mT;
    writer_m2s : in adb3w_ocp_m2sT;
    writer_s2m : out  adb3w_ocp_s2mT);

end dma_ocp_mux256;

architecture synth of dma_ocp_mux256 is

  signal cmd_select : std_logic_vector(2 downto 0) := "000";
  signal resp_select : std_logic := '0';
  
  
begin  -- synth

  external_m2s.Cmd <= desc_fetch_m2s.Cmd when cmd_select = "001" else
                      reader_m2s.Cmd when cmd_select = "010" else
                      writer_m2s.Cmd when cmd_select = "100" else
                      OCP_CMD_IDLE;

  external_m2s.Addr <= desc_fetch_m2s.Addr when cmd_select = "001" else
                       reader_m2s.Addr when cmd_select = "010" else
                       writer_m2s.Addr; -- when cmd_select = "100" else
  --(others => '0');

  external_m2s.BurstLength <= desc_fetch_m2s.BurstLength when cmd_select = "001" else
                              reader_m2s.BurstLength when cmd_select = "010" else
                              writer_m2s.BurstLength; -- when cmd_select = "100" else
  --(others => '0');
  -- Tag ports for response demux
  external_m2s.Tag <= desc_fetch_m2s.Tag(7 downto 2) & "10" when cmd_select = "001" else
                      reader_m2s.Tag(7 downto 2) & "01" when cmd_select = "010" else
                      writer_m2s.Tag(7 downto 2) & "00"; --when cmd_select = "100" else
  --(others => '0');


  desc_fetch_s2m.CmdAccept <= external_s2m.CmdAccept when cmd_select = "001" else '0';
  reader_s2m.CmdAccept <= external_s2m.CmdAccept when cmd_select = "010" else '0';
  writer_s2m.CmdAccept <= external_s2m.CmdAccept when cmd_select = "100" else '0';
  
  
  -- Data only from Writer
  external_m2s.Data <= writer_m2s.Data;
  external_m2s.DataByteEn <= writer_m2s.DataByteEn;
  external_m2s.DataValid <= writer_m2s.DataValid;
  
  writer_s2m.DataAccept <= external_s2m.DataAccept;
  reader_s2m.DataAccept <= '0';
  desc_fetch_s2m.DataAccept <= '0';

  -- Resp select between desc_fetch and reader
  resp_select <= '0' when external_s2m.Tag(1 downto 0) = "10" else '1';
  
  desc_fetch_s2m.Resp <= external_s2m.Resp when resp_select = '0' and enable = '1' else OCP_RESP_NONE;
  reader_s2m.Resp <= external_s2m.Resp when resp_select = '1' and enable = '1' else OCP_RESP_NONE;
  desc_fetch_s2m.Tag <= external_s2m.Tag;
  reader_s2m.Tag <= external_s2m.Tag;  
  desc_fetch_s2m.Data <= external_s2m.Data;
  reader_s2m.Data <= external_s2m.Data;

  external_m2s.RespAccept <= external_s2m.Resp(0) when enable = '0' else
                             desc_fetch_m2s.RespAccept when resp_select = '0' else
                             reader_m2s.RespAccept;

  writer_s2m.Resp <= OCP_RESP_NONE;
  writer_s2m.Data <= (others => '0');
  writer_s2m.Tag <= (others => '0');
  
  switch_ports0: process (clk)
    variable cmd_selected : std_logic;
  begin  -- process switch_ports0
    if clk'event and clk = '1' then  -- rising clock edge
      cmd_selected := '0';

      if enable = '1' then
        -- Switch Command when presented
        if cmd_select(2) = '0' and cmd_select(1) = '0' then
          if desc_fetch_m2s.Cmd /= OCP_CMD_IDLE then
            cmd_select(0) <= '1';
            cmd_selected := '1';
          else
            cmd_select(0) <= '0';
          end if;
        end if;

        if cmd_select(2) = '0' and cmd_select(0) = '0' and cmd_selected = '0'  then
          if reader_m2s.Cmd  /= OCP_CMD_IDLE then
            cmd_select(1) <= '1';
            cmd_selected := '1';
          else
            cmd_select(1) <= '0';
          end if;
        end if;

        if cmd_select(0) = '0' and cmd_select(1) = '0' and cmd_selected = '0' then
          if writer_m2s.Cmd /= OCP_CMD_IDLE then
            cmd_select(2) <= '1';
          else
            cmd_select(2) <= '0';
          end if;
        end if;
    
      else
        cmd_select <= "000";
      end if;


    end if;
  end process switch_ports0;
  

end synth;
