-------------------------------------------------------------------------------
-- Module Name      : write_worker
--
--                    Copyright 2009 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : 
--
--
-- Dependencies     : adb3_ocp,adb3_dma
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_dma.all;


entity write_worker256 is
  generic (
    addr_bits  : natural := 64;
    left_bits  : natural := 39;
    buffer_descriptors : boolean := true;
    registering_for_speed : boolean := true);
  port (
    clk                  : in  std_logic;
    -- ABORT/Cleanup Control
    abort                : in  std_logic;
    cleanup              : in  std_logic;
    -- FSM Status
    fsm_idle             : out std_logic_vector(1 downto 0);
    fsm_aborting         : out std_logic_vector(1 downto 0);
    -- OCP Master Port (to MUX to PCIe)
    ocp_m2s              : out adb3w_ocp_m2sT;
    ocp_s2m              : in  adb3w_ocp_s2mT;
    -- Worker DMA Engine Control
    worker_cmd           : in  adb3_dma_worker_command_type;
    worker_cmd_valid     : in  std_logic;
    worker_ready         : out std_logic;
    worker_completed     : out std_logic;
    -- Burst Length Configuratioon
    max_burst            : in  std_logic_vector(8 downto 0);
    -- IRQ Control
    irq                  : out std_logic;
    irq_ack              : in  std_logic;
    -- Data Path
    fifo_data            : in  std_logic_vector(255 downto 0);
    fifo_empty           : in  std_logic;
    fifo_level           : in  std_logic_vector(8 downto 0);
    fifo_radv            : out std_logic);

end write_worker256;

architecture rtl of write_worker256 is

 
  type state_type is (IDLE,
                      IS_ADDR_ALIGNED,
                      GEN_FIRST_BYTE_WRITE,
                      GEN_SHORT_BYTE_WRITE,
                      FIRST_BURST_WAIT,
                      IS_FIRST_BURST_LEFT,
                      GEN_FIRST_BURST,
                      IS_BURST_LEFT,
                      GEN_BURST,
                      IS_SHORT_BURST_LEFT,
                      GEN_SHORT_BURST,
                      ARE_BYTES_LEFT,
                      GEN_LAST_BYTE_WRITE,
                      GEN_IRQ,
                      S_ABORT);

  signal state : state_type := S_ABORT;


  type wstate_type is (W_IDLE,
                       W_READ_BURST_DESC,
                       W_GEN_IRQ,
                       W_FLUSH,
                       W_GEN_WRITE_CMD,
                       W_ABORT);

  
  
  signal wstate : wstate_type := W_ABORT;
 
  component small_sfifo
    generic (
      width : integer := 64);
    port (
      clk    : in  std_logic;
      rst    : in  std_logic;
      wadv   : in  std_logic;
      wdata  : in  std_logic_vector(width-1 downto 0);
      nfull  : out std_logic;
      radv   : in  std_logic;
      rempty : out std_logic;
      rdata  : out std_logic_vector(width-1 downto 0));
  end component;

  
  component small_sfifo64
    generic (
      width : integer := 64);
    port (
      clk    : in  std_logic;
      rst    : in  std_logic;
      wadv   : in  std_logic;
      wdata  : in  std_logic_vector(width-1 downto 0);
      nfull  : out std_logic;
      radv   : in  std_logic;
      rempty : out std_logic;
      rdata  : out std_logic_vector(width-1 downto 0));
  end component;

   component small_sfifo_reg_op
    generic (
      width : integer := 64);
    port (
      clk    : in  std_logic;
      rst    : in  std_logic;
      wadv   : in  std_logic;
      wdata  : in  std_logic_vector(width-1 downto 0);
      nfull  : out std_logic;
      radv   : in  std_logic;
      rempty : out std_logic;
      rdata  : out std_logic_vector(width-1 downto 0));
  end component;

  signal cmd_fifo_d, cmd_fifo_q : std_logic_vector(106 downto 0) := (others => '0');
  signal worker_cmd_q           : adb3_dma_worker_command_type := ADB3_DMA_WORKER_COMMAND_TYPE_DEFAULT;
  signal cmd_fifo_nfull         : std_logic := '0';
  signal cmd_fifo_empty         : std_logic := '1';
  signal cmd_fifo_radv          : std_logic := '0';
  signal cmd_fifo_wadv          : std_logic := '0';
  signal cmd_fifo_empty_r1         : std_logic := '1';
  signal cmd_fifo_empty_r2         : std_logic := '1';

  signal cmd_fifo_nfull_reg         : std_logic:= '0';
  

  signal current_cmd : adb3_dma_worker_command_type := ADB3_DMA_WORKER_COMMAND_TYPE_DEFAULT;

  signal current_cmd_irq_on_completion_reg : std_logic := '0';
  signal current_cmd_last_descriptor_reg : std_logic := '0';
  
  
  signal addr0, addr1, addr2, addr3, addr4 : std_logic_vector(13 downto 0) := (others => '0');
  signal left0, left0_reg, left1, left2, left3, left4 : std_logic_vector(13 downto 0) := (others => '0');
  signal addr_valid, left_valid     : std_logic := '0';

  signal addr            : std_logic_vector(63 downto 0) := (others => '0');
  --signal xfer_left       : std_logic_vector(63 downto 0);
  signal xfer_left       : std_logic_vector(left_bits-1 downto 0) := (others => '0');
  signal addr_reg        : std_logic_vector(63 downto 0) := (others => '0');
  --signal xfer_left_reg   : std_logic_vector(63 downto 0);
  signal xfer_left_reg   : std_logic_vector(left_bits-1 downto 0) := (others => '0');

  signal lots_left     : std_logic := '0';
--   signal max_burst_reg1 : std_logic_vector(9 downto 0) := "0000000000";
--   signal max_burst_reg2 : std_logic_vector(9 downto 0) := "0000001001";
--   signal max_burst_reg3 : std_logic_vector(9 downto 0) := "0000010010";
--   signal max_burst_reg4 : std_logic_vector(9 downto 0) := "0000100011";
  signal max_burst_reg1 : std_logic_vector(8 downto 0);-- := "000000000";
  signal max_burst_reg2 : std_logic_vector(8 downto 0);-- := "000000000";
  signal max_burst_reg3 : std_logic_vector(8 downto 0);-- := "000000000";
  signal max_burst_reg4 : std_logic_vector(8 downto 0);-- := "000000000";

  signal inc_addr : std_logic := '1';
  
  
  signal irq_cmd    : std_logic := '0';
  signal size       : std_logic_vector(8 downto 0) := (others => '0');
  signal be         : std_logic_vector(31 downto 0) := (others => '0');
  signal byte_write : std_logic := '0';
  signal last_desc  : std_logic := '0';

  signal burst_desc_fifo_ready : std_logic := '0';
  signal burst_desc_fifo_nfull : std_logic := '0';
  signal burst_desc_fifo_nfull_reg : std_logic := '0';
  signal burst_desc_fifo_wadv  : std_logic := '0';
  signal burst_desc_fifo_radv  : std_logic := '0';
  signal burst_desc_fifo_empty : std_logic := '1';
  signal burst_desc_fifo_empty_r1 : std_logic := '1';
  signal burst_desc_fifo_empty_r2 : std_logic := '1';

  signal burst_desc_fifo_d : std_logic_vector(108 downto 0) := (others => '0');
  signal burst_desc_fifo_q : std_logic_vector(108 downto 0):= (others => '0');
  signal burst_desc_fifo_q_r1 : std_logic_vector(108 downto 0):= (others => '0');
  signal burst_desc_fifo_q_r2 : std_logic_vector(108 downto 0):= (others => '0');

  signal w_irq_cmd    : std_logic := '0';
  signal w_size       : std_logic_vector(9 downto 0) := (others => '0');
  signal w_be         : std_logic_vector(31 downto 0) := (others => '0');
--  signal w_addr       : std_logic_vector(63 downto 0) := (others => '0');
  signal w_byte_write : std_logic := '0';
  signal w_last_desc  : std_logic := '0';

  signal w_burst_left : std_logic_vector(9 downto 0) := (others => '0');
  signal fifo_radv_i  : std_logic := '0';
  signal fifo_radv_ii  : std_logic := '0';
  signal fifo_radv_iii  : std_logic := '0';
  signal flush_counter : std_logic_vector(1 downto 0) := "00";

  signal fifo_radv_q  : std_logic := '0';
  
  signal irq_state     : std_logic_vector(2 downto 0) := "000";

  signal local_rst : std_logic := '1';
  signal rst       : std_logic := '0';
  signal rst_ofifo       : std_logic := '0';
  signal fifo_level_reg : std_logic_vector(9 downto 0);
  
  -- Run Continouously if SIZE = 0
  --signal run_continuous : std_logic := '0';
  signal fl_gt_size : std_logic := '0';
  signal fl_gt_size_nidle : std_logic := '0';
  signal fl_gt_size_idle : std_logic := '0';
  signal fl_gt_size_wd_idle : std_logic := '0';
  signal last_state_idle : std_logic := '1';

  signal mcmd         : ocp_CmdT                                          := OCP_CMD_IDLE;
  signal maddr        : std_logic_vector(ADB3_OCP_ADDR_WIDTH-1 downto 0)  := (others => '0');
  signal mburstlength : std_logic_vector(ADB3_OCP_BURST_WIDTH-1 downto 0) := (others => '0');
  signal mdata : std_logic_vector(ADB3W_OCP_DATA_WIDTH-1 downto 0) := (others => '0');
  signal mdatabyteen : std_logic_vector(ADB3W_OCP_DATA_WIDTH/8-1 downto 0) := (others => '0');
  signal mdatavalid : std_logic := '0';

  signal tdata : std_logic_vector(ADB3W_OCP_DATA_WIDTH-1 downto 0) := (others => '0');
  signal tdatabyteen : std_logic_vector(ADB3W_OCP_DATA_WIDTH/8-1 downto 0) := (others => '0');
  signal tdatavalid : std_logic := '0';

  signal tfifo_nfull : std_logic := '0';
  signal tfifo_nfull_r1 : std_logic := '0';
  signal tfifo_empty : std_logic := '0';
  signal tfifo_radv : std_logic := '0';
  
  signal first_word : std_logic := '1';
  signal burst_complete : std_logic := '1';

  signal tdata_wide, mdata_wide : std_logic_vector(ADB3W_OCP_DATA_WIDTH+ADB3W_OCP_DATA_WIDTH/8-1 downto 0) := (others => '0');
  signal tdata_wide_r1 : std_logic_vector(ADB3W_OCP_DATA_WIDTH+ADB3W_OCP_DATA_WIDTH/8-1 downto 0) := (others => '0');
  signal tdatavalid_r1: std_logic := '0';
  
  signal tfifo_flush : std_logic := '0';
  signal abort_dvalid : std_logic := '0';

  signal abort_write_burst_timeout : std_logic := '0';
  signal abort_write_burst_timer : std_logic_vector(11 downto 0) := (others => '0');

  signal state_aborting : std_logic := '0';
  
  signal abort1 : std_logic := '0';
  signal abort2 : std_logic := '0';
  signal cleanup1 : std_logic := '0';
--   attribute use_dsp48 : string;
--   attribute use_dsp48 of left1 : signal is "yes";
--   attribute use_dsp48 of left2 : signal is "yes";
--   attribute use_dsp48 of addr1 : signal is "yes";
--   attribute use_dsp48 of addr2 : signal is "yes";
--   attribute use_dsp48 of addr3 : signal is "yes";
--   attribute use_dsp48 of addr4 : signal is "yes";

  attribute shreg_extract : string;
  attribute shreg_extract of cmd_fifo_d : signal is "no";
  attribute shreg_extract of burst_desc_fifo_q_r1 : signal is "no";
  attribute shreg_extract of burst_desc_fifo_q_r2 : signal is "no";
  attribute shreg_extract of burst_desc_fifo_empty_r1 : signal is "no";
  attribute shreg_extract of burst_desc_fifo_empty_r2 : signal is "no";

  signal irq_delay : std_logic_vector(31 downto 0) := (others => '0');
  signal irq_out : std_logic := '0';

  signal short_xfer : std_logic := '0';

  signal worker_completed_i : std_logic := '0';

  signal first_burst_addr_mask : std_logic_vector(8 downto 0) := (others => '0');
  signal first_burst_length : std_logic_vector(8 downto 0) := (others => '0');

  signal wdstate_is_idle_r1 : std_logic := '1';
  signal wdstate_is_idle_r2 : std_logic := '1';


  signal mcmd_i :  ocp_CmdT := OCP_CMD_IDLE;
  signal mcmd_accept_i : std_logic := '0';
  signal maddr_i        : std_logic_vector(ADB3_OCP_ADDR_WIDTH-1 downto 0)  := (others => '0');
  signal mburstlength_i : std_logic_vector(ADB3_OCP_BURST_WIDTH-1 downto 0) := (others => '0');
  
  signal mcmd_fifo_in,mcmd_fifo_out : std_logic_vector(ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH-1 downto 0) := (others => '0');
  
  signal mcmd_fifo_wadv,mcmd_fifo_radv,mcmd_fifo_empty,mcmd_fifo_nfull : std_logic := '0';

  signal mcmd_fifo_level : std_logic_vector(2 downto 0) := "000";
  signal tfifo_level : std_logic_vector(6 downto 0) := "0000000";
  signal mburstlength_r1 : std_logic_vector(ADB3_OCP_BURST_WIDTH-1 downto 0) := (others => '0');
  signal mburstlength_r1_m1 : std_logic_vector(ADB3_OCP_BURST_WIDTH-1 downto 0) := (others => '0');
  signal mcmd_fifo_wadv_r1 : std_logic := '0';
  
  signal irq_valid : std_logic := '0';
  
begin  -- rtl

  ocp_m2s.Cmd         <= mcmd_i;
  ocp_m2s.Addr        <= maddr_i;
  ocp_m2s.BurstLength <= mburstlength_i;
  ocp_m2s.Data        <= mdata;
  ocp_m2s.DataByteEn  <= mdatabyteen;
  ocp_m2s.DataValid   <= mdatavalid;
  ocp_m2s.RespAccept  <= '0';
  ocp_m2s.Tag         <= (others => '0');
  
  worker_completed <= worker_completed_i;
  
  -- Generate a Power on Reset for FIFOs 
  lrst0 : process (clk)
  begin  -- process lrst0
    if clk'event and clk = '1' then     -- rising clock edge
      local_rst <= '0';
      rst       <= local_rst or cleanup;
      abort1 <= abort;
      abort2 <= abort1;
      cleanup1 <= cleanup;
      if wstate = W_ABORT then
        rst_ofifo <= '1';
      else
        rst_ofifo <= local_rst or cleanup;
      end if;
    end if;
  end process lrst0;
  
  
  -- Small buffer to hold multiple descriptors 
  gen_buf_desc0: if buffer_descriptors generate
    small_sfifo0 : small_sfifo
    generic map (
      width => 107)
    port map (
      clk    => clk,
      rst    => rst,
      wadv   => cmd_fifo_wadv,
      wdata  => cmd_fifo_d,
      nfull  => cmd_fifo_nfull,
      radv   => cmd_fifo_radv,
      rempty => cmd_fifo_empty,
      rdata  => cmd_fifo_q);
    
     cmd_fifo_empty_r2 <= cmd_fifo_empty_r1;
    process (clk) is
    begin  -- process
      if rising_edge(clk) then          -- rising clock edge
        worker_cmd_q <= slv_to_adb3_dma_worker_command(cmd_fifo_q);
        cmd_fifo_empty_r1 <= cmd_fifo_empty;
       
      end if;
    end process;
  
  end generate gen_buf_desc0;
  
  gen_no_buf_desc0: if not buffer_descriptors generate
    worker_cmd_q <= worker_cmd;
    cmd_fifo_empty <= not worker_cmd_valid;
    cmd_fifo_nfull <= '0';
  end generate gen_no_buf_desc0;

  gen_rfs0: if not registering_for_speed generate
    cmd_fifo_d <= adb3_dma_worker_command_to_slv(worker_cmd);
    cmd_fifo_wadv <= worker_cmd_valid;
  end generate gen_rfs0;
  
  
  -- FSM to break descriptor requests into OCP Burst Commands
  fsm0 : process (clk)
    variable short_last_byte : std_logic_vector(5 downto 0);
  begin  -- process fsm0
    if clk'event and clk = '1' then     -- rising clock edge

      if registering_for_speed then
        cmd_fifo_d <= adb3_dma_worker_command_to_slv(worker_cmd);
        cmd_fifo_wadv <= worker_cmd_valid;
        cmd_fifo_nfull_reg    <= cmd_fifo_nfull;
        worker_ready          <= (not cmd_fifo_nfull_reg) and (not state_aborting);
        burst_desc_fifo_nfull_reg <= burst_desc_fifo_nfull;
        burst_desc_fifo_ready <= not burst_desc_fifo_nfull_reg;
      else
        worker_ready          <= (not cmd_fifo_nfull) and (not state_aborting);
        burst_desc_fifo_ready <= not burst_desc_fifo_nfull;
      end if;
      if state = S_ABORT then
        state_aborting <= '1';
      else
        state_aborting <= '0';
      end if;
     
     
      cmd_fifo_radv         <= '0';
      max_burst_reg1         <= max_burst;
      max_burst_reg2         <= max_burst;
      max_burst_reg3         <= max_burst;
      max_burst_reg4         <= max_burst;
      burst_desc_fifo_wadv  <= '0';
      irq_cmd               <= '0';
      byte_write            <= '0';
      last_desc             <= '0';
      xfer_left_reg         <= xfer_left;
      left0_reg             <= left0;
      current_cmd_irq_on_completion_reg <= current_cmd.irq_on_completion;
      current_cmd_last_descriptor_reg <= current_cmd.last_descriptor;

      for i in 1 to 8 loop
        if max_burst_reg1(i) = '1' then
          for j in 8 downto i loop
            first_burst_addr_mask(j) <= '0';
          end loop;  -- j
          for j in i-1 downto 0 loop
            first_burst_addr_mask(j) <= '1';
          end loop;  -- j
        end if;          
      end loop;  -- i
      first_burst_length <= max_burst_reg2 - (first_burst_addr_mask and addr0(13 downto 5));
         
   
      
      case state is
        when IDLE =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if cmd_fifo_empty_r2 = '0' then
              current_cmd   <= worker_cmd_q;
              cmd_fifo_radv <= '1';
              addr0         <= '0' & worker_cmd_q.start_addr(12 downto 0);
              left0         <= '0' & worker_cmd_q.xfer_length(12 downto 0);
              state         <= IS_ADDR_ALIGNED;
              inc_addr <= not worker_cmd_q.fixed_addr;
              short_xfer <= OR_reduce(worker_cmd_q.xfer_length(4 downto 0)) and not OR_reduce(worker_cmd_q.xfer_length(left_bits-1 downto 5));
            end if;
            --run_continuous <= '0';
          end if;
        when IS_ADDR_ALIGNED =>
          short_last_byte := ('0' & addr(4 downto 0))+('0' & left0(4 downto 0));
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if OR_reduce(addr(4 downto 0)) = '1' then
              if (short_last_byte(5) = '0' or short_last_byte = "100000") and short_xfer = '1' then 
                state <= GEN_SHORT_BYTE_WRITE;
              else
                state <= GEN_FIRST_BYTE_WRITE;
              end if;   
            else
              state <= FIRST_BURST_WAIT;
            end if;
            --run_continuous <= not OR_reduce(current_cmd.xfer_length);
          end if;
        when GEN_FIRST_BYTE_WRITE =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if burst_desc_fifo_ready = '1' then
              if inc_addr = '1' then
                addr0    <= (addr0(13 downto 5) & "00000") + CONV_STD_LOGIC_VECTOR(32, 14);
              end if;            
              left0    <= left0_reg - CONV_STD_LOGIC_VECTOR(32, 14) + EXT(addr(4 downto 0), 14);
              size     <= CONV_STD_LOGIC_VECTOR(1, 9);
              for i in 0 to 31 loop
                if addr(4 downto 0) > i then
                  be(i) <= '0';
                else
                  be(i) <= '1';
                end if;
              end loop;  -- i
              state                <= FIRST_BURST_WAIT;
              addr_reg(addr_bits-1 downto 0)             <= addr(addr_bits-1 downto 5) & "00000";
              burst_desc_fifo_wadv <= '1';
              byte_write           <= '1';
            end if;
          end if;
        when GEN_SHORT_BYTE_WRITE =>
          short_last_byte := ('0' & addr(4 downto 0))+('0' & left0(4 downto 0));
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if burst_desc_fifo_ready = '1' then
              left0    <= (others => '0');         
              size     <= CONV_STD_LOGIC_VECTOR(1, 9);
              for i in 0 to 31 loop
                if addr(4 downto 0) > i then
                  be(i) <= '0';
                elsif short_last_byte(5 downto 0) > i then
                  be(i) <= '1';
                else
                  be(i) <= '0';                 
                end if;
              end loop;  -- i
              state                <= GEN_IRQ;
              addr_reg(addr_bits-1 downto 0)             <= addr(addr_bits-1 downto 5) & "00000";
              burst_desc_fifo_wadv <= '1';
              byte_write           <= '1';
            end if;
          end if;
       when FIRST_BURST_WAIT =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            state          <= IS_FIRST_BURST_LEFT;
            if left0(13) = '1' then
              left0(13) <= '0';
            end if;
            if addr0(13) = '1' then
              addr0(13) <= '0';
            end if;
          end if;
       when IS_FIRST_BURST_LEFT =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if addr_valid = '0' or left_valid = '0' then
              if addr0(13) = '1' then
                addr0(13) <= '0';
              end if;
              if left0(13) = '1' then
                left0(13) <= '0';
              end if;
            else
              --if run_continuous = '1' or lots_left = '1' or xfer_left(13 downto 4) >= max_burst_reg1 then
              if lots_left = '1' or xfer_left(13 downto 5) >= first_burst_length then
                state <= GEN_FIRST_BURST;
              else
                state <= IS_SHORT_BURST_LEFT;
              end if;
            end if;
          end if;
        when GEN_FIRST_BURST =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if burst_desc_fifo_ready = '1' then

              if inc_addr = '1' then
                addr0                <= (addr0(13 downto 5) & "00000") + (first_burst_length & "00000");
              end if;
              left0                <= left0_reg - (first_burst_length & "00000");
              size                 <= first_burst_length;
              be                   <= (others => '1');
              state                <= IS_BURST_LEFT;
              addr_reg(addr_bits-1 downto 0)             <= addr(addr_bits-1 downto 0);
              burst_desc_fifo_wadv <= '1';
              byte_write           <= '0';
            end if;
          end if;
        when IS_BURST_LEFT =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if addr_valid = '0' or left_valid = '0' then
              if addr0(13) = '1' then
                addr0(13) <= '0';
              end if;
              if left0(13) = '1' then
                left0(13) <= '0';
              end if;
            else
              --if run_continuous = '1' or lots_left = '1' or xfer_left(13 downto 4) >= max_burst_reg1 then
              if lots_left = '1' or xfer_left(13 downto 5) >= max_burst_reg1 then
                state <= GEN_BURST;
              else
                state <= IS_SHORT_BURST_LEFT;
              end if;
            end if;
          end if;
        when GEN_BURST =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if burst_desc_fifo_ready = '1' then

              if inc_addr = '1' then
                addr0                <= (addr0(13 downto 5) & "00000") + (max_burst_reg2 & "00000");
              end if;
              left0                <= left0_reg - (max_burst_reg3 & "00000");
              size                 <= max_burst_reg4;
              be                   <= (others => '1');
              state                <= IS_BURST_LEFT;
              addr_reg(addr_bits-1 downto 0)             <= addr(addr_bits-1 downto 0);
              burst_desc_fifo_wadv <= '1';
              byte_write           <= '0';
            end if;
          end if;
        when IS_SHORT_BURST_LEFT =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if addr_valid = '0' or left_valid = '0' then
              if addr0(13) = '1' then
                addr0(13) <= '0';
              end if;
              if left0(13) = '1' then
                left0(13) <= '0';
              end if;
            else
              if OR_reduce(xfer_left(13 downto 5)) = '1' then
                state <= GEN_SHORT_BURST;
              else
                state <= ARE_BYTES_LEFT;
              end if;
            end if;
          end if;
        when GEN_SHORT_BURST =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if burst_desc_fifo_ready = '1' then
             if inc_addr = '1' then
                addr0                <= (addr0(13 downto 5) & "00000") + (xfer_left_reg(13 downto 5) & "00000");
              end if;
              left0                <= EXT(left0_reg(4 downto 0), 14);
              size                 <= xfer_left_reg(13 downto 5);
              be                   <= (others => '1');
              state                <= ARE_BYTES_LEFT;
              addr_reg(addr_bits-1 downto 0)             <= addr(addr_bits-1 downto 0);
              burst_desc_fifo_wadv <= '1';
              byte_write           <= '0';
            end if;
          end if;
        when ARE_BYTES_LEFT =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if addr_valid = '0' or left_valid = '0' then
              if addr0(13) = '1' then
                addr0(13) <= '0';
              end if;
              if left0(13) = '1' then
                left0(13) <= '0';
              end if;
            else
              if OR_reduce(xfer_left(4 downto 0)) = '1' then
                state <= GEN_LAST_BYTE_WRITE;
              else
                state <= GEN_IRQ;
              end if;
            end if;
          end if;
        when GEN_LAST_BYTE_WRITE =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if burst_desc_fifo_ready = '1' then
              size     <= CONV_STD_LOGIC_VECTOR(1, 9);
              left0    <= (others => '0');
              for i in 0 to 31 loop
                if xfer_left_reg(4 downto 0) > i then
                  be(i) <= '1';
                else
                  be(i) <= '0';
                end if;
              end loop;  -- i
              state                <= GEN_IRQ;
              addr_reg(addr_bits-1 downto 0)             <= addr(addr_bits-1 downto 0);
              byte_write           <= '1';
              burst_desc_fifo_wadv <= '1';
            end if;
          end if;
        when GEN_IRQ =>
          if abort = '1' then
            state <= S_ABORT;
            left0 <= (others => '-');
          else
            if current_cmd_irq_on_completion_reg = '1' or current_cmd_last_descriptor_reg = '1' then
              if burst_desc_fifo_ready = '1' then
                irq_cmd              <= current_cmd_irq_on_completion_reg;
                last_desc            <= current_cmd_last_descriptor_reg;
                byte_write           <= '0';
                size                 <= (others => '0');
                burst_desc_fifo_wadv <= '1';
                state                <= IDLE;
              end if;
            else
              state <= IDLE;
            end if;
          end if;
        when S_ABORT =>
          if flush_counter = "00" and cmd_fifo_empty_r2 = '0' then
            cmd_fifo_radv <= '1';
          else
            cmd_fifo_radv <= '0';
          end if;
          worker_ready         <= '0';

          burst_desc_fifo_wadv <= '0';
          if cleanup = '1' then
            state <= IDLE;
          end if;
          --run_continuous <= '0';
          left0 <= (others => '0');
        when others =>
          state <= IDLE;
      end case;

      -- Efficiently handle 64 bit arithmetic without 64 bit long carry chains
      if state = IDLE then
        addr1 <= '0' & worker_cmd_q.start_addr(25 downto 13);
        addr2 <= '0' & worker_cmd_q.start_addr(38 downto 26);
        if addr_bits = 64 then
          addr3 <= '0' & worker_cmd_q.start_addr(51 downto 39);
          addr4 <= "00" & worker_cmd_q.start_addr(63 downto 52);
        end if;
        left1 <= '0' & worker_cmd_q.xfer_length(25 downto 13);
        left2 <= '0' & worker_cmd_q.xfer_length(38 downto 26);
      else
        if addr0(13) = '1' then
          addr1 <= addr1+1;
        end if;
        if addr1(13) = '1' then
          addr2 <= addr2+1;
          addr1(13) <= '0';
        end if;
        if addr_bits = 64 then
          if addr2(13) = '1' then
            addr3 <= addr3+1;
            addr2(13) <= '0';
          end if;
          if addr3(13) = '1' then
            addr4 <= addr4+1;
            addr3(13) <= '0';
          end if;
        end if;
        if left0(13) = '1' then
          left1 <= left1-1;
        end if;
        if left1(13) = '1' then
          left2 <= left2-1;
          left1(13) <= '0';
        end if;
        if left_bits = 39 then
          --lots_left <= OR_reduce(left1(12 downto 0)) or OR_reduce(left2(12 downto 0)) or run_continuous;
          lots_left <= OR_reduce(left1(12 downto 0)) or OR_reduce(left2(12 downto 0));
        elsif left_bits = 26 then
          --lots_left <= OR_reduce(left1(12 downto 0)) or run_continuous;
          lots_left <= OR_reduce(left1(12 downto 0));
        else
          lots_left <= '0';
        end if;
      end if;
    end if;
  end process fsm0;


  gen_addr_bits64: if addr_bits = 64 generate
    addr_valid <= not (addr0(13) or addr1(13) or addr2(13) or addr3(13));
    addr       <= addr4(11 downto 0) & addr3(12 downto 0) & addr2(12 downto 0) & addr1(12 downto 0) & addr0(12 downto 0);
  end generate gen_addr_bits64;

  gen_addr_bits39: if addr_bits = 39 generate
    addr_valid <= not (addr0(13) or addr1(13));
    addr       <= EXT(addr2(12 downto 0) & addr1(12 downto 0) & addr0(12 downto 0),64);
  end generate gen_addr_bits39;

  gen_left_bits39: if left_bits = 39 generate
    xfer_left  <= left2(12 downto 0) & left1(12 downto 0) & left0(12 downto 0);
    left_valid <= not (left0(13) or left1(13));
  end generate gen_left_bits39;

  gen_left_bits26: if left_bits = 26 generate
    xfer_left  <= left1(12 downto 0) & left0(12 downto 0);
    left_valid <= not (left0(13));
  end generate gen_left_bits26;

  gen_left_bits14: if left_bits = 14 generate
    xfer_left  <= EXT(left0(12 downto 0),39);
    left_valid <= '1';
  end generate gen_left_bits14;
 

  burst_desc_fifo_d <= last_desc & byte_write & irq_cmd  & '0' & size & be & addr_reg;

  burst_desc_fifo0 : small_sfifo_reg_op
    generic map (
      width => 96)
    port map (
      clk    => clk,
      rst    => rst,
      wadv   => burst_desc_fifo_wadv,
      wdata  => burst_desc_fifo_d(95 downto 0),
      nfull  => open,
      radv   => burst_desc_fifo_radv,
      rempty => open,
      rdata  => burst_desc_fifo_q(95 downto 0));

  burst_desc_fifo1 : small_sfifo_reg_op
    generic map (
      width => 13)
    port map (
      clk    => clk,
      rst    => rst,
      wadv   => burst_desc_fifo_wadv,
      wdata  => burst_desc_fifo_d(108 downto 96),
      nfull  => burst_desc_fifo_nfull,
      radv   => burst_desc_fifo_radv,
      rempty => burst_desc_fifo_empty,
      rdata  => burst_desc_fifo_q(108 downto 96));

  burst_desc_fifo_q_r1 <= burst_desc_fifo_q;
  process (clk) is
  begin  -- process
    if rising_edge(clk) then            -- rising clock edge
      
      burst_desc_fifo_empty_r1 <= burst_desc_fifo_empty or burst_desc_fifo_radv;
      burst_desc_fifo_q_r2 <= burst_desc_fifo_q_r1;
      burst_desc_fifo_empty_r2 <= burst_desc_fifo_empty_r1;
    end if;
  end process;
  

  fl_gt_size <= fl_gt_size_nidle when (OR_reduce(w_burst_left) = '1') else
                fl_gt_size_nidle and fl_gt_size_idle when (wdstate_is_idle_r1='0' or wdstate_is_idle_r2 = '0') and last_state_idle = '1' else
                fl_gt_size_nidle and fl_gt_size_wd_idle when (wdstate_is_idle_r1= '0' or wdstate_is_idle_r2 = '0') and last_state_idle = '0' else
                fl_gt_size_idle when last_state_idle = '1' else
                fl_gt_size_wd_idle;
  
  -- FSM for generating OCP writes form burst descriptions
  fsm1 : process (clk)
    variable next_burst_complete : std_logic := '0';
  begin  -- process fsm1
    if clk'event and clk = '1' then
      -- FIFO Level never checked when fifo_radv = '1' or cycle after
      -- so safe to register

      if fifo_radv_i = '1' then
        fifo_level_reg <= ("0" & (fifo_level-1));
      else
        fifo_level_reg <= ("0" & fifo_level);
      end if;
      
      
      tfifo_flush <= '0';
      abort_dvalid <= '0';

      if fifo_radv_i = '1' and fifo_level_reg > burst_desc_fifo_q_r2(105 downto 96) then
        fl_gt_size_idle <= '1';
      elsif fifo_radv_i = '0' and fifo_level_reg >= burst_desc_fifo_q_r2(105 downto 96) then
        fl_gt_size_idle <= '1';
      else
        fl_gt_size_idle <= '0';
      end if;

      if fifo_radv_i = '1' and fifo_level_reg > w_size then     
        fl_gt_size_wd_idle <= '1';
      elsif fifo_radv_i = '0' and fifo_level_reg >= w_size then     
        fl_gt_size_wd_idle <= '1';  
      else
        fl_gt_size_wd_idle <= '0';
      end if;

      -- Allow 5 extra clock cycles as fifo_level_reg might be out of date
     
      if (wstate = W_IDLE) and (fifo_level_reg >= (burst_desc_fifo_q_r2(105 downto 96) + w_burst_left+5)) and (w_burst_left(9) = '0') then     
        fl_gt_size_nidle <= '1';
      elsif  (wstate /= W_IDLE) and (fifo_level_reg >= (w_size + w_burst_left+5)) and (w_burst_left(9) = '0') then     
        fl_gt_size_nidle <= '1';  
      else
        fl_gt_size_nidle <= '0';
      end if;

      if wstate = W_IDLE then
        last_state_idle <= '1';
      else
        last_state_idle <= '0';
      end if;
      
           
      abort_write_burst_timer <= (others => '0');
      abort_write_burst_timeout <= AND_reduce(abort_write_burst_timer);
                                             
      fifo_radv_ii          <= '0';
      burst_desc_fifo_radv <= '0';
      flush_counter <= flush_counter+1;
      next_burst_complete := burst_complete;
      worker_completed_i <= '0';
      
      case wstate is
        when W_IDLE =>
          if abort1 = '1' then
            wstate <= W_ABORT;
          elsif burst_desc_fifo_empty_r2 = '0' then 
            w_last_desc          <= burst_desc_fifo_q_r2(108);
            
            w_irq_cmd            <= burst_desc_fifo_q_r2(106);
            w_be                 <= burst_desc_fifo_q_r2(95 downto 64);
            w_size               <= burst_desc_fifo_q_r2(105 downto 96);    
--            w_addr(addr_bits-1 downto 0)               <= burst_desc_fifo_q(addr_bits-1 downto 0);
            maddr(addr_bits-1 downto 0) <= burst_desc_fifo_q_r2(addr_bits-1 downto 0);
            --mburstlength <= "00" & burst_desc_fifo_q(89 downto 80);
            if burst_desc_fifo_q_r2(107) = '0' or OR_reduce(w_burst_left) = '0' then
              w_byte_write         <= burst_desc_fifo_q_r2(107);
              burst_desc_fifo_radv <= '1';
              wstate               <= W_READ_BURST_DESC; 
            end if;  
          end if;
          mcmd         <= OCP_CMD_IDLE;
                   
        when W_READ_BURST_DESC =>
          if abort1 = '1' then
            wstate <= W_ABORT;
          else
            if w_irq_cmd = '1' then
              if OR_reduce(w_burst_left) = '0' then
                wstate <= W_GEN_IRQ;
              end if;              
            elsif w_last_desc = '1' then
              if OR_reduce(w_burst_left) = '0' and tfifo_empty = '1' then
                wstate <= W_FLUSH;
              end if;            
            elsif fifo_empty = '0' and fl_gt_size = '1' and tfifo_nfull = '0' and mcmd_fifo_nfull = '0' then         
              wstate       <= W_GEN_WRITE_CMD;
              mcmd         <= OCP_CMD_WRITE;
              
             
--              maddr(addr_bits-1 downto 0)        <= w_addr(addr_bits-1 downto 0);
              mburstlength <= "00" & w_size;
              
            end if;
          end if;
        when W_GEN_IRQ =>
          if abort1 = '1' then
            wstate <= W_ABORT;
          else
            if w_last_desc = '1' then
              if tfifo_empty = '1' then
                wstate <= W_FLUSH;
              end if;
            else
              wstate <= W_IDLE;
            end if;
          end if;
        when W_FLUSH =>
          if abort1 = '1' then
            wstate <= W_ABORT;
          else
            -- Clean up potentially problematic last few bytes in FIFO     
            if fifo_empty = '1' then            
              wstate <= W_IDLE;
              worker_completed_i <= '1';
            else
              if flush_counter = "00" and fifo_empty = '0'then
                fifo_radv_ii <= '1';
              end if;               
            end if;
          end if;
        when W_GEN_WRITE_CMD =>
          if mcmd_accept_i = '1' then
            mcmd <= OCP_CMD_IDLE;
            wstate <= W_IDLE;
          end if;
        
          if abort1 = '1' then
             wstate <= W_ABORT;
          end if;               
        when W_ABORT =>
          abort_dvalid <= '1';
          mcmd         <= OCP_CMD_IDLE;
          
          
          tfifo_flush <= not tfifo_empty and not tfifo_flush and not ocp_s2m.DataAccept;
          if flush_counter = "00" and fifo_empty = '0'then
            fifo_radv_ii <= '1';
          end if;
          if burst_desc_fifo_empty_r2 = '0' and flush_counter = "00" then
            burst_desc_fifo_radv <= '1';
          end if;
          if cleanup1 = '1' then
            wstate <= W_IDLE;
          end if;
        when others =>
          wstate <= W_IDLE;
      end case;


      if wstate = W_ABORT then
        w_burst_left <= (others => '0');
      else
        if OR_reduce(w_burst_left) = '1' then 
          if wstate = W_GEN_WRITE_CMD and mcmd_accept_i = '1' then
            w_burst_left <= w_burst_left+w_size-1;
          else
            w_burst_left <= w_burst_left-1;
          end if;
        else
          if wstate = W_GEN_WRITE_CMD and mcmd_accept_i = '1' then
            w_burst_left <= w_burst_left+w_size;
          end if;
        end if;
      end if;
      if OR_reduce(w_burst_left) = '0' then
        if wstate = W_GEN_WRITE_CMD and mcmd_accept_i = '1' then
          first_word <= w_byte_write;
        end if;
      else
        first_word <= '0';      
      end if;

         
      if rst = '1' then
        tfifo_level <= "0000000";
      else
        if tfifo_radv= '0' then
          if mcmd_fifo_wadv_r1 = '1' then 
            tfifo_level <= tfifo_level + mburstlength_r1(6 downto 0);
          end if;
        else
          if mcmd_fifo_wadv_r1 = '1' then 
            tfifo_level <= tfifo_level + mburstlength_r1_m1(6 downto 0);
          else
            tfifo_level <= tfifo_level - 1;
          end if;         
        end if;
      end if;
      tfifo_nfull <= tfifo_level(6) or tfifo_level(5);
     
      mburstlength_r1 <= mburstlength;
      mburstlength_r1_m1 <= mburstlength-1;
      mcmd_fifo_wadv_r1 <= mcmd_fifo_wadv;

      
      
      
      tdata <= fifo_data;
      tdatavalid <= OR_reduce(w_burst_left);
      if first_word = '1' then
        tdatabyteen <= w_be;
      else
        tdatabyteen <= (others => '1');
      end if;
      
  
      if cleanup1 = '1' then
        irq_state <= "000";
      elsif irq_ack = '1' and irq_state /= "000" then
        irq_state <= irq_state -1;
      elsif wstate = W_GEN_IRQ and (w_last_desc = '0' or tfifo_empty = '1') then
        irq_state <= irq_state+1;
      end if;

      tfifo_nfull_r1 <= tfifo_nfull;
      fifo_radv_iii <= fifo_radv_ii;
     
      if w_byte_write = '1' and w_be(31) = '0' then
        fifo_radv_q <= '0';
      else
        fifo_radv_q <= '1';
      end if;
      
    end if;
  end process fsm1;


  tdata_wide <= tdatabyteen & tdata;

  fifo_radv_i <= fifo_radv_q when OR_reduce(w_burst_left) = '1' else fifo_radv_iii;

  gen_light_tdata_wide: if not registering_for_speed generate
    tdatavalid_r1 <= tdatavalid;
    tdata_wide_r1 <= tdata_wide;
  end generate gen_light_tdata_wide;

  
  gen_fast_tdata_wide: if registering_for_speed generate
    process (clk) is
    begin  -- process
      if rising_edge(clk) then          -- rising clock edge
        tdatavalid_r1 <= tdatavalid;
        tdata_wide_r1 <= tdata_wide;
      end if;
    end process;
  end generate gen_fast_tdata_wide;
                 
  
  small_sfifo2 : small_sfifo64
    generic map (
      width => ADB3W_OCP_DATA_WIDTH+ADB3W_OCP_DATA_WIDTH/8)
    port map (
      clk    => clk,
      rst    => rst,
      wadv   => tdatavalid_r1,
      wdata  => tdata_wide_r1,
      nfull  => open, --tfifo_nfull,
      radv   => tfifo_radv,
      rempty => tfifo_empty,
      rdata  => mdata_wide);


  mdata <= mdata_wide(ADB3W_OCP_DATA_WIDTH-1 downto 0);
  mdatabyteen <= mdata_wide(ADB3W_OCP_DATA_WIDTH+ADB3W_OCP_DATA_WIDTH/8-1 downto ADB3W_OCP_DATA_WIDTH);
  mdatavalid <= (not tfifo_empty);
  tfifo_radv <= tfifo_flush or (ocp_s2m.DataAccept and not tfifo_empty and not abort_dvalid);

  small_sfifo3 : small_sfifo
    generic map (
      width => ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH)
    port map (
      clk    => clk,
      rst    => rst_ofifo,
      wadv   => mcmd_fifo_wadv,
      wdata  => mcmd_fifo_in,
      nfull  => mcmd_fifo_nfull,
      radv   => mcmd_fifo_radv,
      rempty => mcmd_fifo_empty,
      rdata  => mcmd_fifo_out);

  mcmd_fifo_wadv <= '1' when mcmd = OCP_CMD_WRITE and mcmd_fifo_nfull = '0' else '0';
  mcmd_accept_i <= not mcmd_fifo_nfull;
  
  mcmd_i <= OCP_CMD_WRITE when mcmd_fifo_empty = '0' else OCP_CMD_IDLE;
  mcmd_fifo_radv <= ocp_s2m.CmdAccept and not mcmd_fifo_empty;
  mcmd_fifo_in <= mburstlength & maddr;
  mburstlength_i <= mcmd_fifo_out(ADB3_OCP_ADDR_WIDTH + ADB3_OCP_BURST_WIDTH-1 downto ADB3_OCP_ADDR_WIDTH);
  maddr_i <= mcmd_fifo_out(ADB3_OCP_ADDR_WIDTH-1 downto 0);
  
  
  
  
  
  
  irq       <= irq_out;
  fifo_radv <= fifo_radv_i;


  status_out0 : process (clk)
  begin  -- process status_out0
    if clk'event and clk = '1' then     -- rising clock edge
      if state = IDLE then
        fsm_idle(0) <= cmd_fifo_empty;
      else
        fsm_idle(0) <= '0';
      end if;
      if state = S_ABORT then
        fsm_aborting(0) <= '1';
      else
        fsm_aborting(0) <= '0';
      end if;

      if wstate = W_IDLE then
        fsm_idle(1) <= '1';
      else
        fsm_idle(1) <= '0';
      end if;
      if wstate = W_ABORT then
        fsm_aborting(1) <= '1';
      else
        fsm_aborting(1) <= '0';
      end if; 
      
    end if;
  end process status_out0;

  irq_out <= '0' when irq_state = "000" else '1';

  
end rtl;

