--
-- adb3_register_interface32_lite
-- Component of ADB3_OCP Library
--
-- (C) 2009 Alpha Data Ltd.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;

entity adb3_ocp_register_interface32 is
  generic (
    number_of_registers : natural := 1);
  port (
    -- OCP Interface
    ocp_clk : in std_logic;
    ocp_m2s : in adb3_ocp_m2sT;
    ocp_s2m : out adb3_ocp_s2mT;
    -- simple register interface
    registers_out : out register32_array(0 to number_of_registers-1);
    registers_back : in register32_array(0 to number_of_registers-1);
    register_written : out std_logic_vector(number_of_registers-1 downto 0) 

    );

end adb3_ocp_register_interface32;

architecture rtl of adb3_ocp_register_interface32 is

  signal writing : std_logic := '0';
  signal reading : std_logic := '0';
  signal addr    : std_logic_vector(3 downto 0);  

  signal tag : std_logic_vector(ADB3_OCP_TAG_WIDTH-1 downto 0);
  
  signal SCmdAccept : std_logic := '0';
  signal SDataAccept : std_logic := '0';

  signal rdata_valid : std_logic := '0';
  signal rdata_reg : std_logic_vector(127 downto 0);
  
begin  -- rtl

  assert number_of_registers < 65 report "Too many registers specified - use a memory" severity error;
  
  process (ocp_clk)
    variable index_slv : std_logic_vector(5 downto 0);
    variable sub_index : integer;
  begin  -- process
    if ocp_clk'event and ocp_clk = '1' then  -- rising clock edge
      SCmdAccept <= '0';
      SDataAccept <= '0';         
      register_written <= (others => '0');
        
      if writing = '0' and reading = '0' then
        if ocp_m2s.Cmd = OCP_CMD_WRITE then
          SCmdAccept <= '1';
          addr <= ocp_m2s.Addr(7 downto 4);
          writing <= '1';
        end if;
        
        if ocp_m2s.Cmd = OCP_CMD_READ then
          SCmdAccept <= '1';
          addr <= ocp_m2s.Addr(7 downto 4);
          reading <= '1';        
          tag <= ocp_m2s.Tag;
        end if;      
      elsif writing = '1' then 
        if ocp_m2s.DataValid = '1' then
          for i in 0 to number_of_registers-1 loop
            index_slv := CONV_STD_LOGIC_VECTOR(i,6);
            sub_index := i mod 4;
            if addr = index_slv(5 downto 2) then
              register_written(i) <= OR_reduce(ocp_m2s.DataByteEn(sub_index*4+3 downto sub_index*4));
              for j in 0 to 3 loop
                if ocp_m2s.DataByteEn(sub_index*4+j) = '1' then
                  registers_out(i)(8*j+7 downto 8*j) <= ocp_m2s.Data(32*sub_index+8*j+7 downto 32*sub_index+8*j);                                
                end if;              
              end loop;  -- j             
            end if;          
          end loop;  -- i     
          SDataAccept <= '1';
          writing <= '0';
        end if;           
      elsif reading = '1' then
        rdata_valid <= '1';
        rdata_reg <= X"55AADEAD_55AADEAD_55AADEAD_55AADEAD";
        for i in 0 to number_of_registers-1 loop
          index_slv := CONV_STD_LOGIC_VECTOR(i,6);
          sub_index := i mod 4;
          if addr = index_slv(5 downto 2) then
            rdata_reg(32*sub_index+31 downto 32*sub_index) <= registers_back(i);    
          end if;          
        end loop;
        
        if ocp_m2s.RespAccept = '1' and rdata_valid = '1' then
          rdata_valid <= '0'; 
          reading <= '0';
        end if;  
      end if;
    end if;
  end process;

--   gen_rdata: process (registers_back,addr,reading)
--     variable index_slv : std_logic_vector(5 downto 0);
--     variable sub_index : integer;
--   begin  -- process gen_rdata
--      rdata_valid <= reading;
--      rdata_reg <= X"55AADEAD_55AADEAD_55AADEAD_55AADEAD";
--      for i in 0 to number_of_registers-1 loop
--        index_slv := CONV_STD_LOGIC_VECTOR(i,6);
--        sub_index := i mod 4;
--        if addr = index_slv(5 downto 2) then
--          rdata_reg(32*sub_index+31 downto 32*sub_index) <= registers_back(i);    
--        end if;          
--      end loop;
--   end process gen_rdata;

  
  ocp_s2m.CmdAccept <= SCmdAccept;
  ocp_s2m.DataAccept <= SDataAccept;
  ocp_s2m.Tag <= tag;
  ocp_s2m.Data <= rdata_reg;
  ocp_s2m.Resp <= OCP_RESP_NONE when rdata_valid = '0' else OCP_RESP_VALID;
  
end rtl;
