--
-- adb3_ocp_write_fifo_interface
-- Component of ADB3_OCP Library
--
-- (C) 2010 Alpha Data Ltd.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;

entity adb3_ocp_w_byteen_write_fifo_interface256 is
  port (
    -- OCP Interface
    ocp_clk : in std_logic;
    ocp_m2s : in adb3w_ocp_m2sT;
    ocp_s2m : out adb3w_ocp_s2mT;
    -- FIFO Interface
    d   : out std_logic_vector(255 downto 0);
    dv  : out std_Logic;
    pop : in  std_logic);

end adb3_ocp_w_byteen_write_fifo_interface256;

architecture rtl of adb3_ocp_w_byteen_write_fifo_interface256 is

  component small_sfifo
    generic (
      width : integer := 64);
    port (
      clk    : in  std_logic;
      rst    : in  std_logic;
      wadv   : in  std_logic;
      wdata  : in  std_logic_vector(width-1 downto 0);
      nfull  : out std_logic;
      radv   : in  std_logic;
      rempty : out std_logic;
      rdata  : out std_logic_vector(width-1 downto 0));
  end component;

  signal wadv : std_logic_vector(31 downto 0);
  signal nfull : std_logic_vector(31 downto 0);
  signal rempty : std_logic_vector(31 downto 0);

  signal nfull_reg : std_logic := '0';
  signal data_reg : std_logic_vector(255 downto 0);
begin  -- rtl


  gen_fifos: for i in 0 to 31 generate
    small_sfifo_i : small_sfifo
      generic map (
        width => 8)
      port map (
        clk    => ocp_clk,
        rst    => '0',
        wadv   => wadv(i),
        wdata  => data_reg(8*i+7 downto 8*i),
        nfull  => nfull(i),
        radv   => pop,
        rempty => rempty(i),
        rdata  => d(8*i+7 downto 8*i));
    
  end generate gen_fifos;

  -- Reg nfull (4 spaces left in FIFO) to ease timing issues 
  reg_nfulls: process (ocp_clk) is
  begin  -- process reg_nfulls
    if rising_edge(ocp_clk) then        -- rising clock edge
      nfull_reg <= OR_reduce(nfull);
      if ocp_m2s.DataValid = '1' and nfull_reg = '0' then
         wadv <= ocp_m2s.DataByteEn;
      else
         wadv <= (others => '0');
      end if;
      data_reg <= ocp_m2s.Data;
    end if;
  end process reg_nfulls;
  
      
  dv <= not rempty(31);
  -- Only using byte 15 should improve timing, and should always be written
  -- at same time or after all other bytes, alternatively use:
  -- dv <= not OR_reduce(rempty);
  
  ocp_s2m.CmdAccept  <= not nfull_reg when ocp_m2s.Cmd = OCP_CMD_WRITE else '0';
  ocp_s2m.DataAccept <= ocp_m2s.DataValid and not nfull_reg;
  ocp_s2m.Tag        <= (others => '-');
  ocp_s2m.Data       <= (others => '-');
  ocp_s2m.Resp       <= OCP_RESP_NONE;

end rtl;
