--------------------------------------------------------------------------------
--
-- Module Name      : adb3_target_model
--
--                    Copyright 2009 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : 
--
--
-- Dependencies     : adb3_ocp, today
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.STD_LOGIC_MISC.all;

library work;
use work.adb3_ocp.all;
use work.today.all;
use work.pcie_config.all;

library unisim;
use unisim.vcomponents.all;

---------------------------------------------------------------
entity adm_xrc_ku1b_model is
  port (
    ocp_clk              : in    std_logic;
    model_regs_m2s       : in    adb3_ocp_m2sT;
    model_regs_s2m       : out   adb3_ocp_s2mT;
    model_regs_irq       : out   std_logic;
    model_regs_reset     : in    std_logic;
    model_clk_out        : out   std_logic;
    model_clk_locked     : out   std_logic;
    linti_n              : in    std_logic;
    vpd_in            : in    std_logic_vector(47 downto 0));
end adm_xrc_ku1b_model;


---------------------------------------------------------------
architecture rtl of adm_xrc_ku1b_model is

  constant gen_iobs   : boolean := true;
  constant dont_care  : std_logic := '0';
  constant x32 : std_logic_vector(31 downto 0) := (others => dont_care );
  --
  signal ocp_ref_clk_int  : std_logic;
  signal lclk             : std_logic;

  --                      
  signal rst_lclk         : std_logic := '1';
  --
  signal model_regs_m2s_lclk : adb3_ocp_m2sT := ADB3_OCP_DEFAULT_M2S;
  signal model_regs_s2m_lclk : adb3_ocp_s2mT := ADB3_OCP_DEFAULT_S2M;
  --
  signal model_regs_s2m_i    : adb3_ocp_s2mT := ADB3_OCP_DEFAULT_S2M;
  --
  signal addr           : std_logic_vector( 7 downto 4)  := (others => '0');
  signal bank           : std_logic_vector(11 downto 8)  := (others => '0');
  signal re             : std_logic                      := '0';
  signal re_1d          : std_logic                      := '0';
  signal re_2d          : std_logic                      := '0';
  signal re_3d          : std_logic                      := '0';
  signal tag            : std_logic_vector(7 downto 0)   := (others => '0');
  --
  signal we             : std_logic                      := '0';
  --
  signal write          : std_logic                      := '0';
  signal m2sData        : std_logic_vector(127 downto 0) := (others => '0');
  signal m2sBE          : std_logic_vector( 15 downto 0) := (others => '0');
  --
   signal pcbrev            : std_logic_vector(31 downto 0);
  
  
  signal write_lint     : std_logic := '0';

  --
  --
  signal status_power      : std_logic_vector(31 downto 0);
  --
 
  --
  signal status_lint       : std_logic_vector(63 downto 0);
  
  signal address  : std_logic_vector( 13 downto 4 );

  -- buses for the DP BRAM
  signal iicbuf_clk : std_logic;
  signal iicbuf_addra     : std_logic_vector(15 downto 0) := (others => '0');
  signal iicbuf_addrb     : std_logic_vector(15 downto 0) := (others => '0');
  signal iicbuf_dia       : std_logic_vector(31 downto 0) := (others => '0');
  signal iicbuf_q       : std_logic_vector(127 downto 0) := (others => '0');
  
  subtype ram_wea_bus     is std_logic_vector(3 downto 0);
  type    ram_wea_ar_T    is array (0 to 3) of ram_wea_bus;
  signal  ram_wea_ar      : ram_wea_ar_T;
  
-----------------------------------------------------------
begin  -- rtl

   lclk <= ocp_clk;
   model_regs_m2s_lclk <= model_regs_m2s;
   model_regs_s2m <= model_regs_s2m_lclk;

   rst_lclk <= model_regs_reset;

   model_clk_out <= lclk;

   process (lclk) is
   begin  -- process
     if rising_edge(lclk) then          -- rising clock edge
       model_clk_locked <= not model_regs_reset;
     end if;
   end process;
   
   -------------------------------------------------------------------------
   -- register interface for model_regs_m2s
   -------------------------------------------------------------------------

  model_regs_s2m_lclk.CmdAccept  <= model_regs_s2m_i.CmdAccept;
  model_regs_s2m_lclk.DataAccept <= model_regs_s2m_i.DataAccept;
  model_regs_s2m_lclk.Data       <= model_regs_s2m_i.Data;
  model_regs_s2m_lclk.Resp       <= model_regs_s2m_i.Resp;
  model_regs_s2m_lclk.Tag        <= model_regs_s2m_i.Tag;
  
  
  

   -------------------------------------------
   regs_p0 : process (lclk)
   -------------------------------------------
      variable active_addr  : std_logic_vector(3 downto 0);
      variable active_bank  : std_logic_vector(3 downto 0);
      variable write        : std_logic;
      
   begin  -- process general_regs_p0
      if rising_edge(lclk) then
        
         re_3d <= re_2d;
         re_2d <= re_1d;
         re_1d <= re;    -- delay to allow BRAM reads
         
         if (model_regs_m2s_lclk.Cmd = OCP_CMD_WRITE) and 
               (model_regs_s2m_i.CmdAccept = '0') and 
               (we = '0') then
            active_addr                := model_regs_m2s_lclk.Addr(7 downto 4);
            addr                       <= model_regs_m2s_lclk.Addr(7 downto 4);
            active_bank                := model_regs_m2s_lclk.Addr(11 downto 8);
            bank                       <= model_regs_m2s_lclk.Addr(11 downto 8);
            address                    <= model_regs_m2s_lclk.Addr(13 downto 4);
            model_regs_s2m_i.CmdAccept <= '1';
            re                         <= '0';
            tag                        <= tag;
            
         elsif (model_regs_m2s_lclk.Cmd = OCP_CMD_READ) and 
                  (model_regs_s2m_i.CmdAccept = '0') and (we = '0') then
            active_addr                := model_regs_m2s_lclk.Addr(7 downto 4);
            addr                       <= model_regs_m2s_lclk.Addr(7 downto 4);
            active_bank                := model_regs_m2s_lclk.Addr(11 downto 8);
            bank                       <= model_regs_m2s_lclk.Addr(11 downto 8);
            address                    <= model_regs_m2s_lclk.Addr(13 downto 4);
            model_regs_s2m_i.CmdAccept <= '1';
            re                         <= '1';
            tag                        <= model_regs_m2s_lclk.Tag;
            
         else
            address                    <= address;
            active_addr                := addr;
            addr                       <= addr;
            active_bank                := bank;
            bank                       <= bank;
            model_regs_s2m_i.CmdAccept <= '0';
            re                         <= '0';
            tag                        <= tag;
         end if;
         
      
         if (model_regs_m2s_lclk.Cmd = OCP_CMD_WRITE) and 
               (model_regs_s2m_i.CmdAccept = '0') and (we = '0') then
            we <= '1';
         elsif (model_regs_m2s_lclk.DataValid = '1') and (we = '1') then
            we <= '0';
         else
            we <= we;
         end if;
         
         
         if (model_regs_m2s_lclk.DataValid = '1') and (we = '1') then
            model_regs_s2m_i.DataAccept <= '1';
            if ( model_regs_m2s_lclk.Addr( 13 downto 12 ) = "00" ) then
              write       := '1';
            else
              write       := '0';
            end if;
            m2sData     <= model_regs_m2s_lclk.Data(127 downto 0);
            m2sBE       <= model_regs_m2s_lclk.DataByteEn(15 downto 0);
         else
            model_regs_s2m_i.DataAccept <= '0';
            write       := '0';
            m2sData     <= m2sData;
            m2sBE       <= m2sBE;
         end if;


         -- Map Register Write enables
         -- Bank 0
         
         -- defaults       
        
         write_lint    <= '0';
                
         if ( write = '1' ) then
            case active_bank is
               when x"0"   => case active_addr is
                                 when x"0"   => null;
                                 when x"1"   => null;
                                 when x"2"   => null;
                                 when x"3"   => write_lint    <= '1';
                                 when others => null;
                              end case;   -- active_addr
                              
               when others => null;                          
            end case;   -- active_bank
               
         end if;  -- write
         
      end if;  -- lclk
   end process regs_p0;
         
      
   -------------------------------------------
   read_regs_proc : process(lclk)
   -------------------------------------------
      variable regs        : std_logic_vector(127 downto 0);
      variable regs0       : std_logic_vector(127 downto 0);
      variable regs1       : std_logic_vector(127 downto 0);
      variable regs2       : std_logic_vector(127 downto 0);
      variable regs3       : std_logic_vector(127 downto 0);
      variable regs4       : std_logic_vector(127 downto 0);
      variable regs5       : std_logic_vector(127 downto 0);
      variable regs6       : std_logic_vector(127 downto 0);
      variable regs7       : std_logic_vector(127 downto 0);
  begin    -- process read_regs_proc
    if rising_edge( lclk ) then
      if address(11 downto 8) = "0000" then        
        case address( 7 downto 4 ) is
          when x"0" =>
            regs := TODAYS_DATE & x32 & x32;
          when x"1" =>
            regs := x32 & x32 & x32 & x32;
          when x"2" =>
            --regs := serial_no & x32 & x32 & x32;
            regs := x32 & x32 & x32 & x32;
          when x"3" =>
            regs := x32 & x32 & status_lint;
          when others =>            
            regs := ( others => dont_care );
        end case;
      elsif address(11 downto 9) = "011" or address(11 downto 9) = "100" then
        regs := iicbuf_q;
      else
        regs := x32 & x32 & x32 & x32;
      end if;
        if (re_2d = '1') and (re_3d = '0') then
           model_regs_s2m_i.Resp <= "01";
           model_regs_s2m_i.Data <= regs;
           model_regs_s2m_i.Tag  <= tag;
        elsif model_regs_m2s_lclk.RespAccept = '1' then
           model_regs_s2m_i.Resp <= "00";
        end if;
      end if;  -- lclk
   end process read_regs_proc;

 
     -------------------------------------------
   fpga_int_0 : entity work.fpga_int_adb3
   -------------------------------------------
    generic map (
      NUMBER_OF_INTERRUPTS => 2,
      LEVEL_SENSITIVE_DEFAULTS => X"0000",
      ACTIVE_LOW => X"0003")
    port map(
      rst            => rst_lclk,
      lclk           => lclk,
      write          => write_lint,
      be             => m2sBE(7 downto 0),
      data           => m2sData(63 downto 0),
      status         => status_lint,
      int_local      => model_regs_irq,
      interrupts(0)  => linti_n,
      interrupts(1)  => '1');    -- fpga_int_0
  

   

  iicbuf_clk <= vpd_in(0);
  
  iicbuf_addra <= vpd_in(31 downto 16);   
  iicbuf_dia <= vpd_in(15 downto 8) & vpd_in(15 downto 8) & vpd_in(15 downto 8) & vpd_in(15 downto 8);
   ram_wea_ar(0) <= vpd_in(35 downto 32);
   ram_wea_ar(1) <= vpd_in(39 downto 36);
   ram_wea_ar(2) <= vpd_in(43 downto 40);
   ram_wea_ar(3) <= vpd_in(47 downto 44);

  
   iicbuf_addrb  <= x"0" & bank(10 downto 8) & addr & "00000";
     ------------------------------------
  iicbuf_gen : for i in 0 to 3 generate
    ------------------------------------
    
    ------------------------------------
    RAMB36_inst : RAMB36
      ------------------------------------
      generic map (
        DOA_REG             => 0,
        DOB_REG             => 0,
        INIT_A              => X"000000000",
        INIT_B              => X"000000000",
        RAM_EXTENSION_A     => "NONE",
        RAM_EXTENSION_B     => "NONE",
        READ_WIDTH_A        => 9,
        READ_WIDTH_B        => 36,
        SIM_COLLISION_CHECK => "ALL",
        SIM_MODE            => "SAFE",
        SRVAL_A             => X"000000000",
        SRVAL_B             => X"000000000",
        WRITE_MODE_A        => "WRITE_FIRST",
        WRITE_MODE_B        => "WRITE_FIRST",
        WRITE_WIDTH_A       => 9,
        WRITE_WIDTH_B       => 36
        )
      port map (
        CASCADEOUTLATA      => open,
        CASCADEOUTLATB      => open,
        CASCADEOUTREGA      => open,
        CASCADEOUTREGB      => open,
        DOA                 => open,
        DOB                 => iicbuf_q(32*i+31 downto 32*i),
        DOPA                => open,
        DOPB                => open,
        ADDRA               => iicbuf_addra, -- 16-bit A port address input
        ADDRB               => iicbuf_addrb, -- 16-bit B port address input
        CASCADEINLATA       => '0',                       
        CASCADEINLATB       => '0',
        CASCADEINREGA       => '0',
        CASCADEINREGB       => '0',
        CLKA                => iicbuf_clk,
        CLKB                => lclk,
        DIA                 => iicbuf_dia,
        DIB                 => x"00000000",
        DIPA                => x"0",
        DIPB                => x"0",
        ENA                 => '1',
        ENB                 => '1',
        REGCEA              => '1',
        REGCEB              => '1',
        SSRA                => rst_lclk,
        SSRB                => rst_lclk,
        WEA                 => ram_wea_ar(i),
        WEB                 => x"0"
        );
  end generate;  -- iicbuf_gen
 
   
end rtl;   
