--
-- Board Control component definitions package
--
-- SYNTHESIZABLE
--
-- Targets following variants:
--
--   o ADB3-XRC6TL
--   o ADB3-XRC6T1
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

library work;
use work.adb3_model_defs.all;
use work.adb3_board_ctrl_defs.all;


------------------------------------------------------------------
package adb3_board_ctrl is



   --------------------------------------------------------
   component pulse_sync2 is
   --------------------------------------------------------
      port(
         ar : in  std_logic;
         ik : in  std_logic;
         i  : in  std_logic;
         ok : in  std_logic;
         o  : out std_logic
      );
   end component; -- pulse_sync2


   --------------------------------------------------------
   component model_clocks_v5 is
      generic (
         C_FAMILY       : string := "virtex5"
      );                 
      port (
         rclk_p          : in  std_logic;
         rclk_n          : in  std_logic;
         --
         lclk_locked     : out std_logic;
         --
         rclk            : out std_logic;
         lclk            : out std_logic;
         lclk270         : out std_logic;
         lclk2x          : out std_logic;
         lclkEn400k      : out std_logic;
         lclkEn1pps      : out std_logic
      );
   end component; -- model_clocks_v5

   --------------------------------------------------------
   component model_clocks_v6 is
      generic (
         C_FAMILY       : string := "virtex6";
         OCPCLKDIV      : integer := 5
      );                 
      port (
         rclk_p          : in  std_logic;
         rclk_n          : in  std_logic;
         --
         lclk_locked     : out std_logic;
         --
         rclk            : out std_logic;
         ocp_ref_clk     : out std_logic;
         lclk            : out std_logic;
         lclk270         : out std_logic;
         lclk2x          : out std_logic;
         lclkEn400k      : out std_logic;
         lclkEn1pps      : out std_logic
      );
   end component; -- model_clocks_v6


   --------------------------------------------------------
   component progclk_fsm is
        generic (
           C_FAMILY    : string := "virtex4"
        );
        port (
           Clk           : in    std_logic;
           Rst           : in    std_logic;
           Start         : in    std_logic;
           TOCtr_Cy      : in    std_logic;
           DCM_Locked    : in    std_logic;
           DCM_DRDY      : in    std_logic;
           Match         : in    std_logic;
           Busy          : out   std_logic;
           TOCtr_En      : out   std_logic;
           DCM_Rst       : out   std_logic;
           DCM_DEN       : out   std_logic;
           DCM_WE        : out   std_logic;
           SelDiv        : out   std_logic;
           RdDCMStat     : out   std_logic;
           RdLatchEn     : out   std_logic;
           Timeout       : out   std_logic;
           Error         : out   std_logic;
           ClrStat       : out   std_logic;
           ToggleCkSel   : out   std_logic
        );
   end component;   -- progclk_fsm
    
   --------------------------------------------------------
   component progclk_mux
       generic (
           C_FAMILY       : string := "virtex4"
       );                 
       port (             
          rst             : in     std_logic;
          clk             : in     std_logic;
          rclk            : in     std_logic;
          write           : in     std_logic;
          be              : in     std_logic_vector(  3 downto 0);
          data            : in     std_logic_vector( 31 downto 0);
          status          : out    std_logic_vector(127 downto 0);
          --
          reboot_en_i     : in    std_logic;
          --              
          Clk0BootVal     : in     std_logic_vector(31 downto 0);
          Clk0BootVd      : in     std_logic;
          --              
          vclk_o_locked   : out    std_logic;
          vclk_o          : out    std_logic_vector(adb3_num_vclk_o -1 downto 0)
       );
   end component;   -- progclk_mux
     
   --------------------------------------------------------
   component smb_if is
     port (
           -- Clocks and resets
           clk            : in   std_logic;
           clkEn400k      : in  std_logic;
           rst            : in   std_logic;
           -- Control Interface
           SMB_Adr        : in   std_logic_vector(6 downto 0);
           Start          : in   std_logic;
           WrN            : in   std_logic;
           Adr            : in   std_logic_vector(7 downto 0);
           WrDa           : in   std_logic_vector(7 downto 0);
           RdDa           : out  std_logic_vector(7 downto 0);
           Ready          : out  std_logic;
           SMB_Error      : out  std_logic;
           -- SMB Interface
           SMBCLK_o       : out  std_logic;
           SMBCLK_t       : out  std_logic;
           SMBDATA_o      : out  std_logic;
           SMBDATA_t      : out  std_logic;
           SMBDATA_i      : in   std_logic;
           SMB_ALERT_L    : in   std_logic
     );
   end component;   -- smb_if

   --------------------------------------------------------
   component lm87if_fsm is
     port (
           -- Clocks and resets
           clk               : in  std_logic;
           rst               : in  std_logic;
           
           -- Inputs
           TempFlagVd        : in  std_logic;
           LastInit          : in  std_logic;
           CtlReg_NewCmd     : in  std_logic;
           MemCopy           : in  std_logic;
           LastOp            : in  std_logic;
           SMB_IF_Rdy        : in  std_logic;
           SMB_IF_Error      : in  std_logic;
           reboot_en_i       : in  std_logic;
           -- Outputs
           InitPhase         : out std_logic_vector(1 downto 0);
           InitCompleted     : out std_logic;
           NextInitAdr       : out std_logic;
           SMB_IF_Start      : out std_logic;
           ClearAdrCnt       : out std_logic;
           NextAdr           : out std_logic;
           CmdEnd            : out std_logic;
           Timeout           : out std_logic
     );
   end component;   -- lm87if_fsm

     --------------------------------------------------------
   component lm87if_fsm_xrc6tge is
     port (
           -- Clocks and resets
           clk               : in  std_logic;
           rst               : in  std_logic;
           
           -- Inputs
           TempFlagVd        : in  std_logic;
           LastInit          : in  std_logic;
           CtlReg_NewCmd     : in  std_logic;
           MemCopy           : in  std_logic;
           LastOp            : in  std_logic;
           SMB_IF_Rdy        : in  std_logic;
           SMB_IF_Error      : in  std_logic;
           reboot_en_i       : in  std_logic;
           SI5338_Completed  : in  std_logic;
           -- Outputs
           InitPhase         : out std_logic_vector(1 downto 0);
           InitCompleted     : out std_logic;
           NextInitAdr       : out std_logic;
           SMB_IF_Start      : out std_logic;
           ClearAdrCnt       : out std_logic;
           NextAdr           : out std_logic;
           CmdEnd            : out std_logic;
           Timeout           : out std_logic;
           SI5338_CanStart   : out std_logic
     );
   end component;   -- lm87if_fsm

   --------------------------------------------------------
   component iic_arb is
      port (
         -- clocks and resets
         clk                 : in  std_logic;
         clkEn400k           : in  std_logic;
         clkEn1pps           : in  std_logic;
         rst                 : in  std_logic;
         -- to/from Host Interface
         HostNewCmd          : in  std_logic;
         HostMemCopy         : in  std_logic;
         LM87Selected        : in  std_logic;
         HostCmdComplete     : out std_logic;
         -- to/from lm87if_fsm
         InitCompleted       : in  std_logic;
         CmdEnd              : in  std_logic;
         SMB_IF_Start        : in  std_logic;
         StartNewCmd         : out std_logic;
         ArbMemCopy          : out std_logic;
         -- to source select mux
         AutoCopyPhase       : out std_logic
      );
   end component;   -- iic_arb


   --------------------------------------------------------
   component iicif is
     generic (
      -- Temperature sensor limits
      C_COM_INT_HIGHLIM  : std_logic_vector(7 downto 0) := x"4B"; -- +75   
      C_COM_INT_LOWLIM   : std_logic_vector(7 downto 0) := x"FB"; -- -5
      C_COM_EXT_HIGHLIM  : std_logic_vector(7 downto 0) := x"5A"; -- +90
      C_COM_EXT_LOWLIM   : std_logic_vector(7 downto 0) := x"FB"; -- -5
      C_IND_INT_HIGHLIM  : std_logic_vector(7 downto 0) := x"5A"; -- +90
      C_IND_INT_LOWLIM   : std_logic_vector(7 downto 0) := x"D3"; -- -45
      C_IND_EXT_HIGHLIM  : std_logic_vector(7 downto 0) := x"69"; -- +105
      C_IND_EXT_LOWLIM   : std_logic_vector(7 downto 0) := x"D3"  -- -45     
     );
     port (
        -- Clocks and resets
        clk               : in   std_logic;
        clkEn400k         : in   std_logic;
        clkEn1pps         : in   std_logic;
        rst               : in   std_logic;
        -- Host Interface
        HostWr            : in   std_logic_vector(  1 downto 0);
        HostAdr           : in   std_logic_vector( 10 downto 4);
        HostDaIn          : in   std_logic_vector(127 downto 0);
        HostBE            : in   std_logic_vector(  7 downto 0);
        status            : out  std_logic_vector(255 downto 0);
        iicbuf_q          : out  std_logic_vector(127 downto 0);
        -- 
        reboot_en_i       : in   std_logic;
        factory_test_i_n  : in   std_logic;
        -- Temperature Grade Flags
        TempFlagVd        : in   std_logic;
        IndGrade          : in   std_logic;
        -- XRM Voltage
        xrm_fru_da        : out  adb3_xrm_fru_t;
        -- PCB Revision
        pcbrev            : out  std_logic_vector(31 downto 0);
        -- Interrupt
        iic0_alert        : out  std_logic;
        iic1_alert        : out  std_logic;
        temp_alert        : out  std_logic;
        -- SMB Interfaces  
        iic0_scl_i        : in   std_logic;
        iic0_scl_o        : out  std_logic;
        iic0_scl_oe_n     : out  std_logic;
        iic0_sda_i        : in   std_logic;
        iic0_sda_o        : out  std_logic;
        iic0_sda_oe_n     : out  std_logic;
        iic0_alert_i_n    : in   std_logic;
        iic1_scl_i        : in   std_logic;
        iic1_scl_o        : out  std_logic;
        iic1_scl_oe_n     : out  std_logic;
        iic1_sda_i        : in   std_logic;
        iic1_sda_o        : out  std_logic;
        iic1_sda_oe_n     : out  std_logic;
        iic1_alert_i_n    : in   std_logic
     );
   end component;   -- iicif
    
   --------------------------------------------------------
   component iicif_v2 is
     port (
        -- Clocks and resets
        clk               : in   std_logic;
        clkEn400k         : in   std_logic;
        clkEn1pps         : in   std_logic;
        rst               : in   std_logic;
        -- Host Interface
        HostWr            : in   std_logic_vector(  1 downto 0);
        HostAdr           : in   std_logic_vector( 10 downto 4);
        HostDaIn          : in   std_logic_vector(127 downto 0);
        HostBE            : in   std_logic_vector(  7 downto 0);
        status            : out  std_logic_vector(255 downto 0);
        iicbuf_q          : out  std_logic_vector(127 downto 0);
        -- 
        reboot_en_i       : in   std_logic;
        factory_test_i_n  : in   std_logic;
        -- Temperature Grade Flags
        TempGrade         : in   adb3_tempGrade_t;
        TempFlagVd        : in   std_logic;
        -- XRM Voltage
        xrm_fru_da        : out  adb3_xrm_fru_t;
        -- PCB Revision
        pcbrev            : out  std_logic_vector(31 downto 0);
        -- Interrupt
        iic0_alert        : out  std_logic;
        iic1_alert        : out  std_logic;
        temp_alert        : out  std_logic;
        -- SMB Interfaces  
        iic0_scl_i        : in   std_logic;
        iic0_scl_o        : out  std_logic;
        iic0_scl_oe_n     : out  std_logic;
        iic0_sda_i        : in   std_logic;
        iic0_sda_o        : out  std_logic;
        iic0_sda_oe_n     : out  std_logic;
        iic0_alert_i_n    : in   std_logic;
        iic1_scl_i        : in   std_logic;
        iic1_scl_o        : out  std_logic;
        iic1_scl_oe_n     : out  std_logic;
        iic1_sda_i        : in   std_logic;
        iic1_sda_o        : out  std_logic;
        iic1_sda_oe_n     : out  std_logic;
        iic1_alert_i_n    : in   std_logic
     );
   end component;   -- iicif_v2
    
   ----------------------------------------------------------------------------
   component iicif_adb3 is
        generic (
      -- Temperature sensor limits
      C_COM_INT_HIGHLIM  : std_logic_vector(7 downto 0) := x"4B"; -- +75   
      C_COM_INT_LOWLIM   : std_logic_vector(7 downto 0) := x"FB"; -- -5
      C_COM_EXT_HIGHLIM  : std_logic_vector(7 downto 0) := x"5A"; -- +90
      C_COM_EXT_LOWLIM   : std_logic_vector(7 downto 0) := x"FB"; -- -5
      C_IND_INT_HIGHLIM  : std_logic_vector(7 downto 0) := x"5A"; -- +90
      C_IND_INT_LOWLIM   : std_logic_vector(7 downto 0) := x"D3"; -- -45
      C_IND_EXT_HIGHLIM  : std_logic_vector(7 downto 0) := x"69"; -- +105
      C_IND_EXT_LOWLIM   : std_logic_vector(7 downto 0) := x"D3"  -- -45     
   );
   port (
      -- Clocks and resets
      clk               : in  std_logic;
      clkEn400k         : in  std_logic;
      clkEn1pps         : in  std_logic;
      rst               : in  std_logic;
   
      -- Host Interface
      HostWr            : in  std_logic_vector(  1 downto 0);
      HostAdr           : in  std_logic_vector( 10 downto 4);
      HostDaIn          : in  std_logic_vector(127 downto 0);
      HostBE            : in  std_logic_vector(  7 downto 0);
      status            : out std_logic_vector(255 downto 0);
      iicbuf_q          : out std_logic_vector(127 downto 0);
      -- 
      reboot_en_i       : in    std_logic;
      factory_test_i_n  : in    std_logic;
      -- Temperature Grade Flags
      IndGrade          : in  std_logic;
      TempFlagVd        : in  std_logic;
      -- XRM Voltage
      xrm_fru_da        : out  adb3_xrm_fru_t;
      -- PCB Revision
      pcbrev            : out  std_logic_vector(31 downto 0);
      -- Interrupt
      iic0_alert        : out  std_logic;
      iic1_alert        : out  std_logic;
      temp_alert        : out  std_logic;
      iic_done_alert    : out std_logic;
      -- SMB Interfaces  
      iic0_scl_i        : in   std_logic;
      iic0_scl_o        : out  std_logic;
      iic0_scl_oe_n     : out  std_logic;
      iic0_sda_i        : in   std_logic;
      iic0_sda_o        : out  std_logic;
      iic0_sda_oe_n     : out  std_logic;
      iic0_alert_i_n    : in   std_logic;
      iic1_scl_i        : in   std_logic;
      iic1_scl_o        : out  std_logic;
      iic1_scl_oe_n     : out  std_logic;
      iic1_sda_i        : in   std_logic;
      iic1_sda_o        : out  std_logic;
      iic1_sda_oe_n     : out  std_logic;
      iic1_alert_i_n    : in   std_logic
   );
   end component iicif_adb3;

   ----------------------------------------------------------------------------
   component iicif_adb3_v2 is
   port (
      -- Clocks and resets
      clk               : in  std_logic;
      clkEn400k         : in  std_logic;
      clkEn1pps         : in  std_logic;
      rst               : in  std_logic;
   
      -- Host Interface
      HostWr            : in  std_logic_vector(  1 downto 0);
      HostAdr           : in  std_logic_vector( 10 downto 4);
      HostDaIn          : in  std_logic_vector(127 downto 0);
      HostBE            : in  std_logic_vector(  7 downto 0);
      status            : out std_logic_vector(255 downto 0);
      iicbuf_q          : out std_logic_vector(127 downto 0);
      -- 
      reboot_en_i       : in    std_logic;
      factory_test_i_n  : in    std_logic;
      -- Temperature Grade Flags
      TempGrade         : in  adb3_tempGrade_t;
      TempFlagVd        : in  std_logic;
      -- XRM Voltage
      xrm_fru_da        : out  adb3_xrm_fru_t;
      -- PCB Revision
      pcbrev            : out  std_logic_vector(31 downto 0);
      -- Interrupt
      iic0_alert        : out  std_logic;
      iic1_alert        : out  std_logic;
      temp_alert        : out  std_logic;
      iic_done_alert    : out std_logic;
      -- SMB Interfaces  
      iic0_scl_i        : in   std_logic;
      iic0_scl_o        : out  std_logic;
      iic0_scl_oe_n     : out  std_logic;
      iic0_sda_i        : in   std_logic;
      iic0_sda_o        : out  std_logic;
      iic0_sda_oe_n     : out  std_logic;
      iic0_alert_i_n    : in   std_logic;
      iic1_scl_i        : in   std_logic;
      iic1_scl_o        : out  std_logic;
      iic1_scl_oe_n     : out  std_logic;
      iic1_sda_i        : in   std_logic;
      iic1_sda_o        : out  std_logic;
      iic1_sda_oe_n     : out  std_logic;
      iic1_alert_i_n    : in   std_logic
   );
   end component iicif_adb3_v2;

   ----------------------------------------------------------------------------
   component iicif_xrc6tge is
        generic (
      -- Temperature sensor limits
      C_COM_INT_HIGHLIM  : std_logic_vector(7 downto 0) := x"4B"; -- +75   
      C_COM_INT_LOWLIM   : std_logic_vector(7 downto 0) := x"FB"; -- -5
      C_COM_EXT_HIGHLIM  : std_logic_vector(7 downto 0) := x"5A"; -- +90
      C_COM_EXT_LOWLIM   : std_logic_vector(7 downto 0) := x"FB"; -- -5
      C_IND_INT_HIGHLIM  : std_logic_vector(7 downto 0) := x"5A"; -- +90
      C_IND_INT_LOWLIM   : std_logic_vector(7 downto 0) := x"D3"; -- -45
      C_IND_EXT_HIGHLIM  : std_logic_vector(7 downto 0) := x"69"; -- +105
      C_IND_EXT_LOWLIM   : std_logic_vector(7 downto 0) := x"D3"  -- -45     
   );
   port (
      -- Clocks and resets
      clk               : in  std_logic;
      clkEn400k         : in  std_logic;
      clkEn1pps         : in  std_logic;
      rst               : in  std_logic;
   
      -- Host Interface
      HostWr            : in  std_logic_vector(  1 downto 0);
      HostAdr           : in  std_logic_vector( 10 downto 4);
      HostDaIn          : in  std_logic_vector(127 downto 0);
      HostBE            : in  std_logic_vector(  7 downto 0);
      status            : out std_logic_vector(255 downto 0);
      iicbuf_q          : out std_logic_vector(127 downto 0);
      -- 
      reboot_en_i       : in    std_logic;
      factory_test_i_n  : in    std_logic;
      -- Temperature Grade Flags
      IndGrade          : in  std_logic;
      TempFlagVd        : in  std_logic;
      -- Clock Chip Configuration
      SI5338_cfg_data     : in  std_logic_vector(31 downto 0);
      SI5338_cfg_valid    : in   std_logic;
      SI5338_cfg_radv     : out    std_logic;
      -- XRM Voltage
      xrm_fru_da        : out  adb3_xrm_fru_t;
      -- PCB Revision
      pcbrev            : out  std_logic_vector(31 downto 0);
      -- Interrupt
      iic0_alert        : out  std_logic;
      iic1_alert        : out  std_logic;
      temp_alert        : out  std_logic;
      iic_done_alert : out std_logic;
      -- SMB Interfaces  
      iic0_scl_i        : in   std_logic;
      iic0_scl_o        : out  std_logic;
      iic0_scl_oe_n     : out  std_logic;
      iic0_sda_i        : in   std_logic;
      iic0_sda_o        : out  std_logic;
      iic0_sda_oe_n     : out  std_logic;
      iic0_alert_i_n    : in   std_logic;
      iic1_scl_i        : in   std_logic;
      iic1_scl_o        : out  std_logic;
      iic1_scl_oe_n     : out  std_logic;
      iic1_sda_i        : in   std_logic;
      iic1_sda_o        : out  std_logic;
      iic1_sda_oe_n     : out  std_logic;
      iic1_alert_i_n    : in   std_logic
   );
   end component iicif_xrc6tge;
    
   ----------------------------------------------------------------------------
   component iicif_xrc6tge_v2 is
   port (
      -- Clocks and resets
      clk               : in  std_logic;
      clkEn400k         : in  std_logic;
      clkEn1pps         : in  std_logic;
      rst               : in  std_logic;
   
      -- Host Interface
      HostWr            : in  std_logic_vector(  1 downto 0);
      HostAdr           : in  std_logic_vector( 10 downto 4);
      HostDaIn          : in  std_logic_vector(127 downto 0);
      HostBE            : in  std_logic_vector(  7 downto 0);
      status            : out std_logic_vector(255 downto 0);
      iicbuf_q          : out std_logic_vector(127 downto 0);
      -- 
      reboot_en_i       : in  std_logic;
      factory_test_i_n  : in  std_logic;
      -- Temperature Grade Flags
      TempGrade         : in  adb3_tempGrade_t;
      TempFlagVd        : in  std_logic;
      -- Clock Chip Configuration
      SI5338_cfg_data   : in  std_logic_vector(31 downto 0);
      SI5338_cfg_valid  : in  std_logic;
      SI5338_cfg_radv   : out std_logic;
      -- XRM Voltage
      xrm_fru_da        : out  adb3_xrm_fru_t;
      -- PCB Revision
      pcbrev            : out  std_logic_vector(31 downto 0);
      -- Interrupt
      iic0_alert        : out  std_logic;
      iic1_alert        : out  std_logic;
      temp_alert        : out  std_logic;
      iic_done_alert    : out  std_logic;
      -- SMB Interfaces  
      iic0_scl_i        : in   std_logic;
      iic0_scl_o        : out  std_logic;
      iic0_scl_oe_n     : out  std_logic;
      iic0_sda_i        : in   std_logic;
      iic0_sda_o        : out  std_logic;
      iic0_sda_oe_n     : out  std_logic;
      iic0_alert_i_n    : in   std_logic;
      iic1_scl_i        : in   std_logic;
      iic1_scl_o        : out  std_logic;
      iic1_scl_oe_n     : out  std_logic;
      iic1_sda_i        : in   std_logic;
      iic1_sda_o        : out  std_logic;
      iic1_sda_oe_n     : out  std_logic;
      iic1_alert_i_n    : in   std_logic
   );
   end component iicif_xrc6tge_v2;
    
   --------------------------------------------------------
   component flash_pinctl is
      port (
         rst             : in    std_logic;
         clk             : in    std_logic;
         --
         write           : in    std_logic;
         read            : in    std_logic;
         DaVd            : out   std_logic;
         ready           : out   std_logic;
         CmdEnd          : out   std_logic;
         --
         fl_wait_i_n     : in    std_logic;
         fl_d_oe_n       : out   std_logic;
         fl_sel_o_n      : out   std_logic;
         fl_we_o_n       : out   std_logic;
         fl_oe_o_n       : out   std_logic;
         fl_adv_o_n      : out   std_logic      
      );
   end component;   -- flash_pinctl
    
   --------------------------------------------------------
   component flash_bootctl is
      port (
         rst             : in    std_logic;
         clk             : in    std_logic;
         --
         go              : in    std_logic;
         FirstConfigDone : out   std_logic;
         ack_reload      : out   std_logic;
         --
         ClearTgtReq     : in    std_logic;
         ClearTgtAck     : out   std_logic;
         --
         extconfig       : in    std_logic;
         tgt_done        : in    std_logic;
         tgt_init        : in    std_logic;
         tgt_override    : out   std_logic;
         tgt_prog        : out   std_logic;
         -- From FBS pin
         fbs_i_n         : in    std_logic;
         -- flash_pinctl
         ready           : in    std_logic;
         CmdEnd          : in    std_logic;
         write           : out   std_logic;
         read            : out   std_logic;
         --
         BootFgVd        : in    std_logic;
         BitsCnt_Cy      : in    std_logic;
         TGradeLatch_En  : out   std_logic;
         BootFgLatch_En  : out   std_logic;
         BootFgSelMS     : out   std_logic;
         --
         LastSetup       : in    std_logic;
         LastRestore     : in    std_logic;
         LastClkAdr      : in    std_logic;
         CmdROMAdrCtr_Ld : out   std_logic;
         CmdROMAdrCtr_En : out   std_logic;
         LdFlAdr         : out   std_logic;
         WriteOp         : out   std_logic;
         TGradeAdr       : out   std_logic;
         ProgClkAdr      : out   std_logic;
         SelJumpAdr      : out   std_logic;
         IncAdr          : out   std_logic;
         ClkLatch_En     : out   std_logic;
         ChkClkFg        : out   std_logic;
         --
         en_tgt_wr       : out   std_logic
       );
   end component;   -- flash_bootctl

    
   --------------------------------------------------------
   component flashboot_syncif is
      generic (
         USE_BOOTFLAG    : boolean := TRUE
      );
      port (
         rst             : in    std_logic;
         clk             : in    std_logic;
         clkx2           : in    std_logic;
         --
         go              : in    std_logic;
         reload0_latch   : in    std_logic;
         reload1_latch   : in    std_logic;
         FirstConfigDone : out   std_logic;
         ack_reload      : out   std_logic;
         --
         ClearTgt        : in    std_logic;
         TgtCleared      : out   std_logic;
         --
         extconfig       : in    std_logic;
         tgt_done        : in    std_logic;
         tgt_init        : in    std_logic;
         SelAltBits      : in    std_logic;
         tgt_override    : out   std_logic;
         tgtsel          : out   std_logic;
         tgt_prog        : out   std_logic;
         tgt_write       : out   std_logic;
         tgt_data        : out   std_logic_vector(7 downto 0);
         -- From FBS pin
         fbs_i_n         : in    std_logic;
         -- Temperature Grade Flags
         IndGrade        : out   std_logic;
         TempFlagVd      : out   std_logic;
         -- Programmable clock values
         Clk0BootVal     : out   std_logic_vector(31 downto 0);
         Clk0BootVd      : out   std_logic;
         Clk1BootVal     : out   std_logic_vector(31 downto 0);
         Clk1BootVd      : out   std_logic;
         -- To/from Flash pins
         fl_a_o          : out   std_logic_vector(adb3_flash_address_width downto 1);
         fl_d_i          : in    std_logic_vector(adb3_flash_data_width - 1 downto 0);
         fl_d_o          : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
         fl_d_oe_n       : out   std_logic;
         fl_sel_o_n      : out   std_logic;
         fl_we_o_n       : out   std_logic;
         fl_oe_o_n       : out   std_logic;
         fl_adv_o_n      : out   std_logic;
         fl_wait_i_n     : in    std_logic 
      );
   end component;   -- flashboot_syncif
    
   --------------------------------------------------------
   component flashboot_syncif_v2 is
      generic (
         USE_BOOTFLAG    : boolean := TRUE
      );
      port (
         rst             : in    std_logic;
         clk             : in    std_logic;
         clkx2           : in    std_logic;
         --
         go              : in    std_logic;
         reload0_latch   : in    std_logic;
         reload1_latch   : in    std_logic;
         FirstConfigDone : out   std_logic;
         ack_reload      : out   std_logic;
         --
         ClearTgt        : in    std_logic;
         TgtCleared      : out   std_logic;
         --
         extconfig       : in    std_logic;
         tgt_done        : in    std_logic;
         tgt_init        : in    std_logic;
         SelAltBits      : in    std_logic;
         tgt_override    : out   std_logic;
         tgtsel          : out   std_logic;
         tgt_prog        : out   std_logic;
         tgt_write       : out   std_logic;
         tgt_data        : out   std_logic_vector(7 downto 0);
         -- From FBS pin
         fbs_i_n         : in    std_logic;
         -- Temperature Grade Flags
         TempGrade       : out   adb3_tempGrade_t;
         TempFlagVd      : out   std_logic;
         -- Programmable clock values
         Clk0BootVal     : out   std_logic_vector(31 downto 0);
         Clk0BootVd      : out   std_logic;
         Clk1BootVal     : out   std_logic_vector(31 downto 0);
         Clk1BootVd      : out   std_logic;
         -- To/from Flash pins
         fl_a_o          : out   std_logic_vector(adb3_flash_address_width downto 1);
         fl_d_i          : in    std_logic_vector(adb3_flash_data_width - 1 downto 0);
         fl_d_o          : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
         fl_d_oe_n       : out   std_logic;
         fl_sel_o_n      : out   std_logic;
         fl_we_o_n       : out   std_logic;
         fl_oe_o_n       : out   std_logic;
         fl_adv_o_n      : out   std_logic;
         fl_wait_i_n     : in    std_logic 
      );
   end component;   -- flashboot_syncif_v2
    
   --------------------------------------------------------
   component flashboot_syncif_xrc6tge is
      generic (
         USE_BOOTFLAG    : boolean := TRUE
      );
      port (
         rst             : in    std_logic;
         clk             : in    std_logic;
         clkx2           : in    std_logic;
         --
         go              : in    std_logic;
         reload0_latch   : in    std_logic;
         reload1_latch   : in    std_logic;
         FirstConfigDone : out   std_logic;
         ack_reload      : out   std_logic;
         --
         ClearTgt        : in    std_logic;
         TgtCleared      : out   std_logic;
         --
         extconfig       : in    std_logic;
         tgt_done        : in    std_logic;
         tgt_init        : in    std_logic;
         SelAltBits      : in    std_logic;
         tgt_override    : out   std_logic;
         tgtsel          : out   std_logic;
         tgt_prog        : out   std_logic;
         tgt_write       : out   std_logic;
         tgt_data        : out   std_logic_vector(7 downto 0);
         -- From FBS pin
         fbs_i_n         : in    std_logic;
         -- Temperature Grade Flags
         IndGrade        : out   std_logic;
         TempFlagVd      : out   std_logic;
         -- Programmable clock values
         --Clk0BootVal     : out   std_logic_vector(31 downto 0);
         --Clk0BootVd      : out   std_logic;
         --Clk1BootVal     : out   std_logic_vector(31 downto 0);
         --Clk1BootVd      : out   std_logic;
         SI5338_cfg_data   : out   std_logic_vector(31 downto 0);
         SI5338_cfg_valid  : out   std_logic;
         -- To/from Flash pins
         fl_a_o          : out   std_logic_vector(adb3_flash_address_width downto 1);
         fl_d_i          : in    std_logic_vector(adb3_flash_data_width - 1 downto 0);
         fl_d_o          : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
         fl_d_oe_n       : out   std_logic;
         fl_sel_o_n      : out   std_logic;
         fl_we_o_n       : out   std_logic;
         fl_oe_o_n       : out   std_logic;
         fl_adv_o_n      : out   std_logic;
         fl_wait_i_n     : in    std_logic 
      );
   end component;   -- flashboot_syncif_xrc6tge

   --------------------------------------------------------
   component flashboot_syncif_xrc6tge_v2 is
      generic (
         USE_BOOTFLAG    : boolean := TRUE
      );
      port (
         rst             : in    std_logic;
         clk             : in    std_logic;
         clkx2           : in    std_logic;
         --
         go              : in    std_logic;
         reload0_latch   : in    std_logic;
         reload1_latch   : in    std_logic;
         FirstConfigDone : out   std_logic;
         ack_reload      : out   std_logic;
         --
         ClearTgt        : in    std_logic;
         TgtCleared      : out   std_logic;
         --
         extconfig       : in    std_logic;
         tgt_done        : in    std_logic;
         tgt_init        : in    std_logic;
         SelAltBits      : in    std_logic;
         tgt_override    : out   std_logic;
         tgtsel          : out   std_logic;
         tgt_prog        : out   std_logic;
         tgt_write       : out   std_logic;
         tgt_data        : out   std_logic_vector(7 downto 0);
         -- From FBS pin
         fbs_i_n         : in    std_logic;
         -- Temperature Grade Flags
         TempGrade       : out   adb3_tempGrade_t;
         TempFlagVd      : out   std_logic;
         -- Programmable clock values
         --Clk0BootVal     : out   std_logic_vector(31 downto 0);
         --Clk0BootVd      : out   std_logic;
         --Clk1BootVal     : out   std_logic_vector(31 downto 0);
         --Clk1BootVd      : out   std_logic;
         SI5338_cfg_data   : out   std_logic_vector(31 downto 0);
         SI5338_cfg_valid  : out   std_logic;
         -- To/from Flash pins
         fl_a_o          : out   std_logic_vector(adb3_flash_address_width downto 1);
         fl_d_i          : in    std_logic_vector(adb3_flash_data_width - 1 downto 0);
         fl_d_o          : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
         fl_d_oe_n       : out   std_logic;
         fl_sel_o_n      : out   std_logic;
         fl_we_o_n       : out   std_logic;
         fl_oe_o_n       : out   std_logic;
         fl_adv_o_n      : out   std_logic;
         fl_wait_i_n     : in    std_logic 
      );
   end component;   -- flashboot_syncif_xrc6tge_v2
    
   
   --------------------------------------------------------
   component flash_cpld
       generic (
           USE_BOOTFLAG      : boolean := TRUE
       );
       port(
           rst               : in    std_logic;
           -- 'lclk' domain
           lclk              : in    std_logic;
           write             : in    std_logic;
           be                : in    std_logic_vector( 11 downto 0);
           data              : in    std_logic_vector( 95 downto 0);
           status            : out   std_logic_vector( 95 downto 0);
           -- 'smapclk' domain, to/from target FPGA SelectMap
           smapclk           : in    std_logic;
           smapclk_div2      : in    std_logic;
           smapclk_div2_270  : in    std_logic;
           reload            : in    std_logic_vector(1 downto 0);
           extconfig         : in    std_logic;
           aux_done          : in    std_logic;
           aux_init          : in    std_logic;
           aux_override      : out   std_logic;
           aux_tgtsel        : out   std_logic;
           aux_prog          : out   std_logic;
           aux_write         : out   std_logic;
           aux_data          : out   std_logic_vector(7 downto 0);
           TgtCleared        : out   std_logic;
           -- (System) Write inhibit
           mvmro_mp_i        : in    std_logic;
           -- From board switches
           fbs_i_n           : in    std_logic;
           reboot_en_i       : in    std_logic;
           -- From SMB Interface 0
           sysmon_alert      : in    std_logic;
           -- Temperature Grade Flags
           IndGrade          : out   std_logic;
           TempFlagVd        : out   std_logic;
           -- Programmable clock values
           Clk0BootVal       : out   std_logic_vector(31 downto 0);
           Clk0BootVd        : out   std_logic;
           Clk1BootVal       : out   std_logic_vector(31 downto 0);
           Clk1BootVd        : out   std_logic;
           -- To/from Flash pins
           fl_rst_i_n        : in    std_logic;
           fl_rst_o_n        : out   std_logic;
           fl_rst_oe_n       : out   std_logic;
           fl_a_i            : in    std_logic_vector(adb3_flash_address_width  downto 1);
           fl_a_o            : out   std_logic_vector(adb3_flash_address_width  downto 1);
           fl_a_oe_n         : out   std_logic_vector(adb3_flash_address_width  downto 1);
           fl_d_i            : in    std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_d_o            : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_d_oe_n         : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_sel_i_n        : in    std_logic;
           fl_sel_o_n        : out   std_logic;
           fl_sel_oe_n       : out   std_logic;
           fl_we_i_n         : in    std_logic;
           fl_we_o_n         : out   std_logic;
           fl_we_oe_n        : out   std_logic;
           fl_oe_i_n         : in    std_logic;
           fl_oe_o_n         : out   std_logic;
           fl_oe_oe_n        : out   std_logic;
           fl_adv_i_n        : in    std_logic;
           fl_adv_o_n        : out   std_logic;
           fl_adv_oe_n       : out   std_logic;
           fl_clk_i          : in    std_logic;
           fl_clk_o          : out   std_logic;
           fl_clk_oe_n       : out   std_logic;
           fl_wait_i_n       : in    std_logic; 
           fl_wait_o_n       : out   std_logic; 
           fl_wait_oe_n      : out   std_logic);
   end component;   -- flash_cpld

   --------------------------------------------------------
   component flash_cpld_v2
       generic (
           USE_BOOTFLAG      : boolean := TRUE
       );
       port(
           rst               : in    std_logic;
           -- 'lclk' domain
           lclk              : in    std_logic;
           write             : in    std_logic;
           be                : in    std_logic_vector( 11 downto 0);
           data              : in    std_logic_vector( 95 downto 0);
           status            : out   std_logic_vector( 95 downto 0);
           -- 'smapclk' domain, to/from target FPGA SelectMap
           smapclk           : in    std_logic;
           smapclk_div2      : in    std_logic;
           smapclk_div2_270  : in    std_logic;
           reload            : in    std_logic_vector(1 downto 0);
           extconfig         : in    std_logic;
           aux_done          : in    std_logic;
           aux_init          : in    std_logic;
           aux_override      : out   std_logic;
           aux_tgtsel        : out   std_logic;
           aux_prog          : out   std_logic;
           aux_write         : out   std_logic;
           aux_data          : out   std_logic_vector(7 downto 0);
           TgtCleared        : out   std_logic;
           -- (System) Write inhibit
           mvmro_mp_i        : in    std_logic;
           -- From board switches
           fbs_i_n           : in    std_logic;
           reboot_en_i       : in    std_logic;
           -- From SMB Interface 0
           sysmon_alert      : in    std_logic;
           -- Temperature Grade Flags
           TempGrade         : out   adb3_tempGrade_t;
           TempFlagVd        : out   std_logic;
           -- Programmable clock values
           Clk0BootVal       : out   std_logic_vector(31 downto 0);
           Clk0BootVd        : out   std_logic;
           Clk1BootVal       : out   std_logic_vector(31 downto 0);
           Clk1BootVd        : out   std_logic;
           -- To/from Flash pins
           fl_rst_i_n        : in    std_logic;
           fl_rst_o_n        : out   std_logic;
           fl_rst_oe_n       : out   std_logic;
           fl_a_i            : in    std_logic_vector(adb3_flash_address_width  downto 1);
           fl_a_o            : out   std_logic_vector(adb3_flash_address_width  downto 1);
           fl_a_oe_n         : out   std_logic_vector(adb3_flash_address_width  downto 1);
           fl_d_i            : in    std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_d_o            : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_d_oe_n         : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_sel_i_n        : in    std_logic;
           fl_sel_o_n        : out   std_logic;
           fl_sel_oe_n       : out   std_logic;
           fl_we_i_n         : in    std_logic;
           fl_we_o_n         : out   std_logic;
           fl_we_oe_n        : out   std_logic;
           fl_oe_i_n         : in    std_logic;
           fl_oe_o_n         : out   std_logic;
           fl_oe_oe_n        : out   std_logic;
           fl_adv_i_n        : in    std_logic;
           fl_adv_o_n        : out   std_logic;
           fl_adv_oe_n       : out   std_logic;
           fl_clk_i          : in    std_logic;
           fl_clk_o          : out   std_logic;
           fl_clk_oe_n       : out   std_logic;
           fl_wait_i_n       : in    std_logic; 
           fl_wait_o_n       : out   std_logic; 
           fl_wait_oe_n      : out   std_logic);
   end component;   -- flash_cpld_v2

   --------------------------------------------------------
   component flash_cpld_v3
       generic (
           USE_BOOTFLAG      : boolean := TRUE
       );
       port(
           rst               : in    std_logic;
           -- 'lclk' domain
           lclk              : in    std_logic;
           write             : in    std_logic;
           be                : in    std_logic_vector( 11 downto 0);
           data              : in    std_logic_vector( 95 downto 0);
           status            : out   std_logic_vector( 95 downto 0);
           -- 'smapclk' domain, to/from target FPGA SelectMap
           smapclk           : in    std_logic;
           smapclk_div2      : in    std_logic;
           smapclk_div2_270  : in    std_logic;
           reload            : in    std_logic_vector(1 downto 0);
           extconfig         : in    std_logic;
           aux_done          : in    std_logic;
           aux_init          : in    std_logic;
           aux_override      : out   std_logic;
           aux_tgtsel        : out   std_logic;
           aux_prog          : out   std_logic;
           aux_write         : out   std_logic;
           aux_data          : out   std_logic_vector(7 downto 0);
           TgtCleared        : out   std_logic;
           -- (System) Write inhibit
           mvmro_mp_i        : in    std_logic;
           -- From board switches
           fbs_i_n           : in    std_logic;
           reboot_en_i       : in    std_logic;
           -- From SMB Interface 0
           sysmon_alert      : in    std_logic;
           -- Temperature Grade Flags
           TempGrade         : out   adb3_tempGrade_t;
           TempFlagVd        : out   std_logic;
           -- Programmable clock values
           Clk0BootVal       : out   std_logic_vector(31 downto 0);
           Clk0BootVd        : out   std_logic;
           Clk1BootVal       : out   std_logic_vector(31 downto 0);
           Clk1BootVd        : out   std_logic;
           -- To/from Flash pins
           fl_rst_i_n        : in    std_logic;
           fl_rst_o_n        : out   std_logic;
           fl_rst_oe_n       : out   std_logic;
           fl_a_i            : in    std_logic_vector(adb3_flash_address_width  downto 1);
           fl_a_o            : out   std_logic_vector(adb3_flash_address_width  downto 1);
           fl_a_oe_n         : out   std_logic_vector(adb3_flash_address_width  downto 1);
           fl_d_i            : in    std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_d_o            : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_d_oe_n         : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_sel_i_n        : in    std_logic;
           fl_sel_o_n        : out   std_logic;
           fl_sel_oe_n       : out   std_logic;
           fl_we_i_n         : in    std_logic;
           fl_we_o_n         : out   std_logic;
           fl_we_oe_n        : out   std_logic;
           fl_oe_i_n         : in    std_logic;
           fl_oe_o_n         : out   std_logic;
           fl_oe_oe_n        : out   std_logic;
           fl_adv_i_n        : in    std_logic;
           fl_adv_o_n        : out   std_logic;
           fl_adv_oe_n       : out   std_logic;
           fl_clk_i          : in    std_logic;
           fl_clk_o          : out   std_logic;
           fl_clk_oe_n       : out   std_logic;
           fl_wait_i_n       : in    std_logic; 
           fl_wait_o_n       : out   std_logic; 
           fl_wait_oe_n      : out   std_logic);
   end component;   -- flash_cpld_v3

   --------------------------------------------------------
   component flash_cpld_xrc6tge
       generic (
           USE_BOOTFLAG      : boolean := TRUE
       );
       port(
           rst               : in    std_logic;
           -- 'lclk' domain
           lclk              : in    std_logic;
           write             : in    std_logic;
           be                : in    std_logic_vector( 11 downto 0);
           data              : in    std_logic_vector( 95 downto 0);
           status            : out   std_logic_vector( 95 downto 0);
           -- 'smapclk' domain, to/from target FPGA SelectMap
           smapclk           : in    std_logic;
           smapclk_div2      : in    std_logic;
           smapclk_div2_270  : in    std_logic;
           reload            : in    std_logic_vector(1 downto 0);
           extconfig         : in    std_logic;
           aux_done          : in    std_logic;
           aux_init          : in    std_logic;
           aux_override      : out   std_logic;
           aux_tgtsel        : out   std_logic;
           aux_prog          : out   std_logic;
           aux_write         : out   std_logic;
           aux_data          : out   std_logic_vector(7 downto 0);
           TgtCleared        : out   std_logic;
           -- (System) Write inhibit
           mvmro_mp_i        : in    std_logic;
           -- From board switches
           fbs_i_n           : in    std_logic;
           reboot_en_i       : in    std_logic;
           -- From SMB Interface 0
           sysmon_alert      : in    std_logic;
           -- Temperature Grade Flags
           IndGrade          : out   std_logic;
           TempFlagVd        : out   std_logic;
           -- Programmable clock values
--           Clk0BootVal       : out   std_logic_vector(31 downto 0);
--           Clk0BootVd        : out   std_logic;
--           Clk1BootVal       : out   std_logic_vector(31 downto 0);
--           Clk1BootVd        : out   std_logic;
           SI5338_cfg_data     : out   std_logic_vector(31 downto 0);
           SI5338_cfg_valid    : out   std_logic;
           SI5338_cfg_radv     : in    std_logic;
           -- To/from Flash pins
           fl_rst_i_n        : in    std_logic;
           fl_rst_o_n        : out   std_logic;
           fl_rst_oe_n       : out   std_logic;
           fl_a_i            : in    std_logic_vector(adb3_flash_address_width  downto 1);
           fl_a_o            : out   std_logic_vector(adb3_flash_address_width  downto 1);
           fl_a_oe_n         : out   std_logic_vector(adb3_flash_address_width  downto 1);
           fl_d_i            : in    std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_d_o            : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_d_oe_n         : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_sel_i_n        : in    std_logic;
           fl_sel_o_n        : out   std_logic;
           fl_sel_oe_n       : out   std_logic;
           fl_we_i_n         : in    std_logic;
           fl_we_o_n         : out   std_logic;
           fl_we_oe_n        : out   std_logic;
           fl_oe_i_n         : in    std_logic;
           fl_oe_o_n         : out   std_logic;
           fl_oe_oe_n        : out   std_logic;
           fl_adv_i_n        : in    std_logic;
           fl_adv_o_n        : out   std_logic;
           fl_adv_oe_n       : out   std_logic;
           fl_clk_i          : in    std_logic;
           fl_clk_o          : out   std_logic;
           fl_clk_oe_n       : out   std_logic;
           fl_wait_i_n       : in    std_logic; 
           fl_wait_o_n       : out   std_logic; 
           fl_wait_oe_n      : out   std_logic);
   end component;   -- flash_cpld_xrc6tge
   
   --------------------------------------------------------
   component flash_cpld_xrc6tge_v2
       generic (
           USE_BOOTFLAG      : boolean := TRUE
       );
       port(
           rst               : in    std_logic;
           -- 'lclk' domain
           lclk              : in    std_logic;
           write             : in    std_logic;
           be                : in    std_logic_vector( 11 downto 0);
           data              : in    std_logic_vector( 95 downto 0);
           status            : out   std_logic_vector( 95 downto 0);
           -- 'smapclk' domain, to/from target FPGA SelectMap
           smapclk           : in    std_logic;
           smapclk_div2      : in    std_logic;
           smapclk_div2_270  : in    std_logic;
           reload            : in    std_logic_vector(1 downto 0);
           extconfig         : in    std_logic;
           aux_done          : in    std_logic;
           aux_init          : in    std_logic;
           aux_override      : out   std_logic;
           aux_tgtsel        : out   std_logic;
           aux_prog          : out   std_logic;
           aux_write         : out   std_logic;
           aux_data          : out   std_logic_vector(7 downto 0);
           TgtCleared        : out   std_logic;
           -- (System) Write inhibit
           mvmro_mp_i        : in    std_logic;
           -- From board switches
           fbs_i_n           : in    std_logic;
           reboot_en_i       : in    std_logic;
           -- From SMB Interface 0
           sysmon_alert      : in    std_logic;
           -- Temperature Grade Flags
           TempGrade         : out   adb3_tempGrade_t;
           TempFlagVd        : out   std_logic;
           -- Programmable clock values
--           Clk0BootVal       : out   std_logic_vector(31 downto 0);
--           Clk0BootVd        : out   std_logic;
--           Clk1BootVal       : out   std_logic_vector(31 downto 0);
--           Clk1BootVd        : out   std_logic;
           SI5338_cfg_data     : out   std_logic_vector(31 downto 0);
           SI5338_cfg_valid    : out   std_logic;
           SI5338_cfg_radv     : in    std_logic;
           -- To/from Flash pins
           fl_rst_i_n        : in    std_logic;
           fl_rst_o_n        : out   std_logic;
           fl_rst_oe_n       : out   std_logic;
           fl_a_i            : in    std_logic_vector(adb3_flash_address_width  downto 1);
           fl_a_o            : out   std_logic_vector(adb3_flash_address_width  downto 1);
           fl_a_oe_n         : out   std_logic_vector(adb3_flash_address_width  downto 1);
           fl_d_i            : in    std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_d_o            : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_d_oe_n         : out   std_logic_vector(adb3_flash_data_width - 1 downto 0);
           fl_sel_i_n        : in    std_logic;
           fl_sel_o_n        : out   std_logic;
           fl_sel_oe_n       : out   std_logic;
           fl_we_i_n         : in    std_logic;
           fl_we_o_n         : out   std_logic;
           fl_we_oe_n        : out   std_logic;
           fl_oe_i_n         : in    std_logic;
           fl_oe_o_n         : out   std_logic;
           fl_oe_oe_n        : out   std_logic;
           fl_adv_i_n        : in    std_logic;
           fl_adv_o_n        : out   std_logic;
           fl_adv_oe_n       : out   std_logic;
           fl_clk_i          : in    std_logic;
           fl_clk_o          : out   std_logic;
           fl_clk_oe_n       : out   std_logic;
           fl_wait_i_n       : in    std_logic; 
           fl_wait_o_n       : out   std_logic; 
           fl_wait_oe_n      : out   std_logic);
   end component;   -- flash_cpld_xrc6tge_v2
   
   ----------------------------------------------------------------------------
   component si5338_config is
     port (
       rst              : in std_logic;
       clk              : in  std_logic;
       si5338_cfg_data  : in  std_logic_vector(31 downto 0);
       si5338_cfg_valid : in  std_logic;
       si5338_cfg_radv  : out std_logic;
       can_start        : in  std_logic;
       completed        : out std_logic;
       iic_chip_adr     : out std_logic_vector(6 downto 0);
       iic_start        : out std_logic;
       iic_wrn          : out std_logic;
       iic_adr          : out std_logic_vector(7 downto 0);
       iic_wrda         : out std_logic_vector(7 downto 0);
       iic_rdda         : in  std_logic_vector(7 downto 0);
       iic_ready        : in  std_logic;
       iic_error        : in  std_logic);
   end component si5338_config;
   
   --------------------------------------------------------
   component fpga_cfg
       port(
           rst              : in    std_logic;
           -- 'lclk' domain
           lclk             : in    std_logic;
           write            : in    std_logic;
           be               : in    std_logic_vector( 11 downto 0);
           data             : in    std_logic_vector( 95 downto 0);
           status           : out   std_logic_vector( 95 downto 0);
           full             : out   std_logic;
           -- from flash_cpld (in lclk domain)
           TgtCleared       : in    std_logic;
           IndGrade         : in    std_logic;
           -- To power block
           TgtClrLatch      : out   std_logic;
           -- From FBS pin
           fbs_i_n          : in    std_logic;
           reboot_en_i      : in    std_logic;
           -- 'smapclk' domain
           smapclk          : in    std_logic;
           reload           : out   std_logic;
           aux_override     : in    std_logic;
           aux_prog         : in    std_logic;
           aux_write        : in    std_logic;
           aux_data         : in    std_logic_vector(7 downto 0);
           aux_done         : out   std_logic;
           aux_init         : out   std_logic;
           tgt_prog_i_n     : in    std_logic;
           tgt_prog_o_n     : out   std_logic;
           tgt_prog_oe_n    : out   std_logic;
           tgt_init_i_n     : in    std_logic;
           tgt_init_o_n     : out   std_logic;
           tgt_init_oe_n    : out   std_logic;
           tgt_smap_i       : in    std_logic_vector(7 downto 0);
           tgt_smap_o       : out   std_logic_vector(7 downto 0);
           tgt_smap_oe_n    : out   std_logic_vector(7 downto 0);
           tgt_cs_o_n       : out   std_logic;
           tgt_we_o_n       : out   std_logic;
           tgt_hswapen_o    : out   std_logic;
           tgt_mode_i       : in    std_logic_vector(2 downto 0);
           tgt_mode_o       : out   std_logic_vector(2 downto 0);
           tgt_mode_oe_n    : out   std_logic_vector(2 downto 0);
           tgt_done_i       : in    std_logic;
           tgt_cclk_o       : out   std_logic;
           tgt_din_i        : in    std_logic;
           tgt_din_o        : out   std_logic;
           tgt_din_oe_n     : out   std_logic;
           tgt_busy_i       : in    std_logic
         );
   end component;   -- fpga_cfg

   --------------------------------------------------------
   component fpga_cfg_v2
       port(
           rst              : in    std_logic;
           -- 'lclk' domain
           lclk             : in    std_logic;
           write            : in    std_logic;
           be               : in    std_logic_vector( 11 downto 0);
           data             : in    std_logic_vector( 95 downto 0);
           status           : out   std_logic_vector( 95 downto 0);
           full             : out   std_logic;
           -- from flash_cpld (in lclk domain)
           TgtCleared       : in    std_logic;
           TempGrade        : in    adb3_tempGrade_t;
           -- To power block
           TgtClrLatch      : out   std_logic;
           -- From FBS pin
           fbs_i_n          : in    std_logic;
           reboot_en_i      : in    std_logic;
           -- 'smapclk' domain
           smapclk          : in    std_logic;
           reload           : out   std_logic;
           aux_override     : in    std_logic;
           aux_prog         : in    std_logic;
           aux_write        : in    std_logic;
           aux_data         : in    std_logic_vector(7 downto 0);
           aux_done         : out   std_logic;
           aux_init         : out   std_logic;
           tgt_prog_i_n     : in    std_logic;
           tgt_prog_o_n     : out   std_logic;
           tgt_prog_oe_n    : out   std_logic;
           tgt_init_i_n     : in    std_logic;
           tgt_init_o_n     : out   std_logic;
           tgt_init_oe_n    : out   std_logic;
           tgt_smap_i       : in    std_logic_vector(7 downto 0);
           tgt_smap_o       : out   std_logic_vector(7 downto 0);
           tgt_smap_oe_n    : out   std_logic_vector(7 downto 0);
           tgt_cs_o_n       : out   std_logic;
           tgt_we_o_n       : out   std_logic;
           tgt_hswapen_o    : out   std_logic;
           tgt_mode_i       : in    std_logic_vector(2 downto 0);
           tgt_mode_o       : out   std_logic_vector(2 downto 0);
           tgt_mode_oe_n    : out   std_logic_vector(2 downto 0);
           tgt_done_i       : in    std_logic;
           tgt_cclk_o       : out   std_logic;
           tgt_din_i        : in    std_logic;
           tgt_din_o        : out   std_logic;
           tgt_din_oe_n     : out   std_logic;
           tgt_busy_i       : in    std_logic
         );
   end component;   -- fpga_cfg_v2

   
   --------------------------------------------------------
   component fpga_cfg_arcvt1 is
       port(
          rst              : in    std_logic;
          -- 'lclk' domain
          lclk             : in    std_logic;
          write            : in    std_logic;
          be               : in    std_logic_vector(11 downto 0);
          data             : in    std_logic_vector(95 downto 0);
          status           : out   std_logic_vector(95 downto 0);
          full             : out   std_logic;
          -- from flash_cpld (in lclk domain)
          TgtCleared       : in    std_logic;
          TempGrade        : in    adb3_tempGrade_t;
          --  To power block
          TgtClrLatch      : out   std_logic;
          -- From FBS pin
          fbs_i_n          : in    std_logic;
          reboot_en_i      : in    std_logic;
          -- 'smapclk' domain
          smapclk          : in    std_logic;
          reload           : out   std_logic;
          aux_override     : in    std_logic;
          aux_prog         : in    std_logic;
          aux_write        : in    std_logic;
          aux_data         : in    std_logic_vector(7 downto 0);
          aux_done         : out   std_logic;
          aux_init         : out   std_logic;
          --
          tgt_init_done_i    : in    std_logic;
          tgt_conf_done_i    : in    std_logic;
          tgt_status_i_n     : in    std_logic;
          tgt_crcerr_i       : in    std_logic;
          tgt_dclk_o         : out   std_logic;
          tgt_clkusr_o       : out   std_logic;
          tgt_config_i_n     : in    std_logic;
          tgt_config_o_n     : out   std_logic;
          tgt_config_oe_n    : out   std_logic;
          tgt_fppdata_i      : in    std_logic_vector(7 downto 0);
          tgt_fppdata_o      : out   std_logic_vector(7 downto 0);
          tgt_fppdata_oe_n   : out   std_logic_vector(7 downto 0);
          tgt_asdata_i       : in    std_logic_vector(3 downto 0);
          tgt_asdata_o       : out   std_logic_vector(3 downto 0);
          tgt_asdata_oe_n    : out   std_logic_vector(3 downto 0);
          tgt_mode_i         : in    std_logic_vector(4 downto 0);
          tgt_mode_o         : out   std_logic_vector(4 downto 0);
          tgt_mode_oe_n      : out   std_logic_vector(4 downto 0);
          tgt_devclr_i_n     : in    std_logic;
          tgt_devclr_o_n     : out   std_logic;
          tgt_devclr_oe_n    : out   std_logic;
          tgt_dev_oe_i       : in    std_logic;
          tgt_dev_oe_o       : out   std_logic;
          tgt_dev_oe_oe_n    : out   std_logic;
          tgt_iopullup_i_n   : in    std_logic;
          tgt_iopullup_o_n   : out   std_logic;
          tgt_iopullup_oe_n  : out   std_logic;
          spi_flash_cso_i_n  : in    std_logic;
          spi_flash_cso_o_n  : out   std_logic;
          spi_flash_cso_oe_n : out   std_logic;
          tgt_as_rst_i_n     : in    std_logic;
          tgt_as_rst_o_n     : out   std_logic;
          tgt_as_rst_oe_n    : out   std_logic;
          tgt_asmux_i        : in    std_logic;
          tgt_asmux_o        : out   std_logic;
          tgt_asmux_oe_n     : out   std_logic
        );
   end component; -- fpga_cfg_arcvt1
   
   
   
   
   --------------------------------------------------------
   component fpga_int
       port(
           rst              : in    std_logic;
           lclk             : in    std_logic;
           write            : in    std_logic;
           be               : in    std_logic_vector(  7 downto 0);
           data             : in    std_logic_vector( 63 downto 0);
           status           : out   std_logic_vector( 63 downto 0);
           int_local        : out   std_logic;
           iic0_alert       : in    std_logic;
           temp_alert       : in    std_logic;
           --
           linti_i_n        : in    std_logic_vector(1 downto 0);
           linti_o_n        : out   std_logic_vector(1 downto 0);
           linti_oe_n       : out   std_logic_vector(1 downto 0);
           linto_i_n        : in    std_logic_vector(1 downto 0);
           linto_o_n        : out   std_logic_vector(1 downto 0);
           linto_oe_n       : out   std_logic_vector(1 downto 0)
        );
   end component;

   ----------------------------------------------------------------------------
   component fpga_int_adb3 is
     generic(
       NUMBER_OF_INTERRUPTS     : natural                                           := 16;
       LEVEL_SENSITIVE_DEFAULTS : std_logic_vector(15 downto 0) := X"0000";
       ACTIVE_LOW               : std_logic_vector(15 downto 0) := X"0003");
     port(
       rst        : in  std_logic;
       lclk       : in  std_logic;
       write      : in  std_logic;
       be         : in  std_logic_vector(7 downto 0);
       data       : in  std_logic_vector(63 downto 0);
       status     : out std_logic_vector(63 downto 0);
       int_local  : out std_logic;
       interrupts : in  std_logic_vector(NUMBER_OF_INTERRUPTS-1 downto 0));
   end component fpga_int_adb3;

   
   --------------------------------------------------------
   component power
       port(
           rst               : in    std_logic;
           lclk              : in    std_logic;
           status            : out   std_logic_vector(31 downto 0);
           -- From fpga_cfg
           TgtClrLatch       : in    std_logic;
           --
           iic0_alert        : in    std_logic;
           temp_alert        : in    std_logic;
           --
           pwrstat_i         : in    std_logic_vector(adb3_pwrstat_width -1 downto 0);
           xrm_presence_n    : in    std_logic;
           force_xrm2v5_n    : in    std_logic;
           -- Required voltage (from IIC on XRM)
           xrm_fru_da        : in    adb3_xrm_fru_t;
           --
           xrm_vio_en_i      : in    std_logic;
           xrm_vio_en_o      : out   std_logic;
           xrm_vio_en_oe_n   : out   std_logic;
           xrm_vio_vsel_i    : in    std_logic_vector(2 downto 0);
           xrm_vio_vsel_o    : out   std_logic_vector(2 downto 0);
           xrm_vio_vsel_oe_n : out   std_logic_vector(2 downto 0);
           -- to Red LED in CPLD
           hw_fault_n        : out   std_logic
        );
   end component;    -- power

   --------------------------------------------------------
   component power_vpxpam
       port(
           rst               : in    std_logic;
           lclk              : in    std_logic;
           status            : out   std_logic_vector(31 downto 0);
           -- From fpga_cfg
           TgtClrLatch       : in    std_logic;
           --
           iic0_alert        : in    std_logic;
           temp_alert        : in    std_logic;
           --
           pwrstat_i         : in    std_logic_vector(adb3_pwrstat_width -1 downto 0);
           xmc_presence_n    : in    std_logic;
           --
           alt_ledmode_in_n  : in    std_logic;
           -- to Red LED in CPLD
           hw_fault_n        : out   std_logic
        );
   end component;    -- power_vpxpam

   --------------------------------------------------------
   component power_arcvt1
       port(
           rst               : in    std_logic;
           lclk              : in    std_logic;
           status            : out   std_logic_vector(31 downto 0);
           -- From fpga_cfg
           TgtClrLatch       : in    std_logic;
           --
           iic0_alert        : in    std_logic;
           temp_alert        : in    std_logic;
           --
           pwrstat_i         : in    std_logic_vector(adb3_pwrstat_width -1 downto 0);
           xrm_presence_n    : in    std_logic;
           force_xrm2v5_n    : in    std_logic;
           -- Required voltage (from IIC on XRM)
           xrm_fru_da        : in    adb3_xrm_fru_t;
           --
           xrm_vio_en_i      : in    std_logic;
           xrm_vio_en_o      : out   std_logic;
           xrm_vio_en_oe_n   : out   std_logic;
           xrm_vio_vsel_i    : in    std_logic_vector(2 downto 0);
           xrm_vio_vsel_o    : out   std_logic_vector(2 downto 0);
           xrm_vio_vsel_oe_n : out   std_logic_vector(2 downto 0);
           -- to Red LED in CPLD
           hw_fault_n        : out   std_logic;
           --
           pwr_en_i          : in    std_logic_vector(3 downto 0);
           pwr_en_o          : out   std_logic_vector(3 downto 0);
           pwr_en_oe_n       : out   std_logic_vector(3 downto 0)
        );
   end component;    -- power_arcvt1

   --------------------------------------------------------
   component gpio_regs
       port(
           rst                   : in    std_logic;
           lclk                  : in    std_logic;
           ocp_clk               : in    std_logic;
           write                 : in    std_logic;
           be                    : in    std_logic_vector(  7 downto 0);
           data                  : in    std_logic_vector( 63 downto 0);
           status                : out   std_logic_vector( 63 downto 0);
           --
           gpio_t2b              : in    std_logic_vector(15 downto 0);
           gpio_b2t              : out   std_logic_vector(15 downto 0)
        );
   end component;    -- gpio_regs

   
   --------------------------------------------------------
   component mptl_regs
       port(
           rst                   : in    std_logic;
           lclk                  : in    std_logic;
           ocp_clk               : in    std_logic;
           write                 : in    std_logic;
           be                    : in    std_logic_vector( 15 downto 0);
           data                  : in    std_logic_vector(127 downto 0);
           status                : out   std_logic_vector(127 downto 0);
           --
           mptl_status           : in    std_logic_vector( 63 downto 0);
           mptl_config           : out   std_logic_vector( 31 downto 0);
           mptl_recover_status   : in    std_logic_vector( 15 downto 0);
           mptl_recover_control  : out   std_logic_vector( 15 downto 0)
        );
     end component;  -- mptl_regs
     
   --------------------------------------------------------
   component sideserial_regs
       port(
          rst                   : in    std_logic;
          clk                   : in    std_logic;
          write                 : in    std_logic;
          be                    : in    std_logic_vector(  7 downto 0);
          data                  : in    std_logic_vector( 63 downto 0);
          status                : out   std_logic_vector( 95 downto 0);
          --
          serial_clk_o          : out   std_logic;
          serial_t2b_i_n        : in    std_logic;
          serial_b2t_o_n        : out   std_logic
        );
     end component;  -- sideserial_regs

   --------------------------------------------------------
   component xsmon
       port(
          rst                   : in    std_logic;
          clk                   : in    std_logic;
          write                 : in    std_logic;
          be                    : in    std_logic_vector(  7 downto 0);
          data                  : in    std_logic_vector( 63 downto 0);
          status                : out   std_logic_vector( 95 downto 0)
       );
   end component;  -- xsmon
   
   --------------------------------------------------------
   component xsmon_2p0
       port(
          rst                   : in    std_logic;
          clk                   : in    std_logic;
          clkEn400k             : in    std_logic;
          clkEn1pps             : in    std_logic;
          --
          write                 : in    std_logic;
          be                    : in    std_logic_vector(  7 downto 0);
          data                  : in    std_logic_vector( 63 downto 0);
          status                : out   std_logic_vector(383 downto 0);
          --
          xsmonStat             : out   adb3_xsmonStat_t
       );
   end component;  -- xsmon

   
   
end;                  

package body adb3_board_ctrl is

end;
