
library IEEE;
use IEEE.STD_LOGIC_1164.all;

library work;
use work.pciegen3_pkg.all;
use work.pcie_config.all;

entity pcie3_ku_1_wrapper is
  generic (
    pf0_bar2_aperture_size : std_logic_vector(4 downto 0);
    pf0_bar4_aperture_size : std_logic_vector(4 downto 0));
  port (
    sys_reset_n        : in  std_logic;
    sys_clk            : in  std_logic;
    sys_clk_gt         : in  std_logic;
    pci_exp_txn        : out std_logic_vector(7 downto 0);
    pci_exp_txp        : out std_logic_vector(7 downto 0);
    pci_exp_rxn        : in  std_logic_vector(7 downto 0);
    pci_exp_rxp        : in  std_logic_vector(7 downto 0);
    trn_clk            : out std_logic;
    trn_reset_n        : out std_logic;
    -- bar Interface Requester  
    cq_in              : out cq_c2b_t;
    cq_out             : in  cq_b2c_t;
    -- bar Interface Completer
    cc_in              : out cc_c2b_t;
    cc_out             : in  cc_b2c_t;
    -- DMA Interface Requester
    rq_in              : out rq_c2b_t;
    rq_out             : in  rq_b2c_t;
    -- DMA Interface Completer
    rc_in              : out rc_c2b_t;
    rc_out             : in  rc_b2c_t;
    -- Message interface
    messages           : out message_c2b_t;
    -- Interrupts to from core
    int_in             : out interrupts_c2b_t;
    int_out            : in  interrupts_b2c_t;
    -- configuration Interface
    pcie_config_status : out config_c2b_t;
    pcie_config_cfg_fc_sel : in std_logic_vector(2 downto 0)
    );

  
end entity pcie3_ku_1_wrapper;


architecture rtl of pcie3_ku_1_wrapper is

  COMPONENT pcie3_ultrascale_1
     generic (
      pf0_bar2_aperture_size : std_logic_vector(4 downto 0);
      pf0_bar4_aperture_size : std_logic_vector(4 downto 0));
  PORT (
    pci_exp_txn : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    pci_exp_txp : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    pci_exp_rxn : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    pci_exp_rxp : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    --int_pclk_out_slave : out std_logic;
    --int_pipe_rxusrclk_out : out std_logic;
    --int_rxoutclk_out : out std_logic_vector(7 downto 0);
    --int_dclk_out : out std_logic;
    --int_userclk1_out : out std_logic;
    --int_userclk2_out : out std_logic;
    --int_oobclk_out : out std_logic;
    --int_qplllock_out : out std_logic_vector(1 downto 0);
    --int_qplloutclk_out : out std_logic_vector(1 downto 0);
    --int_qplloutrefclk_out : out std_Logic_vector(1 downto 0); 
    --int_pclk_sel_slave : in std_logic_vector(7 downto 0);
    user_clk : OUT STD_LOGIC;
    user_reset : OUT STD_LOGIC;
    user_lnk_up : OUT STD_LOGIC;
    --user_app_rdy : OUT STD_LOGIC;
    s_axis_rq_tlast : IN STD_LOGIC;
    s_axis_rq_tdata : IN STD_LOGIC_VECTOR(255 DOWNTO 0);
    s_axis_rq_tuser : IN STD_LOGIC_VECTOR(59 DOWNTO 0);
    s_axis_rq_tkeep : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    s_axis_rq_tready : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
    s_axis_rq_tvalid : IN STD_LOGIC;
    m_axis_rc_tdata : OUT STD_LOGIC_VECTOR(255 DOWNTO 0);
    m_axis_rc_tuser : OUT STD_LOGIC_VECTOR(74 DOWNTO 0);
    m_axis_rc_tlast : OUT STD_LOGIC;
    m_axis_rc_tkeep : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    m_axis_rc_tvalid : OUT STD_LOGIC;
    m_axis_rc_tready : IN STD_LOGIC; --_VECTOR(21 DOWNTO 0);
    m_axis_cq_tdata : OUT STD_LOGIC_VECTOR(255 DOWNTO 0);
    m_axis_cq_tuser : OUT STD_LOGIC_VECTOR(84 DOWNTO 0);
    m_axis_cq_tlast : OUT STD_LOGIC;
    m_axis_cq_tkeep : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    m_axis_cq_tvalid : OUT STD_LOGIC;
    m_axis_cq_tready : IN STD_LOGIC; --_VECTOR(21 DOWNTO 0);
    s_axis_cc_tdata : IN STD_LOGIC_VECTOR(255 DOWNTO 0);
    s_axis_cc_tuser : IN STD_LOGIC_VECTOR(32 DOWNTO 0);
    s_axis_cc_tlast : IN STD_LOGIC;
    s_axis_cc_tkeep : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    s_axis_cc_tvalid : IN STD_LOGIC;
    s_axis_cc_tready : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
    pcie_rq_seq_num : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
    pcie_rq_seq_num_vld : OUT STD_LOGIC;
    pcie_rq_tag : OUT STD_LOGIC_VECTOR(5 DOWNTO 0);
    pcie_rq_tag_vld : OUT STD_LOGIC;
    pcie_tfc_nph_av : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    pcie_tfc_npd_av : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    pcie_cq_np_req : IN STD_LOGIC;
    pcie_cq_np_req_count : OUT STD_LOGIC_VECTOR(5 DOWNTO 0);
    cfg_phy_link_down : OUT STD_LOGIC;
    cfg_phy_link_status : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    cfg_negotiated_width : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_current_speed : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
    cfg_max_payload : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
    cfg_max_read_req : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
    cfg_function_status : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    cfg_function_power_state : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
    cfg_vf_status : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    cfg_vf_power_state : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
    cfg_link_power_state : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    cfg_mgmt_addr : IN STD_LOGIC_VECTOR(18 DOWNTO 0);
    cfg_mgmt_write : IN STD_LOGIC;
    cfg_mgmt_write_data : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    cfg_mgmt_byte_enable : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_mgmt_read : IN STD_LOGIC;
    cfg_mgmt_read_data : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    cfg_mgmt_read_write_done : OUT STD_LOGIC;
    cfg_mgmt_type1_cfg_reg_access : IN STD_LOGIC;
    cfg_err_cor_out : OUT STD_LOGIC;
    cfg_err_nonfatal_out : OUT STD_LOGIC;
    cfg_err_fatal_out : OUT STD_LOGIC;
    cfg_ltr_enable : OUT STD_LOGIC;
    cfg_ltssm_state : OUT STD_LOGIC_VECTOR(5 DOWNTO 0);
    cfg_rcb_status : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_dpa_substate_change : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_obff_enable : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    cfg_pl_status_change : OUT STD_LOGIC;
    cfg_tph_requester_enable : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_tph_st_mode : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
    cfg_vf_tph_requester_enable : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_vf_tph_st_mode : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
    cfg_msg_received : OUT STD_LOGIC;
    cfg_msg_received_data : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_msg_received_type : OUT STD_LOGIC_VECTOR(4 DOWNTO 0);
    cfg_msg_transmit : IN STD_LOGIC;
    cfg_msg_transmit_type : IN STD_LOGIC_VECTOR(2 DOWNTO 0);
    cfg_msg_transmit_data : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    cfg_msg_transmit_done : OUT STD_LOGIC;
    cfg_fc_ph : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_fc_pd : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
    cfg_fc_nph : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_fc_npd : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
    cfg_fc_cplh : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_fc_cpld : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
    cfg_fc_sel : IN STD_LOGIC_VECTOR(2 DOWNTO 0);
    cfg_per_func_status_control : IN STD_LOGIC_VECTOR(2 DOWNTO 0);
    cfg_per_func_status_data : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    cfg_per_function_number : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_per_function_output_request : IN STD_LOGIC;
    cfg_per_function_update_done : OUT STD_LOGIC;
    --cfg_subsys_vend_id : in std_logic_vector(15 downto 0);
    cfg_dsn : IN STD_LOGIC_VECTOR(63 DOWNTO 0);
    cfg_power_state_change_ack : IN STD_LOGIC;
    cfg_power_state_change_interrupt : OUT STD_LOGIC;
    cfg_err_cor_in : IN STD_LOGIC;
    cfg_err_uncor_in : IN STD_LOGIC;
    cfg_flr_in_process : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_flr_done : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_vf_flr_in_process : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_vf_flr_done : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_link_training_enable : IN STD_LOGIC;
    cfg_ext_read_received : OUT STD_LOGIC;
    cfg_ext_write_received : OUT STD_LOGIC;
    cfg_ext_register_number : OUT STD_LOGIC_VECTOR(9 DOWNTO 0);
    cfg_ext_function_number : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_ext_write_data : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    cfg_ext_write_byte_enable : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_ext_read_data : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    cfg_ext_read_data_valid : IN STD_LOGIC;
    cfg_interrupt_int : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_interrupt_pending : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_interrupt_sent : OUT STD_LOGIC;
    cfg_interrupt_msi_enable : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_interrupt_msi_vf_enable : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_interrupt_msi_mmenable : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
    cfg_interrupt_msi_mask_update : OUT STD_LOGIC;
    cfg_interrupt_msi_data : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    cfg_interrupt_msi_select : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_interrupt_msi_int : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    cfg_interrupt_msi_pending_status : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    cfg_interrupt_msi_pending_status_data_enable : IN STD_LOGIC;
    cfg_interrupt_msi_pending_status_function_num : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_interrupt_msi_sent : OUT STD_LOGIC;
    cfg_interrupt_msi_fail : OUT STD_LOGIC;
    cfg_interrupt_msi_attr : IN STD_LOGIC_VECTOR(2 DOWNTO 0);
    cfg_interrupt_msi_tph_present : IN STD_LOGIC;
    cfg_interrupt_msi_tph_type : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
    cfg_interrupt_msi_tph_st_tag : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
    cfg_interrupt_msi_function_number : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
    cfg_hot_reset_out : OUT STD_LOGIC;
    cfg_config_space_enable : IN STD_LOGIC;
    cfg_req_pm_transition_l23_ready : IN STD_LOGIC;
    cfg_hot_reset_in : IN STD_LOGIC;
    cfg_ds_port_number : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_ds_bus_number : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    cfg_ds_device_number : IN STD_LOGIC_VECTOR(4 DOWNTO 0);
    cfg_ds_function_number : IN STD_LOGIC_VECTOR(2 DOWNTO 0);
    sys_clk : IN STD_LOGIC;
    sys_clk_gt : IN STD_LOGIC;
    sys_reset : IN STD_LOGIC;
    --cfg_vend_id : in std_logic_vector(15 downto 0);
    --cfg_dev_id : in std_logic_vector(15 downto 0);
    --cfg_rev_id : in std_Logic_vector(7 downto 0);
    cfg_subsys_vend_id : in std_logic_vector(15 downto 0);
    --cfg_subsys_id : in std_logic_vector(15 downto 0);
    --pcie_perstn1_in : in std_Logic;
    --pcie_perstn0_out : out std_logic;
    --pcie_perstn1_out : out std_logic;
    int_qpll1lock_out : out std_logic_vector(1 downto 0);
    int_qpll1outrefclk_out : out std_logic_vector(1 downto 0);
    int_qpll1outclk_out : out std_logic_vector(1 downto 0)
    
    --icap_clk : IN STD_LOGIC;
    --icap_csib : IN STD_LOGIC;
    --icap_rdwrb : IN STD_LOGIC;
    --icap_i : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    --icap_o : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    --startup_cfgclk : OUT STD_LOGIC;
    --startup_cfgmclk : OUT STD_LOGIC;
    --startup_eos : OUT STD_LOGIC;
    --startup_preq : OUT STD_LOGIC;
    --startup_clk : IN STD_LOGIC;
    --startup_gsr : IN STD_LOGIC;
    --startup_gts : IN STD_LOGIC;
    --startup_keyclearb : IN STD_LOGIC;
    --startup_pack : IN STD_LOGIC;
    --startup_usrcclko : IN STD_LOGIC;
    --startup_usrcclkts : IN STD_LOGIC;
    --startup_usrdoneo : IN STD_LOGIC;
    --startup_usrdonets : IN STD_LOGIC
  );
END COMPONENT;
--ATTRIBUTE SYN_BLACK_BOX : BOOLEAN;
--ATTRIBUTE SYN_BLACK_BOX OF pcie3_7x_0 : COMPONENT IS TRUE;
--ATTRIBUTE BLACK_BOX_PAD_PIN : STRING;
--ATTRIBUTE BLACK_BOX_PAD_PIN OF pcie3_7x_0 : COMPONENT IS "pci_exp_txn[7:0],pci_exp_txp[7:0],pci_exp_rxn[7:0],pci_exp_rxp[7:0],pipe_pclk_in,pipe_rxusrclk_in,pipe_rxoutclk_in[7:0],pipe_dclk_in,pipe_userclk1_in,pipe_userclk2_in,pipe_oobclk_in,pipe_mmcm_lock_in,pipe_txoutclk_out,pipe_rxoutclk_out[7:0],pipe_pclk_sel_out[7:0],pipe_gen3_out,pipe_mmcm_rst_n,user_clk,user_reset,user_lnk_up,user_app_rdy,s_axis_rq_tlast,s_axis_rq_tdata[255:0],s_axis_rq_tuser[59:0],s_axis_rq_tkeep[7:0],s_axis_rq_tready[3:0],s_axis_rq_tvalid,m_axis_rc_tdata[255:0],m_axis_rc_tuser[74:0],m_axis_rc_tlast,m_axis_rc_tkeep[7:0],m_axis_rc_tvalid,m_axis_rc_tready[21:0],m_axis_cq_tdata[255:0],m_axis_cq_tuser[84:0],m_axis_cq_tlast,m_axis_cq_tkeep[7:0],m_axis_cq_tvalid,m_axis_cq_tready[21:0],s_axis_cc_tdata[255:0],s_axis_cc_tuser[32:0],s_axis_cc_tlast,s_axis_cc_tkeep[7:0],s_axis_cc_tvalid,s_axis_cc_tready[3:0],pcie_rq_seq_num[3:0],pcie_rq_seq_num_vld,pcie_rq_tag[5:0],pcie_rq_tag_vld,pcie_tfc_nph_av[1:0],pcie_tfc_npd_av[1:0],pcie_cq_np_req,pcie_cq_np_req_count[5:0],cfg_phy_link_down,cfg_phy_link_status[1:0],cfg_negotiated_width[3:0],cfg_current_speed[2:0],cfg_max_payload[2:0],cfg_max_read_req[2:0],cfg_function_status[7:0],cfg_function_power_state[5:0],cfg_vf_status[11:0],cfg_vf_power_state[17:0],cfg_link_power_state[1:0],cfg_mgmt_addr[18:0],cfg_mgmt_write,cfg_mgmt_write_data[31:0],cfg_mgmt_byte_enable[3:0],cfg_mgmt_read,cfg_mgmt_read_data[31:0],cfg_mgmt_read_write_done,cfg_mgmt_type1_cfg_reg_access,cfg_err_cor_out,cfg_err_nonfatal_out,cfg_err_fatal_out,cfg_ltr_enable,cfg_ltssm_state[5:0],cfg_rcb_status[1:0],cfg_dpa_substate_change[1:0],cfg_obff_enable[1:0],cfg_pl_status_change,cfg_tph_requester_enable[1:0],cfg_tph_st_mode[5:0],cfg_vf_tph_requester_enable[5:0],cfg_vf_tph_st_mode[17:0],cfg_msg_received,cfg_msg_received_data[7:0],cfg_msg_received_type[4:0],cfg_msg_transmit,cfg_msg_transmit_type[2:0],cfg_msg_transmit_data[31:0],cfg_msg_transmit_done,cfg_fc_ph[7:0],cfg_fc_pd[11:0],cfg_fc_nph[7:0],cfg_fc_npd[11:0],cfg_fc_cplh[7:0],cfg_fc_cpld[11:0],cfg_fc_sel[2:0],cfg_per_func_status_control[2:0],cfg_per_func_status_data[15:0],cfg_per_function_number[2:0],cfg_per_function_output_request,cfg_per_function_update_done,cfg_dsn[63:0],cfg_power_state_change_ack,cfg_power_state_change_interrupt,cfg_err_cor_in,cfg_err_uncor_in,cfg_flr_in_process[1:0],cfg_flr_done[1:0],cfg_vf_flr_in_process[5:0],cfg_vf_flr_done[5:0],cfg_link_training_enable,cfg_ext_read_received,cfg_ext_write_received,cfg_ext_register_number[9:0],cfg_ext_function_number[7:0],cfg_ext_write_data[31:0],cfg_ext_write_byte_enable[3:0],cfg_ext_read_data[31:0],cfg_ext_read_data_valid,cfg_interrupt_int[3:0],cfg_interrupt_pending[1:0],cfg_interrupt_sent,cfg_interrupt_msi_enable[1:0],cfg_interrupt_msi_vf_enable[5:0],cfg_interrupt_msi_mmenable[5:0],cfg_interrupt_msi_mask_update,cfg_interrupt_msi_data[31:0],cfg_interrupt_msi_select[3:0],cfg_interrupt_msi_int[31:0],cfg_interrupt_msi_pending_status[63:0],cfg_interrupt_msi_sent,cfg_interrupt_msi_fail,cfg_interrupt_msi_attr[2:0],cfg_interrupt_msi_tph_present,cfg_interrupt_msi_tph_type[1:0],cfg_interrupt_msi_tph_st_tag[8:0],cfg_interrupt_msi_function_number[2:0],cfg_hot_reset_out,cfg_config_space_enable,cfg_req_pm_transition_l23_ready,cfg_hot_reset_in,cfg_ds_port_number[7:0],cfg_ds_bus_number[7:0],cfg_ds_device_number[4:0],cfg_ds_function_number[2:0],sys_clk,sys_reset,icap_clk,icap_csib,icap_rdwrb,icap_i[31:0],icap_o[31:0],startup_cfgclk,startup_cfgmclk,startup_eos,startup_preq,startup_clk,startup_gsr,startup_gts,startup_keyclearb,startup_pack,startup_usrcclko,startup_usrcclkts,startup_usrdoneo,startup_usrdonets";

  signal user_clk     : std_logic;
  signal user_reset   : std_logic;
  signal user_lnk_up  : std_logic;
  signal user_app_rdy : std_logic;


  signal cfg_mgmt_addr                 : std_logic_vector(18 downto 0) := (others => '0');
  signal cfg_mgmt_write                : std_logic                     := '0';
  signal cfg_mgmt_write_data           : std_logic_vector(31 downto 0) := (others => '0');
  signal cfg_mgmt_byte_enable          : std_logic_vector(3 downto 0)  := (others => '0');
  signal cfg_mgmt_read                 : std_logic                     := '0';
  signal cfg_mgmt_read_data            : std_logic_vector(31 downto 0);
  signal cfg_mgmt_read_write_done      : std_logic;
  signal cfg_mgmt_type1_cfg_reg_access : std_logic                     := '0';
  signal cfg_err_cor_out               : std_logic;
  signal cfg_err_nonfatal_out          : std_logic;
  signal cfg_err_fatal_out             : std_logic;
  signal cfg_ltr_enable                : std_logic;

  signal cfg_rcb_status              : std_logic_vector(3 downto 0);
  signal cfg_dpa_substate_change     : std_logic_vector(3 downto 0);
  signal cfg_obff_enable             : std_logic_vector(1 downto 0);
  signal cfg_pl_status_change        : std_logic;
  signal cfg_tph_requester_enable    : std_logic_vector(3 downto 0);
  signal cfg_tph_st_mode             : std_logic_vector(11 downto 0);
  signal cfg_vf_tph_requester_enable : std_logic_vector(7 downto 0);
  signal cfg_vf_tph_st_mode          : std_logic_vector(23 downto 0);

  signal cfg_msg_transmit                 : std_logic                     := '0';
  signal cfg_msg_transmit_type            : std_logic_vector(2 downto 0)  := (others => '0');
  signal cfg_msg_transmit_data            : std_logic_vector(31 downto 0) := (others => '0');
  signal cfg_msg_transmit_done            : std_logic;
  signal cfg_fc_ph                        : std_logic_vector(7 downto 0);
  signal cfg_fc_pd                        : std_logic_vector(11 downto 0);
  signal cfg_fc_nph                       : std_logic_vector(7 downto 0);
  signal cfg_fc_npd                       : std_logic_vector(11 downto 0);
  signal cfg_fc_cplh                      : std_logic_vector(7 downto 0);
  signal cfg_fc_cpld                      : std_logic_vector(11 downto 0);
  signal cfg_fc_sel                       : std_logic_vector(2 downto 0)  := (others => '0');
  signal cfg_per_func_status_control      : std_logic_vector(2 downto 0)  := (others => '0');
  signal cfg_per_func_status_data         : std_logic_vector(15 downto 0);
  signal cfg_per_function_number          : std_logic_vector(3 downto 0)  := (others => '0');
  signal cfg_per_function_output_request  : std_logic                     := '0';
  signal cfg_per_function_update_done     : std_logic;
  signal cfg_dsn                          : std_logic_vector(63 downto 0) := X"08000A3500000001"; --
  --(others => '0');
  signal cfg_power_state_change_ack       : std_logic                     := '0';
  signal cfg_power_state_change_interrupt : std_logic;
  signal cfg_err_cor_in                   : std_logic                     := '0';
  signal cfg_err_uncor_in                 : std_logic                     := '0';
  signal cfg_flr_in_process               : std_logic_vector(3 downto 0);
  signal cfg_flr_done                     : std_logic_vector(3 downto 0)  := (others => '0');
  signal cfg_flr_done_r0                     : std_logic_vector(3 downto 0)  := (others => '0');
  signal cfg_flr_done_r1                     : std_logic_vector(3 downto 0)  := (others => '0');
  signal cfg_vf_flr_in_process            : std_logic_vector(7 downto 0);
  signal cfg_vf_flr_done                  : std_logic_vector(7 downto 0)  := (others => '0');
  signal cfg_vf_flr_done_r0                  : std_logic_vector(7 downto 0)  := (others => '0');
  signal cfg_vf_flr_done_r1                  : std_logic_vector(7 downto 0)  := (others => '0');
  signal cfg_link_training_enable         : std_logic                     := '1';
  signal cfg_ext_read_received            : std_logic;
  signal cfg_ext_write_received           : std_logic;
  signal cfg_ext_register_number          : std_logic_vector(9 downto 0);
  signal cfg_ext_function_number          : std_logic_vector(7 downto 0);
  signal cfg_ext_write_data               : std_logic_vector(31 downto 0);
  signal cfg_ext_write_byte_enable        : std_logic_vector(3 downto 0);
  signal cfg_ext_read_data                : std_logic_vector(31 downto 0) := (others => '0');
  signal cfg_ext_read_data_valid          : std_logic                     := '0';
  signal cfg_ds_port_number               : std_logic_vector(7 downto 0)  := (others => '0');

  signal cfg_interrupt_msix_enable         : std_logic_vector(1 downto 0);
  signal cfg_interrupt_msix_mask           : std_logic_vector(1 downto 0);
  signal cfg_interrupt_msix_vf_enable      : std_logic_vector(5 downto 0);
  signal cfg_interrupt_msix_vf_mask        : std_logic_vector(5 downto 0);
  signal cfg_interrupt_msix_data           : std_logic_vector(31 downto 0) := (others => '0');
  signal cfg_interrupt_msix_address        : std_logic_vector(63 downto 0) := (others => '0');
  signal cfg_interrupt_msix_int            : std_logic                     := '0';
  signal cfg_interrupt_msix_sent           : std_logic;
  signal cfg_interrupt_msix_fail           : std_logic;
  signal cfg_interrupt_msi_attr            : std_logic_vector(2 downto 0)  := (others => '0');
  signal cfg_interrupt_msi_tph_present     : std_logic                     := '0';
  signal cfg_interrupt_msi_tph_type        : std_logic_vector(1 downto 0)  := (others => '0');
  signal cfg_interrupt_msi_tph_st_tag      : std_logic_vector(8 downto 0)  := (others => '0');
  signal cfg_interrupt_msi_function_number : std_logic_vector(3 downto 0)  := (others => '0');

  signal cfg_hot_reset_out               : std_logic;
  signal cfg_config_space_enable         : std_logic                    := '1';
  signal cfg_req_pm_transition_l23_ready : std_logic                    := '0';
  signal cfg_hot_reset_in                : std_logic                    := '0';
  signal cfg_ds_bus_number               : std_logic_vector(7 downto 0) := (others => '0');
  signal cfg_ds_device_number            : std_logic_vector(4 downto 0) := (others => '0');
  signal cfg_ds_function_number          : std_logic_vector(2 downto 0) := (others => '0');

  signal sys_reset                    : std_logic;
  signal user_tph_stt_address         : std_logic_vector(4 downto 0)  := (others => '0');
  signal user_tph_function_num        : std_logic_vector(2 downto 0)  := (others => '0');
  signal user_tph_stt_read_data       : std_logic_vector(31 downto 0) := (others => '0');
  signal user_tph_stt_read_data_valid : std_logic                     := '0';
  signal user_tph_stt_read_enable     : std_logic                     := '0';


  signal m_axis_rc_tready                  : std_logic; --_vector(21 downto 0);
  signal m_axis_cq_tready                  : std_logic; --_vector(21 downto 0);
  signal s_axis_rq_tready                  : std_logic_vector(3 downto 0);
  signal s_axis_cc_tready                  : std_logic_vector(3 downto 0);

  signal pcie_config_status_i : config_c2b_t;
  
begin  -- architecture rtl

  

  sys_reset   <= not sys_reset_n;
  trn_clk     <= user_clk;
  trn_reset_n <= not user_reset;

  --gen_fix0: for i in 0 to 21 generate
    m_axis_rc_tready <= rc_out.m_axis_rc_tready;
    m_axis_cq_tready <= cq_out.m_axis_cq_tready;
  --end generate gen_fix0;
  cc_in.s_axis_cc_tready <= s_axis_cc_tready(0);
  rq_in.s_axis_rq_tready <= s_axis_rq_tready(0);


  cq_in.m_axis_cq_tparity <= (others => '0');
  rq_in.pcie_rq_tab_av <= (others => '0');
  rc_in.m_axis_rc_tparity <= (others => '0');
  int_in.cfg_interrupt_msi_fail <= '0';
  
  pcie_core_i : pcie3_ultrascale_1
    generic map (
      pf0_bar2_aperture_size => pf0_bar2_aperture_size,
      pf0_bar4_aperture_size => pf0_bar4_aperture_size)
    port map (
      pci_exp_txn                       => pci_exp_txn,
      pci_exp_txp                       => pci_exp_txp,
      pci_exp_rxn                       => pci_exp_rxn,
      pci_exp_rxp                       => pci_exp_rxp,
  --     int_pclk_out_slave => open,
  --int_pipe_rxusrclk_out => open,
  --int_rxoutclk_out => open,
  --int_dclk_out => open,
  --int_userclk1_out => open,
  --int_userclk2_out => open,
  --int_oobclk_out => open,
  --int_qplllock_out => open,
  --int_qplloutclk_out => open,
  --int_qplloutrefclk_out => open,
  --int_pclk_sel_slave => X"00",
      user_clk                          => user_clk,
      user_reset                        => user_reset,
      user_lnk_up                       => user_lnk_up,
    --  user_app_rdy                      => user_app_rdy,
      s_axis_rq_tlast                   => rq_out.s_axis_rq_tlast,
      s_axis_rq_tdata                   => rq_out.s_axis_rq_tdata,
      s_axis_rq_tuser                   => rq_out.s_axis_rq_tuser,
      s_axis_rq_tkeep                   => rq_out.s_axis_rq_tkeep,
      s_axis_rq_tready                  => s_axis_rq_tready,
      s_axis_rq_tvalid                  => rq_out.s_axis_rq_tvalid,
      m_axis_rc_tdata                   => rc_in.m_axis_rc_tdata,
      m_axis_rc_tuser                   => rc_in.m_axis_rc_tuser,
      m_axis_rc_tlast                   => rc_in.m_axis_rc_tlast,
      m_axis_rc_tkeep                   => rc_in.m_axis_rc_tkeep,
      m_axis_rc_tvalid                  => rc_in.m_axis_rc_tvalid,
      m_axis_rc_tready                   => m_axis_rc_tready,
      m_axis_cq_tdata                   => cq_in.m_axis_cq_tdata,
      m_axis_cq_tuser                   => cq_in.m_axis_cq_tuser,
      m_axis_cq_tlast                   => cq_in.m_axis_cq_tlast,
      m_axis_cq_tkeep                   => cq_in.m_axis_cq_tkeep,
      m_axis_cq_tvalid                  => cq_in.m_axis_cq_tvalid,
      m_axis_cq_tready                  => m_axis_cq_tready,
      s_axis_cc_tdata                   => cc_out.s_axis_cc_tdata,
      s_axis_cc_tuser                   => cc_out.s_axis_cc_tuser,
      s_axis_cc_tlast                   => cc_out.s_axis_cc_tlast,
      s_axis_cc_tkeep                   => cc_out.s_axis_cc_tkeep,
      s_axis_cc_tvalid                  => cc_out.s_axis_cc_tvalid,
      s_axis_cc_tready                  => s_axis_cc_tready,
      pcie_rq_seq_num                   => rq_in.pcie_rq_seq_num,
      pcie_rq_seq_num_vld               => rq_in.pcie_rq_seq_num_vld,
      pcie_rq_tag                       => rq_in.pcie_rq_tag,
      pcie_rq_tag_vld                   => rq_in.pcie_rq_tag_vld,
      pcie_tfc_nph_av                   => rq_in.pcie_tfc_nph,
      pcie_tfc_npd_av                   => rq_in.pcie_tfc_npd,
      pcie_cq_np_req                    => '1',
      pcie_cq_np_req_count              => cq_in.pcie_cq_np_req_count,
      cfg_phy_link_down                 => pcie_config_status_i.cfg_phy_link_down,
      cfg_phy_link_status               => pcie_config_status_i.cfg_phy_link_status,
      cfg_negotiated_width              => pcie_config_status_i.cfg_negotiated_width,
      cfg_current_speed                 => pcie_config_status_i.cfg_current_speed,
      cfg_max_payload                   => pcie_config_status_i.cfg_max_payload,
      cfg_max_read_req                  => pcie_config_status_i.cfg_max_read_req,
      cfg_function_status               => pcie_config_status_i.cfg_function_status,
      cfg_function_power_state          => pcie_config_status_i.cfg_function_power_state,
      cfg_vf_status                     => pcie_config_status_i.cfg_vf_status,
      cfg_vf_power_state                => pcie_config_status_i.cfg_vf_power_state,
      cfg_link_power_state              => pcie_config_status_i.cfg_link_power_state,
      cfg_mgmt_addr                     => cfg_mgmt_addr,
      cfg_mgmt_write                    => cfg_mgmt_write,
      cfg_mgmt_write_data               => cfg_mgmt_write_data,
      cfg_mgmt_byte_enable              => cfg_mgmt_byte_enable,
      cfg_mgmt_read                     => cfg_mgmt_read,
      cfg_mgmt_read_data                => cfg_mgmt_read_data,
      cfg_mgmt_read_write_done          => cfg_mgmt_read_write_done,
      cfg_mgmt_type1_cfg_reg_access     => cfg_mgmt_type1_cfg_reg_access,
      cfg_err_cor_out                   => cfg_err_cor_out,
      cfg_err_nonfatal_out              => cfg_err_nonfatal_out,
      cfg_err_fatal_out                 => cfg_err_fatal_out,
      cfg_ltr_enable                    => cfg_ltr_enable,
      cfg_ltssm_state                   => pcie_config_status_i.cfg_ltssm_state,
      cfg_rcb_status                    => cfg_rcb_status,
      cfg_dpa_substate_change           => cfg_dpa_substate_change,
      cfg_obff_enable                   => cfg_obff_enable,
      cfg_pl_status_change              => cfg_pl_status_change,
      cfg_tph_requester_enable          => cfg_tph_requester_enable,
      cfg_tph_st_mode                   => cfg_tph_st_mode,
      cfg_vf_tph_requester_enable       => cfg_vf_tph_requester_enable,
      cfg_vf_tph_st_mode                => cfg_vf_tph_st_mode,
      cfg_msg_received                  => messages.cfg_msg_received,
      cfg_msg_received_data             => messages.cfg_msg_received_data,
      cfg_msg_received_type             => messages.cfg_msg_received_type,
      cfg_msg_transmit                  => cfg_msg_transmit,
      cfg_msg_transmit_type             => cfg_msg_transmit_type,
      cfg_msg_transmit_data             => cfg_msg_transmit_data,
      cfg_msg_transmit_done             => cfg_msg_transmit_done,
      cfg_fc_ph                         => cfg_fc_ph,
      cfg_fc_pd                         => cfg_fc_pd,
      cfg_fc_nph                        => cfg_fc_nph,
      cfg_fc_npd                        => cfg_fc_npd,
      cfg_fc_cplh                       => cfg_fc_cplh,
      cfg_fc_cpld                       => cfg_fc_cpld,
      cfg_fc_sel                        => pcie_config_cfg_fc_sel,
      cfg_per_func_status_control       => cfg_per_func_status_control,
      cfg_per_func_status_data          => cfg_per_func_status_data,
      cfg_per_function_number           => cfg_per_function_number,
      cfg_per_function_output_request   => cfg_per_function_output_request,
      cfg_per_function_update_done      => cfg_per_function_update_done,
      --cfg_subsys_vend_id                => X"4144",
      cfg_dsn                           => cfg_dsn,
      cfg_power_state_change_ack        => cfg_power_state_change_ack,
      cfg_power_state_change_interrupt  => cfg_power_state_change_interrupt,
      cfg_err_cor_in                    => cfg_err_cor_in,
      cfg_err_uncor_in                  => cfg_err_uncor_in,
      cfg_flr_in_process                => cfg_flr_in_process,
      cfg_flr_done                      => cfg_flr_done,
      cfg_vf_flr_in_process             => cfg_vf_flr_in_process,
      cfg_vf_flr_done                   => cfg_vf_flr_done,
      cfg_link_training_enable          => cfg_link_training_enable,
      cfg_ext_read_received             => cfg_ext_read_received,
      cfg_ext_write_received            => cfg_ext_write_received,
      cfg_ext_register_number           => cfg_ext_register_number,
      cfg_ext_function_number           => cfg_ext_function_number,
      cfg_ext_write_data                => cfg_ext_write_data,
      cfg_ext_write_byte_enable         => cfg_ext_write_byte_enable,
      cfg_ext_read_data                 => cfg_ext_read_data,
      cfg_ext_read_data_valid           => cfg_ext_read_data_valid,
      cfg_ds_port_number                => cfg_ds_port_number,
      cfg_interrupt_int                 => int_out.cfg_interrupt_int,
      cfg_interrupt_pending             => int_out.cfg_interrupt_pending,
      cfg_interrupt_sent                => int_in.cfg_interrupt_sent,
      cfg_interrupt_msi_enable          => int_in.cfg_interrupt_msi_enable,
      cfg_interrupt_msi_vf_enable       => open,
      cfg_interrupt_msi_mmenable        => int_in.cfg_interrupt_msi_mmenable,
      cfg_interrupt_msi_mask_update     => int_in.cfg_interrupt_msi_mask_update,
      cfg_interrupt_msi_data            => int_in.cfg_interrupt_msi_data,
      cfg_interrupt_msi_select          => int_out.cfg_interrupt_msi_select,
      cfg_interrupt_msi_int             => int_out.cfg_interrupt_msi_int,
      cfg_interrupt_msi_pending_status  => int_out.cfg_interrupt_msi_pending_status,
      cfg_interrupt_msi_pending_status_data_enable => '0',
      cfg_interrupt_msi_pending_status_function_num => "0000",
      cfg_interrupt_msi_sent            => int_in.cfg_interrupt_msi_sent,
      --cfg_interrupt_msi_fail            => int_in.cfg_interrupt_msi_fail,
      --cfg_interrupt_msix_enable         => cfg_interrupt_msix_enable,
      --cfg_interrupt_msix_mask           => cfg_interrupt_msix_mask,
      --cfg_interrupt_msix_vf_enable      => cfg_interrupt_msix_vf_enable,
      --cfg_interrupt_msix_vf_mask        => cfg_interrupt_msix_vf_mask,
      --cfg_interrupt_msix_data           => cfg_interrupt_msix_data,
      --cfg_interrupt_msix_address        => cfg_interrupt_msix_address,
      --cfg_interrupt_msix_int            => cfg_interrupt_msix_int,
      --cfg_interrupt_msix_sent           => cfg_interrupt_msix_sent,
      --cfg_interrupt_msix_fail           => cfg_interrupt_msix_fail,
      cfg_interrupt_msi_attr            => cfg_interrupt_msi_attr,
      cfg_interrupt_msi_tph_present     => cfg_interrupt_msi_tph_present,
      cfg_interrupt_msi_tph_type        => cfg_interrupt_msi_tph_type,
      cfg_interrupt_msi_tph_st_tag      => cfg_interrupt_msi_tph_st_tag,
      cfg_interrupt_msi_function_number => cfg_interrupt_msi_function_number,
      cfg_hot_reset_out                 => cfg_hot_reset_out,
      cfg_config_space_enable           => cfg_config_space_enable,
      cfg_req_pm_transition_l23_ready   => cfg_req_pm_transition_l23_ready,
      cfg_hot_reset_in                  => cfg_hot_reset_in,
      cfg_ds_bus_number                 => cfg_ds_bus_number,
      cfg_ds_device_number              => cfg_ds_device_number,
      cfg_ds_function_number            => cfg_ds_function_number,

      --startup_cfgclk               => open,
      --startup_cfgmclk              => open,
      --startup_eos                  => open,
      --startup_preq                 => open,
      --startup_clk                  => '0',
      --startup_gsr                  => '0',
      --startup_gts                  => '0',
      --startup_keyclearb            => '1',
      --startup_pack                 => '0',
      --startup_usrcclko             => '0',
      --startup_usrcclkts            => '1',
      --startup_usrdoneo             => '0',
      --startup_usrdonets            => '1',
      --icap_clk                     => '0',
      --icap_csib                    => '0',
      --icap_rdwrb                   => '0',
      --icap_i                       => X"00000000",
      --icap_o                       => open,
      --cfg_vend_id => to_stdlogicvector(VENDOR_ID),
      --cfg_dev_id => to_stdlogicvector(DEVICE_ID),
      --cfg_rev_id => to_stdlogicvector(REVISION_ID),
      cfg_subsys_vend_id => to_stdlogicvector(SUBSYSTEM_VENDOR_ID),
      --cfg_subsys_id => to_stdlogicvector(SUBSYSTEM_ID),
      
      
      sys_clk                      => sys_clk,
      sys_clk_gt                   => sys_clk_gt,
      sys_reset                    => sys_reset_n,
      --pcie_perstn1_in              => '0',
      --pcie_perstn0_out => open,
      --pcie_perstn1_out => open,
      int_qpll1lock_out => open,
      int_qpll1outrefclk_out => open,
      int_qpll1outclk_out => open
      --user_tph_stt_address         => user_tph_stt_address,
      --user_tph_function_num        => user_tph_function_num,
      --user_tph_stt_read_data       => user_tph_stt_read_data,
      --user_tph_stt_read_data_valid => user_tph_stt_read_data_valid,
      --user_tph_stt_read_enable     => user_tph_stt_read_enable
      );

  feedback_flr_done: process (user_clk) is
  begin  -- process feedback_flr_done
    if rising_edge(user_clk) then            -- rising clock edge
      if user_reset = '1' then
        cfg_flr_done_r0 <= (others => '0');
        cfg_flr_done_r1 <= (others => '0');
        cfg_vf_flr_done_r0 <= (others => '0');
        cfg_vf_flr_done_r1 <= (others => '0');       
      else
        cfg_flr_done_r0 <= cfg_flr_in_process;
        cfg_vf_flr_done_r0 <= cfg_vf_flr_in_process;
        cfg_flr_done_r1 <= cfg_flr_done_r0;
        cfg_vf_flr_done_r1 <= cfg_vf_flr_done_r0;
      end if;

      pcie_config_status <= pcie_config_status_i;
    end if;
  end process feedback_flr_done;

  gen_flr_done0: for i in 0 to 1 generate
    cfg_flr_done(i) <= (not cfg_flr_done_r1(i)) and (cfg_flr_done_r0(i));
  end generate gen_flr_done0;
  cfg_flr_done(3 downto 2) <= "00";
  
  gen_vf_flr_done0: for i in 0 to 5 generate
    cfg_vf_flr_done(i) <= (not cfg_vf_flr_done_r1(i)) and (cfg_vf_flr_done_r0(i));
  end generate gen_vf_flr_done0;
  cfg_vf_flr_done(7 downto 6) <= "00";
  
  cfg_power_state_change_ack <= cfg_power_state_change_interrupt;

  
end architecture rtl;
