--------------------------------------------------------------------------------
--
-- Package Name      : adb3_ocp_axi4
--
--                    Copyright 2010 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : Package For Interfacing ADB3 OCP Type Logic With
--                    AXI-4 Compatible IP
--
--
-- Dependencies     : adb3_ocp
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

library work;
use work.adb3_ocp.all;

package adb3_ocp_axi4 is

  type axi4_m2sT is record
    -- Write Address/Control Channel
    awid : std_Logic_vector(0 downto 0);
    awaddr : std_logic_vector(63 downto 0);
    awlen : std_logic_vector(7 downto 0);
    awsize : std_Logic_vector(2 downto 0);
    awburst : std_logic_vector(1 downto 0);
    awlock : std_logic_vector(1 downto 0);
    awcache : std_logic_vector(3 downto 0);
    awprot : std_Logic_vector(2 downto 0);
    awqos : std_logic_vector(3 downto 0);
    awregion : std_logic_vector(3 downto 0);
    awvalid : std_logic;
    -- Write Data Channel
    --wid : std_logic_vector(0 downto 0);
    wdata : std_logic_vector(127 downto 0);
    wstrb : std_logic_vector(15 downto 0);
    wlast : std_logic;
    wvalid : std_logic;
    -- Write response Channel
    bready : std_logic;
    -- Read Address/Control Channel
    arid : std_logic_vector(0 downto 0);
    araddr : std_logic_vector(63 downto 0);
    arlen : std_logic_vector(7 downto 0);
    arsize : std_logic_vector(2 downto 0);
    arburst : std_logic_vector(1 downto 0);
    arlock : std_logic_vector(1 downto 0);
    arcache : std_logic_vector(3 downto 0);
    arprot : std_logic_vector(2 downto 0);
    arqos : std_logic_vector(3 downto 0);
    arregion : std_logic_vector(3 downto 0);
    arvalid : std_logic;
    -- Read Data Channel
    rready : std_logic;
  end record axi4_m2sT;
  
  type axi4_s2mT is record
    -- Write Address/Control Channel
    awready : std_logic;
    -- Write Data Channel
    wready : std_logic;
    -- Write response Channel
    bid : std_logic_vector(0 downto 0);
    bresp : std_logic_vector(1 downto 0);
    bvalid : std_logic;
    -- Read Address/Control Channel
    arready : std_logic;
    -- Read Data Channel
    rid : std_logic_vector(0 downto 0);
    rdata : std_logic_vector(127 downto 0);
    rresp : std_logic_vector(1 downto 0);
    rlast : std_logic;
    rvalid : std_logic;
  end record axi4_s2mT;


  type axi4_s2mT_array is array (natural range <>) of axi4_s2mT;
  type axi4_m2sT_array is array (natural range <>) of axi4_m2sT;

  constant AXI4_M2ST_ZEROS : axi4_m2sT :=
    (awaddr       => (others => '0'),
          awlen   => (others => '0'),
          awsize  => (others => '0'),
          awburst => (others => '0'),
          awcache => (others => '0'),
          awprot  => (others => '0'),
          awvalid => '0',
          awid   => (others => '0'),
          awlock  => (others => '0'),
          awqos  => (others => '0'),
          awregion  => (others => '0'),
          wdata   => (others => '0'),
          wstrb   => (others => '0'),
          wlast   => '0',
          wvalid  => '0',
          --wid   => (others => '0'),
          bready  => '0',
          araddr  => (others => '0'),
          arlen   => (others => '0'),
          arsize  => (others => '0'),
          arburst => (others => '0'),
          arcache => (others => '0'),
          arprot  => (others => '0'),
          arid   => (others => '0'),
          arlock  => (others => '0'),
          arqos  => (others => '0'),
          arregion  => (others => '0'),
          arvalid => '0',
          rready  => '0');
  constant AXI4_S2MT_ZEROS : axi4_s2mT :=
    (awready   => '0',
       wready  => '0',
       bresp   => (others => '0'),
       bvalid  => '0',
       bid   => (others => '0'),
       arready => '0',
       rdata   => (others => '0'),
       rresp   => (others => '0'),
       rlast   => '0',
       rvalid  => '0',
       rid   => (others => '0')
       );
  
  component adb3_ocp_to_axi4 is
    port (
      clk             : in  std_logic;
      rst             : in  std_logic;
      ocp_m2s         : in  adb3_ocp_m2sT;
      ocp_s2m         : out adb3_ocp_s2mT;
      axi_m2s         : out axi4_m2sT;
      axi_s2m         : in  axi4_s2mT;
      illegal_ocp_cmd : out std_logic);
  end component adb3_ocp_to_axi4;

  component axi4_to_adb3_ocp is
    port (
      clk     : in  std_logic;
      rst     : in  std_logic;
      axi_m2s : in  axi4_m2sT;
      axi_s2m : out axi4_s2mT;
      ocp_m2s : out adb3_ocp_m2sT;
      ocp_s2m : in  adb3_ocp_s2mT);
  end component axi4_to_adb3_ocp;

end package adb3_ocp_axi4;



-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- ADB3OCP2AXI4 module
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_axi4.all;

entity adb3_ocp_to_axi4 is
  port (
    clk     : in  std_logic;
    rst     : in  std_logic;
    ocp_m2s : in  adb3_ocp_m2sT;
    ocp_s2m : out adb3_ocp_s2mT;
    axi_m2s : out axi4_m2sT;
    axi_s2m : in  axi4_s2mT;
    illegal_ocp_cmd : out std_logic := '0');

end entity adb3_ocp_to_axi4;

architecture rtl of adb3_ocp_to_axi4 is

  signal burst_counter : std_logic_vector(7 downto 0) := (others => '0');

  type tag_table_type is array (0 to 15) of std_logic_vector(ADB3_OCP_TAG_WIDTH-1 downto 0);
  signal tag_table : tag_table_type := (others => (others => '0'));
  signal tag_counter : std_logic_vector(3 downto 0) := "0000";
  
  signal read_busy : std_logic := '0';
  signal write_busy : std_logic := '0';

  signal last_tag_response : std_logic_vector(3 downto 0) := (others => '0');
  signal tags_in_flight : std_logic_vector(3 downto 0) := (others => '0');
  signal write_tag : std_logic_vector(3 downto 0) := (others => '0');
  signal writes_in_flight : std_logic_vector(3 downto 0) := (others => '0');
  signal no_writes_in_flight : std_logic := '1';
  
begin  -- architecture rtl

  -- Track Tags to map 8 bit OCP tags to 4 bit AXI tags
  -- Only needed on Reads
  tag_table0: process (clk) is
  begin  -- process tag_table
    if rising_edge(clk) then            -- rising clock edge
      if ocp_m2s.Cmd = OCP_CMD_READ and axi_s2m.arready = '1' and read_busy = '0'  and no_writes_in_flight = '1' then
        tag_table(CONV_INTEGER(tag_counter)) <= ocp_m2s.Tag;
        tag_counter <= tag_counter+1;
      end if;
      if axi_s2m.rvalid = '1' and axi_s2m.rlast = '1' and ocp_m2s.RespAccept = '1' then
        last_tag_response <= last_tag_response+1;
      end if;
      tags_in_flight <= tag_counter - last_tag_response;
      read_busy <= tags_in_flight(3);
      if rst = '1' then
        tag_counter <= (others => '0');
        last_tag_response <= (others => '0');
      end if;
      no_writes_in_flight <= not OR_reduce(writes_in_flight);
    end if;
  end process tag_table0;

  -- Track write burst data count
  -- Only needed on writes
  -- Simpler lower perforamnce option, write data waits until 1 cycle after
  -- CMD
  bc0: process (clk,rst) is
   begin  -- process bc0
     if rst = '1' then
       burst_counter <= (others => '0');
       illegal_ocp_cmd <= '0';
       writes_in_flight <= (others => '0');
     elsif rising_edge(clk) then           -- rising clock edge
       if ocp_m2s.Cmd = OCP_CMD_WRITE and axi_s2m.awready = '1' and write_busy = '0' then
         burst_counter <= ocp_m2s.BurstLength(7 downto 0);
         if OR_reduce(ocp_m2s.BurstLength(ADB3_OCP_BURST_WIDTH-1 downto 8)) = '1'  then
           illegal_ocp_cmd <= '1';
         end if;
         if axi_s2m.bvalid /= '1' then
           -- Ignore BRESP as decrement count even if error
           writes_in_flight <= writes_in_flight+1;
         end if;
        
       else
         if ocp_m2s.DataValid = '1' and axi_s2m.wready = '1' and write_busy = '1'then
           burst_counter <= burst_counter -1;
         end if;
         if axi_s2m.bvalid ='1' then
           -- Ignore BRESP as decrement count even if error
           writes_in_flight <= writes_in_flight-1;
         end if;
       end if;
     end if;
   end process bc0; 

   write_busy <= OR_reduce(burst_counter);


   axi_m2s.awid <= "0";
   axi_m2s.awaddr <= ocp_m2s.Addr; --(31 downto 0);
   axi_m2s.awlen <= ocp_m2s.BurstLength(7 downto 0)-1;
   axi_m2s.awsize <= "100";             -- 128 bits
   axi_m2s.awburst <= "01";             -- INCR
   axi_m2s.awlock <= "00";               -- Not Used
   axi_m2s.awcache <= "0011";           -- Xilinx recommended setting
   axi_m2s.awprot <= "000";             -- Xilinx recommended setting           
   axi_m2s.awqos <= "0000";             -- Not Used
   axi_m2s.awregion <= "0000";          -- Not Used
   axi_m2s.awvalid <= '1' when write_busy = '0' and ocp_m2s.Cmd = OCP_CMD_WRITE else '0';

   --axi_m2s.wid <= "0";
   axi_m2s.wdata <= ocp_m2s.Data;
   axi_m2s.wstrb <= ocp_m2s.DataByteEn;
   axi_m2s.wlast <= '1' when burst_counter = "00000001" else '0';
   axi_m2s.wvalid <= ocp_m2s.DataValid and write_busy;

   axi_m2s.bready <= '1';               -- Always accept and ignore BRESP

   axi_m2s.arid <= "0";
   axi_m2s.araddr <= ocp_m2s.Addr; --(31 downto 0);
   axi_m2s.arlen <= ocp_m2s.BurstLength(7 downto 0)-1;
   axi_m2s.arsize <= "100";             -- 128 bits
   axi_m2s.arburst <= "01";             -- INCR
   axi_m2s.arlock <= "00";               -- Not Used
   axi_m2s.arcache <= "0011";           -- Xilinx recommended setting
   axi_m2s.arprot <= "000";             -- Xilinx recommended setting           
   axi_m2s.arqos <= "0000";             -- Not Used
   axi_m2s.arregion <= "0000";          -- Not Used
   axi_m2s.arvalid <= '1' when read_busy = '0' and ocp_m2s.Cmd = OCP_CMD_READ and no_writes_in_flight = '1' else '0';

   axi_m2s.rready <= ocp_m2s.RespAccept;
   
   ocp_s2m.CmdAccept <= axi_s2m.arready when read_busy = '0' and ocp_m2s.Cmd = OCP_CMD_READ and no_writes_in_flight = '1' else
                        axi_s2m.awready when write_busy = '0' and ocp_m2s.Cmd = OCP_CMD_WRITE else
                        '0';
   ocp_s2m.DataAccept  <= axi_s2m.wready and write_busy;
   ocp_s2m.Data        <= axi_s2m.rdata;
   ocp_s2m.Resp        <= OCP_RESP_NONE when axi_s2m.rvalid = '0' else
                          OCP_RESP_VALID; -- when axi_s2m.rvalid = '1' and axi_s2m.rresp(1) = '0' else
                          --OCP_RESP_ERROR;

   ocp_s2m.Tag <= tag_table(CONV_INTEGER(last_tag_response));
   
   


   
end architecture rtl;


-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- ADB3OCP2AXI4 module
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_axi4.all;

entity axi4_to_adb3_ocp is
  port (
    clk     : in  std_logic;
    rst     : in  std_logic;
    axi_m2s : in  axi4_m2sT;
    axi_s2m : out axi4_s2mT;
    ocp_m2s : out adb3_ocp_m2sT;
    ocp_s2m : in  adb3_ocp_s2mT);

end entity axi4_to_adb3_ocp;

architecture rtl of axi4_to_adb3_ocp is

  signal read_preempting : std_logic;
  signal write_busy : std_logic := '0';

  type burst_table_type is array (0 to 15) of std_logic_vector(7 downto 0);
  signal burst_table : burst_table_type := (others => (others => '0'));

  signal bid : std_logic_vector(0 downto 0) := "0";
  signal bvalid : std_logic := '0';
  
  signal artag : std_logic_vector(3 downto 0) := "0000";
  
begin  -- architecture rtl

  axi_s2m.awready <= ocp_s2m.CmdAccept when read_preempting = '0' and write_busy = '0' else '0';
  axi_s2m.wready  <= ocp_s2m.DataAccept;
  axi_s2m.bid <= bid;
  axi_s2m.bresp <= "00";                -- OK
  axi_s2m.bvalid <= bvalid;
  axi_s2m.arready <= ocp_s2m.CmdAccept;
  axi_s2m.rid <= (others => '0');
  axi_s2m.rdata <= ocp_s2m.Data;
  axi_s2m.rresp <= "00";                -- OKAY
  axi_s2m.rvalid <= '1' when ocp_s2m.Resp = OCP_RESP_VALID else '0';
  axi_s2m.rlast <= '1' when burst_table(CONV_INTEGER(ocp_s2m.Tag(3 downto 0))) = "00000001" else '0';

  read_preempting <= axi_m2s.arvalid;

  ocp_m2s.Cmd <= OCP_CMD_READ when axi_m2s.arvalid = '1' else
                 OCP_CMD_WRITE when axi_m2s.awvalid = '1' and write_busy = '0' else
                 OCP_CMD_IDLE;

  --ocp_m2s.Addr <= X"00000000" & axi_m2s.araddr when axi_m2s.arvalid = '1' else
   --               X"00000000" & axi_m2s.awaddr;
  ocp_m2s.Addr <= axi_m2s.araddr when axi_m2s.arvalid = '1' else
                 axi_m2s.awaddr;
  ocp_m2s.BurstLength <= (X"0" & axi_m2s.arlen)+1 when axi_m2s.arvalid = '1' else
                        (X"0" & axi_m2s.awlen)+1;
  ocp_m2s.Data <= axi_m2s.wdata;
  ocp_m2s.DataByteEn <= axi_m2s.wstrb;
  ocp_m2s.DataValid <= axi_m2s.wvalid;

  ocp_m2s.RespAccept <= axi_m2s.rready;
  ocp_m2s.Tag <= X"0" & artag;
  
   track_rbursts0: process (clk) is
  begin  -- process track_rbursts0
    if rising_edge(clk) then            -- rising clock edge
      if axi_m2s.arvalid = '1' and ocp_s2m.CmdAccept = '1' then
        burst_table(CONV_INTEGER(artag)) <= axi_m2s.arlen+1;
        artag <= artag+1;
      end if;
      for i in 0 to 15 loop
        if axi_m2s.rready = '1' and ocp_s2m.Resp = OCP_RESP_VALID and ocp_s2m.Tag(3 downto 0) = i then
          burst_table(i) <= burst_table(i)-1;
        end if;
      end loop;  -- i
      if rst = '1' then
        artag <= (others => '0');
      end if;
    end if;
  end process track_rbursts0;


  gen_wr_resp0: process (clk) is
  begin  -- process gen_wr_resp0
    if rising_edge(clk) then            -- rising clock edge
      if write_busy = '0' then
        if axi_m2s.arvalid = '0' and axi_m2s.awvalid = '1' and ocp_s2m.CmdAccept = '1' then
          write_busy <= '1';
          bid <= axi_m2s.awid;
        end if;
      else
        if axi_m2s.wlast = '1' then
          bvalid <= '1';
        end if;
        if bvalid = '1' and axi_m2s.bready = '1' then
          bvalid <= '0';
          write_busy <= '0';
        end if;
      end if;
    end if;
  end process gen_wr_resp0;
  
end architecture rtl;
