
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.mptl.all;
use work.mptl_pins.all;
use work.adb3_ocp_axi4.all;

entity mptl128_interface_target_ku_axi is
  generic (
    enable_bridge_master     : boolean := true;
    enable_target_master     : boolean := false;
    enable_dma               : boolean := true;
    fabric_reset_in          : boolean := true;
    output_mptlclk           : boolean := true;
    behav_sim_axi4_over_mptl : boolean := false);
  port (
    -- Optional ASYNC reset from Fabric
    async_sys_rst            : in  std_logic;
    -- AXI4 Interfaces
    aclk                     : out std_logic;
    aresetn                  : out std_logic;
    -- Direct Slave
    ds_axi_awaddr            : out std_logic_vector(63 downto 0);
    ds_axi_awlen             : out std_logic_vector(7 downto 0);
    ds_axi_awsize            : out std_logic_vector(2 downto 0);
    ds_axi_awburst           : out std_logic_vector(1 downto 0);
    ds_axi_awcache           : out std_logic_vector(3 downto 0);
    ds_axi_awprot            : out std_logic_vector(2 downto 0);
    ds_axi_awvalid           : out std_logic;
    ds_axi_wdata             : out std_logic_vector(127 downto 0);
    ds_axi_wstrb             : out std_logic_vector(15 downto 0);
    ds_axi_wlast             : out std_logic;
    ds_axi_wvalid            : out std_logic;
    ds_axi_bready            : out std_logic;
    ds_axi_araddr            : out std_logic_vector(63 downto 0);
    ds_axi_arlen             : out std_logic_vector(7 downto 0);
    ds_axi_arsize            : out std_logic_vector(2 downto 0);
    ds_axi_arburst           : out std_logic_vector(1 downto 0);
    ds_axi_arcache           : out std_logic_vector(3 downto 0);
    ds_axi_arprot            : out std_logic_vector(2 downto 0);
    ds_axi_arvalid           : out std_logic;
    ds_axi_rready            : out std_logic;
    ds_axi_awready           : in  std_logic;
    ds_axi_wready            : in  std_logic;
    ds_axi_bresp             : in  std_logic_vector(1 downto 0);
    ds_axi_bvalid            : in  std_logic;
    ds_axi_arready           : in  std_logic;
    ds_axi_rdata             : in  std_logic_vector(127 downto 0);
    ds_axi_rresp             : in  std_logic_vector(1 downto 0);
    ds_axi_rlast             : in  std_logic;
    ds_axi_rvalid            : in  std_logic;
    ds_axi_awlock            : out std_logic_vector(0 downto 0);
    ds_axi_awqos             : out std_logic_vector (3 downto 0);
    ds_axi_arlock            : out std_logic_vector(0 downto 0);
    ds_axi_arqos             : out std_logic_vector (3 downto 0);
    ds_axi_awregion          : out std_logic_vector (3 downto 0);
    ds_axi_arregion          : out std_logic_vector (3 downto 0);
    -- DMA Channel 0
    dma0_axi_awaddr          : out std_logic_vector(38 downto 0);
    dma0_axi_awlen           : out std_logic_vector(7 downto 0);
    dma0_axi_awsize          : out std_logic_vector(2 downto 0);
    dma0_axi_awburst         : out std_logic_vector(1 downto 0);
    dma0_axi_awcache         : out std_logic_vector(3 downto 0);
    dma0_axi_awprot          : out std_logic_vector(2 downto 0);
    dma0_axi_awvalid         : out std_logic;
    dma0_axi_wdata           : out std_logic_vector(127 downto 0);
    dma0_axi_wstrb           : out std_logic_vector(15 downto 0);
    dma0_axi_wlast           : out std_logic;
    dma0_axi_wvalid          : out std_logic;
    dma0_axi_bready          : out std_logic;
    dma0_axi_araddr          : out std_logic_vector(38 downto 0);
    dma0_axi_arlen           : out std_logic_vector(7 downto 0);
    dma0_axi_arsize          : out std_logic_vector(2 downto 0);
    dma0_axi_arburst         : out std_logic_vector(1 downto 0);
    dma0_axi_arcache         : out std_logic_vector(3 downto 0);
    dma0_axi_arprot          : out std_logic_vector(2 downto 0);
    dma0_axi_arvalid         : out std_logic;
    dma0_axi_rready          : out std_logic;
    dma0_axi_awready         : in  std_logic;
    dma0_axi_wready          : in  std_logic;
    dma0_axi_bresp           : in  std_logic_vector(1 downto 0);
    dma0_axi_bvalid          : in  std_logic;
    dma0_axi_arready         : in  std_logic;
    dma0_axi_rdata           : in  std_logic_vector(127 downto 0);
    dma0_axi_rresp           : in  std_logic_vector(1 downto 0);
    dma0_axi_rlast           : in  std_logic;
    dma0_axi_rvalid          : in  std_logic;
    dma0_axi_awlock          : out std_logic_vector(0 downto 0);
    dma0_axi_awqos           : out std_logic_vector (3 downto 0);
    dma0_axi_arlock          : out std_logic_vector(0 downto 0);
    dma0_axi_arqos           : out std_logic_vector (3 downto 0);
    dma0_axi_awregion        : out std_logic_vector (3 downto 0);
    dma0_axi_arregion        : out std_logic_vector (3 downto 0);
    -- DMA Channel 1
    dma1_axi_awaddr          : out std_logic_vector(38 downto 0);
    dma1_axi_awlen           : out std_logic_vector(7 downto 0);
    dma1_axi_awsize          : out std_logic_vector(2 downto 0);
    dma1_axi_awburst         : out std_logic_vector(1 downto 0);
    dma1_axi_awcache         : out std_logic_vector(3 downto 0);
    dma1_axi_awprot          : out std_logic_vector(2 downto 0);
    dma1_axi_awvalid         : out std_logic;
    dma1_axi_wdata           : out std_logic_vector(127 downto 0);
    dma1_axi_wstrb           : out std_logic_vector(15 downto 0);
    dma1_axi_wlast           : out std_logic;
    dma1_axi_wvalid          : out std_logic;
    dma1_axi_bready          : out std_logic;
    dma1_axi_araddr          : out std_logic_vector(38 downto 0);
    dma1_axi_arlen           : out std_logic_vector(7 downto 0);
    dma1_axi_arsize          : out std_logic_vector(2 downto 0);
    dma1_axi_arburst         : out std_logic_vector(1 downto 0);
    dma1_axi_arcache         : out std_logic_vector(3 downto 0);
    dma1_axi_arprot          : out std_logic_vector(2 downto 0);
    dma1_axi_arvalid         : out std_logic;
    dma1_axi_rready          : out std_logic;
    dma1_axi_awready         : in  std_logic;
    dma1_axi_wready          : in  std_logic;
    dma1_axi_bresp           : in  std_logic_vector(1 downto 0);
    dma1_axi_bvalid          : in  std_logic;
    dma1_axi_arready         : in  std_logic;
    dma1_axi_rdata           : in  std_logic_vector(127 downto 0);
    dma1_axi_rresp           : in  std_logic_vector(1 downto 0);
    dma1_axi_rlast           : in  std_logic;
    dma1_axi_rvalid          : in  std_logic;
    dma1_axi_awlock          : out std_logic_vector(0 downto 0);
    dma1_axi_awqos           : out std_logic_vector (3 downto 0);
    dma1_axi_arlock          : out std_logic_vector(0 downto 0);
    dma1_axi_arqos           : out std_logic_vector (3 downto 0);
    dma1_axi_awregion        : out std_logic_vector (3 downto 0);
    dma1_axi_arregion        : out std_logic_vector (3 downto 0);
    -- ABMI Interface      
    abmi_axi_awaddr          : in  std_logic_vector(11 downto 0);
    abmi_axi_awvalid         : in  std_logic;
    abmi_axi_wdata           : in  std_logic_vector(31 downto 0);
    abmi_axi_wstrb           : in  std_logic_vector(3 downto 0);
    abmi_axi_wvalid          : in  std_logic;
    abmi_axi_bready          : in  std_logic;
    abmi_axi_araddr          : in  std_logic_vector(11 downto 0);
    abmi_axi_arvalid         : in  std_logic;
    abmi_axi_rready          : in  std_logic;
    abmi_axi_awready         : out std_logic;
    abmi_axi_wready          : out std_logic;
    abmi_axi_bresp           : out std_logic_vector(1 downto 0);
    abmi_axi_bvalid          : out std_logic;
    abmi_axi_arready         : out std_logic;
    abmi_axi_rdata           : out std_logic_vector(31 downto 0);
    abmi_axi_rresp           : out std_logic_vector(1 downto 0);
    abmi_axi_rvalid          : out std_logic;
    -- MPTL Interface
    mptl_tx_p                : out std_logic_vector(3 downto 0);
    mptl_tx_n                : out std_logic_vector(3 downto 0);
    mptl_rx_p                : in  std_logic_vector(3 downto 0);
    mptl_rx_n                : in  std_logic_vector(3 downto 0);
    mptl_clk_p               : in  std_logic;
    mptl_clk_n               : in  std_logic;
    mptl_clk_out             : out std_logic;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out std_logic;
    mptl_target_gtp_online_n : out std_logic;
    mptl_bridge_gtp_online_n : in  std_logic;
    -- MPTL128 Sideband Interface
    gpio_t2b                 : in  std_logic_vector(15 downto 0);
    gpio_b2t                 : out std_logic_vector(15 downto 0));

end mptl128_interface_target_ku_axi;

architecture rtl of mptl128_interface_target_ku_axi is

  
  signal ds_axi_m2s : axi4_m2sT := AXI4_M2ST_ZEROS;
  signal ds_axi_s2m : axi4_s2mT := AXI4_S2MT_ZEROS;

  signal dma_axi_m2s : axi4_m2sT_array(0 to 1) := (others => AXI4_M2ST_ZEROS);
  signal dma_axi_s2m : axi4_s2mT_array(0 to 1) := (others => AXI4_S2MT_ZEROS);

  signal direct_slave_m2s : adb3_ocp_m2sT               := ADB3_OCP_DEFAULT_M2S;
  signal direct_slave_s2m : adb3_ocp_s2mT               := ADB3_OCP_DEFAULT_S2M;
  signal dma_channels_m2s : adb3_ocp_m2sT_array(0 to 1) := (others => ADB3_OCP_DEFAULT_M2S);
  signal dma_channels_s2m : adb3_ocp_s2mT_array(0 to 1) := (others => ADB3_OCP_DEFAULT_S2M);

  signal ocpclk                               : std_logic;
  signal irst, irst_r1, irst_r2, srst, srst_n : std_logic                    := '1';
  signal dma_abort                            : std_logic_vector(1 downto 0) := "00";

  signal mptl_target_gtp_online_i_n : std_logic;
  signal mptl_target_configured_i_n : std_logic;

  signal bridge_online_stable_n : std_logic;
  signal target_online_stable_n : std_logic;
  signal bridge_online_n_sr     : std_logic_vector(31 downto 0);
  signal target_online_n_sr     : std_logic_vector(31 downto 0);

  -- Convert a non '0' or '1' bit of std_logic to defined
  function make_defined_sl(
    constant x : in std_logic;
    constant z : in std_logic)
    return std_logic is
  begin
    if (x = '0')or(x = '1') then
      return x;
    else
      return z;
    end if;
  end;


  -- Convert any non '0' or '1' bits of std_logic_vector to '0'
  function make_defined_slv(
    constant x : in std_logic_vector)
    return std_logic_vector is
    variable m : std_logic_vector(x'range);
  begin
    for i in x'range loop
      m(i) := make_defined_sl(x(i), '0');
    end loop;
    return m;
  end;


  component clk_wiz_0
    port
      (                                 -- Clock in ports
        clk_in1  : in  std_logic;
        -- Clock out ports
        clk_out1 : out std_logic;
        -- Status and control signals
        reset    : in  std_logic;
        locked   : out std_logic
        );
  end component;

  signal abmi_m2s_i : adb3_ocp_m2sT  := ADB3_OCP_DEFAULT_M2S;
  signal abmi_s2m_i : adb3_ocp_s2mT  := ADB3_OCP_DEFAULT_S2M;
  signal abmi_l_m2s : adb3l_ocp_m2sT := ADB3L_OCP_DEFAULT_M2S;
  signal abmi_l_s2m : adb3l_ocp_s2mT := ADB3L_OCP_DEFAULT_S2M;

  signal mptl_clk_out_i : std_logic;

  signal async_rst_debounce0 : std_logic := '1';
  signal async_rst_debounce1 : std_logic := '1';
  signal async_rst_debounce2 : std_logic := '1';
  signal async_rst_req       : std_logic := '1';

  signal async_rst_req_count : std_logic_vector(7 downto 0) := (others => '1');
  signal sync_rst_req        : std_logic                    := '1';

  signal ocp_clk_locked, ocp_clk_locked_mclk : std_logic := '0';



  
  function behav_sim_decision_func(
    constant behav_sim : boolean := false)
    return boolean is
  begin
    return behav_sim;
  end;

  constant behav_sim : boolean := behav_sim_decision_func
                                  -- synthesis translate_off
                                  (behav_sim => behav_sim_axi4_over_mptl)
                                  -- synthesis translate_on
;

  
  component mptl_tgt_ku_axi_bm is
    generic (
      enable_bridge_master : boolean;
      enable_target_master : boolean;
      enable_dma           : boolean;
      fabric_reset_in      : boolean;
      output_mptlclk       : boolean);
    port (
      mptl_tx_p                : out std_logic_vector(3 downto 0);
      mptl_tx_n                : out std_logic_vector(3 downto 0);
      mptl_rx_p                : in  std_logic_vector(3 downto 0);
      mptl_rx_n                : in  std_logic_vector(3 downto 0);
      mptl_clk_p               : in  std_logic;
      mptl_clk_n               : in  std_logic;
      mptl_clk_out             : out std_logic;
      mptl_target_configured_n : out std_logic;
      mptl_target_gtp_online_n : out std_logic;
      mptl_bridge_gtp_online_n : in  std_logic;
      async_sys_rst            : in  std_logic                      := '0';
      aclk                     : out std_logic;
      aresetn                  : out std_logic;
      ds_axi_awaddr            : out std_logic_vector(63 downto 0);
      ds_axi_awlen             : out std_logic_vector(7 downto 0);
      ds_axi_awsize            : out std_logic_vector(2 downto 0);
      ds_axi_awburst           : out std_logic_vector(1 downto 0);
      ds_axi_awcache           : out std_logic_vector(3 downto 0);
      ds_axi_awprot            : out std_logic_vector(2 downto 0);
      ds_axi_awvalid           : out std_logic;
      ds_axi_wdata             : out std_logic_vector(127 downto 0);
      ds_axi_wstrb             : out std_logic_vector(15 downto 0);
      ds_axi_wlast             : out std_logic;
      ds_axi_wvalid            : out std_logic;
      ds_axi_bready            : out std_logic;
      ds_axi_araddr            : out std_logic_vector(63 downto 0);
      ds_axi_arlen             : out std_logic_vector(7 downto 0);
      ds_axi_arsize            : out std_logic_vector(2 downto 0);
      ds_axi_arburst           : out std_logic_vector(1 downto 0);
      ds_axi_arcache           : out std_logic_vector(3 downto 0);
      ds_axi_arprot            : out std_logic_vector(2 downto 0);
      ds_axi_arvalid           : out std_logic;
      ds_axi_rready            : out std_logic;
      ds_axi_awlock            : out std_logic_vector(0 downto 0);
      ds_axi_awqos             : out std_logic_vector(3 downto 0);
      ds_axi_arlock            : out std_logic_vector(0 downto 0);
      ds_axi_arqos             : out std_logic_vector(3 downto 0);
      ds_axi_awregion          : out std_logic_vector(3 downto 0);
      ds_axi_arregion          : out std_logic_vector(3 downto 0);
      ds_axi_awready           : in  std_logic                      := '0';
      ds_axi_wready            : in  std_logic                      := '0';
      ds_axi_bresp             : in  std_logic_vector(1 downto 0)   := (others => '0');
      ds_axi_bvalid            : in  std_logic                      := '0';
      ds_axi_arready           : in  std_logic                      := '0';
      ds_axi_rdata             : in  std_logic_vector(127 downto 0) := (others => '0');
      ds_axi_rresp             : in  std_logic_vector(1 downto 0)   := (others => '0');
      ds_axi_rlast             : in  std_logic                      := '0';
      ds_axi_rvalid            : in  std_logic                      := '0';
      dma0_axi_awaddr          : out std_logic_vector(38 downto 0);
      dma0_axi_awlen           : out std_logic_vector(7 downto 0);
      dma0_axi_awsize          : out std_logic_vector(2 downto 0);
      dma0_axi_awburst         : out std_logic_vector(1 downto 0);
      dma0_axi_awcache         : out std_logic_vector(3 downto 0);
      dma0_axi_awprot          : out std_logic_vector(2 downto 0);
      dma0_axi_awvalid         : out std_logic;
      dma0_axi_wdata           : out std_logic_vector(127 downto 0);
      dma0_axi_wstrb           : out std_logic_vector(15 downto 0);
      dma0_axi_wlast           : out std_logic;
      dma0_axi_wvalid          : out std_logic;
      dma0_axi_bready          : out std_logic;
      dma0_axi_araddr          : out std_logic_vector(38 downto 0);
      dma0_axi_arlen           : out std_logic_vector(7 downto 0);
      dma0_axi_arsize          : out std_logic_vector(2 downto 0);
      dma0_axi_arburst         : out std_logic_vector(1 downto 0);
      dma0_axi_arcache         : out std_logic_vector(3 downto 0);
      dma0_axi_arprot          : out std_logic_vector(2 downto 0);
      dma0_axi_arvalid         : out std_logic;
      dma0_axi_rready          : out std_logic;
      dma0_axi_awlock          : out std_logic_vector(0 downto 0);
      dma0_axi_awqos           : out std_logic_vector(3 downto 0);
      dma0_axi_arlock          : out std_logic_vector(0 downto 0);
      dma0_axi_arqos           : out std_logic_vector(3 downto 0);
      dma0_axi_awregion        : out std_logic_vector(3 downto 0);
      dma0_axi_arregion        : out std_logic_vector(3 downto 0);
      dma0_axi_awready         : in  std_logic                      := '0';
      dma0_axi_wready          : in  std_logic                      := '0';
      dma0_axi_bresp           : in  std_logic_vector(1 downto 0)   := (others => '0');
      dma0_axi_bvalid          : in  std_logic                      := '0';
      dma0_axi_arready         : in  std_logic                      := '0';
      dma0_axi_rdata           : in  std_logic_vector(127 downto 0) := (others => '0');
      dma0_axi_rresp           : in  std_logic_vector(1 downto 0)   := (others => '0');
      dma0_axi_rlast           : in  std_logic                      := '0';
      dma0_axi_rvalid          : in  std_logic                      := '0';
      dma1_axi_awaddr          : out std_logic_vector(38 downto 0);
      dma1_axi_awlen           : out std_logic_vector(7 downto 0);
      dma1_axi_awsize          : out std_logic_vector(2 downto 0);
      dma1_axi_awburst         : out std_logic_vector(1 downto 0);
      dma1_axi_awcache         : out std_logic_vector(3 downto 0);
      dma1_axi_awprot          : out std_logic_vector(2 downto 0);
      dma1_axi_awvalid         : out std_logic;
      dma1_axi_wdata           : out std_logic_vector(127 downto 0);
      dma1_axi_wstrb           : out std_logic_vector(15 downto 0);
      dma1_axi_wlast           : out std_logic;
      dma1_axi_wvalid          : out std_logic;
      dma1_axi_bready          : out std_logic;
      dma1_axi_araddr          : out std_logic_vector(38 downto 0);
      dma1_axi_arlen           : out std_logic_vector(7 downto 0);
      dma1_axi_arsize          : out std_logic_vector(2 downto 0);
      dma1_axi_arburst         : out std_logic_vector(1 downto 0);
      dma1_axi_arcache         : out std_logic_vector(3 downto 0);
      dma1_axi_arprot          : out std_logic_vector(2 downto 0);
      dma1_axi_arvalid         : out std_logic;
      dma1_axi_rready          : out std_logic;
      dma1_axi_awlock          : out std_logic_vector(0 downto 0);
      dma1_axi_awqos           : out std_logic_vector(3 downto 0);
      dma1_axi_arlock          : out std_logic_vector(0 downto 0);
      dma1_axi_arqos           : out std_logic_vector(3 downto 0);
      dma1_axi_awregion        : out std_logic_vector(3 downto 0);
      dma1_axi_arregion        : out std_logic_vector(3 downto 0);
      dma1_axi_awready         : in  std_logic                      := '0';
      dma1_axi_wready          : in  std_logic                      := '0';
      dma1_axi_bresp           : in  std_logic_vector(1 downto 0)   := (others => '0');
      dma1_axi_bvalid          : in  std_logic                      := '0';
      dma1_axi_arready         : in  std_logic                      := '0';
      dma1_axi_rdata           : in  std_logic_vector(127 downto 0) := (others => '0');
      dma1_axi_rresp           : in  std_logic_vector(1 downto 0)   := (others => '0');
      dma1_axi_rlast           : in  std_logic                      := '0';
      dma1_axi_rvalid          : in  std_logic                      := '0';
      abmi_axi_awaddr          : in  std_logic_vector(11 downto 0)  := (others => '0');
      abmi_axi_awvalid         : in  std_logic                      := '0';
      abmi_axi_wdata           : in  std_logic_vector(31 downto 0)  := (others => '0');
      abmi_axi_wstrb           : in  std_logic_vector(3 downto 0)   := (others => '0');
      abmi_axi_wvalid          : in  std_logic                      := '0';
      abmi_axi_bready          : in  std_logic                      := '0';
      abmi_axi_araddr          : in  std_logic_vector(11 downto 0)  := (others => '0');
      abmi_axi_arvalid         : in  std_logic                      := '0';
      abmi_axi_rready          : in  std_logic                      := '0';
      abmi_axi_awready         : out std_logic;
      abmi_axi_wready          : out std_logic;
      abmi_axi_bresp           : out std_logic_vector(1 downto 0);
      abmi_axi_bvalid          : out std_logic;
      abmi_axi_arready         : out std_logic;
      abmi_axi_rdata           : out std_logic_vector(31 downto 0);
      abmi_axi_rresp           : out std_logic_vector(1 downto 0);
      abmi_axi_rvalid          : out std_logic;
      gpio_t2b                 : in  std_logic_vector(15 downto 0);
      gpio_b2t                 : out std_logic_vector(15 downto 0));
  end component mptl_tgt_ku_axi_bm;


  signal irst1 : std_logic;
  signal mptl_target_gtp_online_i_n1 : std_logic;
  signal mptl_target_configured_i_n1 : std_logic;
  signal mptl_target_gtp_online_i_n2 : std_logic;
  signal mptl_target_configured_i_n2 : std_logic;

  attribute ASYNC_REG : boolean;
  attribute ASYNC_REG of irst1,mptl_target_gtp_online_i_n1, mptl_target_configured_i_n1: signal is TRUE;

  
begin  -- rtl

  gen_synth : if behav_sim = false generate
    
    aclk    <= ocpclk;
    aresetn <= srst_n;

    srst_n <= not srst;

    mptl_target_gtp_online_n <= mptl_target_gtp_online_i_n when ocp_clk_locked = '1' and sync_rst_req = '0' else '1';
    mptl_target_configured_n <= mptl_target_configured_i_n when ocp_clk_locked = '1' and sync_rst_req = '0' else '1';
    mptl_clk_out             <= mptl_clk_out_i             when output_mptlclk                              else '0';


    retime_reset : process (ocpclk, async_rst_req) is
    begin  -- process retime_reset
      if async_rst_req = '1' then
        srst <= '1';
        irst1 <= '1';
      elsif rising_edge(ocpclk) then    -- rising clock edge
        irst1 <= irst;
        srst <= irst1;
      end if;
    end process retime_reset;

    process (mptl_clk_out_i) is
    begin  -- process
      if rising_edge(mptl_clk_out_i) then  -- rising clock edge
        mptl_target_gtp_online_i_n1 <= mptl_target_gtp_online_i_n;
        mptl_target_configured_i_n1 <= mptl_target_configured_i_n;
        mptl_target_gtp_online_i_n2 <= mptl_target_gtp_online_i_n1;
        mptl_target_configured_i_n2 <= mptl_target_configured_i_n1;
        bridge_online_n_sr <= mptl_bridge_gtp_online_n & bridge_online_n_sr(31 downto 1);
        target_online_n_sr <= mptl_target_gtp_online_i_n2 & target_online_n_sr(31 downto 1);

        bridge_online_stable_n <= bridge_online_n_sr(0) and bridge_online_n_sr(31);
        target_online_stable_n <= target_online_n_sr(0) and target_online_n_sr(31);

        ocp_clk_locked_mclk <= ocp_clk_locked;

        if mptl_target_configured_i_n2 = '1' or target_online_stable_n = '1' or bridge_online_stable_n = '1' or async_rst_req = '1' or sync_rst_req = '1' or ocp_clk_locked_mclk = '0' then
          irst <= '1';
        else
          irst <= '0';
        end if;

        if fabric_reset_in = true then
          -- Debounce ASYNC reset to convert to glitch free signal
          -- Must be asserted for > 16ns
          async_rst_debounce0 <= async_sys_rst;
          async_rst_debounce1 <= async_rst_debounce0;
          async_rst_debounce2 <= async_rst_debounce1;
          async_rst_req       <= async_rst_debounce2 or async_rst_debounce1;
        else
          async_rst_debounce0 <= '0';
          async_rst_debounce1 <= '0';
          async_rst_debounce2 <= '0';
          async_rst_req       <= '0';
        end if;

        if async_rst_req = '1' then
          async_rst_req_count <= (others => '1');
          sync_rst_req        <= '1';
        elsif async_rst_req_count /= "00000000" then
          async_rst_req_count <= async_rst_req_count -1;
          sync_rst_req        <= '1';
        else
          sync_rst_req <= '0';
        end if;
      end if;
    end process;

    dma_abort <= "00";

    gen_axi_clk : clk_wiz_0
      port map (

        -- Clock in ports
        clk_in1  => mptl_clk_out_i,
        -- Clock out ports  
        clk_out1 => ocpclk,
        -- Status and control signals                
        reset    => sync_rst_req,
        locked   => ocp_clk_locked
        );

    
    ds_ocp2axi : adb3_ocp_to_axi4
      port map (
        clk             => ocpclk,
        rst             => srst,
        ocp_m2s         => direct_slave_m2s,
        ocp_s2m         => direct_slave_s2m,
        axi_m2s         => ds_axi_m2s,
        axi_s2m         => ds_axi_s2m,
        illegal_ocp_cmd => open);

    gen_dma_ocp2axi : for i in 0 to 1 generate
      dma_ocp2axi : adb3_ocp_to_axi4
        port map (
          clk             => ocpclk,
          rst             => srst,
          ocp_m2s         => dma_channels_m2s(i),
          ocp_s2m         => dma_channels_s2m(i),
          axi_m2s         => dma_axi_m2s(i),
          axi_s2m         => dma_axi_s2m(i),
          illegal_ocp_cmd => open);
    end generate gen_dma_ocp2axi;


    mptl128_interface_target_ku_1 : entity work.mptl128_interface_target_ku
      generic map (
        enable_bridge_master => enable_bridge_master,
        enable_target_master => enable_target_master,
        enable_dma           => enable_dma)
      port map (
        ocp_clk                  => ocpclk,
        direct_slave_m2s         => direct_slave_m2s,
        direct_slave_s2m         => direct_slave_s2m,
        dma_channels_m2s         => dma_channels_m2s,
        dma_channels_s2m         => dma_channels_s2m,
        dma_abort                => dma_abort,
        abmi_m2s                 => abmi_m2s_i,
        abmi_s2m                 => abmi_s2m_i,
        mptl_tx.p                => mptl_tx_p,
        mptl_tx.n                => mptl_tx_n,
        mptl_rx.p                => mptl_rx_p,
        mptl_rx.n                => mptl_rx_n,
        mptl_clk.p               => mptl_clk_p,
        mptl_clk.n               => mptl_clk_n,
        mptl_clk_out             => mptl_clk_out_i,
        mptl_target_configured_n => mptl_target_configured_i_n,
        mptl_target_gtp_online_n => mptl_target_gtp_online_i_n,
        mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_n,
        gpio_t2b                 => gpio_t2b,
        gpio_b2t                 => gpio_b2t);



    ds_axi_awaddr      <= ds_axi_m2s.awaddr;
    ds_axi_awlen       <= ds_axi_m2s.awlen;
    ds_axi_awsize      <= ds_axi_m2s.awsize;
    ds_axi_awburst     <= ds_axi_m2s.awburst;
    ds_axi_awcache     <= ds_axi_m2s.awcache;
    ds_axi_awprot      <= ds_axi_m2s.awprot;
    ds_axi_awvalid     <= ds_axi_m2s.awvalid;
    ds_axi_wdata       <= ds_axi_m2s.wdata;
    ds_axi_wstrb       <= ds_axi_m2s.wstrb;
    ds_axi_wlast       <= ds_axi_m2s.wlast;
    ds_axi_wvalid      <= ds_axi_m2s.wvalid;
    ds_axi_bready      <= ds_axi_m2s.bready;
    ds_axi_araddr      <= ds_axi_m2s.araddr;
    ds_axi_arlen       <= ds_axi_m2s.arlen;
    ds_axi_arsize      <= ds_axi_m2s.arsize;
    ds_axi_arburst     <= ds_axi_m2s.arburst;
    ds_axi_arcache     <= ds_axi_m2s.arcache;
    ds_axi_arprot      <= ds_axi_m2s.arprot;
    ds_axi_arvalid     <= ds_axi_m2s.arvalid;
    ds_axi_rready      <= ds_axi_m2s.rready;
    ds_axi_s2m.awready <= make_defined_sl(ds_axi_awready, '0');
    ds_axi_s2m.wready  <= make_defined_sl(ds_axi_wready, '0');
    ds_axi_s2m.bresp   <= make_defined_slv(ds_axi_bresp);
    ds_axi_s2m.bvalid  <= make_defined_sl(ds_axi_bvalid, '0');
    ds_axi_s2m.arready <= make_defined_sl(ds_axi_arready, '0');
    ds_axi_s2m.rdata   <= make_defined_slv(ds_axi_rdata);
    ds_axi_s2m.rresp   <= make_defined_slv(ds_axi_rresp);
    ds_axi_s2m.rlast   <= make_defined_sl(ds_axi_rlast, '0');
    ds_axi_s2m.rvalid  <= make_defined_sl(ds_axi_rvalid, '0');

    dma0_axi_awaddr        <= dma_axi_m2s(0).awaddr(38 downto 0);
    dma0_axi_awlen         <= dma_axi_m2s(0).awlen;
    dma0_axi_awsize        <= dma_axi_m2s(0).awsize;
    dma0_axi_awburst       <= dma_axi_m2s(0).awburst;
    dma0_axi_awcache       <= dma_axi_m2s(0).awcache;
    dma0_axi_awprot        <= dma_axi_m2s(0).awprot;
    dma0_axi_awvalid       <= dma_axi_m2s(0).awvalid;
    dma0_axi_wdata         <= dma_axi_m2s(0).wdata;
    dma0_axi_wstrb         <= dma_axi_m2s(0).wstrb;
    dma0_axi_wlast         <= dma_axi_m2s(0).wlast;
    dma0_axi_wvalid        <= dma_axi_m2s(0).wvalid;
    dma0_axi_bready        <= dma_axi_m2s(0).bready;
    dma0_axi_araddr        <= dma_axi_m2s(0).araddr(38 downto 0);
    dma0_axi_arlen         <= dma_axi_m2s(0).arlen;
    dma0_axi_arsize        <= dma_axi_m2s(0).arsize;
    dma0_axi_arburst       <= dma_axi_m2s(0).arburst;
    dma0_axi_arcache       <= dma_axi_m2s(0).arcache;
    dma0_axi_arprot        <= dma_axi_m2s(0).arprot;
    dma0_axi_arvalid       <= dma_axi_m2s(0).arvalid;
    dma0_axi_rready        <= dma_axi_m2s(0).rready;
    dma_axi_s2m(0).awready <= make_defined_sl(dma0_axi_awready, '0');
    dma_axi_s2m(0).wready  <= make_defined_sl(dma0_axi_wready, '0');
    dma_axi_s2m(0).bresp   <= make_defined_slv(dma0_axi_bresp);
    dma_axi_s2m(0).bvalid  <= make_defined_sl(dma0_axi_bvalid, '0');
    dma_axi_s2m(0).arready <= make_defined_sl(dma0_axi_arready, '0');
    dma_axi_s2m(0).rdata   <= make_defined_slv(dma0_axi_rdata);
    dma_axi_s2m(0).rresp   <= make_defined_slv(dma0_axi_rresp);
    dma_axi_s2m(0).rlast   <= make_defined_sl(dma0_axi_rlast, '0');
    dma_axi_s2m(0).rvalid  <= make_defined_sl(dma0_axi_rvalid, '0');


    dma1_axi_awaddr        <= dma_axi_m2s(1).awaddr(38 downto 0);
    dma1_axi_awlen         <= dma_axi_m2s(1).awlen;
    dma1_axi_awsize        <= dma_axi_m2s(1).awsize;
    dma1_axi_awburst       <= dma_axi_m2s(1).awburst;
    dma1_axi_awcache       <= dma_axi_m2s(1).awcache;
    dma1_axi_awprot        <= dma_axi_m2s(1).awprot;
    dma1_axi_awvalid       <= dma_axi_m2s(1).awvalid;
    dma1_axi_wdata         <= dma_axi_m2s(1).wdata;
    dma1_axi_wstrb         <= dma_axi_m2s(1).wstrb;
    dma1_axi_wlast         <= dma_axi_m2s(1).wlast;
    dma1_axi_wvalid        <= dma_axi_m2s(1).wvalid;
    dma1_axi_bready        <= dma_axi_m2s(1).bready;
    dma1_axi_araddr        <= dma_axi_m2s(1).araddr(38 downto 0);
    dma1_axi_arlen         <= dma_axi_m2s(1).arlen;
    dma1_axi_arsize        <= dma_axi_m2s(1).arsize;
    dma1_axi_arburst       <= dma_axi_m2s(1).arburst;
    dma1_axi_arcache       <= dma_axi_m2s(1).arcache;
    dma1_axi_arprot        <= dma_axi_m2s(1).arprot;
    dma1_axi_arvalid       <= dma_axi_m2s(1).arvalid;
    dma1_axi_rready        <= dma_axi_m2s(1).rready;
    dma_axi_s2m(1).awready <= make_defined_sl(dma1_axi_awready, '0');
    dma_axi_s2m(1).wready  <= make_defined_sl(dma1_axi_wready, '0');
    dma_axi_s2m(1).bresp   <= make_defined_slv(dma1_axi_bresp);
    dma_axi_s2m(1).bvalid  <= make_defined_sl(dma1_axi_bvalid, '0');
    dma_axi_s2m(1).arready <= make_defined_sl(dma1_axi_arready, '0');
    dma_axi_s2m(1).rdata   <= make_defined_slv(dma1_axi_rdata);
    dma_axi_s2m(1).rresp   <= make_defined_slv(dma1_axi_rresp);
    dma_axi_s2m(1).rlast   <= make_defined_sl(dma1_axi_rlast, '0');
    dma_axi_s2m(1).rvalid  <= make_defined_sl(dma1_axi_rvalid, '0');


    gen_tgt_master : if enable_target_master generate
      
      
      axi4_lite_to_adb3_ocpl_1 : entity work.axi4_lite_to_adb3_ocpl
        port map (
          S_AXI_ACLK                 => ocpclk,
          S_AXI_ARESETN              => srst_n,
          S_AXI_AWADDR(11 downto 0)  => ABMI_AXI_AWADDR,
          S_AXI_AWADDR(31 downto 12) => "00000000000000000000",
          S_AXI_AWVALID              => ABMI_AXI_AWVALID,
          S_AXI_WDATA                => ABMI_AXI_WDATA,
          S_AXI_WSTRB                => ABMI_AXI_WSTRB,
          S_AXI_WVALID               => ABMI_AXI_WVALID,
          S_AXI_BREADY               => ABMI_AXI_BREADY,
          S_AXI_ARADDR(11 downto 0)  => ABMI_AXI_ARADDR,
          S_AXI_ARADDR(31 downto 12) => "00000000000000000000",
          S_AXI_ARVALID              => ABMI_AXI_ARVALID,
          S_AXI_RREADY               => ABMI_AXI_RREADY,
          S_AXI_ARREADY              => ABMI_AXI_ARREADY,
          S_AXI_RDATA                => ABMI_AXI_RDATA,
          S_AXI_RRESP                => ABMI_AXI_RRESP,
          S_AXI_RVALID               => ABMI_AXI_RVALID,
          S_AXI_WREADY               => ABMI_AXI_WREADY,
          S_AXI_BRESP                => ABMI_AXI_BRESP,
          S_AXI_BVALID               => ABMI_AXI_BVALID,
          S_AXI_AWREADY              => ABMI_AXI_AWREADY,
          ocpl_cmd                   => abmi_l_m2s.cmd,
          ocpl_addr                  => abmi_l_m2s.addr,
          ocpl_m2sdata               => abmi_l_m2s.data,
          ocpl_databyteen            => abmi_l_m2s.databyteen,
          ocpl_datavalid             => abmi_l_m2s.datavalid,
          ocpl_respaccept            => abmi_l_m2s.respaccept,
          ocpl_cmdaccept             => abmi_l_s2m.cmdaccept,
          ocpl_dataaccept            => abmi_l_s2m.dataaccept,
          ocpl_resp                  => abmi_l_s2m.resp,
          ocpl_s2mdata               => abmi_l_s2m.data);

      adb3l_to_adb3_1 : entity work.adb3l_to_adb3
        port map (
          pri_m2s => abmi_l_m2s,
          pri_s2m => abmi_l_s2m,
          sec_m2s => abmi_m2s_i,
          sec_s2m => abmi_s2m_i);

    end generate gen_tgt_master;

    ds_axi_awlock   <= (others => '0');
    ds_axi_awqos    <= (others => '0');
    ds_axi_awregion <= (others => '0');
    ds_axi_arlock   <= (others => '0');
    ds_axi_arqos    <= (others => '0');
    ds_axi_arregion <= (others => '0');

    dma0_axi_awlock   <= (others => '0');
    dma0_axi_awqos    <= (others => '0');
    dma0_axi_awregion <= (others => '0');
    dma0_axi_arlock   <= (others => '0');
    dma0_axi_arqos    <= (others => '0');
    dma0_axi_arregion <= (others => '0');

    dma1_axi_awlock   <= (others => '0');
    dma1_axi_awqos    <= (others => '0');
    dma1_axi_awregion <= (others => '0');
    dma1_axi_arlock   <= (others => '0');
    dma1_axi_arqos    <= (others => '0');
    dma1_axi_arregion <= (others => '0');


    gen_no_tgt_master : if not enable_target_master generate
      abmi_axi_awready <= '0';
      abmi_axi_wready  <= '0';
      abmi_axi_bresp   <= (others => '0');
      abmi_axi_bvalid  <= '0';
      abmi_axi_arready <= '0';
      abmi_axi_rdata   <= (others => '0');
      abmi_axi_rresp   <= (others => '0');
      abmi_axi_rvalid  <= '0';
      abmi_m2s_i       <= ADB3_OCP_DEFAULT_M2S;
    end generate gen_no_tgt_master;

  end generate gen_synth;


  -- synthesis translate_off
  gen_fast_sim : if behav_sim = true generate
    
    mptl_tgt_ku_axi_bm_1: mptl_tgt_ku_axi_bm
      generic map (
        enable_bridge_master => enable_bridge_master,
        enable_target_master => enable_target_master,
        enable_dma           => enable_dma,
        fabric_reset_in      => fabric_reset_in,
        output_mptlclk       => output_mptlclk)
      port map (
        mptl_tx_p                => mptl_tx_p,
        mptl_tx_n                => mptl_tx_n,
        mptl_rx_p                => mptl_rx_p,
        mptl_rx_n                => mptl_rx_n,
        mptl_clk_p               => mptl_clk_p,
        mptl_clk_n               => mptl_clk_n,
        mptl_clk_out             => mptl_clk_out,
        mptl_target_configured_n => mptl_target_configured_n,
        mptl_target_gtp_online_n => mptl_target_gtp_online_n,
        mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_n,
        async_sys_rst            => async_sys_rst,
        aclk                     => aclk,
        aresetn                  => aresetn,
        ds_axi_awaddr            => ds_axi_awaddr,
        ds_axi_awlen             => ds_axi_awlen,
        ds_axi_awsize            => ds_axi_awsize,
        ds_axi_awburst           => ds_axi_awburst,
        ds_axi_awcache           => ds_axi_awcache,
        ds_axi_awprot            => ds_axi_awprot,
        ds_axi_awvalid           => ds_axi_awvalid,
        ds_axi_wdata             => ds_axi_wdata,
        ds_axi_wstrb             => ds_axi_wstrb,
        ds_axi_wlast             => ds_axi_wlast,
        ds_axi_wvalid            => ds_axi_wvalid,
        ds_axi_bready            => ds_axi_bready,
        ds_axi_araddr            => ds_axi_araddr,
        ds_axi_arlen             => ds_axi_arlen,
        ds_axi_arsize            => ds_axi_arsize,
        ds_axi_arburst           => ds_axi_arburst,
        ds_axi_arcache           => ds_axi_arcache,
        ds_axi_arprot            => ds_axi_arprot,
        ds_axi_arvalid           => ds_axi_arvalid,
        ds_axi_rready            => ds_axi_rready,
        ds_axi_awlock            => ds_axi_awlock,
        ds_axi_awqos             => ds_axi_awqos,
        ds_axi_arlock            => ds_axi_arlock,
        ds_axi_arqos             => ds_axi_arqos,
        ds_axi_awregion          => ds_axi_awregion,
        ds_axi_arregion          => ds_axi_arregion,
        ds_axi_awready           => ds_axi_awready,
        ds_axi_wready            => ds_axi_wready,
        ds_axi_bresp             => ds_axi_bresp,
        ds_axi_bvalid            => ds_axi_bvalid,
        ds_axi_arready           => ds_axi_arready,
        ds_axi_rdata             => ds_axi_rdata,
        ds_axi_rresp             => ds_axi_rresp,
        ds_axi_rlast             => ds_axi_rlast,
        ds_axi_rvalid            => ds_axi_rvalid,
        dma0_axi_awaddr          => dma0_axi_awaddr,
        dma0_axi_awlen           => dma0_axi_awlen,
        dma0_axi_awsize          => dma0_axi_awsize,
        dma0_axi_awburst         => dma0_axi_awburst,
        dma0_axi_awcache         => dma0_axi_awcache,
        dma0_axi_awprot          => dma0_axi_awprot,
        dma0_axi_awvalid         => dma0_axi_awvalid,
        dma0_axi_wdata           => dma0_axi_wdata,
        dma0_axi_wstrb           => dma0_axi_wstrb,
        dma0_axi_wlast           => dma0_axi_wlast,
        dma0_axi_wvalid          => dma0_axi_wvalid,
        dma0_axi_bready          => dma0_axi_bready,
        dma0_axi_araddr          => dma0_axi_araddr,
        dma0_axi_arlen           => dma0_axi_arlen,
        dma0_axi_arsize          => dma0_axi_arsize,
        dma0_axi_arburst         => dma0_axi_arburst,
        dma0_axi_arcache         => dma0_axi_arcache,
        dma0_axi_arprot          => dma0_axi_arprot,
        dma0_axi_arvalid         => dma0_axi_arvalid,
        dma0_axi_rready          => dma0_axi_rready,
        dma0_axi_awlock          => dma0_axi_awlock,
        dma0_axi_awqos           => dma0_axi_awqos,
        dma0_axi_arlock          => dma0_axi_arlock,
        dma0_axi_arqos           => dma0_axi_arqos,
        dma0_axi_awregion        => dma0_axi_awregion,
        dma0_axi_arregion        => dma0_axi_arregion,
        dma0_axi_awready         => dma0_axi_awready,
        dma0_axi_wready          => dma0_axi_wready,
        dma0_axi_bresp           => dma0_axi_bresp,
        dma0_axi_bvalid          => dma0_axi_bvalid,
        dma0_axi_arready         => dma0_axi_arready,
        dma0_axi_rdata           => dma0_axi_rdata,
        dma0_axi_rresp           => dma0_axi_rresp,
        dma0_axi_rlast           => dma0_axi_rlast,
        dma0_axi_rvalid          => dma0_axi_rvalid,
        dma1_axi_awaddr          => dma1_axi_awaddr,
        dma1_axi_awlen           => dma1_axi_awlen,
        dma1_axi_awsize          => dma1_axi_awsize,
        dma1_axi_awburst         => dma1_axi_awburst,
        dma1_axi_awcache         => dma1_axi_awcache,
        dma1_axi_awprot          => dma1_axi_awprot,
        dma1_axi_awvalid         => dma1_axi_awvalid,
        dma1_axi_wdata           => dma1_axi_wdata,
        dma1_axi_wstrb           => dma1_axi_wstrb,
        dma1_axi_wlast           => dma1_axi_wlast,
        dma1_axi_wvalid          => dma1_axi_wvalid,
        dma1_axi_bready          => dma1_axi_bready,
        dma1_axi_araddr          => dma1_axi_araddr,
        dma1_axi_arlen           => dma1_axi_arlen,
        dma1_axi_arsize          => dma1_axi_arsize,
        dma1_axi_arburst         => dma1_axi_arburst,
        dma1_axi_arcache         => dma1_axi_arcache,
        dma1_axi_arprot          => dma1_axi_arprot,
        dma1_axi_arvalid         => dma1_axi_arvalid,
        dma1_axi_rready          => dma1_axi_rready,
        dma1_axi_awlock          => dma1_axi_awlock,
        dma1_axi_awqos           => dma1_axi_awqos,
        dma1_axi_arlock          => dma1_axi_arlock,
        dma1_axi_arqos           => dma1_axi_arqos,
        dma1_axi_awregion        => dma1_axi_awregion,
        dma1_axi_arregion        => dma1_axi_arregion,
        dma1_axi_awready         => dma1_axi_awready,
        dma1_axi_wready          => dma1_axi_wready,
        dma1_axi_bresp           => dma1_axi_bresp,
        dma1_axi_bvalid          => dma1_axi_bvalid,
        dma1_axi_arready         => dma1_axi_arready,
        dma1_axi_rdata           => dma1_axi_rdata,
        dma1_axi_rresp           => dma1_axi_rresp,
        dma1_axi_rlast           => dma1_axi_rlast,
        dma1_axi_rvalid          => dma1_axi_rvalid,
        abmi_axi_awaddr          => abmi_axi_awaddr,
        abmi_axi_awvalid         => abmi_axi_awvalid,
        abmi_axi_wdata           => abmi_axi_wdata,
        abmi_axi_wstrb           => abmi_axi_wstrb,
        abmi_axi_wvalid          => abmi_axi_wvalid,
        abmi_axi_bready          => abmi_axi_bready,
        abmi_axi_araddr          => abmi_axi_araddr,
        abmi_axi_arvalid         => abmi_axi_arvalid,
        abmi_axi_rready          => abmi_axi_rready,
        abmi_axi_awready         => abmi_axi_awready,
        abmi_axi_wready          => abmi_axi_wready,
        abmi_axi_bresp           => abmi_axi_bresp,
        abmi_axi_bvalid          => abmi_axi_bvalid,
        abmi_axi_arready         => abmi_axi_arready,
        abmi_axi_rdata           => abmi_axi_rdata,
        abmi_axi_rresp           => abmi_axi_rresp,
        abmi_axi_rvalid          => abmi_axi_rvalid,
        gpio_t2b                 => gpio_t2b,
        gpio_b2t                 => gpio_b2t);
  end generate gen_fast_sim;
  -- synthesis translate_on
  
end rtl;

