--------------------------------------------------------------------------------
--
-- Module Name      : mptl128_packet_demux
--
--                    Copyright 2009 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : DeMUX for MPTL
--
--
-- Dependencies     : mptl
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------
--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.mptl.all;

entity mptl128_packet_demux is

  generic (
    number_of_ports : natural := 5);
  port (
    -- OCP Interface
    clk            : in  std_logic;
    online         : in  std_logic;
    -- MPTL Multi-port output interface
    mptl128_data_out        : out  mptl128_data_array(0 to number_of_ports-1);
    mptl128_valid_out       : out std_logic_vector(number_of_ports-1 downto 0);
    -- MPTL Single-port input
    mptl128_data_in         : in std_logic_vector(127 downto 0);
    mptl128_valid_in        : in std_logic;
    -- Error Detection
    error_detected         : out std_logic);

end mptl128_packet_demux;

architecture rtl of mptl128_packet_demux is
  
  signal packet_words_left : std_logic_vector(4 downto 0) := (others => '0');
  signal destination : std_logic_vector(3 downto 0);
  
  signal data_reg : std_logic_vector(127 downto 0);
  signal data_valid : std_logic;

  signal err_det : std_logic;

  signal chksum : std_logic_vector(3 downto 0);

  signal online_reg : std_logic := '0';
  
  
begin  -- rtl

  chksum <= mptl128_data_in(3 downto 0) xor mptl128_data_in(7 downto 4) xor mptl128_data_in(11 downto 8) xor mptl128_data_in(15 downto 12) xor
            mptl128_data_in(19 downto 16) xor mptl128_data_in(23 downto 20) xor mptl128_data_in(27 downto 24) xor mptl128_data_in(31 downto 28) xor
            mptl128_data_in(35 downto 32) xor mptl128_data_in(43 downto 40) xor mptl128_data_in(47 downto 44);

  demux0: process (clk)
  begin  -- process mux0
    if clk'event and clk = '1' then  -- rising clock edge
      err_det <= '0';
      online_reg <= online;
      if packet_words_left = "00000" or packet_words_left = "00001" then
        if mptl128_valid_in  = '1' then
          -- Verify its a legal packet
          if (mptl128_data_in(3 downto 0) = MPTL_SINGLE_BEAT_WRITE or
            mptl128_data_in(3 downto 0) = MPTL_READ_REQUEST or
            mptl128_data_in(3 downto 0) = MPTL_BURST_WRITE_CMD or
            mptl128_data_in(3 downto 0) = MPTL_BURST_WRITE_DATA or
            mptl128_data_in(3 downto 0) = MPTL_READ_DATA or
            mptl128_data_in(3 downto 0) = MPTL_STATUS or
            mptl128_data_in(3 downto 0) = MPTL_COMMAND) and
            chksum = mptl128_data_in(39 downto 36) then
            packet_words_left <= mptl128_data_in(12 downto 8);
            destination <=  mptl128_data_in(7 downto 4);
          else
            err_det <= '1';
            packet_words_left <= (others => '0');
          end if;        
        end if;
      else
        if mptl128_valid_in  = '1' or online_reg = '0'then
          packet_words_left <= packet_words_left-1;      
        end if;    
      end if;
     
      data_reg <= mptl128_data_in;
      data_valid <= mptl128_valid_in;

      for i in 0 to number_of_ports-1 loop
        if CONV_STD_LOGIC_VECTOR(i,4) = destination then
          mptl128_data_out(i) <= data_reg;
          mptl128_valid_out(i) <= data_valid;
        else
          mptl128_data_out(i) <= (others => '0');
          mptl128_valid_out(i) <= '0';
        end if;
      end loop;  -- i
      
      
      error_detected <= err_det;
    end if;
  end process demux0;


end rtl;
