--------------------------------------------------------------------------------
--
-- Module Name      : mpt1l128_packet_mux
--
--                    Copyright 2009 by
--                    Alpha Data Parallel Systems Ltd.
--                    All Rights Reserved.
--
-- Original Author  : Andrew McCormick
-- Created          : 
--
-- Modified By      : 
-- Date             :
-- Change Notes     :     
--
-- Description      : Packet MUX for MPTL 
--
--
-- Dependencies     : mptl
--
-- Disclaimer       : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                    WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                    WARRANTIES IMPLIED OF MERCHANTABILITY, FITNESS FOR A
--                    PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--------------------------------------------------------------------------------



library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.mptl.all;

entity mptl128_packet_mux is

  generic (
    number_of_ports : natural := 5);
  port (
    -- OCP Interface
    clk            : in  std_logic;
    online         : in  std_logic;
    -- MPTL Multi-port input interface
    mptl128_data_in        : in  mptl128_data_array(0 to number_of_ports-1);
    mptl128_valid_in       : in  std_logic_vector(number_of_ports-1 downto 0);
    mptl128_accept_in      : out std_logic_vector(number_of_ports-1 downto 0);
    -- MPTL Single-port output
    mptl128_data_out       : out std_logic_vector(127 downto 0);
    mptl128_valid_out      : out std_logic;
    mptl128_fifo_nfull     : in  std_logic);

end mptl128_packet_mux;

architecture rtl of mptl128_packet_mux is
  
  signal packet_words_left : std_logic_vector(4 downto 0) := (others => '0');
  signal packet_words_left_is_not0 : std_logic := '0';

  signal mptl_mask : std_logic_vector(number_of_ports-1 downto 0) := (others => '0');
  signal next_mptl_mask : std_logic_vector(number_of_ports-1 downto 0) := (others => '0');

  signal valid_in : std_logic;

  signal  mptl128_data_out_i       : mptl128_data_array(0 to number_of_ports-1);
  signal  mptl128_valid_out_i      : std_logic_vector(number_of_ports-1 downto 0);

  signal nfull_reg : std_logic := '0';
  
begin  -- rtl


  --valid_in <= OR_reduce(mptl_mask and mptl128_valid_in) and not nfull_reg;
  valid_in <= OR_reduce(mptl_mask and mptl128_valid_in);-- and not nfull_reg;

  --mptl128_accept_in <= (mptl_mask and mptl128_valid_in) when packet_words_left = "00000" else (others => '0');
  mptl128_accept_in <= (mptl_mask and mptl128_valid_in) when packet_words_left_is_not0 = '0' and nfull_reg = '0' else (others => '0');
  
  mux0: process (clk)
  begin  -- process mux0
    if clk'event and clk = '1' then  -- rising clock edge      
      for i in 0 to number_of_ports-1 loop
        if mptl128_valid_out_i(i) = '1' then
          mptl128_data_out <= mptl128_data_out_i(i);
        end if;
        mptl128_data_out_i(i) <= mptl128_data_in(i);
      end loop;
      mptl128_valid_out <=  OR_reduce(mptl128_valid_out_i);     
      mptl128_valid_out_i <= (others => '0');
      
      if packet_words_left_is_not0 = '0' then
        if valid_in  = '1' then         
          for i in 0 to number_of_ports-1 loop
            if mptl_mask(i) = '1' then           
              --mptl128_data_out_i(i) <= mptl128_data_in(i);
              mptl128_valid_out_i(i) <= '1';
              packet_words_left <= mptl128_data_in(i)(12 downto 8)-1;
              if mptl128_data_in(i)(12 downto 8) /= "00001" then
                packet_words_left_is_not0 <= '1';
              end if;              
            end if;         
          end loop;  -- i
        else
          mptl_mask <= next_mptl_mask;  
        end if;
      else      
        if valid_in  = '1' then
          packet_words_left <= packet_words_left-1;
          if packet_words_left = "00001" then
            packet_words_left_is_not0 <= '0';
          end if; 
          for i in 0 to number_of_ports-1 loop
            if mptl_mask(i) = '1' then            
              --mptl128_data_out_i(i) <= mptl128_data_in(i);
              mptl128_valid_out_i(i) <= '1';
            end if;         
          end loop;  -- i       
        end if;
        if online = '0' then
          packet_words_left_is_not0 <= '0';
        end if;
      end if;
      
      -- next_mptl_mask
      for i in  number_of_ports-1 downto 0 loop
        if mptl128_valid_in(i)= '1' then
          next_mptl_mask <= (others => '0');
          next_mptl_mask(i) <= '1';
        end if;
      end loop;  -- i
      if mptl128_fifo_nfull = '1' then
        next_mptl_mask <= (others => '0');
      end if; 
      nfull_reg <= mptl128_fifo_nfull; 
    end if;
  end process mux0;


end rtl;
