#ifndef _ADATA_COMMONTHREAD_H
#define _ADATA_COMMONTHREAD_H

#include <windows.h>
#include <stdlib.h>

typedef void* (PortableThreadFunction)(void*);

typedef struct _PortableThread {
	HANDLE hThread;
	DWORD dwThreadId;
	PortableThreadFunction*	pFunction;
	void* pArgument;
} PortableThread;

static DWORD WINAPI
_portableThreadStub(
  void* pArgument)
{
  PortableThread* pThread = (PortableThread*)pArgument;

  return (DWORD)(uintptr_t)pThread->pFunction(pThread->pArgument);
}

__inline static PortableThread*
portableThreadCreate(
  PortableThreadFunction* pFunction,
  void* pArgument)
{
  PortableThread* pNew;
  HANDLE hThread;

  pNew = (PortableThread*)malloc(sizeof(*pNew));
  if (NULL == pNew) {
    return NULL;
  }
  pNew->pFunction = pFunction;
  pNew->pArgument = pArgument;

  hThread = CreateThread(NULL, 0, &_portableThreadStub, (void*)pNew, 0, &pNew->dwThreadId);
  if (NULL == hThread) {
    free(pNew);
    return NULL;
  }
  pNew->hThread = hThread;

  return pNew;
}

__inline static BOOL
portableThreadWait(
  PortableThread* pThread,
  unsigned int timeoutMs,
  void** ppResult)
{
  DWORD dwRes, exitCode;

  dwRes = WaitForSingleObject(pThread->hThread, timeoutMs);
  if (WAIT_OBJECT_0 == dwRes) {
    GetExitCodeThread(pThread->hThread, &exitCode);
    CloseHandle(pThread->hThread);
    if (NULL != ppResult) {
      *ppResult = (void*)(uintptr_t)exitCode;
    }
    free(pThread);
    return TRUE;
  }
  return FALSE;
}

__inline static BOOL
portableThreadWaitInfinite(
  PortableThread* pThread,
  void** ppResult)
{
  DWORD dwRes, exitCode;

  dwRes = WaitForSingleObject(pThread->hThread, INFINITE);
  if (WAIT_OBJECT_0 == dwRes) {
    GetExitCodeThread(pThread->hThread, &exitCode);
    CloseHandle(pThread->hThread);
    if (NULL != ppResult) {
      *ppResult = (void*)(uintptr_t)exitCode;
    }
    free(pThread);
    return TRUE;
  }
  return FALSE;
}

__inline static BOOL
portableThreadKill(
  PortableThread* pThread)
{
  DWORD dwRes;

  dwRes = TerminateThread(pThread->hThread, INFINITE);
  if (!dwRes) {
    return FALSE;
  }
  CloseHandle(pThread->hThread);
  free(pThread);
  return TRUE;
}

#endif
