#pragma once

#include <gtkmm/drawingarea.h>

#include <objmodel.h>
#include <tracebuffer.h>
#include "stripedline.h"

class CScopeWnd : public Gtk::DrawingArea, public IScopeWnd
{
public:
  CScopeWnd(IGraphWnd* pParent);
  virtual ~CScopeWnd();
  // IScopeWnd interface
  void addTrace(unsigned int sensorIndex, int at = -1);
  void addTraceDone();
  void adjustScale(float value);
  void deviceChanged();
  void enableTrace(unsigned int traceIndex, bool_t bEnable);
  control_handle_t getNativeControl() { return *this; }
  unsigned int getNumTrace() { return m_numTrace; }
  IGraphWnd* getParent(void) const { return m_pParent; }
  unsigned int getTrace(unsigned int traceIndex) {
    ASSERT(traceIndex < m_numTrace);
    return m_pTraceInfo[traceIndex].m_pTraceBuffer->getSensorIndex();
  };
  void idle();
  void removeAllTraces();
  void update(timestamp_t timestamp);
  void updatePeriodChanged();

protected:
  // Per-trace information
  struct CTraceInfo {
    CTraceBuffer* m_pTraceBuffer;
    bool_t m_bDisplayEnabled;
  };
  // Information about how multiple traces are scaled to this scope
  struct Scaling {
    int m_numYDivision;
    float m_yDivision;
    float m_yMin, m_yMax;
    float m_yScaling;
    bool_t m_bYValid;
  };
  // Layout of axes etc. on scope; all values in pixels relative to client rectangle
  struct Layout{
    int m_width;
    int m_height;
    int m_axisYTop;
    int m_axisXYBottom;
    int m_axisXYLeft;
    int m_axisXRight;
    unsigned int m_axisYHeight;
    unsigned int m_axisXWidth;
  };
  // 2D point
  struct Point {
    int m_x, m_y;
  };

  IGraphWnd* m_pParent;
  Scaling m_scaling;
  Layout m_layout;
  CTraceInfo* m_pTraceInfo;
  unsigned int m_numTrace;
  bool_t m_bMouseOver;
  Point m_mousePos;
#if (GTK_MAJOR_VERSION >= 3)
  Glib::RefPtr<Gdk::Cursor> m_cursor;
#else
  Gdk::Cursor m_cursor;
#endif
  float m_minimum, m_maximum;
  CStripedLine m_lineRenderer;

protected:
  typedef const CTraceBuffer* CTraceBufferConstPtr;
  void calcLayout(Layout& layout);
  void calcYScaling(Scaling& scaling, const CTraceInfo* pTraceInfo, const Layout& layout, unsigned int numTrace);
  void drawAxes(const Cairo::RefPtr<Cairo::Context>& cr);
  void drawGuides(const Cairo::RefPtr<Cairo::Context>& cr, const DeviceInfo& devInfo);
  void drawMessage(const Cairo::RefPtr<Cairo::Context>& cr, const DeviceInfo& devInfo);
  void drawMinimalTicks(const Cairo::RefPtr<Cairo::Context>& cr);
  void drawTicksAndLabels(const Cairo::RefPtr<Cairo::Context>& cr, const SensorInfo* pSensorInfo);
  void drawTrace(const Cairo::RefPtr<Cairo::Context>& cr, SensorInfo* pSensorInfo, CTraceBuffer* pSensorData, timestamp_t t0);
  void needRedraw();
  bool onDraw(const Cairo::RefPtr<Cairo::Context>& cr);
#if (GTK_MAJOR_VERSION < 3)
  bool onExpose(GdkEventExpose* pEvent);
#endif
  bool onMouseEnter(GdkEventCrossing* pEvent);
  bool onMouseLeave(GdkEventCrossing* pEvent);
  bool onMouseMove(GdkEventMotion* pEvent);
  void onRealize();
  void onSizeAllocate(Gtk::Allocation& allocation);
  bool xToTimestamp(int mouseX, timestamp_t t0, unsigned int updatePeriod, timestamp_t& timestamp);
  bool timestampToX(timestamp_t timestamp, timestamp_t t0, unsigned int updatePeriod, int& x);
  bool valueToY(float value, int& y);
};
