#pragma once

#include <objmodel.h>

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

class CDeviceInfoWnd : public CWnd, public IDeviceInfoWnd
{
	DECLARE_DYNAMIC(CDeviceInfoWnd)

public:
	CDeviceInfoWnd(ISysMonWnd* pParent, bool_t bVisible);
	virtual ~CDeviceInfoWnd();
  // IDeviceInfoWnd interface
  void deviceChanged();
  void numDeviceChanged();
  control_handle_t getNativeControl() { return *this; }
  ISysMonWnd* getParent() const { return m_pParent; }

protected:
  static const unsigned int m_maxNumFlashBank = 8;
  static const unsigned int m_maxNumMemoryBank = 16;
  static const unsigned int m_maxNumModuleSite = 4;
  static const unsigned int m_maxNumSensor = 64;
  static const unsigned int m_maxNumTargetFpga = 16;
  static const unsigned int m_maxNumWindow = 16;

  ISysMonWnd* m_pParent;
  unsigned int m_numDevice;
  CFont m_font;
  CBrush m_bgBrush;
  CTreeCtrl m_treeCtl;
  struct {
    HTREEITEM m_hApi;
    HTREEITEM m_hApiItem[2];
    HTREEITEM m_hSummary;
    HTREEITEM m_hSummaryItem[11];
    HTREEITEM m_hTargetFpga;
    struct {
      HTREEITEM m_hFpga;
      HTREEITEM m_hIdentifier;
      HTREEITEM m_hDeviceCode;
      HTREEITEM m_hFamilyCode;
      HTREEITEM m_hPackageCode;
      HTREEITEM m_hDevice;
      HTREEITEM m_hPackage;
      HTREEITEM m_hFlags;
      struct {
        HTREEITEM m_speed;
        HTREEITEM m_stepping;
        HTREEITEM m_notConfigurable;
      } m_flagsItem;
      HTREEITEM m_hSpeedGrade;
      HTREEITEM m_hStepping;
      HTREEITEM m_hPresent;
    } m_targetFpgaItem[m_maxNumTargetFpga];
    HTREEITEM m_hWindow;
    struct {
      HTREEITEM m_hWindow;
      HTREEITEM m_hBusBase;
      HTREEITEM m_hBusLimit;
      HTREEITEM m_hBusSize;
      HTREEITEM m_hLocalBase;
      HTREEITEM m_hLocalLimit;
      HTREEITEM m_hLocalSize;
      HTREEITEM m_hVirtualSize;
    } m_windowItem[m_maxNumWindow];
    HTREEITEM m_hFlash;
    struct {
      HTREEITEM m_hFlash;
      HTREEITEM m_hIdentifier;
      HTREEITEM m_hSize;
      HTREEITEM m_hUseableStart;
      HTREEITEM m_hUseableLimit;
      HTREEITEM m_hUseableLength;
    } m_flashItem[m_maxNumFlashBank];
    HTREEITEM m_hMemory;
    struct {
      HTREEITEM m_hMemoryBank;
      HTREEITEM m_hMaximumFrequency;
      HTREEITEM m_hMinimumFrequency;
      HTREEITEM m_hPhysicalSize;
      HTREEITEM m_hPhysicalDataWidth;
      HTREEITEM m_hPhysicalEccWidth;
      HTREEITEM m_hPhysicalWidth;
      HTREEITEM m_hTypeMask;
      struct {
        HTREEITEM m_hZbtFt;
        HTREEITEM m_hZbtP;
        HTREEITEM m_hSdramDdr;
        HTREEITEM m_hSsramDdr2;
        HTREEITEM m_hSdramSdr;
        HTREEITEM m_hSdramDdr2;
        HTREEITEM m_hSsramQdr;
        HTREEITEM m_hSdramDdr3;
      } m_typeItem;
      HTREEITEM m_hConnectivityMask;
      HTREEITEM m_hPresent;
    } m_memoryItem[m_maxNumMemoryBank];
    HTREEITEM m_hSensor;
    struct {
      HTREEITEM m_hSensor;
      HTREEITEM m_hDescription;
      HTREEITEM m_hUnit;
      HTREEITEM m_hDatatype;
      HTREEITEM m_hExponent;
      HTREEITEM m_hError;
      HTREEITEM m_hCapabilities;
      struct {
        HTREEITEM m_hSoftware;
        HTREEITEM m_hHwMinimum;
        HTREEITEM m_hHwMaximum;
        HTREEITEM m_hHwUnderAlarm;
        HTREEITEM m_hHwOverAlarm;
      } m_capabilitiesItem;
    } m_sensorItem[m_maxNumSensor];
    HTREEITEM m_hModule;
    struct {
      HTREEITEM m_hModule;
      HTREEITEM m_hProduct;
      HTREEITEM m_hPartNumber;
      HTREEITEM m_hManufacturer;
      HTREEITEM m_hSerialNumber;
      HTREEITEM m_hIoVoltage;
      HTREEITEM m_hManufactureDate;
      HTREEITEM m_hFlags;
      struct {
        HTREEITEM m_hLegacyXrm;
        HTREEITEM m_hForce2V5;
        HTREEITEM m_hRomError;
        HTREEITEM m_hCSumError;
      } m_flagsItem;
      HTREEITEM m_hPresent;
    } m_moduleItem[m_maxNumModuleSite];
  } m_treeItem;

protected:
  void setTree(HTREEITEM, const TCHAR* pString);

  DECLARE_MESSAGE_MAP()

protected:
  afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
  afx_msg void OnSize(UINT nType, int cx, int cy);
};
